#!/bin/sh

echo "@@@@@ START: `date`"

INSTDIR=/usr/local/c33
PREFIX=c33-epson-elf-
#ARCHIVE_DIR=$HOME/EpsonC33/archive
ARCHIVE_DIR=/home/c33-gdb/archive
ARCHIVE_DATE=`date +%y%m%d`
DEBUG=${GDB17_DEBUG-no}

SRCDIR=insight-5.1.1
SRCLIST="MAKE.sh EX $SRCDIR utility/share"

EXE=
DLLS=
case `uname -s` in
CYGWIN*)
	EXE=.exe
	DLLS="c33-insight/itcl/itcl/win/cygitcl30.dll \
		c33-insight/itcl/itk/win/cygitk30.dll \
		c33-insight/tcl/win/cygtcl80.dll \
		c33-insight/tcl/win/cygtclpip80.dll \
		c33-insight/tcl/win/cygtclreg80.dll \
		c33-insight/tix/win/tix4180.dll \
		c33-insight/tk/win/cygtk80.dll \
		"
	;;
Linux*)
	;;
*)
	;;
esac
LANG=C

clean=no
config=no
build=no
install=no
tags=no
archive=no
diff=no
copybin=no

if [ $# -eq 0 ]; then
	clean=yes
	config=yes
	build=yes
	install=yes
	tags=yes
fi

while [ $# -gt 0 ]; do
	case $1 in
	all)
		clean=yes
		config=yes
		build=yes
		install=yes
		tags=yes
		;;
	clean)
		clean=yes
		;;
	config)
		config=yes
		;;
	build)
		build=yes
		;;
	install)
		install=yes
		;;
	tags)
		tags=yes
		;;
	archive)
		archive=yes
		;;
	copybin)
		copybin=yes
		;;
	diff)
		diff=yes
		shift
		if [ $# -gt 0 ] && [ -d $1 ]; then
			last=$1
		else
			echo "diff error"
			exit 1
		fi
		;;
	esac

	shift
done

if [ "$clean" = "yes" ]; then
	echo "@@@@@ CLEAN: `date`"

	if [ -e c33-insight ]; then
		mv c33-insight c33-insight-gomi
		rm -rf c33-insight-gomi
	fi
fi

if [ "$config" = "yes" ]; then
	echo "@@@@@ CONFIG: `date`"

	mkdir c33-insight
	(cd c33-insight; \
		../$SRCDIR/configure \
			--target=c33-epson-elf \
			--prefix=$INSTDIR ) || exit 1
fi

if [ "$build" = "yes" ]; then
	echo "@@@@@ BUILD: `date`"

	(cd c33-insight; make) || exit 1

	cp c33-insight/gdb/gdb$EXE c33-gdb$EXE
	for i in gdb; do
		if [ "$DEBUG" = "yes" ]; then
			cp -p c33-$i$EXE c33-$i.dbg$EXE
		fi
		strip c33-$i$EXE
	done

	if [ -n "$DLLS" ]; then
		cp -p $DLLS .
		if [ "$DEBUG" != "yes" ]; then
			strip *.dll
		fi
	fi
fi

if [ "$install" = "yes" ]; then
	echo "@@@@@ INSTALL: `date`"

	#rm -rf c33
	#mkdir c33
	#(cd c33-insight; make install) || exit 1

	mkdir -p $INSTDIR/bin
	for i in gdb; do
#		cp -p c33-$i$EXE $INSTDIR/bin/$PREFIX$i$EXE
		cp -p c33-$i$EXE $INSTDIR/bin/$i$EXE
		if [ "$copybin" = "yes" ]; then
			cp -p c33-$i$EXE c:/gnu33/$i$EXE
		fi
		if [ "$DEBUG" = "yes" ]; then
			cp -p c33-$i.dbg$EXE $INSTDIR/bin/$PREFIX$i.dbg$EXE
		fi
	done
	mv $INSTDIR/share $INSTDIR/share-gomi
	rm -rf $INSTDIR/share-gomi
	cp -rip utility/share $INSTDIR

	if [ -n "$DLLS" ]; then
		cp -p *.dll $INSTDIR/bin/
	fi
fi

if [ "$tags" = "yes" ]; then
	echo "@@@@@ TAGS: `date`"

	rm -rf tags
	ctags -R $SRCDIR
fi

if [ "$archive" = "yes" ]; then
	echo "@@@@@ ARCHIVE: `date`"

	t=$ARCHIVE_DIR/$SRCDIR-$ARCHIVE_DATE.tgz
	echo "Archive to '$t'."
	if [ -f $t ]; then
		echo -n "This file is exist.  Really over write? "
		read x
		if [ -z $x ] || [ $x != "y" ]; then
			echo ""
			echo "Stop archive."
			exit 1
		fi
		rm -rf $t
	fi
	tar cfz $t --exclude-from EX $SRCLIST
fi

if [ "$diff" = "yes" ]; then
	echo "@@@@@ ARCHIVE: `date`"

	t=$ARCHIVE_DIR/`echo $last | sed -e 's:/:-:g'`-$ARCHIVE_DATE.diff
	echo "Diff from $last to '$t'."
	if [ -f $t ]; then
		echo -n "This file is exist.  Really over write? "
		read x
		if [ -z $x ] || [ $x != "y" ]; then
			echo ""
			echo "Stop diff."
			exit 1
		fi
		rm -rf $t
	fi
	for i in $SRCLIST; do
		diff -ruN --exclude-from=EX $last/$i $i >> $t
	done
fi

echo "@@@@@ DONE: `date`"

exit 0
