/* Target-dependent code for the EPSON E0C33 for GDB, the GNU debugger.
   Copyright 1996, Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "frame.h"
#include "inferior.h"
#include "obstack.h"
#include "target.h"
#include "value.h"
#include "bfd.h"
#include "gdb_string.h"
#include "gdbcore.h"
#include "symfile.h"
#include "arch-utils.h"
#include "regcache.h"


static char *c33_generic_reg_names[] = REGISTER_NAMES;

char **c33_register_names = c33_generic_reg_names;

/* add T.Tazaki 2004/04/27 : Reg window >>> */
static char *c33_generic_reg_names_ssp[] = REGISTER_NAMES_SSP;

char **c33_register_names_ssp = c33_generic_reg_names_ssp;

static char *c33_generic_reg_names_usp[] = REGISTER_NAMES_USP;

char **c33_register_names_usp = c33_generic_reg_names_usp;

/* add T.Tazaki 2004/04/27 : Reg window <<< */


/* v[O̖ */
#define INSN_PUSHN      0x0200
#define INSN_LD_RD_SS   0xA400
#define INSN_LD_SD_RS   0xA000
#define INSN_EXT        0xC000
#define INSN_SUB_SP     0x8400
#define INSN_SUB_RD     0x6400
#define INSN_LDB_RD_SPMEM   0x4000
#define INSN_LDUB_RD_SPMEM  0x4400
#define INSN_LDH_RD_SPMEM   0x4800
#define INSN_LDUH_RD_SPMEM  0x4c00
#define INSN_LDW_RD_SPMEM   0x5000

/* Local function declarations.  */

static CORE_ADDR examine_prologue ();
void    c33_frame_find_saved_regs (struct frame_info *,struct frame_saved_regs *);



/******************************************************************************
    INPUT   int             gp
            struct type*    \̖߂l̃^Cv\̂ւ̃|C^
    RETURN  int             1:X^bN蓖ā@0:WX^蓖
        \̖߂lX^bN֊蓖Ă邩ǂ肷
******************************************************************************/
/* Should call_function allocate stack space for a struct return?  */
int
c33_use_struct_convention (gcc_p, type)
     int gcc_p;
     struct type *type;
{
  return (TYPE_NFIELDS (type) > 1 || TYPE_LENGTH (type) > 4);
}


/******************************************************************************
    INPUT   struct frame_info*  t[ւ̃|C^
    RETURN  CORE_ADDR           ֐̃^[AhX
        X^bNt[chain邽߂Ɋ֐̃^[AhX߂֐
            h8300p
******************************************************************************/
/* Function: frame_chain */
/* Given a GDB frame, determine the address of the calling function's frame.
   This will be used to create a new GDB frame struct, and then
   INIT_EXTRA_FRAME_INFO and INIT_FRAME_PC will be called for the new frame.

   For us, the frame address is its stack pointer value, so we look up
   the function prologue to determine the caller's sp value, and return it.  */
CORE_ADDR
c33_frame_chain (thisframe)
     struct frame_info *thisframe;
{
  if (PC_IN_CALL_DUMMY (thisframe->pc, thisframe->frame, thisframe->frame))
    {               /* initialize the from_pc now */
      thisframe->from_pc = generic_read_register_dummy (thisframe->pc,
                            thisframe->frame,
                            PC_REGNUM);
      return thisframe->frame;
    }
  c33_frame_find_saved_regs (thisframe, (struct frame_saved_regs *) 0);
  return thisframe->fsr->regs[SP_REGNUM];
}


/******************************************************************************
    INPUT   CORE_ADDR   v[O̊JnAhX
    RETURN  CORE_ADDR   v[O̖̎߂̃AhX
        v[O̖߃R[hǂݔ΂֐
            h8300QlC33pɕύX
******************************************************************************/
/* Function: skip_prologue
   Return the address of the first code past the prologue of the function.  */
CORE_ADDR
c33_skip_prologue (start_pc)
     CORE_ADDR start_pc;
{
    short insn_word;
    unsigned char   ucScrRegNo;
    int adjust;
    
    /* v[Ȍ */
        /* 1.Z[uhWX^̑ޔpushn */
        /* 2.alr, ahr X^bNɑޔ */
            /* ld.w     %r0, %alr   */
            /* ld.w     %r1, %ahr   */
            /* pushn    %r1         */
        /* 3.[Jϐ̈Ä̊m */
            /* ld.w     %r0, %sp    */
            /* xsub     %r0, imm32  */  /*  sub %sp,imm6    */
            /* ld.w     %sp, %r0    */
        /* 4.WX^ړȂ */
            /* xld.w    %r0,[%sp+24] */
            /* xld.w    %r1,[%sp+28] */

    /* v[Ȍ͂āAv[O̎̃AhX擾 */

    /* ߂o */
    insn_word = read_memory_unsigned_integer (start_pc, 2);

    /* IF pushn߂ */
    if ((insn_word & 0xff00) == INSN_PUSHN){
        /* ̖߂o */
        start_pc += 2;
        insn_word = read_memory_unsigned_integer (start_pc, 2);
    }

    /* WHILE alr,ahr̃WX^ړ߂ */
    while ((insn_word & 0xff00) == INSN_LD_RD_SS){
        /* ̖߂o */
        start_pc += 2;
        insn_word = read_memory_unsigned_integer (start_pc, 2);
    }

    /* IF pushn߂(alr,ahr̃X^bNޔ) */
    if ((insn_word & 0xff00) == INSN_PUSHN){
        /* ̖߂o */
        start_pc += 2;
        insn_word = read_memory_unsigned_integer (start_pc, 2);
    }
    
    /* IF [Jϐ̈Ä̊mۂ̂߂̃X^bN|C^ړ */
    if ((insn_word & 0xfc00) == INSN_SUB_SP){   /* sub %sp,imm10 */
    /* ̈悪A4KB̏ꍇ */
        /* ̖߂o */
        start_pc += 2;
        insn_word = read_memory_unsigned_integer (start_pc, 2);
    }
    else if ((insn_word & 0xff00) == INSN_LD_RD_SS){
    /* ̈悪4KBȏ̏ꍇ */

        /* IF SP̈ړ߂ */
        if ((insn_word & 0x00f0) == 1){ /* insn sp register number */
            /* spړ̂߂ɎgpXNb`WX^ێēꃌWX^ŏĂ邩`FbN */
            ucScrRegNo = insn_word & 0x000f;
            
            /* ̖߂o */
            start_pc += 2;
            insn_word = read_memory_unsigned_integer (start_pc, 2);
            
            /* while xsub߂(ۂext߂ɓWJꂽ) */
            /* ext߂͂P̏ꍇƂQ̏ꍇ̂while */
            while ((insn_word & 0xe000) == INSN_EXT){
                /* ̖߂o */
                start_pc += 2;
                insn_word = read_memory_unsigned_integer (start_pc, 2);
            }
            if ((insn_word & 0xfc00) == INSN_SUB_RD){   /* sub %rd,imm6 */
                
                /* IF spړ̂߂̃XNb`WX^ */
                if (ucScrRegNo == (insn_word & 0x000f)){
                    /* ̖߂o */
                    start_pc += 2;
                    insn_word = read_memory_unsigned_integer (start_pc, 2);
                }
            }
            if ((insn_word & 0xff00) == INSN_LD_SD_RS){ /* ld.w %sp,%rs */
                /* IF spړ̂߂̃XNb`WX^ */
                if (((insn_word & 0x00f0) == ucScrRegNo) && ((insn_word & 0x000f) == 1)){
                    start_pc += 2;
                    insn_word = read_memory_unsigned_integer (start_pc, 2);
                }
            }
        }
    }

    /* ̈̃WX^ړ߂ */
    while (1){
        adjust = 0;

        /* WHILE x߂ */
        /* ext߂͂P̏ꍇƂQ̏ꍇ̂while */
        while ((insn_word & 0xe000) == INSN_EXT){
            /* ̖߂o */
            adjust += 2;
            insn_word = read_memory_unsigned_integer (start_pc+adjust, 2);
        }
        /* IF ̃WX^ړ߂ */
        if (((insn_word & 0xfc00) == INSN_LDB_RD_SPMEM)     /* ld.b %rd,[%sp+imm6] */
            || ((insn_word & 0xfc00) == INSN_LDUB_RD_SPMEM) /* ld.ub %rd,[%sp+imm6] */
            || ((insn_word & 0xfc00) == INSN_LDH_RD_SPMEM)  /* ld.h %rd,[%sp+imm6] */
            || ((insn_word & 0xfc00) == INSN_LDUH_RD_SPMEM) /* ld.uh %rd,[%sp+imm6] */
            || ((insn_word & 0xfc00) == INSN_LDW_RD_SPMEM)){/* ld.w %rd,[%sp+imm6] */
            start_pc += adjust+2;
            insn_word = read_memory_unsigned_integer (start_pc, 2);
        }
        else
            break;
    }

    return start_pc;
}


/******************************************************************************
    INPUT   struct frame_info*          t[i[GAւ̃|C^
            struct frame_saved_regs*    Z[uhWX^i[GAւ̃|C^
    RETURN  void
        t[ۑ֐
            h8300p
******************************************************************************/
/* Put here the code to store, into a struct frame_saved_regs,
   the addresses of the saved registers of frame described by FRAME_INFO.
   This includes special registers such as pc and fp saved in special
   ways in the stack frame.  sp is even more special:
   the address we return for it IS the sp for the next frame.

   We cache the result of doing this in the frame_obstack, since it is
   fairly expensive.  */
void
c33_frame_find_saved_regs (fi, fsr)
     struct frame_info *fi;
     struct frame_saved_regs *fsr;
{
  register struct frame_saved_regs *cache_fsr;
  CORE_ADDR ip;
  struct symtab_and_line sal;
  CORE_ADDR limit;

  if (!fi->fsr)
    {
      cache_fsr = (struct frame_saved_regs *)
    frame_obstack_alloc (sizeof (struct frame_saved_regs));
      memset (cache_fsr, '\0', sizeof (struct frame_saved_regs));

      fi->fsr = cache_fsr;

      if (PC_IN_CALL_DUMMY (fi->pc, fi->frame, fi->frame))
    {           /* no more to do. */
      if (fsr)
        *fsr = *fi->fsr;
      return;
    }
      /* Find the start and end of the function prologue.  If the PC
         is in the function prologue, we only consider the part that
         has executed already.  */

      ip = get_pc_function_start (fi->pc);
      sal = find_pc_line (ip, 0);
      limit = (sal.end && sal.end < fi->pc) ? sal.end : fi->pc;

      /* This will fill in fields in *fi as well as in cache_fsr.  */
      examine_prologue (ip, limit, fi->frame, cache_fsr, fi);
    }

  if (fsr)
    *fsr = *fi->fsr;
}

/******************************************************************************
    INPUT   CORE_ADDR   ߂̊i[ĂAhX
            CORE_ADDR   v[ȌIAhX
            unsigned short  gp
    RETURN  CORE_ADDR   擾߂̖̎߂̃AhX
        v[O̖߂擾 
            h8300p
******************************************************************************/
/* Fetch the instruction at ADDR, returning 0 if ADDR is beyond LIM or
   is not the address of a valid instruction, the address of the next
   instruction beyond ADDR otherwise.  *PWORD1 receives the first word
   of the instruction. */
CORE_ADDR
NEXT_PROLOGUE_INSN (addr, lim, pword1)
     CORE_ADDR addr;
     CORE_ADDR lim;
     unsigned short *pword1;
{
    char buf[2];
    if (addr < lim + 8)
    {
        read_memory (addr, buf, 2);
        *pword1 = extract_signed_integer (buf, 2);
        return addr + 2;
    }
    return 0;
}

/******************************************************************************
    INPUT   CORE_ADDR                   v[O̍ŏ̖߃AhX
            CORE_ADDR                   v[O̍Ō̖߃AhX
            CORE_ADDR                   SPAhXl
            struct frame_saved_regs*    t[̃Z[uhWX^\̂ւ̃|C^
            struct frame_info*          t[\̂ւ̃|C^
    RETURN  CORE_ADDR                   擾߂̖̎߂̃AhX
        v[O𒲍
            h8300QlC33pɕύX
******************************************************************************/
/* Examine the prologue of a function.  `ip' points to the first instruction.
   `limit' is the limit of the prologue (e.g. the addr of the first
   linenumber, or perhaps the program counter if we're stepping through).
   `frame_sp' is the stack pointer value in use in this frame.
   `fsr' is a pointer to a frame_saved_regs structure into which we put
   info about the registers saved by this frame.
   `fi' is a struct frame_info pointer; we fill in various fields in it
   to reflect the offsets of the arg pointer and the locals pointer.  */
static CORE_ADDR
examine_prologue (ip, limit, after_prolog_fp, fsr, fi)
     register CORE_ADDR ip;
     register CORE_ADDR limit;
     CORE_ADDR after_prolog_fp;
     struct frame_saved_regs *fsr;
     struct frame_info *fi;
{
  register CORE_ADDR next_ip;
  int r;
  int have_fp = 0;
  unsigned short insn_word;
  /* Number of things pushed onto stack, starts at 2/4, 'cause the
     PC is already there */
//  unsigned int reg_save_depth = 4;

    unsigned int auto_depth = 0;    /* Number of bytes of autos */
    unsigned int tmp_depth = 0;
    int count;
    int iPushnCnt = 0;
    int i;
    unsigned char   ucScrRegNo;
    
// char in_frame[11];       /* One for each reg */

  int adjust = 0;

//  memset (in_frame, 1, 11);

    /*  */
    for (r = 0; r < NUM_REGS; r++)
    {
        fsr->regs[r] = 0;
    }

    if (after_prolog_fp == 0)
    {
        /* SPWX^lǂݍ */
        after_prolog_fp = read_register (SP_REGNUM);
    }

    /* If the PC isn't valid, quit now.  */
    if (ip == 0)
        return 0;

    /* v[Ȍ */
        /* 1.Z[uhWX^̑ޔpushn */
        /* 2.alr, ahr X^bNɑޔ */
            /* ld.w     %r0, %alr   */
            /* ld.w     %r1, %ahr   */
            /* pushn    %r1         */
        /* 3.[Jϐ̈Ä̊m */
            /* ld.w     %r0, %sp    */
            /* xsub     %r0, imm32  */  /*  sub %sp,imm6    */
            /* ld.w     %sp, %r0    */
        /* 4.WX^ړȂ(X^bNt[ɂ͊֌WȂ) */

    /* v[Ȍ͂āAX^bNt[擾 */

    /* ߂o */
    next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);

    /* IF pushn߂ */
    if ((insn_word & 0xff00) == INSN_PUSHN){
        iPushnCnt = insn_word & 0x0f;
        /* ̖߂o */
        ip = next_ip;
        next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
    }

    /* WHILE alr,ahr̃WX^ړ߂ */
    while (next_ip && ((insn_word & 0xff00) == INSN_LD_RD_SS)){
        /* ̖߂o */
        ip = next_ip;
        next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
    }

    /* IF pushn߂(alr,ahr̃X^bNޔ) */
    if ((insn_word & 0xff00) == INSN_PUSHN){
        count = insn_word & 0x0f;

        /* ^[AhX܂ł̃X^bNTCYXV */
        auto_depth += 4 * (1+count);

        /* ̖߂o */
        ip = next_ip;
        next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
    }
    
    /* IF [Jϐ̈Ä̊mۂ̂߂̃X^bN|C^ړ */
    if ((insn_word & 0xfc00) == INSN_SUB_SP){   /* sub %sp,imm10 */
    /* ̈悪A4KB̏ꍇ */
        count = insn_word & 0x3ff;
        auto_depth += count * 4;
    }
    else if (next_ip && ((insn_word & 0xff00) == INSN_LD_RD_SS)){
    /* ̈悪4KBȏ̏ꍇ */

        /* IF SP̈ړ߂ */
        if ((insn_word & 0x00f0) == 1){ /* insn sp register number */
            /* spړ̂߂ɎgpXNb`WX^ێēꃌWX^ŏĂ邩`FbN */
            ucScrRegNo = insn_word & 0x000f;
            
            /* ̖߂o */
            ip = next_ip;
            next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
            
            /* while xsub߂(ۂext߂ɓWJꂽ) */
            /* ext߂͂P̏ꍇƂQ̏ꍇ̂while */
            while (next_ip && ((insn_word & 0xe000) == INSN_EXT)){
                tmp_depth += (insn_word & 0x1fff);
                /* ̖߂o */
                ip = next_ip;
                next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
            }
            if (next_ip && ((insn_word & 0xfc00) == INSN_SUB_RD)){  /* sub %rd,imm6 */
                tmp_depth += (insn_word & 0x03f0) >> 4;
                
                /* IF spړ̂߂̃XNb`WX^łȂWX^sub߂ */
                if (ucScrRegNo != (insn_word & 0x000f)){
                    tmp_depth = 0;
                }
                /* ̖߂o */
                ip = next_ip;
                next_ip = NEXT_PROLOGUE_INSN (ip, limit, &insn_word);
            }
            if ((insn_word & 0xff00) == INSN_LD_SD_RS){ /* ld.w %sp,%rs */
                /* IF spړ̂߂̃XNb`WX^łȂWX^ld߂ */
                if (((insn_word & 0x00f0) != ucScrRegNo) || ((insn_word & 0x000f) != 1)){
                    tmp_depth = 0;
                }
                auto_depth += tmp_depth * 4;
            }
        }
    }

    /* Z[uhWX^i[AhXۑ */
    if (iPushnCnt > 0){
        for (i = 0;i<=iPushnCnt;i++){
            fsr->regs[i] = after_prolog_fp + auto_depth;

            /* ^[AhX܂ł̃X^bNTCYXV */
            auto_depth += 4;
        }
    }

    /* The args are always reffed based from the stack pointer */
    fi->args_pointer = after_prolog_fp;
    /* Locals are always reffed based from the fp */
    fi->locals_pointer = after_prolog_fp;
    /* The PC is at a known place */
    fi->from_pc = read_memory_unsigned_integer (after_prolog_fp + auto_depth, 4);

    fsr->regs[SP_REGNUM] = after_prolog_fp + auto_depth+4;

    return (ip);
}

/******************************************************************************
    INPUT   struct frame_info*  t[\̂ւ̃|C^
    RETURN  void
        X^bNt[̏
            h8300p
******************************************************************************/
void
c33_init_extra_frame_info (fi)
     struct frame_info *fi;
{
  fi->fsr = 0;          /* Not yet allocated */
  fi->args_pointer = 0;     /* Unknown */
  fi->locals_pointer = 0;   /* Unknown */
  fi->from_pc = 0;
  if (PC_IN_CALL_DUMMY (fi->pc, fi->frame, fi->frame))
    {               /* anything special to do? */
      return;
    }
}

/******************************************************************************
    INPUT   struct frame_info*  t[\̂ւ̃|C^
    RETURN  CORE_ADDR           ^[AhX
        Z[uhPC(^[AhXj擾֐
            h8300p
******************************************************************************/
/* Function: frame_saved_pc 
   Return the saved PC from this frame.
   Find the caller of this frame.  We do this by seeing if RP_REGNUM
   is saved in the stack anywhere, otherwise we get it from the
   registers.  If the inner frame is a dummy frame, return its PC
   instead of RP, because that's where "caller" of the dummy-frame
   will be found.  */
CORE_ADDR
c33_frame_saved_pc (frame)
     struct frame_info *frame;
{
    if (PC_IN_CALL_DUMMY (frame->pc, frame->frame, frame->frame))
        return generic_read_register_dummy (frame->pc, frame->frame, PC_REGNUM);
    else{
        if (!frame->from_pc)
        {
            struct frame_saved_regs ignore;
            get_frame_saved_regs (frame, &ignore);
        }
        return frame->from_pc;
    }
}


/******************************************************************************
    INPUT   struct frame_info*  t[\̂ւ̃|C^
    RETURN  CORE_ADDR           X^bNt[̃[Jϐ̈AhX
        X^bNt[̃[Jϐ̈̃AhX擾֐
            h8300p
******************************************************************************/
CORE_ADDR
c33_frame_locals_address (fi)
     struct frame_info *fi;
{
  if (PC_IN_CALL_DUMMY (fi->pc, fi->frame, fi->frame))
    return (CORE_ADDR) 0;   /* Not sure what else to do... */
  if (!fi->locals_pointer)
    {
      struct frame_saved_regs ignore;

      get_frame_saved_regs (fi, &ignore);

    }
  return fi->locals_pointer;
}

/******************************************************************************
    INPUT   struct frame_info*  t[\̂ւ̃|C^
    RETURN  CORE_ADDR           X^bNt[̈̈AhX
        X^bNt[̈̈̃AhX擾֐
            h8300p
******************************************************************************/
/* Return the address of the argument block for the frame
   described by FI.  Returns 0 if the address is unknown.  */

CORE_ADDR
c33_frame_args_address (fi)
     struct frame_info *fi;
{
  if (PC_IN_CALL_DUMMY (fi->pc, fi->frame, fi->frame))
    return (CORE_ADDR) 0;   /* Not sure what else to do... */
  if (!fi->args_pointer)
    {
      struct frame_saved_regs ignore;

      get_frame_saved_regs (fi, &ignore);

    }
  return fi->args_pointer;
}

/******************************************************************************
    INPUT   int             ̐
            value_ptr*      
            CORE_ADDR       
            unsigned char   
            CORE_ADDR       
    RETURN  CORE_ADDR       
        WX^AX^bNt[ɐς
            h8300p
******************************************************************************/
/* Function: push_arguments
   Setup arguments and RP for a call to the target.  First four args
   go in R6->R9, subsequent args go into sp + 16 -> sp + ...  Structs
   are passed by reference.  64 bit quantities (doubles and long
   longs) may be split between the regs and the stack.  When calling a
   function that returns a struct, a pointer to the struct is passed
   in as a secret first argument (always in R6).

   Stack space for the args has NOT been allocated: that job is up to us.
 */
CORE_ADDR
c33_push_arguments (nargs, args, sp, struct_return, struct_addr)
     int nargs;
/*     value_ptr *args;      Modify 2002.07.10 T.Tazaki */
     struct value **args; /* Modify 2002.07.10 T.Tazaki */
     CORE_ADDR sp;
     unsigned char struct_return;
     CORE_ADDR struct_addr;
{
  int argreg;
  int argnum;
  int len = 0;
  int stack_offset;

  /* First, just for safety, make sure stack is aligned */
  sp &= ~3;

#if 1   /* c33 */
  /* Now make space on the stack for the args. */
  for (argnum = 0; argnum < nargs; argnum++)
    len += ((TYPE_LENGTH (VALUE_TYPE (args[argnum])) + 3) & ~3);
  sp -= len;            /* possibly over-allocating, but it works... */
#else

    if (nargs > 4){
  for (argnum = 4; argnum < nargs; argnum++)
    len += ((TYPE_LENGTH (VALUE_TYPE (args[argnum])) + 3) & ~3);
  sp -= len;            /* possibly over-allocating, but it works... */
    }
#endif


  /* (you might think we could allocate 16 bytes */
  /* less, but the ABI seems to use it all! )  */
  argreg = ARG0_REGNUM;

  /* the struct_return pointer occupies the first parameter-passing reg */
  if (struct_return)
    write_register (argreg++, struct_addr);

  stack_offset = 0; /* c33 */
  /* The offset onto the stack at which we will start copying parameters
     (after the registers are used up) begins at 16 rather than at zero.
     I don't really know why, that's just the way it seems to work.  */

  /* Now load as many as possible of the first arguments into
     registers, and push the rest onto the stack.  There are 16 bytes
     in four registers available.  Loop thru args from first to last.  */
    for (argnum = 0; argnum < nargs; argnum++)
    {
        int len;
        char *val;
        char valbuf[REGISTER_RAW_SIZE (ARG0_REGNUM)];

        /* \̈ŃTCY8傫 */
        if (TYPE_CODE (VALUE_TYPE (*args)) == TYPE_CODE_STRUCT
            && TYPE_LENGTH (VALUE_TYPE (*args)) > 8)
        {
            store_address (valbuf, 4, VALUE_ADDRESS (*args));
            len = 4;
            val = valbuf;
        }
        else
        {
        /* NO TCY8ȉ */
        
            len = TYPE_LENGTH (VALUE_TYPE (*args));
            val = (char *) VALUE_CONTENTS (*args);
        }

        while (len > 0){
            if (argreg <= ARGLAST_REGNUM)
            {
                CORE_ADDR regval;

                regval = extract_address (val, REGISTER_RAW_SIZE (argreg));

                /* WX^Ɉi[ */
                write_register (argreg, regval);
                len -= REGISTER_RAW_SIZE (argreg);
                val += REGISTER_RAW_SIZE (argreg);
                argreg++;
            }
            else
            {
                /* X^bNɈi[ */
                write_memory (sp + stack_offset, val, 4);
                len -= 4;
                val += 4;
                stack_offset += 4;
            }
        }
        args++;
    }
    return sp;
}


/* Function: push_return_address
   Setup the return address for a dummy frame, as called by
   call_function_by_hand.  Only necessary when you are using an
   empty CALL_DUMMY, ie. the target will not actually be executing
   a JSR/BSR instruction.  */
/* h8300p */
/* X^bNt[ɖ߂AhXZbg */
CORE_ADDR
c33_push_return_address (pc, sp)
     CORE_ADDR pc;
     CORE_ADDR sp;
{
    unsigned char buf[4];
    int wordsize;

    wordsize = 4;
    sp -= wordsize;
    store_unsigned_integer (buf, wordsize, CALL_DUMMY_ADDRESS ());
    write_memory (sp, buf, wordsize);
    return sp;
}


/* Function: pop_frame
   Restore the machine to the state it had before the current frame 
   was created.  Usually used either by the "RETURN" command, or by
   call_function_by_hand after the dummy_frame is finished. */
/* h8300QlC33pɕύX */
/* X^bNt[̓eWX^ɖ߂ */
void
c33_pop_frame ()
{
  unsigned regnum;
  struct frame_saved_regs fsr;
  struct frame_info *frame = get_current_frame ();

  if (PC_IN_CALL_DUMMY (frame->pc, frame->frame, frame->frame))
    {
      generic_pop_dummy_frame ();
    }
  else
    {
      get_frame_saved_regs (frame, &fsr);

      for (regnum = 0; regnum < NUM_REGS; regnum++)
    {
      /* Don't forget SP_REGNUM is a frame_saved_regs struct is the
         actual value we want, not the address of the value we want.  */
      if (fsr.regs[regnum] && regnum != SP_REGNUM)
        write_register (regnum,
                read_memory_integer (fsr.regs[regnum], 4));
      else if (fsr.regs[regnum] && regnum == SP_REGNUM)
        write_register (regnum, fsr.regs[regnum]);

    }
    /* Don't forget the update the PC too!  */
    write_pc (frame->from_pc);

    }
  flush_cached_frames ();
}


/* Change the register names based on the current machine type. */
/* ^[QbgA[LeN`𒲂ׂ */
static int
c33_target_architecture_hook (ap)
     const bfd_arch_info_type *ap;
{
    /* A[LeN`c33łȂ΁AOԂ */
    /* ɂAgdbŃ[jO\ */
    if (ap->arch != bfd_arch_c33)
        return 0;

    tm_print_insn_info.mach = bfd_mach_c33;
    return 1;
}

/* ֐ */
void
_initialize_c33_tdep ()
{
    /* disassemble̊֐ݒ肷 */
    tm_print_insn = print_insn_c33;

    /* target architecture`FbN֐ݒ肷 */
    target_architecture_hook = c33_target_architecture_hook;
}

