/* Parameters for execution on an EPSON C33 processor.
   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002
   Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "regcache.h"

#define TARGET_BYTE_ORDER LITTLE_ENDIAN

//#define NUM_REGS 29	//change T.Tazaki 2003/11/26
#define NUM_REGS 31		//change T.Tazaki 2003/11/26
#define NUM_REGS_STD 21

//add %idir, %dbbr  T.Tazaki 2003/12/03
#define REGISTER_NAMES \
{ "r0",  "r1",  "r2",   "r3",   "r4",   "r5",   "r6",   "r7",  \
  "r8",  "r9",  "r10",  "r11",  "r12",  "r13",  "r14",  "r15", \
  "psr", "sp",  "alr",  "ahr",  "lco" , "lsa" , "lea" , "sor", \
  "ttbr","dp" ,"idir","dbbr", "usp" , "ssp" , "pc" \
}

//for GUI Mode add T.Tazaki 2004/04/27 >>>
#define REGISTER_NAMES_SSP \
{ "r0",  "r1",  "r2",   "r3",   "r4",   "r5",   "r6",   "r7",  \
  "r8",  "r9",  "r10",  "r11",  "r12",  "r13",  "r14",  "r15", \
  "psr", "  ",  "alr",  "ahr",  "lco" , "lsa" , "lea" , "sor", \
  "ttbr","dp" ,"idir","dbbr", "usp" , "ssp(sp)" , "pc" \
}

#define REGISTER_NAMES_USP \
{ "r0",  "r1",  "r2",   "r3",   "r4",   "r5",   "r6",   "r7",  \
  "r8",  "r9",  "r10",  "r11",  "r12",  "r13",  "r14",  "r15", \
  "psr", "  ",  "alr",  "ahr",  "lco" , "lsa" , "lea" , "sor", \
  "ttbr","dp" ,"idir","dbbr", "usp(sp)" , "ssp" , "pc" \
}
//for GUI Mode add T.Tazaki 2004/04/27 <<<

/* Initializer for an array of names of registers.
   Entries beyond the first NUM_REGS are ignored.  */

extern char **c33_register_names;
#define REGISTER_NAME(i) c33_register_names[i]

//for GUI Mode add T.Tazaki 2004/04/27 >>>
extern char **c33_register_names_ssp;
extern char **c33_register_names_usp;
#define REGISTER_NAME_SSP(i) c33_register_names_ssp[i]
#define REGISTER_NAME_USP(i) c33_register_names_usp[i]
//for GUI Mode add T.Tazaki 2004/04/27 <<<

#define REGISTER_BYTES (NUM_REGS * 4)

#define REGISTER_SIZE 4
#define MAX_REGISTER_RAW_SIZE 4

#define R0_REGNUM 0
#define ARG0_REGNUM 6
#define ARGLAST_REGNUM 9
#define PSR_REGNUM 16
#define SP_REGNUM 17
#define ALR_REGNUM 18
#define AHR_REGNUM 19
#define LCO_REGNUM 20
#define LSA_REGNUM 21
#define LEA_REGNUM 22
#define SOR_REGNUM 23
#define TTBR_REGNUM 24
#define DP_REGNUM 25
#define IDIR_REGNUM 26
#define DBBR_REGNUM 27
#define USP_REGNUM 28
#define SSP_REGNUM 29
#define PC_REGNUM 30
#define RETURN_REGNUM 4
#define FP_REGNUM 0

#define REGISTER_VIRTUAL_TYPE(REG) builtin_type_int

#define REGISTER_BYTE(REG) ((REG) * 4)
#define REGISTER_VIRTUAL_SIZE(REG) 4
#define REGISTER_RAW_SIZE(REG) 4

#define MAX_REGISTER_VIRTUAL_SIZE 4

#define BREAKPOINT {0x00, 0x04} /* little-ended */

#define FUNCTION_START_OFFSET 0

#define DECR_PC_AFTER_BREAK 0

#define INNER_THAN(lhs,rhs) ((lhs) < (rhs))

#define SAVED_PC_AFTER_CALL(frame) (read_memory_integer (read_register (SP_REGNUM), 4))

struct frame_info;
struct frame_saved_regs;
struct type;
struct value;

#define EXTRA_FRAME_INFO struct frame_saved_regs *fsr; \
                         CORE_ADDR from_pc;            \
                         CORE_ADDR args_pointer;       \
                         CORE_ADDR locals_pointer;

extern void c33_init_extra_frame_info PARAMS ((struct frame_info * fi));
#define INIT_EXTRA_FRAME_INFO(fromleaf, fi) c33_init_extra_frame_info (fi)

/* Put here the code to store, into a struct frame_saved_regs,
   the addresses of the saved registers of frame described by FRAME_INFO.
   This includes special registers such as pc and fp saved in special
   ways in the stack frame.  sp is even more special:
   the address we return for it IS the sp for the next frame.  */

#define FRAME_FIND_SAVED_REGS(frame_info, frame_saved_regs) \
        c33_frame_find_saved_regs(frame_info, &(frame_saved_regs))

extern CORE_ADDR c33_frame_chain PARAMS ((struct frame_info * fi));
#define FRAME_CHAIN(fi) c33_frame_chain (fi)
#define FRAME_CHAIN_VALID(FP, FI)	generic_file_frame_chain_valid (FP, FI)

#define EXTRACT_RETURN_VALUE(TYPE, REGBUF, VALBUF) \
        memcpy (VALBUF, REGBUF + REGISTER_BYTE (RETURN_REGNUM), TYPE_LENGTH (TYPE))

#define EXTRACT_STRUCT_VALUE_ADDRESS(REGBUF) \
        extract_address (REGBUF + REGISTER_BYTE (RETURN_REGNUM), REGISTER_RAW_SIZE (RETURN_REGNUM))

#define STORE_RETURN_VALUE(TYPE, VALBUF) \
        write_register_bytes(REGISTER_BYTE (RETURN_REGNUM), VALBUF, TYPE_LENGTH (TYPE));

extern CORE_ADDR c33_skip_prologue PARAMS ((CORE_ADDR pc));
#define SKIP_PROLOGUE(pc) (c33_skip_prologue (pc))

#define FRAME_ARGS_SKIP 0

/* Define other aspects of the stack frame.  */

/* A macro that tells us whether the function invocation represented
   by FI does not have a frame on the stack associated with it.  If it
   does not, FRAMELESS is set to 1, else 0.  */
#define FRAMELESS_FUNCTION_INVOCATION(FI) (frameless_look_for_prologue (FI))

extern CORE_ADDR c33_frame_saved_pc PARAMS ((struct frame_info *));
extern CORE_ADDR c33_frame_args_address PARAMS ((struct frame_info *));
extern CORE_ADDR c33_frame_locals_address PARAMS ((struct frame_info *));
#define FRAME_SAVED_PC(FRAME) c33_frame_saved_pc(FRAME)
#define FRAME_ARGS_ADDRESS(fi) c33_frame_args_address(fi)
#define FRAME_LOCALS_ADDRESS(fi) c33_frame_locals_address(fi)

#define FRAME_NUM_ARGS(fi) (-1)

extern void c33_pop_frame ();
#define POP_FRAME c33_pop_frame ()

#define USE_GENERIC_DUMMY_FRAMES 1
#define CALL_DUMMY                   {0}
#define CALL_DUMMY_START_OFFSET      (0)
#define CALL_DUMMY_BREAKPOINT_OFFSET (0)
#define CALL_DUMMY_LOCATION          AT_ENTRY_POINT
#define FIX_CALL_DUMMY(DUMMY, START, FUNADDR, NARGS, ARGS, TYPE, GCCP)
#define CALL_DUMMY_ADDRESS()         entry_point_address ()

extern CORE_ADDR c33_push_return_address PARAMS ((CORE_ADDR, CORE_ADDR));
#define PUSH_RETURN_ADDRESS(PC, SP) c33_push_return_address (PC, SP)

#define PUSH_DUMMY_FRAME	generic_push_dummy_frame ()

extern CORE_ADDR
c33_push_arguments PARAMS ((int nargs, struct value ** args, CORE_ADDR sp,
                            unsigned char struct_return,
                            CORE_ADDR struct_addr));
#define PUSH_ARGUMENTS(NARGS, ARGS, SP, STRUCT_RETURN, STRUCT_ADDR) \
        (c33_push_arguments (NARGS, ARGS, SP, STRUCT_RETURN, STRUCT_ADDR))

#define STORE_STRUCT_RETURN(STRUCT_ADDR, SP)

#define PC_IN_CALL_DUMMY(PC, SP, FP) generic_pc_in_call_dummy (PC, SP, FP)

extern use_struct_convention_fn c33_use_struct_convention;
#define USE_STRUCT_CONVENTION(GCC_P, TYPE) c33_use_struct_convention (GCC_P, TYPE);

/* override the default get_saved_register function with
   one that takes account of generic CALL_DUMMY frames */

#define GET_SAVED_REGISTER(raw_buffer, optimized, addrp, frame, regnum, lval) \
        generic_get_saved_register (raw_buffer, optimized, addrp, frame, regnum, lval)

/* hardware debugging registers */
#define TARGET_HAS_HARDWARE_WATCHPOINTS

#define TARGET_CAN_USE_HARDWARE_WATCHPOINT(type, cnt, ot) 1

#define target_insert_hw_breakpoint(addr, shadow) c33_insert_hw_breakpoint(addr, shadow)
extern int c33_insert_hw_breakpoint (CORE_ADDR, void *);

#define target_remove_hw_breakpoint(addr, shadow) c33_remove_hw_breakpoint(addr, shadow)
extern int c33_remove_hw_breakpoint (CORE_ADDR, void *);

#define target_insert_watchpoint(addr, len, type) c33_insert_watchpoint (addr, len, type)
extern int c33_insert_watchpoint (CORE_ADDR, int, int);

#define target_remove_watchpoint(addr, len, type) c33_remove_watchpoint (addr, len, type)
extern int c33_remove_watchpoint (CORE_ADDR, int, int);

/* Define this for Wingdb */

#define	TARGET_C33
