/* Generic memory read/write for hardware simulator models.
   Copyright (C) 1997 Free Software Foundation, Inc.
   Contributed by Cygnus Support.

This file is part of GDB, the GNU debugger.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#include "sim-main.h"
#include "sim-assert.h"

/* Generic implementation of sim_read that works with simulators
   modeling real hardware */

/***************************************************
 * c33Common Target Information define and struct *
****************************************************/
#define CORE_ADDR   unsigned long
#include "../../gdb/c33common.h"

/* >>>>> change iruma m.takeishi '03.02.27 */
/*
static struct c33_target_info copy_c33TargetInfo; // target information //
*/
static struct c33_target_info *p_c33TargetInfo;   // target information //
/* <<<<< change iruma m.takeishi '03.02.27 */
static int iMapSetted = 0;

/***************************************************
 * c33Common Target Information struct Get *
****************************************************/
/* >>>>> change iruma m.takeishi '03.02.27 */
/*
void
sim_map_set2( struct c33_target_info org_c33TargetInfo )
{
*/
void
sim_map_set2( struct c33_target_info *org_c33TargetInfo )
{
    iMapSetted = 1;

    //copy_c33TargetInfo = org_c33TargetInfo;
    p_c33TargetInfo = org_c33TargetInfo;
}
/* <<<<< change iruma m.takeishi '03.02.27 */

/***************************************************
 * Simulation memory read *
****************************************************/
int
sim_read (SIM_DESC sd, SIM_ADDR mem, unsigned char *buf, int length)
{
    unsigned char ucData1,ucData2,ucData3,ucData4;
    struct c33_map_info *pstMapInfo;
    int iEndian = C33_LITTLE;
    int iType;
    int iStatus;

    SIM_ASSERT (STATE_MAGIC (sd) == SIM_MAGIC_NUMBER);

    iStatus = sim_core_read_buffer (sd, NULL, read_map,
                   buf, mem, length);

    if( iMapSetted == 1 )
    {
        /* >>>>> change iruma m.takeishi '03.02.27 */
        /*
        if( copy_c33TargetInfo.pMapInfo != NULL )
        */
        if( p_c33TargetInfo->pMapInfo != NULL )
        /* <<<<< change iruma m.takeishi '03.02.27 */
        {

            iType = C33_BYTE;
            if( ( length % 2 ) == 0 ) iType = C33_HALF_WORD;
            if( ( length % 4 ) == 0 ) iType = C33_WORD;

            /* >>>>> change iruma m.takeishi '03.02.27 */
            /*
            pstMapInfo = (struct c33_map_info *)copy_c33TargetInfo.pMapInfo;
            */
            pstMapInfo = (struct c33_map_info *)(p_c33TargetInfo->pMapInfo);
            /* <<<<< change iruma m.takeishi '03.02.27 */
            
            while( pstMapInfo )
            {
                /* check if address renge */
                if(( mem >= pstMapInfo->ulStartAddr ) && ( mem <= pstMapInfo->ulEndAddr ))
                {
                    iEndian = pstMapInfo->iEndian;
                    break;
                }
                
                pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
            }
        
            if( iEndian == C33_BIG ){
                /* data swap */
                switch( iType )
                {
                    case C33_BYTE:
                        break;
                        
                    case C33_HALF_WORD:
                        ucData1 = *buf;
                        ucData2 = *( buf + 1 );
                
                        *buf = ucData2;
                        *( buf + 1 ) = ucData1;
                        break;
                        
                    case C33_WORD:
                        ucData1 = *buf;
                        ucData2 = *( buf + 1 );
                        ucData3 = *( buf + 2 );
                        ucData4 = *( buf + 3 );
                
                        *buf = ucData4;
                        *( buf + 1 ) = ucData3;
                        *( buf + 2 ) = ucData2;
                        *( buf + 3 ) = ucData1;
                        break;
                }
            }
        }
    }

    return iStatus;
}

/***************************************************
 * Simulation memory write *
****************************************************/
int
sim_write (SIM_DESC sd, SIM_ADDR mem, unsigned char *buf, int length)
{
    unsigned char ucData1,ucData2,ucData3,ucData4;
    struct c33_map_info *pstMapInfo;
    int iEndian = C33_LITTLE;
    int iType;
    unsigned char  szbuf[20];
    unsigned char  *pszbuf;
    
    pszbuf = &szbuf[0];

    SIM_ASSERT (STATE_MAGIC (sd) == SIM_MAGIC_NUMBER);
  
    
    if( iMapSetted == 1 )
    {
        /* >>>>> change iruma m.takeishi '03.02.27 */
        /*
        if( copy_c33TargetInfo.pMapInfo != NULL )
        */
        if( p_c33TargetInfo->pMapInfo != NULL )
        /* >>>>> change iruma m.takeishi '03.02.27 */
        {

            iType = C33_BYTE;
            if( ( length % 2 ) == 0 ) iType = C33_HALF_WORD;
            if( ( length % 4 ) == 0 ) iType = C33_WORD;

            /* >>>>> change iruma m.takeishi '03.02.27 */
            /*
            pstMapInfo = (struct c33_map_info *)copy_c33TargetInfo.pMapInfo;
            */
            pstMapInfo = (struct c33_map_info *)(p_c33TargetInfo->pMapInfo);
            /* <<<<< change iruma m.takeishi '03.02.27 */
            
            while( pstMapInfo )
            {
                /* check if address renge */
                if(( mem >= pstMapInfo->ulStartAddr ) && ( mem <= pstMapInfo->ulEndAddr ))
                {
                    iEndian = pstMapInfo->iEndian;
                    break;
                }
                
                pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
            }
        
            if( iEndian == C33_BIG ){
                /* data swap */
                switch( iType )
                {
                    case C33_BYTE:
                        *pszbuf = *buf;
                        break;

                    case C33_HALF_WORD:
                        ucData1 = *buf;
                        ucData2 = *( buf + 1 );
                
                        *pszbuf = ucData2;
                        *( pszbuf + 1 ) = ucData1;
                        break;
                        
                    case C33_WORD:
                        ucData1 = *buf;
                        ucData2 = *( buf + 1 );
                        ucData3 = *( buf + 2 );
                        ucData4 = *( buf + 3 );
                
                        *pszbuf = ucData4;
                        *( pszbuf + 1 ) = ucData3;
                        *( pszbuf + 2 ) = ucData2;
                        *( pszbuf + 3 ) = ucData1;
                        break;
                }

                return sim_core_write_buffer (sd, NULL, write_map, pszbuf, mem, length);
            }
        }
    }

    return sim_core_write_buffer (sd, NULL, write_map, buf, mem, length);
}

