/*
 * Created on 2005/03/31
 * Modified on 2005/06/17 by D.Fujimoto added method autoUpdateSections() and
 * 										modified create().
 * Modified on 2005/07/07 by D.Fujimoto modified create()
 * 										added C++ nature configs
 * 										added method addCppNature().
 * Modified on 2005/10/11 by D.Fujimoto added renameProject(),
 * 											replaceLinkerScriptFile(),
 * 											replaceMakeArgs().
 * 										refactored deleteLaunchConfig().
 * Modified on 2005/11/17 by D.Fujimoto modified deleteLaunchConfig() to not to
 * 											delete when .launch file is not shared.
 * Modified on 2005/11/18 by D.Fujimoto modified create(), addGnu33Descriptor() for
 * 											initial vector section settings.
 * Modified on 2005/12/07 by D.Fujimoto modified deleteLaunchConfig() to rewrite history
 * 											when a Shared configuration gets deleted.
 * Modified on 2005/12/37 by D.Fujimoto modified configureExternalTool() variablized tool path.
 * Modified on 2006/01/26 by D.Fujimoto modified configureExternalTool() to save launch config
 * 											to project directory.
 * Modified on 2006/07/21 by D.Fujimoto modifed createCmdFile() to create a default cmd file. 
 * Modified on 2006/11/27 by D.Fujimoto modifed configureExternalTool() arguments.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/03/22 by D.Fujimoto overloaded deleteLaunchConfig() and 
 * 											modified renameProject() to delete launch configs.
 */
package jp.co.epson.ide.eclipse.gnu33;

import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptUpdater;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.make.core.IMakeCommonBuildInfo;
import org.eclipse.cdt.make.core.MakeBuilder;
import org.eclipse.cdt.make.core.scannerconfig.IScannerConfigBuilderInfo2;
import org.eclipse.cdt.make.internal.core.scannerconfig2.ScannerConfigProfileManager;
import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.internal.ui.DebugUIPlugin;
import org.eclipse.debug.internal.ui.launchConfigurations.LaunchConfigurationManager;
import org.eclipse.debug.internal.ui.launchConfigurations.LaunchHistory;

/**
 * @author D.Fujimoto
 * This class will create a new project.
 */
public class Gnu33ProjectFactory {

	// common suffix for files
	private static final String SUFFIX = ResourceProvider.getResourceString("common.suffix");
	
	// String attributes taken from the external tools plugin
	// IExternalToolConstants.ID_PROGRAM_LAUNCH_CONFIGURATION_TYPE
	public static final String LAUNCH_TYPE =  "org.eclipse.ui.externaltools.ProgramLaunchConfigurationType"; //$NON-NLS-1$
	// IExternalToolConstants.ID_EXTERNAL_TOOLS_LAUNCH_GROUP
	public static final String LAUNCH_GROUP = "org.eclipse.ui.externaltools.launchGroup";
	// IExternalToolConstants.ATTR_LOCATION
	public static final String ATTR_LOCATION =  "org.eclipse.ui.externaltools.ATTR_LOCATION";
	// IExternalToolConstants.ATTR_WORKING_DIRECTORY
	public static final String ATTR_WORKING_DIRECTORY =  "org.eclipse.ui.externaltools.ATTR_WORKING_DIRECTORY";
	//IExternalToolConstants.ATTR_TOOL_ARGUMENTS
	public static final String ATTR_TOOL_ARGUMENTS =  "org.eclipse.ui.externaltools.ATTR_TOOL_ARGUMENTS";
	
	/**
	 * Creates a new Project.
	 * Call this after the C make project is created.
	 * This method will:
	 * 		add a C++ nature so the project can make C classes
	 * 		add a nature to the project description file
	 * 		add a Gnu33 project description file
	 * 		modify the CDT make settings
	 * 		creates a launch config as an external tool
	 * @param project	the project to create
	 * @param	general	contains general info for creating a project
	 * 				cpuType:the cpu type of this project
	 * 				useHighPrecision:flag for use of library(not in use)
	 * @param vector	the vector section setting
	 * @param monitor	progress monitor
	 * @return	the created project
	 * @throws CoreException
	 * 
	 * This method now takes a GeneralInfo object for certain CPU dependant settings. 
	 */
	public static IProject create(IProject project, 
			GeneralInfo general, SectionData vector, IProgressMonitor monitor) 
				throws CoreException {
		
		return create(project, general, vector, false, monitor);
	}
	
	
	/**
	 * Create project for a TEST
	 * Convenient method that does not change the scanner config
	 * JUnit tests should call this method 
	 * @param project
	 * @param general
	 * @param vector
	 * @param isTest		true if this is called from a test
	 * @param monitor
	 * @return
	 * @throws CoreException
	 */
	public static IProject create(IProject project, 
			GeneralInfo general, SectionData vector, boolean isTest, IProgressMonitor monitor) 
				throws CoreException {
		
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}

		// add the C++ nature
		addCppNature(project, monitor);
		monitor.worked(1);
		
		// add a Gnu33 nature
		Gnu33ProjectNature.addNature(project);
		monitor.worked(1);

		// add the .gnu33project file
		addDescriptor(project, general, vector);
		monitor.worked(1);
		
		// auto update the sections
		autoUpdateSections(project);
		
		// modifies the make setting
		modifyMakeSettings(project);
		monitor.worked(1);

		// configure an external tool setting
		configureExternalTool(project);
		monitor.worked(1);	
	
		// create default command file after the descriptor has been made
		createCmdFile(project);
		monitor.worked(1);

		if (!isTest) {
			// disable the scanner config setting
			// invoke this here to avoid dead lock
			disableScannerConfig(project);
			monitor.worked(1);
		}
		
		return project;

	}

	
	/**
	 * Adds a C++ nature to this project.
	 * A C nature must be added before this method is invoked.
	 * @throws CoreException
	 * @since 1.1.1 
	 *
	 */
	private static void addCppNature(IProject project, IProgressMonitor monitor) throws CoreException {

		if (project != null) {
			// Add C++ Nature to the newly created project.
			CCorePlugin.getDefault().convertProjectFromCtoCC(project, new SubProgressMonitor(monitor, 1));
		}

	}
	
	/**
	 * Create a descriptor file and add it to the project.
	 * @param general GeneralInfo 
	 */
	private static void addDescriptor(IProject project, GeneralInfo general, SectionData vector) throws CoreException {
		
		// Create via DescriptorManager 
		Gnu33ProjectDescriptorManager manager = new Gnu33ProjectDescriptorManager(project, true);
		manager.createProjectDescriptor(general, vector);
		
	} 
	
	/**
	 * Add the libraries to the linkerscript setting if the section is 
	 * in auto update.
	 * This can be invoked after the project has been created.
	 * @param project
	 */
	private static void autoUpdateSections(IProject project) {
		
		new LinkerScriptUpdater(project.getName()).schedule(); 
	}
	
	/**
	 * Rewrite the .project file, and have the CDT use the make command 
	 * we set here.
	 * 
	 */
	private static void modifyMakeSettings(IProject project) {
		
		IProject newProj = project;
		IProjectDescription projDesc = null;

		try {
			// get Project Descriptor(.project file) and rewrite it
			projDesc = newProj.getDescription();
			ICommand[] commands = projDesc.getBuildSpec();
			for (int i = 0; i < commands.length; i++) {
				if (commands[i].getBuilderName().equals(MakeBuilder.BUILDER_ID)) {
					Map argMap = commands[i].getArguments();					
					
					// set build command to make -f projName_gnu33IDE.mak
					argMap.put("org.eclipse.cdt.make.core.useDefaultBuildCmd", "false");  //$NON-NLS-1$ //$NON-NLS-2$
					
					argMap.put(IMakeCommonBuildInfo.BUILD_COMMAND, "${" + ToolPathVarResolver.TOOL_LOC + "}/" + ResourceProvider.getResourceString("gnu33tools.make.exe"));
					argMap.put(IMakeCommonBuildInfo.BUILD_ARGUMENTS, "-f " + newProj.getName() + //$NON-NLS-1$ 
							SUFFIX + ResourceProvider.getResourceString("makefile.extension")); //$NON-NLS-1$
					// the probable path to make will be ${gnu33_loc}\make.exe
					// a relative path from the .project file
					
					// set auto build on (on a project level)
					argMap.put("org.eclipse.cdt.make.core.enableAutoBuild", "true");
					
					commands[i].setArguments(argMap);
				}

			}

			// save it back to .project file
			projDesc.setBuildSpec(commands);
			newProj.setDescription(projDesc, null);

		} catch (CoreException e) {
			Gnu33CorePlugin.log(e);
		}
	
	}

	/**
	 * Disables the Scanner output commnd parser.
	 * This method must not be invoked just after modifyMakeSettings()
	 * otherwise a dead lock may occur.
	 * @param newProj
	 * @throws CoreException
	 * @since 1.4.0
	 */
	private static void disableScannerConfig(IProject newProj) throws CoreException {
		// disable the discovery scanner info parser
		// this might cause an error during a JUnit test
		IScannerConfigBuilderInfo2 info = ScannerConfigProfileManager.createScannerConfigBuildInfo2(newProj);
		List provList = info.getProviderIdList();
		for (int i = 0; i < provList.size(); i++) {
			info.setProviderOutputParserEnabled((String) provList.get(i), false);
		}
		info.save();
	}
	
  
	/**
	 * Automatically creates an External Tool Configuration.
	 * The tool location will a relative path to gdb.exe.
	 * The working directory will be the project folder.
	 * The argument will be "-x XXXX.cmd". 
	 * @throws CoreException
	 */
	private static void configureExternalTool(IProject project) throws CoreException {
	
		IProject newProj = project;
 		// Prepare location, workingDir, arguments for launch config
		// create location with a variable
		String location = "${" + ToolPathVarResolver.TOOL_LOC + "}/" + ResourceProvider.getResourceString("gnu33tools.gdb.exe");
		String workingDir = "${workspace_loc:/" + newProj.getName() + "}";
		String arguments = "-x ${project_name}" +
			SUFFIX + ResourceProvider.getResourceString("commandfile.extension");
				
		// Obtain the launch manager
		ILaunchManager manager= DebugPlugin.getDefault().getLaunchManager();
		
		// Obtain the configuration typeiExternal toolsj
		ILaunchConfigurationType programType= manager.getLaunchConfigurationType(LAUNCH_TYPE);

		String launchName = manager.generateUniqueLaunchConfigurationNameFrom(
				ResourceProvider.getResourceString("launch.configName") + newProj.getName());
		
		// Make a working copy
		ILaunchConfigurationWorkingCopy wc = programType.newInstance(newProj, launchName);
		
		// Set the location, workingDir, arguments
		wc.setAttribute(ATTR_LOCATION, location);
		wc.setAttribute(ATTR_WORKING_DIRECTORY, workingDir);
		wc.setAttribute(ATTR_TOOL_ARGUMENTS, arguments);
		
		// save launch config to project dir
		wc.setContainer(newProj);

		// Save it
		try {
			wc.doSave();
		} catch (CoreException e) {
			// ignore ResourceExceptions because the underlying file
			// may already exist
		} 
	}
	
	
	/**
	 * Automatically renames the External Tool Configuration.
	 * The working directory will be updated to the new project folder 
	 * if the format matches ${workspace_loc:/pjName}
	 * @throws CoreException
	 * @since 1.4.0
	 */
	private static void updateExternalTool(IProject project, String previousName) throws CoreException {

		IProject newProj = project;
		String newName = newProj.getName();
		
		// working dir attribute format
		String wksp_loc = "${workspace_loc:/";
		String wksp_loc_regex = "\\$\\{workspace_loc:/";
				
		// Obtain the launch manager
		ILaunchManager manager= DebugPlugin.getDefault().getLaunchManager();
		
		// Obtain the configuration typeiExternal toolsj
		ILaunchConfigurationType externalTools= manager.getLaunchConfigurationType(LAUNCH_TYPE);

		
		// get all congigs for External Tools
		ILaunchConfiguration[] configs = manager.getLaunchConfigurations(externalTools);

		// get the Launch Configuration name
		for (int i = 0; i < configs.length; i++) {
			String name = configs[i].getName();

			if (name.equals(ResourceProvider.getResourceString("launch.configName") + previousName)) {

				// rename the previous launch file to the new name 
				// if it does no already exist 
				ILaunchConfigurationWorkingCopy wc = configs[i].getWorkingCopy();
				
				// try replacing the working dir attribute
				String workdirAttr = wc.getAttribute(ATTR_WORKING_DIRECTORY, wksp_loc + newName + "}");
				if (workdirAttr.startsWith(wksp_loc + previousName)) {
					workdirAttr = workdirAttr.replaceFirst(wksp_loc_regex + previousName, wksp_loc_regex + newName);
					wc.setAttribute(ATTR_WORKING_DIRECTORY, workdirAttr);
				}

				wc.rename(ResourceProvider.getResourceString("launch.configName") + newName);
				wc.doSave();
			}
		}
 
	}
	
	/**
	 * Creates a default command file in the project
	 * 
	 * @param project
	 * @throws CoreException
	 */
	private static void createCmdFile(IProject project) throws CoreException {
		// create default command file after the descriptor has been made
		Gnu33ProjectDescriptorManager man = Gnu33ProjectDescriptorManager.getDescriptorManager(project);
		man.createDefaultCmdFile();
	}
	
	/**
	 * Deletes the External Tools Launch Configuration for this project.
	 * Only delete is if it is local(maintained by Eclipse).
	 * Shared projects will get deleted when the <code>force</code> flag is set.
	 * 
	 * @param project	this should be a name of a Gnu33 Project
	 * @param force	the project will be deleted even if it is a shared one.
	 */
	public static void deleteLaunchConfig(String projectName, boolean force) {
		// get the LaunchManager
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();

		// get the External Tools
		ILaunchConfigurationType externalTools = 
			manager.getLaunchConfigurationType(LAUNCH_TYPE);
		
		try {

			// get all congigs for External Tools
			ILaunchConfiguration[] configs = manager.getLaunchConfigurations(externalTools);

			// get the Launch Configuration name
			for (int i = 0; i < configs.length; i++) {
				String name = configs[i].getName();

				if (name.equals(ResourceProvider.getResourceString("launch.configName") + projectName)) {
					if (configs[i].isLocal()) {

						if (Gnu33CorePlugin.isDebug) {
							System.out.println("Deleting " + name);
						}

						// delete the config
						configs[i].delete();

					} else {
						// if the LaunchConfiguration is Shared, rewrite the history
						// ignore class access warnings
						LaunchConfigurationManager lMan = DebugUIPlugin.getDefault().getLaunchConfigurationManager();
						LaunchHistory history = lMan.getLaunchHistory(LAUNCH_GROUP);
						history.launchConfigurationRemoved(configs[i]);
						
/* >>>>> ADDED by D.Fujimoto 2007/03/22 force deletion at project rename */
						// also delete the config itself when flag is set
						if (force) {
							configs[i].delete();
						}
/* <<<<< ADDED by D.Fujimoto 2007/03/22 force deletion at project rename */
					}
				}
			}

		} catch (CoreException e) {
			Gnu33CorePlugin.log(ResourceProvider.getResourceString("delete.launchConfig.failed") + projectName);
		}

	}
	
	/**
	 * Deletes the External Tools Launch Configuration for this project.
	 * Only delete is if it is local(maintained by Eclipse).
	 * 
	 * Convenience method for compatiblity reasons.
	 * @param project	this should be a Gnu33 Project
	 */
	public static void deleteLaunchConfig(IProject project) {
		deleteLaunchConfig(project.getName(), false);
	}
	
	/**
	 * Operations when a project gets renamed.
	 * 
	 * 
	 * @param project			the new Gnu33 project
	 * @param previousName		the old project name
	 * @throws CoreException
	 * @since 1.2.0
	 */
	public static void renameProject(IProject project, String previousName) throws CoreException {
		
		String newProjName = project.getName();
		
		Gnu33ProjectDescriptorManager manager = Gnu33ProjectDescriptorManager.getDescriptorManager(project);	  	
	
		// build settings
		List buildList = manager.getBuildList();
		BuildSetting build = (BuildSetting) buildList.get(0);
		build.setDefaultSettingName(newProjName);
		
		// linkerscript file option
		replaceLinkerScriptFileOption(build, previousName, newProjName);

		manager.updateBuildSetting(buildList);
		
		// parameter settings
		List parList = manager.getParameterList();
		ParameterSetting par = (ParameterSetting) parList.get(0);
		par.setDefaultSettingName(newProjName);
		manager.updateParameterSetting(parList);
		
		// linker settings
		LinkerScriptSetting linkerscript = manager.getLdsSetting();
		linkerscript.setDefaultSettingName(newProjName);
		manager.updateLinkerScriptSetting(linkerscript);
		
		// debugger settings
		DebuggerSetting debug = manager.getGDBSetting();
		debug.setDefaultSettingName(newProjName);
		manager.updateDebuggerSetting(debug);
		
/* >>>>> MODIFIED by D.Fujimoto 2007/04/05 do not delete the launch file, but copy it */
//		// create new launch config
//		configureExternalTool(project);
//		
///* >>>>> ADDED by D.Fujimoto 2007/03/22 delete old launch config */
//		// force delete the previous project's launch config
//		deleteLaunchConfig(previousName, true);
///* <<<<< ADDED by D.Fujimoto 2007/03/22 delete old launch config */

		updateExternalTool(project, previousName);
/* <<<<< MODIFIED by D.Fujimoto 2007/04/05 do not delete the launch file, but copy it */
		
		// update the build arg in project file
		replaceMakeArgs(project, previousName);

		// create a new command file 
		createCmdFile(project);

	}
	
	/**
	 * Replaces the user setting option to the renamed option.<br>
	 * Common suffixes will be added automatically.
	 * @param build		the build setting
	 * @param oldName		the old setting name
	 * @param newName		the new setting name.
	 * @since 1.2.0
	 */
	private static void replaceLinkerScriptFileOption(BuildSetting build, String oldName, String newName) {
		
		String ext = ResourceProvider.getResourceString("linkerscriptfile.extension");

		
		// lds file name
		IBuildTool linker = build.getLinker();
		IBuildOption userOp = linker.getOption(BuildSettingFactory.LD_USER_OPT_ID);
		
		String value = userOp.getStringValue();
		String newValue = "";
		
		StringTokenizer tok = new StringTokenizer(value);
		while (tok.hasMoreTokens()) {
			String token = tok.nextToken();
			if (token.equals(oldName + SUFFIX + ext)) {
				newValue += " " + newName + SUFFIX + ext;
			} else {
				newValue += " " + token;
			}
		
		}
		
		// set the new option string
		newValue = newValue.trim();
		userOp.setStringValue(newValue);
	
	}

	/**
	 * Replaces the makefile option in the project description file.<br>
	 * Common suffixes will be added automatically.
	 * @param project			the new project
	 * @param previousName		the old project name
	 * @throws CoreException
	 * @since 1.2.0
	 */
	private static void replaceMakeArgs(IProject project, String previousName) throws CoreException {
		
		String makeCmdProp = IMakeCommonBuildInfo.BUILD_ARGUMENTS;
		String ext = ResourceProvider.getResourceString("makefile.extension");
		
		IProjectDescription desc = project.getDescription();
		ICommand[] commands = desc.getBuildSpec();
		for (int i = 0; i < commands.length; i++) {
			if (commands[i].getBuilderName().equals(MakeBuilder.BUILDER_ID)) {
				Map argMap = commands[i].getArguments();
				String buildArg = (String) argMap.get(makeCmdProp);
				
				String newValue = "";
				
				StringTokenizer tok = new StringTokenizer(buildArg);
				while (tok.hasMoreTokens()) {
					String token = tok.nextToken();
					if (token.endsWith(SUFFIX + ext)) {
						newValue += " " + project.getName() + SUFFIX + ext;
					} else {
						newValue += " " + token; 
					}
				}
				newValue = newValue.trim();
				
				argMap.put(makeCmdProp, newValue);
				commands[i].setArguments(argMap);
			}
		}
		
		
		// rewrite the project description file
		desc.setBuildSpec(commands);
		project.setDescription(desc, null);
		
	} 
	
}
