/*
 * Created on 2004/07/28
 * Modified on 2005/05/25 by D.Fujimoto added isNatureActive()
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

/**
 * This class represents the Nature of a Project
 * If a nature is registered in the .project file, 
 * it is a Gnu33 project.
 * @author D.Fujimoto
 *
 */
public class Gnu33ProjectNature implements IProjectNature {

	IProject project;
	
	public static final String NATURE_ID = "jp.co.epson.ide.eclipse.gnu33.gnu33nature"; //$NON-NLS-1$
	public static final String BUILDER_ID = "jp.co.epson.ide.eclipse.gnu33.gnu33filebuilder"; //$NON-NLS-1$
  
	/**
	 * Constructor.
	 */
	public Gnu33ProjectNature() {
	}

	/**
	 * Adds the nature to the given project.
	 * @param newProj	the project to add this nature
	 * @throws CoreException
	 */
	public static void addNature(IProject newProj) throws CoreException {
				
		IProjectDescription desc = newProj.getDescription();
		if (!desc.hasNature(Gnu33ProjectNature.NATURE_ID)) {
			String[] ids = desc.getNatureIds();
			String[] newIds = new String[ids.length + 1];
			System.arraycopy(ids, 0, newIds, 0, ids.length);
			newIds[ids.length] = Gnu33ProjectNature.NATURE_ID;
			desc.setNatureIds(newIds);
			newProj.setDescription(desc, IResource.FORCE, null);
		}
		
	}
   
	/**
	 * Called when nature is added.
	 * The Gnu33FilesBuilder will be added.
	 * @see org.eclipse.core.resources.IProjectNature#configure()
	 */
	public void configure() throws CoreException {
		
		// add builder
		IProjectDescription desc = project.getDescription();
		ICommand[] commands = desc.getBuildSpec();
		
		// do not add the builder if it is already there
		for (int i = 0; i < commands.length; i++) {
			if (commands[i].getBuilderName().equals(BUILDER_ID)) {
				// nothing to do
				return;
			}
		}
		
		ICommand command = desc.newCommand();
		command.setBuilderName(BUILDER_ID);

		ICommand[] newCommands = new ICommand[commands.length + 1];
		// add before other builders
		System.arraycopy(commands, 0, newCommands, 1, commands.length);
		newCommands[0] = command;
		desc.setBuildSpec(newCommands);
		project.setDescription(desc, null);
		
		//Gnu33CorePlugin.log("gnu33 project nature added");
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
	}

	/** 
	 * Returns the project handle
	 * @see org.eclipse.core.resources.IProjectNature#getProject()
	 */
	public IProject getProject() {
		return project;
	}

	/** 
	 * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
	 */
	public void setProject(IProject project) {
	
		this.project = project;
	}
	
	/**
	 * Checks if this project has the nature and is enabled
	 * @param project	the project in question. should exist and be open.
	 * @return	true if this project has the nature and is enabled
	 * @since 1.1.1
	 */
	public static boolean isNatureActive(IProject project) {
		boolean result = false;
		
		try {
			result = project.hasNature(NATURE_ID) && project.isNatureEnabled(NATURE_ID);
		} catch (CoreException e) {
			// ignore
		}
		
		return result;
		
	} 

}
