/*
 * Created on 2004/08/17
 * Modified on 2005/03/30 by D.Fujimoto in createCompiler() (new option)
 * 										createAssembler() (new option)
 * 										createLinker() (path to ansi library)
 * 										alterUserOps() (new option)
 * Modified on 2005/07/12 by D.Fujimoto in createLinker() (path to libraries plus c++ libs)
 * Modified on 2005/07/14 by D.Fujimoto in createCompiler() and createAssembler() (removed -B option)
 * Modified on 2005/07/25 by D.Fujimoto in createCompiler() for include path with variable
 * 										in createLinker() for libpath with variable
 * Modified on 2005/08/05 by D.Fujimoto in createCompiler() deleted data pointer options along with 
 * 											their constant Strings.
 * Modified on 2005/08/10 by D.Fujimoto in createCompiler() added Kanji filter.
 * Modified on 2005/11/22 by D.Fujimoto in createCompiler() added -Os option.
 * Modified on 2006/02/16 by D.Fujimoto in createCompiler() default for kanji filter 
 * 											obtained from store.
 * Modified on 2006/03/28 by D.Fujimoto added createEnvVars().
 * Modified on 2006/06/15 by D.Fujimoto added getReservedEnvVars().
 * Modified on 2006/10/31 by D.Fujimoto debug options get default.
 * Migrated for GNU33 on 2007/01/19 from GNU17(1.0.0)
 * Modified on 2007/06/29 by D.Fujimoto in createLinker() added lib for simio
 * Modified on 2007/07/18 by D.Fujimoto added cp command
 * Modified on 2007/08/23 by D.Fujimoto added shouldAlterMiscOptions().
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptor;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.cdt.managedbuilder.core.IOption;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.preference.IPreferenceStore;


/**
 * Factory for creating IBuildTools.
 * @author D.Fujimoto
 * 
 */
public class BuildSettingFactory {

	public static final String COMPILER_CMD = ResourceProvider.getResourceString("gnu33tools.compiler");
	public static final String ASSEMBLER_CMD = ResourceProvider.getResourceString("gnu33tools.assembler");
	public static final String LINKER_CMD = ResourceProvider.getResourceString("gnu33tools.linker");
	public static final String RM_CMD = ResourceProvider.getResourceString("gnu33tools.rm");
	public static final String SED_CMD = ResourceProvider.getResourceString("gnu33tools.sed"); 
	public static final String CP_CMD = ResourceProvider.getResourceString("gnu33tools.cp");
	public static final String KNJFILTER_CMD = ResourceProvider.getResourceString("gnu33tools.kanjifilter");
	public static final String OBJDUMP_CMD = ResourceProvider.getResourceString("gnu33tools.objdump");
	public static final String CASM_CMD = ResourceProvider.getResourceString("gnu33tools.as");

	public static final String COMPILER_ID = BuildSettingResources.getString("ccompiler.id");
	public static final String COMPILER_CAT = BuildSettingResources.getString("ccompiler.cat"); 
	public static final String PREPRO_CAT = BuildSettingResources.getString("ccompiler.prepro.cat");
	public static final String CC_GENERAL_CAT = BuildSettingResources.getString("ccompiler.general.cat");
	public static final String OPT_CAT = BuildSettingResources.getString("ccompiler.opt.cat");
	public static final String DEBUG_CAT = BuildSettingResources.getString("ccompiler.debug.cat");
	public static final String INC_CAT = BuildSettingResources.getString("ccompiler.inc.cat");
	public static final String DEFINE_CAT = BuildSettingResources.getString("ccompiler.define.cat");
	public static final String CODEGEN_CAT = BuildSettingResources.getString("ccompiler.codegen.cat");
	public static final String MISC_CAT = BuildSettingResources.getString("ccompiler.misc.cat");;
	public static final String C_MEDDA_OPT_ID = BuildSettingResources.getString("ccompiler.access32bit"); 
	public static final String KNJFILT_OPT_ID = BuildSettingResources.getString("ccompiler.knjfilt");
	public static final String OPT_LEVEL_OPT_ID = BuildSettingResources.getString("ccompiler.optLevel");
	public static final String C_DEBUG_STAB_OPT_ID = BuildSettingResources.getString("ccompiler.debugStab");
	public static final String INC_PATH_OPT_ID = BuildSettingResources.getString("ccompiler.includePath");
	public static final String DEFINE_OPT_ID = BuildSettingResources.getString("ccompiler.symbols");
	public static final String NO_BUILTIN_OPT_ID = BuildSettingResources.getString("ccompiler.noBuiltIn");
	public static final String NO_EXPAND_MEMCPY_OPT_ID = BuildSettingResources.getString("ccompiler.noExpandMemcpy");
	public static final String FPIC_OPT_ID = BuildSettingResources.getString("ccompiler.fPIC");
	public static final String LONG_CALL_OPT_ID = BuildSettingResources.getString("ccompiler.longCalls");
	public static final String C_USER_OPT_ID = BuildSettingResources.getString("ccompiler.userOps");
	
	public static final String ASSEMBLER_ID = BuildSettingResources.getString("asm.id");
	public static final String ASSEMBLER_CAT = BuildSettingResources.getString("asm.cat");
	public static final String AS_GENERAL_CAT = BuildSettingResources.getString("asm.general.cat");
	public static final String ASM_MEDDA_OPT_ID = BuildSettingResources.getString("asm.access32bit");
	public static final String ASM_DEBUG_STAB_OPT_ID = BuildSettingResources.getString("asm.debugStabs");
	public static final String ASM_USER_OPT_ID = BuildSettingResources.getString("asm.userOpts");
	
	public static final String LINKER_ID = BuildSettingResources.getString("linker.id");
	public static final String LINKER_CAT = BuildSettingResources.getString("ld.cat");
	public static final String LD_GENERAL_CAT = BuildSettingResources.getString("ld.general.cat");
	public static final String LD_LIB_CAT = BuildSettingResources.getString("ld_lib.cat");
	public static final String GEN_MAP_OPT_ID = BuildSettingResources.getString("linker.genMap");
	public static final String NO_ALIGN_CHECK_OPT_ID = BuildSettingResources.getString("linker.noAlignCheck");
	public static final String LD_LIB_OPT_ID = BuildSettingResources.getString("linker.libraries");
	public static final String LD_USER_OPT_ID = BuildSettingResources.getString("linker.userOps");

	public static final String CASM_ID = BuildSettingResources.getString("casm.id");
	public static final String CASM_CAT = BuildSettingResources.getString("casm.cat");
	public static final String CASM_GENERAL_CAT = BuildSettingResources.getString("asm.general.cat");

	// assembler debug stabs option
	public static final String AS_GSTABS = "--gstabs";
	
	public static final String MC33PE = "-mc33pe";
	public static final String MC33ADV = "-mc33adv";

	// additional compiler and assembler option for ADV
	public static final String MC33401 = ResourceProvider.getResourceString("default.value.mc33401");
	
	// 2pass option
	public static final String MCEXT = "-mc33_ext";
	
	/**
	 * Create an IBuildTool object for GNU33 compiler.
	 * @param cpuType	CPU type of GeneralInfo
	 * @return The compiler as an IBuildTool
	 */
	public static IBuildTool createCompiler(int cpuType) {
		
		IBuildTool compiler = new BuildTool(COMPILER_ID, COMPILER_CMD);
		compiler.setDisabled(false);
		compiler.setName(COMPILER_CAT);
		
		// general (medda32)
		IBuildCategory general = new BuildCategory(CC_GENERAL_CAT, compiler);
		IBuildOption medda32 = new BuildOption(C_MEDDA_OPT_ID, "-medda32", IBuildOption.TYPE_BOOLEAN, general);
		medda32.setLabel(BuildSettingResources.getString("ccompiler.medda32.label"));
		
		// Kanji filter
		IBuildOption knj = new BuildOption(KNJFILT_OPT_ID, "", IBuildOption.TYPE_BOOLEAN, general);
		knj.setLabel(BuildSettingResources.getString("ccompiler.knjfilt.label"));
		
		// optimize
		IBuildCategory optimize = new BuildCategory(OPT_CAT, compiler);
		BuildOption optLevel = new BuildOption(OPT_LEVEL_OPT_ID, "-O", 
				IBuildOption.TYPE_ENUM, optimize);
		optLevel.setLabel(BuildSettingResources.getString("ccompiler.optLevel.label"));
		List commandList = new ArrayList();
		commandList.add(new String[] {"-O0", BuildSettingResources.getString("ccompiler.optLevel0.label") });
		commandList.add(new String[] {"-O1", BuildSettingResources.getString("ccompiler.optLevel1.label") });
		commandList.add(new String[] {"-O2", BuildSettingResources.getString("ccompiler.optLevel2.label") });
		commandList.add(new String[] {"-O3", BuildSettingResources.getString("ccompiler.optLevel3.label") });
		commandList.add(new String[] {"-Os", BuildSettingResources.getString("ccompiler.optLevel4.label") });
		optLevel.setCommandList(commandList);
	

		// include paths
		IBuildCategory includes = new BuildCategory(INC_CAT, compiler);
		BuildOption includePath = new BuildOption(INC_PATH_OPT_ID, "-I", 
				IBuildOption.TYPE_LIST, includes);
		includePath.setLabel(BuildSettingResources.getString("ccompiler.includePath.label"));
		includePath.setBrowseType(IOption.BROWSE_DIR);
		
		IBuildCategory symbols = new BuildCategory(DEFINE_CAT, compiler);
		IBuildOption defines = new BuildOption(DEFINE_OPT_ID, "-D", 
				IBuildOption.TYPE_LIST, symbols);
		defines.setLabel(BuildSettingResources.getString("ccompiler.defines.label"));
				
		
		// code generating
		IBuildCategory codeGen = new BuildCategory(CODEGEN_CAT, compiler);
		IBuildOption noBuiltIn = new BuildOption(NO_BUILTIN_OPT_ID, "-fno-builtin", 
				IBuildOption.TYPE_BOOLEAN, codeGen);
		noBuiltIn.setLabel(BuildSettingResources.getString("ccompiler.noBuiltIn.label"));
		
		IBuildOption noExpandMemcpy = new BuildOption(NO_EXPAND_MEMCPY_OPT_ID, "-mno-memcpy", 
				IBuildOption.TYPE_BOOLEAN, codeGen);
		noExpandMemcpy.setLabel(BuildSettingResources.getString("ccompiler.noExpandMemcpy.label"));
		
		IBuildOption fPic = new BuildOption(FPIC_OPT_ID, "-fPIC", 
				IBuildOption.TYPE_BOOLEAN, codeGen);
		fPic.setLabel(BuildSettingResources.getString("ccompiler.fPic.label"));
		
		IBuildOption longCalls = new BuildOption(LONG_CALL_OPT_ID, "-mlong-calls", 
				IBuildOption.TYPE_BOOLEAN, codeGen);
		longCalls.setLabel(BuildSettingResources.getString("ccompiler.longCalls.label"));
	 
		// other user defined options
		IBuildCategory compilerOthers = new BuildCategory(MISC_CAT, compiler);
		IBuildOption miscellaneous = new BuildOption(C_USER_OPT_ID, "", 
				IBuildOption.TYPE_STRING_MULTI, compilerOthers);
		miscellaneous.setLabel(BuildSettingResources.getString("ccompiler.misc.label"));
		
		
		
		// set default settings from store 
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		if (cpuType == GeneralInfo.CPU_STD || cpuType == GeneralInfo.CPU_PE) {
			medda32.setBooleanValue(true);
		} else if (cpuType == GeneralInfo.CPU_ADV) {
			medda32.setBooleanValue(false);
		}
		
		knj.setBooleanValue(store.getBoolean("ccompiler.knjfilt.value"));
		
		optLevel.setEnumValue(store.getInt(OPT_LEVEL_OPT_ID));
   
		String includeDir = 
			MakeFileGenerator.expandMacro(MakeFileGenerator.MACRO_TOOL_DIR) +
			ResourceProvider.getResourceString("gnu33tools.includePath");
		includePath.setListValue(new String[] {includeDir});

		String defineStr = store.getString(DEFINE_OPT_ID);
		defines.setListValue(defineStr.split(IBuildOption.LIST_DELIMITER));
		
		noBuiltIn.setBooleanValue(store.getBoolean(NO_BUILTIN_OPT_ID));
		noExpandMemcpy.setBooleanValue(store.getBoolean(NO_EXPAND_MEMCPY_OPT_ID));
		longCalls.setBooleanValue(store.getBoolean(LONG_CALL_OPT_ID));
		
		
		String defaultMisc = "";

/* >>>>> DELETED by D.Fujimoto 2007/03/13 cpu options moved to switchCpuOptions() */		
//		if (cpuType == GeneralInfo.CPU_PE) {
//			defaultMisc += " " + MC33PE;
//		} else if (cpuType == GeneralInfo.CPU_ADV) {
//			defaultMisc += " " + MC33ADV;
//			defaultMisc += " " + MC33401;
//		}
/* <<<<< DELETED by D.Fujimoto 2007/03/13 cpu options moved to switchCpuOptions() */
		miscellaneous.setStringValue(defaultMisc + " " + store.getString(C_USER_OPT_ID));

/* >>>>> ADDED by D.Fujimoto 2006/10/31 debug option */		
		// debug option is set as default 	  
		compiler.setDefaultOptions(new String[] {"-gstabs"});
/* <<<<< ADDED by D.Fujimoto 2006/10/31 debug option */
		
		return compiler;
	}
	 
	/**
	 * Creates an IBuildTool object for GNU33 assembler.
	 * @param CPU type of GeneralInfo
	 * @return The assembler as an IBuildTool
	 */
	public static IBuildTool createAssembler(int cpuType) {
		
		IBuildTool assembler = new BuildTool(ASSEMBLER_ID, ASSEMBLER_CMD);
		assembler.setDisabled(false);
		assembler.setName(ASSEMBLER_CAT);
		
		// generals
		IBuildCategory general = new BuildCategory(AS_GENERAL_CAT, assembler);
		IBuildOption medda32 = new BuildOption(ASM_MEDDA_OPT_ID, "-medda32", IBuildOption.TYPE_BOOLEAN, general);
		medda32.setLabel(BuildSettingResources.getString("asm.medda32.label"));
		
		IBuildOption asFlags = new BuildOption(ASM_USER_OPT_ID, "", 
				IBuildOption.TYPE_STRING_MULTI, general);
		asFlags.setLabel(BuildSettingResources.getString("asm.asFlags.label"));
		
		
		
		// set default settings from store	
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		medda32.setBooleanValue(true);

		// debug option set as default					   
		assembler.setDefaultOptions(new String[] {"-c", "-xassembler-with-cpp", "-Wa," + AS_GSTABS});

		String defaultFlags = "";
/* >>>>> DELETED by D.Fujimoto 2007/03/13 cpu options moved to switchCpuOptions() */		
//		if (cpuType == GeneralInfo.CPU_PE) {
//			defaultFlags += " " + MC33PE;
//		} else if (cpuType == GeneralInfo.CPU_ADV) {
//			defaultFlags += " " + MC33ADV;
//			defaultFlags += " " + MC33401;
//		}
/* <<<<< DELETED by D.Fujimoto 2007/03/13 cpu options moved to switchCpuOptions() */
		asFlags.setStringValue(defaultFlags + " " + store.getString(ASM_USER_OPT_ID));
	
		return assembler;
	}
	
	/**
	 * Creates an IBuildTool object for GNU33 linker.
	 * 
	 * @param ldsFileName 	the -T option String
	 * @param generalInfo 	for retrieving emulib paths 
	 * @return The linker as an IBuildTool
	 * 
	 * The Map and alignment options are hidden from the UI.
	 */
	public static IBuildTool createLinker(String ldsFileName, GeneralInfo generalInfo) {
		IBuildTool linker = new BuildTool(LINKER_ID, LINKER_CMD);
		linker.setDisabled(false);
		linker.setName(LINKER_CAT);
		
		// generals
		IBuildCategory general = new BuildCategory(LD_GENERAL_CAT, linker);
		IBuildOption genMap = new BuildOption(GEN_MAP_OPT_ID, "-Map", 
				IBuildOption.TYPE_BOOLEAN, general);
		genMap.setLabel(BuildSettingResources.getString("ld.genMap.label"));
		genMap.setInvisible(true);
		
		IBuildOption alignPaging = new BuildOption(NO_ALIGN_CHECK_OPT_ID, "-N", 
				IBuildOption.TYPE_BOOLEAN, general);
		alignPaging.setLabel(BuildSettingResources.getString("ld.alignPaging.label"));
		alignPaging.setInvisible(true);
		
		IBuildOption ldFlags = new BuildOption(LD_USER_OPT_ID, "", 
				IBuildOption.TYPE_STRING_MULTI, general);
		ldFlags.setLabel(BuildSettingResources.getString("ld.ldFlags.label"));
		

		// libraries
		IBuildCategory libs = new BuildCategory(LD_LIB_CAT, linker);
		BuildOption libraries = new BuildOption(LD_LIB_OPT_ID, "",
				IBuildOption.TYPE_LIST, libs);
		libraries.setLabel(BuildSettingResources.getString("ld.libraries.label"));
		libraries.setBrowseType(IOption.BROWSE_FILE);
		
		
		
		
		// set default settings from store
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		genMap.setBooleanValue(store.getBoolean(GEN_MAP_OPT_ID));
		alignPaging.setBooleanValue(store.getBoolean(NO_ALIGN_CHECK_OPT_ID));
		ldFlags.setStringValue(
				"-T " + ldsFileName + ResourceProvider.getResourceString("linkerscriptfile.extension") + " " +
				store.getString(LD_USER_OPT_ID)
		);
		

		IPath libPath = generalInfo.getLibPath();
		IPath cpplibPath = libPath.append(generalInfo.getCppLibName());
		IPath longlonglibPath = libPath.append(generalInfo.getLongLongLibName());
		IPath ansilibPath = libPath.append(generalInfo.getAnsiLibName());
		IPath emulibPath = libPath.append(generalInfo.getEmulibName());
/* >>>>> ADDED by D.Fujimoto 2007/06/29 lib for simio */
		IPath simIOlibPath = libPath.append(generalInfo.getSimIOLibName());
/* <<<<< ADDED by D.Fujimoto 2007/06/29 lib for simio */
		
		String toolDirVar = MakeFileGenerator.expandMacro(MakeFileGenerator.MACRO_TOOL_DIR);
/* >>>>> ADDED by D.Fujimoto 2007/06/29 lib for simio */
		String[] libStrings = null;
		if (store.getBoolean("addSimIOLib")) {
			// add the simio lib
			libStrings = new String[] {
					toolDirVar + simIOlibPath.toString(),
					toolDirVar + cpplibPath.toString(), toolDirVar + longlonglibPath.toString(),
					toolDirVar + ansilibPath.toString(), toolDirVar + emulibPath.toString()
			};
		} else {
			libStrings = new String[] {
					toolDirVar + cpplibPath.toString(), toolDirVar + longlonglibPath.toString(),
					toolDirVar + ansilibPath.toString(), toolDirVar + emulibPath.toString()
			};
		}
/* <<<<< ADDED by D.Fujimoto 2007/06/29 lib for simio */
		libraries.setListValue(libStrings); 
		
		return linker;
	}
	
	/**
	 * Creates an environment substitution setting.
	 * 
	 * @return a List of IEnvVar
	 * @since 1.3.0
	 */
	public static List createEnvVars() {
	
		// create a collection
		List envVars = new ArrayList();
		
		// add default settings here
		// (none)
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		// split with delimiter
		String envString = store.getString(Gnu33ProjectDescriptor.TAG_ENV_VAR);
		String[] splitValues = envString.split(IBuildOption.LIST_DELIMITER);
		for (int i = 0; i < splitValues.length; i++) {
			// get key and value pair
			String[] keyValue = splitValues[i].split("=");
			if (keyValue.length == 2) {
				envVars.add(new EnvVar(keyValue[0], keyValue[1]));
			}
		}
		
		return envVars;
	}
	
	
	/**
	 * Returns an list of reserved environment subst settings.
	 * @return	a List of reserved IEnvVars
	 * @since 1.3.0
	 * @see MakeFileGenerator#RESERVED_MACROS
	 */
	public static List getReservedEnvVars() {
		List reservedList = new ArrayList();
		
		IEnvVar[] reserved = {
			new EnvVar(MakeFileGenerator.MACRO_TOOL_DIR, ToolPathVarResolver.getGnuToolsPath().toString(), true, true)
		};
		
		reservedList.addAll(Arrays.asList(reserved));
		
		return reservedList;
	}
	
	/**
	 * Creates an IBuildTool object for GNU33 C source assembler for 2 pass make.
	 * @return The assembler as an IBuildTool for 2 pass make
	 * @since 1.4.0
	 */
	public static IBuildTool createCAssembler() {
		
		IBuildTool assembler = new BuildTool(CASM_ID, CASM_CMD);
		assembler.setDisabled(false);
		assembler.setName(CASM_CAT);
		
		// generals
		IBuildCategory general = new BuildCategory(CASM_GENERAL_CAT, assembler);
		
		IBuildOption asFlags = new BuildOption(ASM_USER_OPT_ID, "", 
				IBuildOption.TYPE_STRING_MULTI, general);
		asFlags.setLabel(BuildSettingResources.getString("casm.asFlags.label"));
		
		
		
		// set default settings
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		assembler.setDefaultOptions(new String[0]);

		return assembler;
	}
	
	/**
	 * Alter the user options depending on CPU type.
	 * If the cpu type is STD, delete any -mc33pe or -mc33adv or -mc33401 options.
	 * If PE, delete any -mc33adv,-mc33401 and add -mc33pe.
	 * If ADV, delete any -mc33pe and add -mc33adv and -mc33401.
	 * @param userOps the user options String
	 * @param cpuType the CPU type 
	 * @return the altered user option String
	 * @deprecated 1.4.0
	 */
	public static String alterUserOps(String userOps, int cpuType) {
		userOps = userOps.trim();
		userOps = userOps.replaceAll(MC33PE, "");
		userOps = userOps.replaceAll(MC33ADV, "");
		userOps = userOps.replaceAll(MC33401, "");
		
		if (cpuType == GeneralInfo.CPU_PE) {
			userOps += " " + MC33PE;
		} else if (cpuType == GeneralInfo.CPU_ADV) {
			userOps += " " + MC33ADV;
			userOps += " " + MC33401;

		}

		return userOps.trim();
	}

	/**
	 * Alters the CPU specific options
	 * @param setting	the build setting to modify
	 * @param cpuType	the new CPU type to switch to
	 * @since 1.4.0
	 */
	public static void switchCpuOptions(BuildSetting setting, int cpuType) {
		
		// compiler options
		IBuildTool compiler = setting.getCompiler();
		switchCpuOptions(compiler, cpuType);
		
		// assembler options
		IBuildTool assembler = setting.getAssembler();
		switchCpuOptions(assembler, cpuType);
	}
	
	/**
	 * Alters the CPU specific options for the given tool.
	 * This method will alter the default options for an IBuildTool.
	 * @param tool		the tool to modify the options for
	 * @param cpuType	the new CPU type to swtich to
	 * @since 1.4.0
	 */
	public static void switchCpuOptions(IBuildTool tool, int cpuType) {
		
		// default options for a tool
		List defOptions = new ArrayList();
		String[] options;
		// the tool ID
		String toolID = tool.getToolID();
		
		// user options for a tool
		IBuildOption misc = null;
		String optionStr = null;
		
		
		// add the basic default options
		options = tool.getDefaultOptions();
		for (int i = 0; i < options.length; i++) {
			defOptions.add(options[i]);
		}
		
		// remove all the current CPU options
		defOptions.remove(MC33PE);
		defOptions.remove(MC33ADV);
		defOptions.remove(MC33401);
		
		
		
		if (toolID.equals(COMPILER_ID)) {
			// remove from the user option(for compatibility)
			misc = tool.getOption(C_USER_OPT_ID);
		} else if (toolID.equals(ASSEMBLER_ID)) {
			// remove from the user option(for compatibility)
			misc = tool.getOption(ASM_USER_OPT_ID);
		}
	
		if (misc != null) {
			// erase from the user options (including -c)
			optionStr = misc.getStringValue().trim();
			
/* >>>>> DELETED by D.Fujimoto 2008/06/13 do not delete '-mc33pe2' like '2' */					
//			optionStr = optionStr.replaceAll(MC33PE, "");
//			optionStr = optionStr.replaceAll(MC33ADV, "");
//			optionStr = optionStr.replaceAll(MC33401, "");
/* <<<<< DELETED by D.Fujimoto 2008/06/13 do not delete '-mc33pe2' like '2' */
			
/* >>>>> MODIFIED by D.Fujimoto 2008/06/02 do not delete when option contains a '-c' like -fsigned-char */		
//			optionStr = optionStr.replaceAll("-c", "");
//			misc.setStringValue(optionStr);


			// delete when there is no leading nor trailing spaces
			String newOptionStr = "";
			StringTokenizer tokenizer = new StringTokenizer(optionStr);
			while (tokenizer.hasMoreTokens()) {
				String token = tokenizer.nextToken();
				if (token.equals("-c") || token.equals(MC33PE) || token.equals(MC33ADV) || token.equals(MC33401)) {
					newOptionStr += " ";
				} else {
					newOptionStr += " " + token;
				}
			}

			misc.setStringValue(newOptionStr);
/* <<<<< MODIFIED by D.Fujimoto 2008/06/02 do not delete when option contains a '-c' like -fsigned-char */
		}
		

		
		
		// add to the default options(same for all tools)
		if (cpuType == GeneralInfo.CPU_PE) {
			defOptions.add(0, MC33PE);
		} else if (cpuType == GeneralInfo.CPU_ADV) {
			defOptions.add(0, MC33401);
			defOptions.add(0, MC33ADV);
		}
		
		// set CPU options to the default options
		tool.setDefaultOptions((String[]) defOptions.toArray(new String[0]));
		
	}

	
	/**
	 * Returns whether the user options may need migrating.
	 * @param	setting	the build setting
	 * @return true if the options need to be modified
	 * @since 1.4.0
	 */
	public static boolean shouldAlterMiscOptions(BuildSetting setting) {
	
		boolean shouldDo = false;
		
		IBuildTool[] tools = { setting.getCompiler(), setting.getAssembler() }; 
		
		// user options for a tool
		IBuildOption misc = null;
		String optionStr = null;
		
		
		for (int i = 0; i < tools.length; i++) {
			String toolID = tools[i].getToolID();
			
			if (toolID.equals(COMPILER_ID)) {
				// check the user option(for compatibility)
				misc = tools[i].getOption(C_USER_OPT_ID);
			} else if (toolID.equals(ASSEMBLER_ID)) {
				// check the user option(for compatibility)
				misc = tools[i].getOption(ASM_USER_OPT_ID);
			}
		
			if (misc != null) {
				optionStr = misc.getStringValue().trim();
				if (optionStr.indexOf(MC33PE) != -1 
						|| optionStr.indexOf(MC33ADV) != -1
						|| optionStr.indexOf(MC33401) != -1 
						|| optionStr.indexOf("-c") != -1) {
					shouldDo = true;
					break;
				}
			}	
			
		}
		
		return shouldDo;
		
	}
	
	/**
	 * Returns the option for 2pass make
	 * @param
	 * @return	the 2pass option as String
	 * @since 1.4.0
	 */
	public static String get2passOptions() {
		return get2passOptions(true);
	}

	/**
	 * Returns the option for 2pass make
	 * @param	needsPrefix		adds -Wa, when true
	 * @return	the 2pass option as String
	 * @since 1.4.0
	 */
	public static String get2passOptions(boolean needsPrefix) {
		String prefix = "";
		if (needsPrefix) {
			prefix = "-Wa,";
		}
		
		return prefix + MCEXT 
			+ " " + prefix + MakeFileGenerator.expandMacro(MakeFileGenerator.MACRO_TARGET)
			+ ResourceProvider.getResourceString("objdumpfile.extension")
			+ " " + prefix + MakeFileGenerator.expandMacro(MakeFileGenerator.MACRO_ALLOBJDUMP_FILE) 
			+ ResourceProvider.getResourceString("objdumpfile.extension");
			
	}
	
	/**
	 * Returns the 2pass flag default
	 * @return	
	 */
	public static boolean is2passDefault() {
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		return store.getDefaultBoolean("perform2pass");
	}
	
	/**
	 * Returns the build default
	 * @return
	 */
	public static int getDefaultGoal() {
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		return store.getDefaultInt("defaultGoal");
	}

}
