/*
 * Created on 2004/08/17
 *
 */
package jp.co.epson.ide.eclipse.gnu33.build;

import java.util.List;

/**
 * Interface for a tool for building a project.
 * Represents a node in the tree in BuildSettingsPropertyPage. 
 * @author D.Fujimoto
 *
 */
public interface IBuildTool {

	public static final String SPACE = " ";  //$NON-NLS-1$
	
	/**
	 * Returns the tool's command.
	 */
	public String getCommand();
	
	/**
	 * Sets the tool's command.
	 * @param command
	 */
	public void setCommand(String command);
	
	/**
	 * Returns the tool's name.
	 */
	public String getName();
	
	/**
	 * Sets the tool's name.
	 * @param name
	 */
	public void setName(String name);
	
	/**
	 * Returns whether the tool is disabled.
	 * @return true if the tool is disabled
	 */
	public boolean isDisabled();
	
	/**
	 * Sets the tool to be disabled.
	 * @param isDisabled
	 */
	public void setDisabled(boolean isDisabled);
	
	/**
	 * Returns the tool ID.
	 * @return the tool ID
	 */
	public String getToolID();
	
	/**
	 * Add a category to the tool.
	 * @param category
	 */
	public void addOptionCategory(IBuildCategory category);
	
	/**
	 * Returns a List of categories in this tool.
	 */
	public List getOptionCategories();
	
	/**
	 * Gets the option in this tool by the given ID.
	 * @param id
	 */
	public IBuildOption getOption(String id);
	
	/**
	 * Returns an array of options in this tool.
	 */
	public IBuildOption[] getOptions();

	/**
	 * Returns default options that will be output to
	 * the make file
	 *
	 * @return	default options in String array
	 */
	public String[] getDefaultOptions();

	/**
	 * Sets default options for this tool.
	 * @param defaultOptions
	 */
	public void setDefaultOptions(String[] defaultOptions);

}
