/*
 * Created on 2004/10/05
 * Modified on 2006/03/14 by D.Fujimoto modified makeContents() to output user defined commands.
 * Modified on 2006/03/14 by D.Fujimoto modified(refactored) makeContents()
 * 											baud rate updates moved to Gnu33FilesBuilder
 * 										added new Constructor.
 * Modified on 2006/05/16 by D.Fujimoto added property "mode" and makeContents().
 * Modified on 2006/07/21 by D.Fujimoto deleted "mode" and erased from makeContents().
 * Modified on 2007/07/19 by D.Fujimoto modified makeContents() for icd ver6
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Creates a command file(.cmd) during a build.
 * @author D.Fujimoto
 *	
 */
public class CommandFileGenerator extends AbstractFileGenerator {

	public static final String ERROR_BAUD_RATE_WRITE = ResourceProvider.getResourceString("baudrate.failed"); 
	
	private static final String HEADER = "# Command file generated by Gnu33 Plug-in for Eclipse";
	public static final String CMD_PREFIX = "c33 ";
	
	private DebuggerSetting gdbSetting;
	
	private ParameterSetting parSetting;

	private IPreferenceStore store;
	
	/**
	 * Constructor. Get the DebuggerSetting from the descriptor file.
	 * 
	 * @see AbstractFileGenerator
	 */
	public CommandFileGenerator(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		super(project, manager, fileName + EXT_CMD); //$NON-NLS-1$

		gdbSetting = manager.getGDBSetting();
		
		// get the first ParameterSetting's name
		parSetting = (ParameterSetting) manager.getParameterList().get(0);

		// get store settings
		store = Gnu33CorePlugin.getDefault().getPreferenceStore();

	}
	

	/**
	 * Constructor for generating file contents on the spot.
	 * @param project		the project to generate the file for 
	 * @param manager		the project's descriptor manager
	 * @param gdbSetting	the setting to generate the commands from
	 */
	public CommandFileGenerator(IProject project, Gnu33ProjectDescriptorManager manager, DebuggerSetting gdbSetting) {
		super(project, manager, gdbSetting.getName() + ResourceProvider.getResourceString("commandfile.extension"));
		this.gdbSetting = gdbSetting;
		this.parSetting = (ParameterSetting) manager.getParameterList().get(0);
	}
	


	/**
	 * Makes the contents of the file using the descriptor file.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator#makeContents()
	 */
	public String makeContents() {

/* >>>>> DELETED by D.Fujimoto 2006/07/21 no more modes */
/* >>>>> MODIFIED by D.Fujimoto 2006/05/16 return the raw contents at BUILD_MODE */
//		// 
//		if (mode == BUILD_MODE) {
//
//			// if raw commands are empty, create commands automatically
//			String raw = gdbSetting.getRawCommands();
//			if (raw != null && !raw.equals("")) {
//				return raw;
//			} 
//		}
/* <<<<< MODIFIED by D.Fujimoto 2006/05/16 return the raw contents at BUILD_MODE */
/* <<<<< DELETED by D.Fujimoto 2006/07/21 no more modes */
		
		StringBuffer buf = new StringBuffer(1024);

		int debuggerType = gdbSetting.getDebuggerType();

		// file header
		buf.append(HEADER).append(NEW_LINE);

		// file (elf for symbol information)
		buf.append("# load debugging information in program").append(NEW_LINE);
		buf.append("file ").append(project.getName())
			.append(EXT_EXE).append(NEW_LINE); //$NON-NLS-1$

		// c33 rpf (parameter file)
		buf.append("# set the memory configuration map file to the debugger").append(NEW_LINE);
		buf.append(CMD_PREFIX + "rpf ").append(parSetting.getName())
			.append(EXT_PAR).append(NEW_LINE); //$NON-NLS-1$

		// c33 das (debug RAM address)
/* >>>>> MODIFIED by D.Fujimoto 2007/07/19 icd6 */
//		if (cpuType == GeneralInfo.CPU_PE
//				&& debuggerType == DebuggerSetting.TYPE_ICD3) {
		if (cpuType == GeneralInfo.CPU_PE) {
			if (debuggerType == DebuggerSetting.TYPE_ICD3 || debuggerType == DebuggerSetting.TYPE_ICD6) {
/* <<<<< MODIFIED by D.Fujimoto 2007/07/19 icd6 */
				if (gdbSetting.getDebugRamAddr() != DebuggerSetting.RAM_ADDR_DEFAULT) {
					buf.append("# set debug RAM address configuration").append(NEW_LINE);
					buf.append(CMD_PREFIX + "das 0x060000 ");
					buf.append("0x" + UIUtil.getAddressString(gdbSetting.getDebugRamAddr()));
					buf.append(" 1").append(NEW_LINE); // STD:0 PE:1 ADV:3
				}
			}
		}

		// target (debugger and port) 
		buf.append("# connect to the debugger with specified mode and port").append(NEW_LINE);
		buf.append("# target [icd usb | icd6 usb | icd com | sim]").append(NEW_LINE);
		buf.append("target ");
		if (debuggerType == DebuggerSetting.TYPE_ICD3) {
			buf.append("icd usb").append(NEW_LINE);
/* >>>>> ADDED by D.Fujimoto 2007/07/19 icd6 */			
		} else if (debuggerType == DebuggerSetting.TYPE_ICD6) {
			buf.append("icd6 usb").append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2007/07/19 icd6 */
		} else if (debuggerType == DebuggerSetting.TYPE_SIM) {
			buf.append("sim").append(NEW_LINE);
		} else if (debuggerType == DebuggerSetting.TYPE_ICD2 || debuggerType == DebuggerSetting.TYPE_MON) {
			buf.append("icd com").append(gdbSetting.getComPortNo()).append(" ")
					.append(gdbSetting.getPortWait()).append(NEW_LINE);
		}

		// c33 lpt (parallel port)
		if (debuggerType == DebuggerSetting.TYPE_ICD2) {
			if (gdbSetting.getParallelPort() != 0) {
				buf.append("# use parallel ports").append(NEW_LINE);
				buf.append(CMD_PREFIX + "lpt ").append(gdbSetting.getParallelPort()).append(NEW_LINE);
			}
		}

		// load (executable to debug)
		buf.append("# load program to memory").append(NEW_LINE);
		buf.append("load ").append(project.getName())
			.append(EXT_EXE).append(NEW_LINE); //$NON-NLS-1$

		// set (ttbr register)
		buf.append("# set ttbr register").append(NEW_LINE);
		if (gdbSetting.isUseVectorAddress()) {
			if (cpuType == GeneralInfo.CPU_STD
					|| (cpuType == GeneralInfo.CPU_PE && gdbSetting.getDebuggerType() == DebuggerSetting.TYPE_ICD2)) {
				// cpu is STD or PE with ICD2
				buf.append("set {char}0x4812d=0x59").append(NEW_LINE);
				buf.append("set {long}0x48134=");
			} else {
				// cpu is ADV or PE with ICDx/SIM
				buf.append("set $ttbr=");
			}
			buf.append("0x" + UIUtil.getAddressString(gdbSetting.getVectorAddress())).append(NEW_LINE);
		}

		// c33 rsth (reset command)
		buf.append("# hot reset").append(NEW_LINE);
		buf.append(CMD_PREFIX + "rsth").append(NEW_LINE);
		
		
/* >>>>> MODIFIED by D.Fujimoto 2006/03/14 baud rate moved to Gnu33FilesBuilder */
		// baud rate is set to another file
//		  setBaudRateToFile();
/* <<<<< MODIFIED by D.Fujimoto 2006/03/14 baud rate moved to Gnu33FilesBuilder */

		return new String(buf);

	}
	
	/**
	 * Sets the baud rate value to GDB33's ini file.
	 * Reads the file, and rewrites it.
	 * Does nothing if the file does not exist, or if the property does not exist in file.
	 */
	public void setBaudRateToFile() {
		String iniFileName = ResourceProvider.getResourceString("gdb33.inifile"); //$NON-NLS-1$
		String baudRateProperty = 
			ResourceProvider.getResourceString("gdb33.inifile.baudRate") + "="; //$NON-NLS-1$  //$NON-NLS-2$
		
		int baudRate = gdbSetting.getBaudRate();
		
		IPath gdbtkPath = ToolPathVarResolver.getGnuToolsPath().append(iniFileName);
		File gdbtk = gdbtkPath.toFile();

		try {
			if (gdbtk.exists() && gdbtk.canRead() && gdbtk.canWrite()) {
				 StringBuffer buf = new StringBuffer();
			   
				 // read the file and modify baud rate
				 BufferedReader br = new BufferedReader(new FileReader(gdbtk));
				 while (br.ready()) {
					 String line = br.readLine();
					 if (line.startsWith(baudRateProperty)) {
						 buf.append(baudRateProperty).append(baudRate);
					 } else {
						 buf.append(line);
					 }
					 buf.append(NEW_LINE);
				 }
				 br.close();
				 
				 // rewrite file
				 BufferedWriter bw = new BufferedWriter(new FileWriter(gdbtk));
				 bw.write(new String(buf));
				 bw.flush();
				 bw.close();
		 
			} else {
				Logger.log(ERROR_BAUD_RATE_WRITE + iniFileName);
			}
		} catch (IOException ioe) {
			Logger.log(ERROR_BAUD_RATE_WRITE + iniFileName);
		}
		
	
	}
	
	
	/**
	 * Append the given commands to the commands generated by makeContents()
	 * @param commands		commands to append
	 * @return	full command line  
	 * @since 1.3.0
	 */
	public String appendCommands(String commands) {
		return makeContents() + commands;
	}

	/**
	 * Prepends the given commands to the commands generated by makeContents()
	 * @param commands		commands to prepend
	 * @return	full command line
	 * @since 1.3.0
	 */
	public String prependCommands(String commands) {
		return commands + NEW_LINE + makeContents();
	}
	
}
