/*
 * Created on 2004/10/06
 * Modified on 2005/07/11 by D.Fujimoto modified makeContents() for writing commands for
 * 												c++ files. 
 * Modified on 2005/07/14 by D.Fujimoto made -B option to be a regularly added.
 * Modified on 2005/07/15 by D.Fujimoto modified various parts in makeContents() for creating 
 * 											dependency files.
 * 											Also added static final Strings.
 * Modified on 2005/08/04 by D.Fujimoto added MACRO_CXXFLAGS.
 * Modified on 2005/08/05 by D.Fujimoto deleted makeCommandLineForDataPointers() and 
 * 										modified parseOptions() for data pointer removal.
 * Modified on 2005/08/10 by D.Fujimoto modified makeContents() for compiler command when using 
 * 											kanji filter.
 * Modified on 2005/08/11 by D.Fujimoto added MACRO_CXXCOMPILER.
 * Modified on 2005/09/15 by D.Fujimoto modified makeContents() to use kanji filter 
 * 											on creating .d files. 
 * Modified on 2006/03/14 by D.Fujimoto modified(refactored) makeContents().
 * Modified on 2006/03/15 by D.Fujimoto modified makeContents() for additional sed.exe
 * 											commands.
 * Modified on 2006/03/30 by D.Fujimoto modified makeContents() to replace user envs,
 * 											added RESERVED_MACROS.
 * Modified on 2006/10/31 by D.Fujimoto for parsing assembler options.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.text.MessageFormat;
import java.util.List;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.LinkerScriptSetting;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;
import jp.co.epson.ide.eclipse.gnu33.variables.ToolPathVarResolver;

import org.eclipse.cdt.core.model.CoreModel;
import org.eclipse.cdt.core.model.ITranslationUnit;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;

/**
 * Creates a make file(.mak) during a build.
 * @author D.Fujimoto
 */
public class MakeFileGenerator extends AbstractFileGenerator {

	public static final String MACRO_TARGET = "TARGET";
	public static final String MACRO_GOAL = "GOAL";
	public static final String MACRO_TOOL_DIR = "TOOL_DIR";
	public static final String MACRO_ANSILIB_DIR = "ANSILIB_DIR";
	public static final String MACRO_EMULIB_DIR = "EMULIB_DIR";
	public static final String MACRO_COMPILER = "CC";
	public static final String MACRO_CXXCOMPILER = "CXX";
	public static final String MACRO_ASSEMBLER = "AS";
	public static final String MACRO_LINKER = "LD";
	public static final String MACRO_RM = "RM";
	public static final String MACRO_SED = "SED";
	public static final String MACRO_CP = "CP";
	public static final String MACRO_MOVE = "MOVE";
	public static final String MACRO_KNJFILTER = "CC_KFILT";
	public static final String MACRO_OBJDUMP = "OBJDUMP";
	public static final String MACRO_OBJCOPY = "OBJCOPY";
	public static final String MACRO_MOTO2FF = "MOTO2FF";
	public static final String MACRO_SCONV = "SCONV";

	public static final String MACRO_CFLAGS = "CFLAGS";
	public static final String MACRO_CXXFLAGS = "CXXFLAGS";
	public static final String MACRO_ASFLAGS = "ASFLAGS";
	public static final String MACRO_LDFLAGS = "LDFLAGS";
	public static final String MACRO_OBJECTS = "OBJS";
	public static final String MACRO_USR_LIB = "OBJLDS";
	public static final String MACRO_DEF_LIB = "LIBS";
	public static final String MACRO_DEPS = "DEPS";
	public static final String MACRO_DEPCMD_CC = "DEPCMD_CC";
	public static final String MACRO_DEPCMD_AS = "DEPCMD_AS";
	public static final String MACRO_SED_PTN = "SED_PTN";
	public static final String MACRO_SED_PTN2 = "SED_PTN2";
	
	// macros for ext
	public static final String MACRO_EXTFLAGS = "EXTFLAGS";
	public static final String MACRO_CEXTTEMPS = "CEXTTEMPS";
	public static final String MACRO_ASMEXTTEMPS = "ASMEXTTEMPS";
	public static final String MACRO_OBJDUMPFLAGS = "OBJDUMPFLAGS";
	public static final String MACRO_AS_CC = "AS_CC";
	public static final String MACRO_ASFLAGS_CC = "ASFLAGS_CC";
	public static final String MACRO_EXTFLAGS_CC = "EXTFLAGS_CC";
	public static final String MACRO_ALLOBJDUMP_FILE = "ALLOBJDUMP_FILE";
	public static final String MACRO_PASS = "PASS";
	
	// macros for psa
	public static final String MACRO_OBJCOPYFLAGS = "OBJCOPYFLAGS";
	public static final String MACRO_SCONVFLAGS = "SCONVFLAGS";

	
	// macro names which cannot be specified for IEnv
	public static String[] RESERVED_MACROS = {
			MACRO_TARGET,
			MACRO_GOAL,
			MACRO_TOOL_DIR,
			MACRO_ANSILIB_DIR,
			MACRO_EMULIB_DIR,
			MACRO_COMPILER,
			MACRO_CXXCOMPILER,
			MACRO_ASSEMBLER,
			MACRO_LINKER,
			MACRO_RM,
			MACRO_SED,
			MACRO_CP,
			MACRO_MOVE,
			MACRO_KNJFILTER,
			MACRO_OBJDUMP,
			MACRO_CFLAGS,
			MACRO_CXXFLAGS,
			MACRO_ASFLAGS,
			MACRO_LDFLAGS,
			MACRO_OBJECTS,
			MACRO_USR_LIB,
			MACRO_DEF_LIB,
			MACRO_DEPS,
			MACRO_DEPCMD_CC,
			MACRO_DEPCMD_AS,
			MACRO_SED_PTN,
			MACRO_SED_PTN2,
			MACRO_EXTFLAGS,
			MACRO_CEXTTEMPS,
			MACRO_ASMEXTTEMPS,
			MACRO_OBJDUMPFLAGS,
			MACRO_AS_CC,
			MACRO_ASFLAGS_CC,
			MACRO_EXTFLAGS_CC,
			MACRO_ALLOBJDUMP_FILE,
			MACRO_PASS
	}; 
	
	// for hierarchial dependency files
	// the sed command is sed.exe
	public final String DEPCMD 
		= "@{0} -M -MG {1} $< | " + expandMacro(MACRO_SED) + " -e " + expandMacro(MACRO_SED_PTN) 
		+ " | " +  expandMacro(MACRO_SED) + " -e " + expandMacro(MACRO_SED_PTN2)
		+ " >$(@:%.o=%.d)";


	// path to the tool directory
	protected IPath toolDir;
	// -B option string for the compiler
	protected String toolDirOption;
	
	// pattern string for sed
	protected String sedPattern = ResourceProvider.getResourceString("sed.pattern");
	protected String sedPattern2 = ResourceProvider.getResourceString("sed.pattern2");
	
	protected GeneralInfo generalInfo;
	protected BuildSetting buildSetting;
	protected LinkerScriptSetting ldsSetting;
	
	// the tools
	IBuildTool compiler;
	IBuildTool assembler;
	IBuildTool linker;
	
	/**
	 * Constructor. Get the BuildSetting from the descriptor file.
	 * 
	 * @see AbstractFileGenerator
	 */
	public MakeFileGenerator(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		super(project, manager, fileName + EXT_MAK);
		
		buildSetting = (BuildSetting) manager.getBuildList().get(0);
		generalInfo = manager.getGeneralInfo();
		ldsSetting = manager.getLdsSetting();
		
		// prepare the path to tools directory 
		toolDir = ToolPathVarResolver.getGnuToolsPath();
		toolDirOption = "-B" + expandMacro(MACRO_TOOL_DIR) + IPath.SEPARATOR + SPACE;
		
		// prepare the tools
		compiler = buildSetting.getCompiler();
		assembler = buildSetting.getAssembler();
		linker = buildSetting.getLinker();
		
	}

	/**
	 * Makes the contents of the file using the descriptor file.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator#makeContents()
	 *
	 */
	public String makeContents() {

		StringBuffer buf = new StringBuffer();
		
		// check the use of Kanji filter
		boolean useKanjiFilt = compiler.getOption(BuildSettingFactory.KNJFILT_OPT_ID).getBooleanValue();
		// set -c
		setExtraOption("-c");
		
		
		// make some Strings
		String executable = expandMacro(MACRO_TARGET) 
			+ EXT_EXE;
		String makFile = buildSetting.getMakefileName()
			+ EXT_MAK;
		String ldsFile = ldsSetting.getSettingName()
			+ EXT_LNK;
		
		// file header
		buf.append("# Make file generated by Gnu33 Plug-in for Eclipse")
			.append(NEW_LINE);
		buf.append("# This file should be placed directly under the project folder").append(NEW_LINE);
		
		buf.append(NEW_LINE);
	 
		// macro TARGET
		buf.append("# macro definitions for target file").append(NEW_LINE);
		buf.append(MACRO_TARGET + "= ").append(buildSetting.getSettingName()).append(NEW_LINE);
		buf.append(NEW_LINE);
		
		// macro for tools
		buf.append("# macro definitions for tools").append(NEW_LINE);
		
		// place TOOL_DIR
		buf.append(MACRO_TOOL_DIR + "= " + toolDir.toString()).append(NEW_LINE);
		
		buf.append(MACRO_COMPILER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(compiler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_CXXCOMPILER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(compiler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_ASSEMBLER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(assembler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_LINKER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(linker.getCommand()).append(NEW_LINE);
		buf.append(MACRO_RM + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.RM_CMD).append(NEW_LINE);
		// creating dependency files
		buf.append(MACRO_SED + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.SED_CMD).append(NEW_LINE);
		// Kanji filter
		buf.append(MACRO_KNJFILTER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.KNJFILTER_CMD).append(NEW_LINE);
		
		buf.append(NEW_LINE);
		

		// macro for tool options
		buf.append("# macro definitions for tool flags").append(NEW_LINE);
		
		// command line for compiler
		buf.append(MACRO_CFLAGS + "= ");
		buf.append(toolDirOption);
		parseOptions(buf, compiler);
		buf.append(NEW_LINE);
		
		// command line for c++ compiler
		buf.append(MACRO_CXXFLAGS + "= ");
		buf.append("${" + MACRO_CFLAGS + "}");
		buf.append(NEW_LINE);
		
		// command line for assembler
		buf.append(MACRO_ASFLAGS + "= ");
		buf.append(toolDirOption);
		parseOptions(buf, assembler);
		buf.append(NEW_LINE);
		
		// command line for linker
		buf.append(MACRO_LDFLAGS + "= ");
		parseOptions(buf, linker);
		buf.append(NEW_LINE);
		
		buf.append(NEW_LINE);
		
		// macro for object files
		buf.append("# macro definitions for object files").append(NEW_LINE);
		buf.append(MACRO_OBJECTS + "= ");
		// project accessing utility
		ProjectAccessor util = new ProjectAccessor(project);
		List sourceFiles = util.getSourceFilesFromProject();
		List objs = util.getAssumedObjectFiles();
		for (int i = 0; i < objs.size(); i++) {
			buf.append((String) objs.get(i)).append(" \\");
			buf.append(NEW_LINE);
			buf.append("      ");
		}
		buf.append(NEW_LINE);
		
		// macro for user libraries
		buf.append("# macro definitions for library files").append(NEW_LINE);
		buf.append(MACRO_USR_LIB + "= ");
		String[] libraries = linker.getOption(BuildSettingFactory.LD_LIB_OPT_ID).getListValue();
		for (int i = 0; i < libraries.length; i++) {
			buf.append(libraries[i]).append(" \\");
			buf.append(NEW_LINE);
			buf.append("        ");
		}
		buf.append(NEW_LINE);
		
		// macro for dependency file list
		buf.append("# macro definitions for dependency files").append(NEW_LINE);
		buf.append(MACRO_DEPS + "= ");
		buf.append("$(" + MACRO_OBJECTS + ":%.o=%.d)").append(NEW_LINE);
		
		// macro for replacement pattern
		buf.append(MACRO_SED_PTN + "= ");
		buf.append("\'").append(sedPattern).append("\'").append(NEW_LINE);
		
		// hierarchial dependency files
		buf.append(MACRO_SED_PTN2 + "= ");
		buf.append("\'").append(sedPattern2).append("\'").append(NEW_LINE);
		
		// macro for creating dependency files
		buf.append(NEW_LINE);
		buf.append("# macro definitions for creating dependency files").append(NEW_LINE);
		buf.append(MACRO_DEPCMD_CC + "= ");

		// use Kanji filter for creating dep files
		String[] depCmdArgs = null;
		if (useKanjiFilt) {
			depCmdArgs = new String[] {expandMacro(MACRO_KNJFILTER), expandMacro(MACRO_CFLAGS)};
		} else {
			depCmdArgs = new String[] {expandMacro(MACRO_COMPILER), expandMacro(MACRO_CFLAGS)};
		}
		buf.append(MessageFormat.format(DEPCMD, depCmdArgs)).append(NEW_LINE);
		buf.append(MACRO_DEPCMD_AS + "= ");
		buf.append(MessageFormat.format(DEPCMD, new String[] {
				expandMacro(MACRO_ASSEMBLER), expandMacro(MACRO_ASFLAGS)
		})).append(NEW_LINE);
		buf.append(NEW_LINE);
		
		// targets and dependencies
		buf.append("# targets and dependencies").append(NEW_LINE);
		buf.append(".PHONY : all clean").append(NEW_LINE);
		buf.append(NEW_LINE);

		
		// all
		buf.append("all : ").append(executable).append(NEW_LINE);
		buf.append(NEW_LINE);
		
		// $(TARGET).elf
		buf.append(executable).append(" : ")
			.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
			.append(makFile).append(SPACE)
			.append(ldsFile)
			.append(NEW_LINE);
		buf.append(TAB).append(expandMacro(MACRO_LINKER)).append(SPACE)
			.append(expandMacro(MACRO_LDFLAGS)).append(SPACE)
			.append("-o $@ ")
			.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
			.append(expandMacro(MACRO_USR_LIB)).append(SPACE)
			.append(NEW_LINE);
		
		buf.append(NEW_LINE);
		
		// source files
		for (int i = 0; i < objs.size(); i++) {
			IFile sourceFile = ((IFile) sourceFiles.get(i));
			buf.append("## ").append(sourceFile.getProjectRelativePath().toString()).append(NEW_LINE);

			buf.append((String) objs.get(i)).append(" : ")
				.append(sourceFile.getProjectRelativePath()).append(NEW_LINE);

			if (CoreModel.isTranslationUnit(sourceFile)) {
				ITranslationUnit cUnit = (ITranslationUnit) CoreModel.getDefault().create(sourceFile);
				if (cUnit.isSourceUnit()) {
					if (cUnit.isSourceUnit() && (cUnit.isCLanguage() || cUnit.isCXXLanguage())) {
						// c/c++ source
						buf.append(TAB);

						// the compiler command varies on the kanji filter setting
						if (useKanjiFilt) {
							buf.append(expandMacro(MACRO_KNJFILTER));
						} else {
							buf.append(expandMacro(MACRO_COMPILER));
						}
						buf.append(SPACE)
							.append(expandMacro(MACRO_CFLAGS)).append(SPACE)
							.append("-o $@ $<").append(NEW_LINE);
						
						// dependency file creation
						buf.append(TAB);
						buf.append(expandMacro(MACRO_DEPCMD_CC)).append(NEW_LINE);
						
					} else if (cUnit.isASMLanguage()) {
						// asm source
						buf.append(TAB);
						buf.append(expandMacro(MACRO_ASSEMBLER)).append(SPACE)
							.append(expandMacro(MACRO_ASFLAGS)).append(SPACE)
							.append("-o $@ $<").append(NEW_LINE);

						// dependency file creation
						buf.append(TAB);
						buf.append(expandMacro(MACRO_DEPCMD_AS)).append(NEW_LINE);
						
					}
				}
			}
			
			buf.append(NEW_LINE);
		}
		
		// include the generated dependency files
		buf.append("# include dependency files").append(NEW_LINE);
		buf.append("-include ").append(expandMacro(MACRO_DEPS)).append(NEW_LINE).append(NEW_LINE);
		
		// clean
		buf.append("# clean files").append(NEW_LINE);
		buf.append("clean :").append(NEW_LINE);
		buf.append(TAB)
			.append(expandMacro(MACRO_RM)).append(SPACE)
			.append("-f").append(SPACE)
			.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
			.append(expandMacro(MACRO_TARGET))
			.append(EXT_EXE).append(SPACE)
			.append(expandMacro(MACRO_TARGET))
			.append(EXT_MAP).append(SPACE);
		buf.append(expandMacro(MACRO_DEPS)).append(SPACE);
		buf.append(NEW_LINE);

		// replacing user envs
		return replaceEnvVars(new String(buf), buildSetting.getEnvVars());
		
	}
	
	/**
	 * Sets an extra option to the compiler default options.
	 * @param option
	 * @since 1.4.0
	 */
	public void setExtraOption(String option) {
		// add the new option to the array of default options
		String[] defaults = compiler.getDefaultOptions();
		String[] newDefaults = new String[defaults.length + 1];
		System.arraycopy(defaults, 0, newDefaults, 0, defaults.length);
		
		newDefaults[newDefaults.length - 1] = option;
		compiler.setDefaultOptions(newDefaults);
	}
	
	/**
	 * Converts the given String to a GNU macro variable , like
	 * $(macro).
	 * 
	 * @param macro the macro name
	 * @return formatted String as $(macro)
	 */
	public static String expandMacro(String macro) {
		return new String("$(" + macro + ")");
	}
	
	/**
	 * Creates command line with the build settings.
	 * @param buf
	 * @param tool
	 */
	public void parseOptions(StringBuffer buf, IBuildTool tool) {
		
		StringBuffer cmdBuf = new StringBuffer();
		
		List categoryList = tool.getOptionCategories();
		for (int i = 0; i < categoryList.size(); i++) {
			IBuildCategory category = (IBuildCategory) categoryList.get(i);
			List optionList = category.getOptions();
			IBuildOption[] options = (IBuildOption[]) optionList.toArray(new IBuildOption[0]);
		
			makeCommandLine(cmdBuf, options);
		}
		
		
		// append the default options
		String[] defOptions = tool.getDefaultOptions();
		StringBuffer defaultBuf = new StringBuffer();
		for (int i = 0; i < defOptions.length; i++) {
			defaultBuf.append(defOptions[i] + " "); 
		}

		// parsing assembler options
		if (tool.getToolID().equals(BuildSettingFactory.ASSEMBLER_ID)) {
			StringBuffer optBuf = new StringBuffer();
			
			// add -Wa,options,...
			if (cmdBuf.length() > 0) {
				optBuf.append("-Wa,");
				
				StringTokenizer tok = new StringTokenizer(new String(cmdBuf));
				while (tok.hasMoreTokens()) {
					optBuf.append(tok.nextToken());
					if (tok.hasMoreTokens()) {
						optBuf.append(",");
					}
				}
			}
			
			cmdBuf = optBuf;
			
		}
		
		// default options + regular options
		cmdBuf = defaultBuf.append(" ").append(cmdBuf);
		
		buf.append(cmdBuf);

	}
	
   
	/**
	 * Parses the options to a command line.
	 * Almost the same as 
	 * jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build.BuildSettingsToolPage#
	 * makeCommandLine(StringBuffer, IBuildOption[], IPreferenceStore)
	 * @param buf
	 * @param options
	 */
	public void makeCommandLine(StringBuffer buf, IBuildOption[] options) {
		
		for (int i = 0; i < options.length; i++) {
			
			IBuildOption option = options[i];
			switch (option.getType()) {
				case IBuildOption.TYPE_BOOLEAN :
					if (option.getBooleanValue()) {
						buf.append(option.getCommand() + IBuildTool.SPACE);
						if (option.getId().equals(BuildSettingFactory.GEN_MAP_OPT_ID)) {
							// for -Map option
							buf.append(buildSetting.getSettingName()
									+ EXT_MAP).append(IBuildTool.SPACE);
						}
					}
					break;
				case IBuildOption.TYPE_STRING :
				case IBuildOption.TYPE_STRING_MULTI :
					String val = option.getStringValue();
					if (val.length() > 0) {
						buf.append(val + IBuildTool.SPACE);
					}
					break;
				case IBuildOption.TYPE_ENUM :
					String enumCmd = option.getEnumCommand();
					if (enumCmd != null) {
						buf.append(enumCmd + IBuildTool.SPACE);
					}						
					break;
				case IBuildOption.TYPE_LIST :
					// do not show the libraries
					if (option.getId().equals(BuildSettingFactory.LD_LIB_OPT_ID)) {
						break;
					}
					String[] listVal = option.getListValue();
					String listCmd = option.getCommand();
					for (int j = 0; j < listVal.length; j++) {
						buf.append(listCmd + listVal[j]).append(IBuildTool.SPACE);
					}
					break;
			}
		}
	}
	
}
