/*
 * Created on 2007/01/19
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * This class is applicable for GNU33 only.
 * 
 * Modified on 2007/10/17 by D.Fujimoto ignore linker error code to show message.
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.io.File;
import java.text.MessageFormat;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;

import org.eclipse.cdt.core.model.CoreModel;
import org.eclipse.cdt.core.model.ITranslationUnit;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;

/**
 * Alternate Make file generator for a 2 pass make.
 * @author D.Fujimoto
 */
public class MakeFileGenerator2 extends MakeFileGenerator {

	// extension for the temporary assembler input file generated from c files
	public static final String EXT_2PASS = "ext0";
	
/* >>>>> ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
	public static final String ALLOBJDUMP_FILE = ResourceProvider.getResourceString("allobjdump.filename");
/* <<<<< ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
	
	// file for linker error
	public static final String LINK_ERR_FILE = "lderr"; 

	// shell command for checking the linker error file
	public static final String CHECK_LINK_ERR_CMD = 
		TAB + "@if [ -s " + LINK_ERR_FILE + " ]; then \\\n" +
		TAB + TAB +	"cmd /c \"type " + LINK_ERR_FILE + "\" \\\n" +
		TAB + TAB + "&& " + expandMacro(MACRO_RM) + " -f " + expandMacro(MACRO_TARGET) + EXT_EXE + " \\\n" +
		TAB + TAB + "&& exit 1; \\\n" +
		TAB + "else " + expandMacro(MACRO_RM) + " -f " + LINK_ERR_FILE + " ; \\\n" +
		TAB + "fi";
	
	// directory name for 1pass object files
	public static final String OBJ_1PASS_DIR = "obj1pass";
	
	// shell command for removing the 1pass object dir
	public static final String CLEAN_1PASS_DIR_CMD = 
		TAB + "@if [ -e " + OBJ_1PASS_DIR + " ]; then \\\n" +
		TAB + TAB +	"cmd /c \"rd /s /q " + OBJ_1PASS_DIR + "\" ; \\\n" +
		TAB + "fi";

	// shell command for making the save dir
	public static final String MAKE_OBJ_1PASS_DIR_CMD =
		TAB + "cmd /c \"md "+ OBJ_1PASS_DIR + "\"";
		
	
	// shell command for saving 1pass objects
	public static final String SAVE_OBJ_1PASS_CMD =
		TAB + "for NAME in " + expandMacro(MACRO_OBJECTS) + " ; do \\\n" + 
		TAB + TAB + expandMacro(MACRO_CP) + " -pf " + "$$NAME " + OBJ_1PASS_DIR + "/$$NAME ; done \\\n" +
		TAB + "&& " + expandMacro(MACRO_RM) + " -f " + expandMacro(MACRO_OBJECTS);

	// shell command for saving 2pass objects
	public static final String RESTORE_OBJ_1PASS_CMD =
		TAB + expandMacro(MACRO_RM) + " -f " + expandMacro(MACRO_OBJECTS) + " \\\n" +
		TAB + "&& \\\n" +
		TAB + "for NAME in " + expandMacro(MACRO_OBJECTS) + " ; do \\\n" +
		TAB + TAB + expandMacro(MACRO_CP) + " -pf " + OBJ_1PASS_DIR + "/$$NAME $$NAME ; done \\\n" +
		TAB + "&& cmd /c \"rd /s /q " + OBJ_1PASS_DIR + "\""; 

	// flag indicating 2pass/1pass 
	public static final String FLAG_2PASS = "2pass";
	public static final String FLAG_1PASS = "1pass";
	
	
	// plain assembler for 2pass 
	IBuildTool cassembler = BuildSettingFactory.createCAssembler();
	
	/**
	 * Constructor. Get the BuildSetting from the descriptor file.
	 * 
	 * @see AbstractFileGenerator
	 */
	public MakeFileGenerator2(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		super(project, manager, fileName); 
		
	}

	/**
	 * Makes the contents of the file using the descriptor file.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator#makeContents()
	 * 
	 */
	public String makeContents() {

		StringBuffer buf = new StringBuffer(2048);
		
		// check the use of Kanji filter
		boolean useKanjiFilt = compiler.getOption(BuildSettingFactory.KNJFILT_OPT_ID).getBooleanValue();
		// set -S
		setExtraOption("-S");
		
		
		// make some Strings
		String executable = expandMacro(MACRO_TARGET) 
			+ EXT_EXE; 
		String makFile = buildSetting.getMakefileName()
			+ EXT_MAK; 
		String ldsFile = ldsSetting.getSettingName()
			+ EXT_LNK; 
		
		// file header
		buf.append("# Make file generated by Gnu33 Plug-in for Eclipse")
			.append(NEW_LINE);
		buf.append("# This file should be placed directly under the project folder").append(NEW_LINE);
		
		buf.append(NEW_LINE);
	 
		// macro TARGET
		buf.append("# macro definitions for target file").append(NEW_LINE);
		buf.append(MACRO_TARGET + "= ").append(buildSetting.getSettingName()).append(NEW_LINE);
		
/* >>>>> ADDED by D.Fujimoto 2008/06/03 goal */
		// macro GOAL
		buf.append(MACRO_GOAL + "= ");
		buf.append(executable).append(NEW_LINE);		// executable only for GNU33
		buf.append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2008/06/03 goal */
		
		// macro for tools
		buf.append("# macro definitions for tools").append(NEW_LINE);
		
		// place TOOL_DIR
		buf.append(MACRO_TOOL_DIR + "= " + toolDir.toString()).append(NEW_LINE);
		
		buf.append(MACRO_COMPILER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(compiler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_CXXCOMPILER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
		.append(compiler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_ASSEMBLER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(assembler.getCommand()).append(NEW_LINE);
		buf.append(MACRO_AS_CC + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(cassembler.getCommand()).append(NEW_LINE);	// for ext0 src 2pass
		buf.append(MACRO_LINKER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(linker.getCommand()).append(NEW_LINE);
		buf.append(MACRO_RM + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.RM_CMD).append(NEW_LINE);
		// creating dependency files
		buf.append(MACRO_SED + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.SED_CMD).append(NEW_LINE);
		// create copy utility
		buf.append(MACRO_CP + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.CP_CMD).append(NEW_LINE);
		// Kanji filter
		buf.append(MACRO_KNJFILTER + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.KNJFILTER_CMD).append(NEW_LINE);
		// objdump
		buf.append(MACRO_OBJDUMP + "= ").append(expandMacro(MACRO_TOOL_DIR)).append(IPath.SEPARATOR)
			.append(BuildSettingFactory.OBJDUMP_CMD).append(NEW_LINE);

		buf.append(NEW_LINE);
		

		// macro for tool options
		buf.append("# macro definitions for tool flags").append(NEW_LINE);
		
		// command line for compiler
		buf.append(MACRO_CFLAGS + "= ");
		buf.append(toolDirOption);
		parseOptions(buf, compiler);
		buf.append(NEW_LINE);
		
		// command line for c++ compiler
		buf.append(MACRO_CXXFLAGS + "= ");
		buf.append("${" + MACRO_CFLAGS + "}");
		buf.append(NEW_LINE);

		// command line for assembler
		buf.append(MACRO_ASFLAGS + "= ");
		buf.append(toolDirOption);
		parseOptions(buf, assembler);
		buf.append(NEW_LINE);
		
		// command line for ext0 files 
		buf.append(MACRO_ASFLAGS_CC + "= ");
		// add CPU specific options
		BuildSettingFactory.switchCpuOptions(cassembler, generalInfo.getCpuType());
		parseOptions(buf, cassembler);
		// also add the medda option here(special)
		addSpecialOption(buf);
		buf.append(NEW_LINE);
		
		// command line for linker
		buf.append(MACRO_LDFLAGS + "= ");
		parseOptions(buf, linker);
		buf.append(NEW_LINE);

/* >>>>> ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
		// file name for allobjects' dump
		buf.append(MACRO_ALLOBJDUMP_FILE + "= ");
		buf.append(ALLOBJDUMP_FILE);
		buf.append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
		
		// command line for ext
		buf.append(MACRO_EXTFLAGS + "= ");
		buf.append(BuildSettingFactory.get2passOptions());
		buf.append(NEW_LINE);
		
		// command line for ext0 files 
		buf.append(MACRO_EXTFLAGS_CC + "= ");
		buf.append(BuildSettingFactory.get2passOptions(false));
		buf.append(NEW_LINE);
		
		buf.append(MACRO_OBJDUMPFLAGS + "= ");
		buf.append("-t");
		buf.append(NEW_LINE);
		
		buf.append(NEW_LINE);
		
/* >>>>> ADDED by D.Fujimoto 2008/06/03 2pass 1pass switch */
		buf.append("# macro for switching 2pass or 1pass build").append(NEW_LINE);
		buf.append(MACRO_PASS + "= ");
		if (buildSetting.is2passBuild()) {
			buf.append(FLAG_2PASS);
		} else {
			buf.append(FLAG_1PASS);
		}
		buf.append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2008/06/03 2pass 1pass switch */

		buf.append(NEW_LINE);
		
		// macro for object files
		buf.append("# macro definitions for object files").append(NEW_LINE);
		buf.append(MACRO_OBJECTS + "= ");
		// project accessing utility
		ProjectAccessor util = new ProjectAccessor(project);
		List sourceFiles = util.getSourceFilesFromProject();
		List objs = util.getAssumedObjectFiles();
		for (int i = 0; i < objs.size(); i++) {
			buf.append((String) objs.get(i)).append(" \\");
			buf.append(NEW_LINE);
			buf.append("      ");
		}
		buf.append(NEW_LINE);
		
		// macro for user libraries
		buf.append("# macro definitions for library files").append(NEW_LINE);
		buf.append(MACRO_USR_LIB + "= ");
		String[] libraries = linker.getOption(BuildSettingFactory.LD_LIB_OPT_ID).getListValue(); 
		for (int i = 0; i < libraries.length; i++) {
			buf.append(libraries[i]).append(" \\");
			buf.append(NEW_LINE);
			buf.append("        ");
		}
		buf.append(NEW_LINE);
		
		// macro for c source 
		buf.append("# macro definitions for assembly files generated from c source files").append(NEW_LINE);
		buf.append(MACRO_CEXTTEMPS + "= ");
		for (int i = 0; i < objs.size(); i++) {
			IFile sourceFile = ((IFile) sourceFiles.get(i));

			if (CoreModel.isTranslationUnit(sourceFile)) {
				ITranslationUnit cUnit = (ITranslationUnit) CoreModel.getDefault().create(sourceFile);
				if (cUnit.isCLanguage() || cUnit.isCXXLanguage()) {
					buf.append(sourceFile.getProjectRelativePath().removeFileExtension().addFileExtension(EXT_2PASS))
						.append(" \\")
						.append(NEW_LINE);
				}
			}
			
			buf.append("      ");
		}
		buf.append(NEW_LINE);
		buf.append(NEW_LINE);

		// macro for asm files
		buf.append("# macro definitions for original assembly files").append(NEW_LINE);
		buf.append(MACRO_ASMEXTTEMPS + "= ");
		for (int i = 0; i < objs.size(); i++) {
			IFile sourceFile = ((IFile) sourceFiles.get(i));

			if (CoreModel.isTranslationUnit(sourceFile)) {
				ITranslationUnit cUnit = (ITranslationUnit) CoreModel.getDefault().create(sourceFile);
				if (cUnit.isASMLanguage()) {
					buf.append(sourceFile.getProjectRelativePath())
						.append(" \\")
						.append(NEW_LINE);
				}
			}
			
			buf.append("      ");
		}
		buf.append(NEW_LINE);
		buf.append(NEW_LINE);
		
		
		// macro for dependency file list
		buf.append("# macro definitions for dependency files").append(NEW_LINE);
		buf.append(MACRO_DEPS + "= ");
		buf.append("$(" + MACRO_OBJECTS + ":%.o=%.d)").append(NEW_LINE);
		
		// macro for replacement pattern
		buf.append(MACRO_SED_PTN + "= ");
		buf.append("\'").append(sedPattern).append("\'").append(NEW_LINE);
		
		// hierarchial dependency files
		buf.append(MACRO_SED_PTN2 + "= ");
		buf.append("\'").append(sedPattern2).append("\'").append(NEW_LINE);
		
		// macro for creating dependency files
		buf.append(NEW_LINE);
		buf.append("# macro definitions for creating dependency files").append(NEW_LINE);
		buf.append(MACRO_DEPCMD_CC + "= ");

		// use Kanji filter for creating dep files
		String[] depCmdArgs = null;
		if (useKanjiFilt) {
			depCmdArgs = new String[] {expandMacro(MACRO_KNJFILTER), expandMacro(MACRO_CFLAGS)};
		} else {
			depCmdArgs = new String[] {expandMacro(MACRO_COMPILER), expandMacro(MACRO_CFLAGS)};
		}
		buf.append(MessageFormat.format(DEPCMD, depCmdArgs)).append(NEW_LINE);
		buf.append(MACRO_DEPCMD_AS + "= ");
		buf.append(MessageFormat.format(DEPCMD, new String[] {
				expandMacro(MACRO_ASSEMBLER), expandMacro(MACRO_ASFLAGS)
		})).append(NEW_LINE);
		buf.append(NEW_LINE);
		
		// targets and dependencies
		buf.append("# targets and dependencies").append(NEW_LINE);
		buf.append(".PHONY : all clean").append(NEW_LINE);
		buf.append(NEW_LINE);

		
		// all
/* >>>>> MODIFIED by D.Fujimoto 2008/06/03 goal */
//		buf.append("all : ").append(executable).append(NEW_LINE);
		buf.append("all : ").append(expandMacro(MACRO_GOAL)).append(NEW_LINE);
/* <<<<< MODIFIED by D.Fujimoto 2008/06/03 goal */
		buf.append(NEW_LINE);
		
		// $(TARGET).elf
		buf.append(executable).append(" : ")
			.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
			.append(makFile).append(SPACE)
			.append(ldsFile)
			.append(NEW_LINE);

/* >>>>> MODIFIED by D.Fujimoto 2008/06/03 2pass 1pass switch */
		// 1pass/2pass switch
		buf.append("ifeq (" + expandMacro(MACRO_PASS) + ", " + FLAG_1PASS + ")").append(NEW_LINE);
		new MakeFile1passPortion().appendComamnds(buf);
		buf.append("else").append(NEW_LINE);
		new MakeFile2passPortion().appendComamnds(buf);
		buf.append("endif").append(NEW_LINE);
/* <<<<< MODIFIED by D.Fujimoto 2008/06/03 2pass 1pass switch */
		
		buf.append(NEW_LINE);
		
		
		// source files
		for (int i = 0; i < objs.size(); i++) {
			IFile sourceFile = ((IFile) sourceFiles.get(i));
			buf.append("## ").append(sourceFile.getProjectRelativePath().toString()).append(NEW_LINE);

			if (CoreModel.isTranslationUnit(sourceFile)) {
				ITranslationUnit cUnit = (ITranslationUnit) CoreModel.getDefault().create(sourceFile);

				buf.append((String) objs.get(i)).append(" : ")
					.append(sourceFile.getProjectRelativePath());
				if (cUnit.isCLanguage() || cUnit.isCXXLanguage()) {
					buf.append(SPACE).append(
							sourceFile.getProjectRelativePath().removeFileExtension().addFileExtension(EXT_2PASS));
				}
				buf.append(NEW_LINE);
				
				if (cUnit.isSourceUnit()) {
					if (cUnit.isSourceUnit() && (cUnit.isCLanguage() || cUnit.isCXXLanguage())) {
						// c/c++ source
						buf.append(TAB);

						// the compiler command varies on the kanji filter setting
						if (useKanjiFilt) {
							buf.append(expandMacro(MACRO_KNJFILTER));
						} else {
							buf.append(expandMacro(MACRO_COMPILER));
						}
						// compile c source
						buf.append(SPACE)
							.append(expandMacro(MACRO_CFLAGS)).append(SPACE)
							.append("-o $(@:%.o=%." + EXT_2PASS + ") $<").append(NEW_LINE);
						 
						// assemble c source generated ext0 files
						buf.append(TAB);
						buf.append(expandMacro(MACRO_AS_CC)).append(SPACE)
							.append(expandMacro(MACRO_ASFLAGS_CC)).append(SPACE)
							.append("-o $@ $(@:%.o=%." + EXT_2PASS + ") ").append(NEW_LINE);
						
						// dependency file creation
						buf.append(TAB);
						buf.append(expandMacro(MACRO_DEPCMD_CC)).append(NEW_LINE);
						
					} else if (cUnit.isASMLanguage()) {
						// asm source
						buf.append(TAB);
						buf.append(expandMacro(MACRO_ASSEMBLER)).append(SPACE)
							.append(expandMacro(MACRO_ASFLAGS)).append(SPACE)
							.append("-o $@ $<").append(NEW_LINE);

						// dependency file creation
						buf.append(TAB);
						buf.append(expandMacro(MACRO_DEPCMD_AS)).append(NEW_LINE);
						
					}
				}
			}
			
			buf.append(NEW_LINE);
		}
		
		buf.append(NEW_LINE);
		
		// ext file dependecies (different from GNU17)
		buf.append("# dependecies for assembled c source files").append(NEW_LINE);
		for (int i = 0; i < objs.size(); i++) {
			IFile sourceFile = ((IFile) sourceFiles.get(i));

			if (CoreModel.isTranslationUnit(sourceFile)) {
				ITranslationUnit cUnit = (ITranslationUnit) CoreModel.getDefault().create(sourceFile);
				if (cUnit.isCLanguage() || cUnit.isCXXLanguage()) {
					buf.append(sourceFile.getProjectRelativePath().removeFileExtension().addFileExtension(EXT_2PASS))
						.append(" : ")
						.append(sourceFile.getProjectRelativePath())
						.append(NEW_LINE);
				}
			}
			
		}
		buf.append(NEW_LINE);
		
		
		// include the generated dependency files
		buf.append("# include dependency files").append(NEW_LINE);
		buf.append("-include ").append(expandMacro(MACRO_DEPS)).append(NEW_LINE).append(NEW_LINE);
		
		// clean
		buf.append("# clean files").append(NEW_LINE);
		buf.append("clean :").append(NEW_LINE);
		buf.append(TAB)
			.append(expandMacro(MACRO_RM)).append(SPACE)
			.append("-f").append(SPACE)
			.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
			.append(expandMacro(MACRO_TARGET))
			.append(EXT_EXE).append(SPACE)  
			.append(expandMacro(MACRO_TARGET))
			.append(EXT_MAP).append(SPACE);	
		buf.append(expandMacro(MACRO_DEPS)).append(SPACE);
		buf.append(expandMacro(MACRO_CEXTTEMPS)).append(SPACE);
		buf.append(expandMacro(MACRO_TARGET)).append(EXT_DMP).append(SPACE);
/* >>>>> ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
		buf.append(expandMacro(MACRO_ALLOBJDUMP_FILE)).append(EXT_DMP).append(SPACE);
/* <<<<< ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
		buf.append(LINK_ERR_FILE).append(SPACE);
		buf.append(NEW_LINE);

		buf.append(CLEAN_1PASS_DIR_CMD);
		buf.append(NEW_LINE);

		// replacing user envs
		return replaceEnvVars(new String(buf), buildSetting.getEnvVars());
		
	}
	
	/**
	 * Special case for adding the medda32 option. 
	 * NOTE: this is only a patch method
	 * 
	 * @return the medda32 option set for assembling compiler generated files 
	 */
	private final void addSpecialOption(StringBuffer buf) {
		IBuildOption option = compiler.getOption(BuildSettingFactory.C_MEDDA_OPT_ID);
		if (option.getBooleanValue()) {
			makeCommandLine(buf, new IBuildOption[] {option});	
		}
		
	}
	
	
	
	/**
	 * Command appender for building in 2pass. 
	 * 
	 * @author D.Fujimoto
	 * @since 1.4.1	refactored from makeContents()
	 */
	class MakeFile2passPortion {
		
		/**
		 * Appends command snippets for build 
		 * @param buf		the buffer to append the commands to
		 */
		public void appendComamnds(StringBuffer buf) {
			
			buf.append("# 1pass linking").append(NEW_LINE);
			buf.append(TAB)
/* >>>>> ADDED by D.Fujimoto 2007/10/17 ignore linker error code to show message */
				.append("-")
/* <<<<< ADDED by D.Fujimoto 2007/10/17 ignore linker error code to show message */
				.append(expandMacro(MACRO_LINKER)).append(SPACE)
				.append(expandMacro(MACRO_LDFLAGS)).append(SPACE)
				.append("-o $@ ")
				.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
				.append(expandMacro(MACRO_USR_LIB)).append(SPACE)
				.append("2>").append(LINK_ERR_FILE).append(SPACE)
				.append(NEW_LINE);

			// check linker errors
			buf.append(CHECK_LINK_ERR_CMD).append(NEW_LINE);
			
			
			// create symbol table file
			buf.append(TAB).append(expandMacro(MACRO_OBJDUMP)).append(SPACE)
				.append(expandMacro(MACRO_OBJDUMPFLAGS)).append(SPACE)
				.append("$@").append(SPACE)
				.append(">").append(SPACE)
				.append(expandMacro(MACRO_TARGET)).append(EXT_DMP).append(SPACE)
				.append(NEW_LINE);
			
			// remove 1pass executable
			buf.append(TAB).append(expandMacro(MACRO_RM)).append(SPACE)
				.append("-f").append(SPACE)
				.append(expandMacro(MACRO_TARGET))
				.append(EXT_EXE).append(SPACE);  
			buf.append(NEW_LINE);

/* >>>>> ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
			// create all objects dump file
			buf.append("# create all objets dump file").append(NEW_LINE);
			buf.append(TAB).append(expandMacro(MACRO_RM)).append(SPACE)
				.append("-f").append(SPACE)
				.append(expandMacro(MACRO_ALLOBJDUMP_FILE))
				.append(EXT_DMP).append(SPACE)
				.append(NEW_LINE);
			buf.append(TAB).append("for NAME in ").append(expandMacro(MACRO_OBJECTS)).append(" ; do \\").append(NEW_LINE);
			buf.append(TAB).append(TAB).append(expandMacro(MACRO_OBJDUMP)).append(SPACE)
				.append(expandMacro(MACRO_OBJDUMPFLAGS)).append(SPACE)
				.append("$$NAME").append(SPACE).append(">>").append(SPACE)
				.append(expandMacro(MACRO_ALLOBJDUMP_FILE)).append(EXT_DMP)
				.append("  ; done")
				.append(NEW_LINE);
/* <<<<< ADDED by D.Fujimoto 2008/01/09 2pass 2nd arg */
			
			// save 1pass objects
			buf.append("# save 1pass object files").append(NEW_LINE);
			buf.append(CLEAN_1PASS_DIR_CMD).append(NEW_LINE);
			buf.append(MAKE_OBJ_1PASS_DIR_CMD).append(NEW_LINE);
			// create copies for source folders
			ProjectAccessor util = new ProjectAccessor(project);	// added
			List folders = util.getSourceFoldersFromProject(true);
			if (folders.size() > 0) {
				for (int i = 0; i < folders.size(); i++) {
					IResource folder = (IResource) folders.get(i);
					IPath path = folder.getProjectRelativePath();
					
					buf.append(TAB).append("-cmd /c \"md ");
					buf.append(OBJ_1PASS_DIR).append(File.separator).append(path.toOSString());
					buf.append("\"").append(NEW_LINE);
				}
				
			}
			buf.append(SAVE_OBJ_1PASS_CMD).append(NEW_LINE);
			
			
			// assembler 2pass
			buf.append("# 2pass for assembly files").append(NEW_LINE);
			buf.append(TAB).append("for NAME in $(basename " + expandMacro(MACRO_ASMEXTTEMPS) + ")  ; do \\").append(NEW_LINE);
			buf.append(TAB).append(TAB).append(expandMacro(MACRO_ASSEMBLER)).append(SPACE)
				.append(expandMacro(MACRO_ASFLAGS)).append(SPACE)
				.append(expandMacro(MACRO_EXTFLAGS)).append(SPACE)
				.append("-o $$NAME.o $$NAME.s ; done").append(NEW_LINE);
			
			// c 2pass 
			buf.append("# 2pass for c files").append(NEW_LINE);
			buf.append(TAB).append("for NAME in $(basename " + expandMacro(MACRO_CEXTTEMPS) + ")  ; do \\").append(NEW_LINE);
			buf.append(TAB).append(TAB).append(expandMacro(MACRO_AS_CC)).append(SPACE)
				.append(expandMacro(MACRO_ASFLAGS_CC)).append(SPACE)
				.append(expandMacro(MACRO_EXTFLAGS_CC)).append(SPACE)
				.append("-o $$NAME.o $$NAME." + EXT_2PASS + " ; done").append(NEW_LINE);
			
			// remove map
			buf.append(TAB).append(expandMacro(MACRO_RM)).append(SPACE)
				.append("-f").append(SPACE)
				.append(expandMacro(MACRO_TARGET))
				.append(EXT_MAP).append(SPACE);	
			buf.append(NEW_LINE);

			// 2nd link
			buf.append("# 2pass linking").append(NEW_LINE);
			buf.append(TAB).append(expandMacro(MACRO_LINKER)).append(SPACE)
				.append(expandMacro(MACRO_LDFLAGS)).append(SPACE)
				.append("-o $@ ")
				.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
				.append(expandMacro(MACRO_USR_LIB)).append(SPACE)
				.append(NEW_LINE);
		
			// restore 1pass objects
			buf.append("# restore 1pass object files").append(NEW_LINE);
			buf.append(RESTORE_OBJ_1PASS_CMD).append(NEW_LINE);
			
		}
	}

	
	/**
	 * Command appender for building 1pass. 
	 * 
	 * @author D.Fujimoto
	 * @since 1.4.1
	 */
	class MakeFile1passPortion {
		
		
		/**
		 * Appends command snippets for build 
		 * @param buf		the buffer to append the commands to
		 */
		public void appendComamnds(StringBuffer buf) {
			
			buf.append("# 1pass linking").append(NEW_LINE);
			buf.append(TAB).append(expandMacro(MACRO_LINKER)).append(SPACE)
				.append(expandMacro(MACRO_LDFLAGS)).append(SPACE)
				.append("-o $@ ")
				.append(expandMacro(MACRO_OBJECTS)).append(SPACE)
				.append(expandMacro(MACRO_USR_LIB)).append(SPACE)
				.append(NEW_LINE);
	
		}
	}

}
