/*
 * Created on 2004/10/06
 * Modified on 2006/03/14 by .D.Fujimoto modified(refactored) makeContents().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.file;

import java.util.List;
import java.util.StringTokenizer;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterArea;
import jp.co.epson.ide.eclipse.gnu33.parameter.ParameterSetting;

import org.eclipse.core.resources.IProject;

/**
 * Creates a parameter file(.par) during a build.
 * @author D.Fujimoto
 */
public class ParameterFileGenerator extends AbstractFileGenerator {

	private ParameterSetting parSetting;

	/**
	 * Constructor. Get the ParameterSetting from the descriptor file.
	 * 
	 * @see AbstractFileGenerator
	 */
	public ParameterFileGenerator(IProject project,
			Gnu33ProjectDescriptorManager manager, String fileName) {
		super(project, manager, fileName + EXT_PAR);

		parSetting = (ParameterSetting) manager.getParameterList().get(0);
	}

	/**
	 * Makes the contents of the file using the descriptor file.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator#makeContents()
	 */
	public String makeContents() {

		StringBuffer buf = new StringBuffer();

		// file header
		buf.append("# Parameter file generated by Gnu33 Plug-in for Eclipse")
				.append(NEW_LINE);

		// setting comment
		String comment = parSetting.getComment();
		StringTokenizer tokenizer = new StringTokenizer(comment, "\n");
		while (tokenizer.hasMoreTokens()) {
			buf.append("# " + tokenizer.nextToken()).append(NEW_LINE);
		}
		buf.append(NEW_LINE);

		// CPU type
		if (cpuType == GeneralInfo.CPU_PE) {
			buf.append("C33_PE").append(NEW_LINE);
		} else if (cpuType == GeneralInfo.CPU_ADV) {
			buf.append("C33_ADVANCED").append(NEW_LINE);
		}
		buf.append(NEW_LINE);

		List areaList = parSetting.getAreaList();
		for (int i = 0; i < areaList.size(); i++) {
			ParameterArea area = (ParameterArea) areaList.get(i);

			// area type
			buf.append(area.getAreaTypeString()).append(SPACE).append(SPACE)
					.append(TAB);
			// start and end address
			buf.append(UIUtil.getAddressString(area.getStartAddress())).append(SPACE);
			buf.append(UIUtil.getAddressString(area.getEndAddress())).append(SPACE);
			if (!area.isStackArea()) {
				// wait cycles
				buf.append(area.getWaitCycleString(area.getWaitCycleRead()));
				buf.append(area.getWaitCycleString(area.getWaitCycleWrite()));

				// access type
				buf.append(area.getAccessTypeChar());
				buf.append(SPACE);

				// use big endian
				if (area.isUseBigEndian()) {
					buf.append("B").append("  ");
				} else {
					buf.append("   ");
				}
			} else {
				buf.append("       ");
			}
			buf.append(TAB);

			// area comment
			tokenizer = new StringTokenizer(area.getComment(), "\n\r\f");
			buf.append("# ");
			while (tokenizer.hasMoreTokens()) {
				buf.append(tokenizer.nextToken());
				buf.append(" ");
			}
			buf.append(NEW_LINE);
		}

		return new String(buf);
	}

}
