/*
 * Created on 2007/04/13
 * 
 * This class should be common to GNU33 and GNU17.
 */
package jp.co.epson.ide.eclipse.gnu33.others;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;

/**
 * Job for building a project
 * This class originally came from jdt's CoreUtility.BuildJob 
 * 
 * This class 
 * @author D.Fujimoto
 * 
 */
public class CleanJob extends Job {

	/**
	 * the target project 
	 */
	private final IProject fProject;

	/**
	 * @param name	name of the job
	 * @param project	project to build
	 */
	public CleanJob(IProject project) {
		super("Clean Job");
		fProject= project;
		
		setRule(ResourcesPlugin.getWorkspace().getRuleFactory().buildRule());
		setUser(true);
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IStatus run(IProgressMonitor monitor) {
		synchronized (getClass()) {
			if (monitor.isCanceled()) {
				return Status.CANCEL_STATUS;
			}
		}

		try {
			if (fProject != null) {
				monitor.beginTask("clean " + fProject.getName(), 2);

				// build the project
				fProject.build(IncrementalProjectBuilder.CLEAN_BUILD, new SubProgressMonitor(monitor,1));
			}
		} catch (CoreException e) {
			return e.getStatus();
		} catch (OperationCanceledException e) {
			return Status.CANCEL_STATUS;
		} finally {
			monitor.done();
		}
		return Status.OK_STATUS;
	}
	
	public boolean belongsTo(Object family) {
		return ResourcesPlugin.FAMILY_MANUAL_BUILD == family;
	}
}
