/*
 * Created on 2004/08/09
 * Modified on 2005/11/10 by D.Fujimoto added startsWithDecimalNumber().
 * Modified on 2006/03/17 by D.Fujimoto added checkVecAddrBits().
 * Modified on 2006/03/28 by D.Fujimoto added isPathString().
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.others;

import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.events.KeyEvent;

/**
 * Class containing utility static methods for UI handling.
 * @author D.Fujimoto
 *
 */
public class UIUtil {

	static final int MAX_ADDR_CHARS = 8;

	/**
	 * Enables input for digits, A-F, a-f, control codes.
	 * Add this method to a Text widgets KeyListener#keyPressed() 
	 * @param e - KeyEvent
	 */
	public static void enableHexInput(KeyEvent e) {
		char c = e.character;
		if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x46) || (0x61 <= c && c <= 0x66) || 
			Character.isISOControl(e.character)) {
			e.doit = true;
		} else {
			e.doit = false;
		}
	}
	
	/**
	 * Checks if argument String consists of digits, A-F, a-f.
	 * @param str 	String to check  
	 * @return 	true if argument is a valid hex String, otherwise false
	 */
	public static boolean isHexString(String str) {
		if (str.length() == 0) {
			return false;
		}
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x46) || (0x61 <= c && c <= 0x66)) {
				continue;
			}
			return false;
			
		}
		return true;
	}
	
	/**
	 * Enables input for digits, alphabet and _.
	 * Add this method to a Text widget's KeyListener#keyPressed() 
	 * @param e - KeyEvent
	 */
	public static void enableAlphNumInput(KeyEvent e) {
		char c = e.character;
		if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) || 
			c == 0x5F || 
			Character.isISOControl(c)) {
			e.doit = true;	  
		} else {
			e.doit = false;
		}
	}

	/**
	 * Checks if argument String consists of digits, alphabet and _.
	 * @param str 	String to check 
	 * @return		true if argument is valid, otherwise false.
	 */
	public static boolean isAlphNumString(String str) {
		if (str.length() == 0) {
			return false;
		}
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) ||
					c == 0x5F) {
				continue;
			}
			return false;
			
		} 
		return true;
	}
	
	/**
	 * Enables input for digits, alphabet, _ / : and \.
	 * Others are not allowed since they might conflict with
	 * makefile characters.
	 * @param e
	 * @since 1.3.0
	 */
	public static void enablePathInput(KeyEvent e) {
		char c = e.character;
		if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) ||
				c == 0x5F || c == 0x2F || c == 0x3A || c == 0x5C || 
				Character.isISOControl(c)) {
			e.doit = true;
		} else {
			e.doit = false;
		}
	}
	
	/**
	 * Checks if argument String consists of characters only used for paths
	 * Allowed characters are, alphanumerics, _ / : and \
	 * Others are not allowed since they might conflict with
	 * makefile characters.
	 *		
	 * @param str	String to check
	 * @return		true if argument is valid, otherwise false.
	 * @since 1.3.0
	 */
	public static boolean isPathString(String str) {
		
		if (str.length() == 0) {
			return false;
		}
		
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) ||
					c == 0x5F || c == 0x2F || c == 0x3A || c == 0x5C) {
				continue;
			}
			return false;
		}
		return true;
	}
	
	/**
	 * Enables input for digits, alphabet and "_" and ".".
	 * Add this method to a Text widget's KeyListener#keyPressed()
	 * @param e KeyEvent
	 */
	public static void enableSectionInput(KeyEvent e) {
		char c = e.character;
		if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) || 
			c == 0x5F ||
			c == 0x2E ||
			Character.isISOControl(c)) {
			e.doit = true;
		} else {
			e.doit = false;
		}
	}
	
	/**
	 * Checks if the String is qualified as an input for the 
	 * Combo's in LinkerScriptDialog.
	 * Which means, it consists of digits, alphabet and "_" and ".".
	 * If other characters are contained in the String, returns false.
	 * @return true if it is a qualified input 
	 */
	public static boolean isSectionString(String str) {
		if (str.length() == 0) {
			return false;
		}
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if ((0x30 <= c && c <= 0x39) || (0x41 <= c && c <= 0x5A) || (0x61 <= c && c <= 0x7A) ||
					c == 0x5F ||
					c == 0x2E) {
				continue;
			} 
			return false;
			
		} 
		return true;
	}
	
	
	/**
	 * Enables input for ASCII codes (0x00 - 0x7F)
	 * Add this method to a Text widget's KeyListener#keyPressed()
	 * @param e KeyEvent
	 * @since 1.4.0
	 */
	public static void enableAsciiInput(KeyEvent e) {
		char c = e.character;

		if (0x00 <= c && c <= 0x7F) {
			e.doit = true;
		} else {
			e.doit = false;
		}
		
	}
	
	/**
	 * Checks if the String consists only of ASCII chars 
	 * @param str
	 * @return	whether str consists only of ASCII chars
	 */
	public static boolean isAscii(String str) {
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if (0x00 <= c && c <= 0x7F) {
				continue;
			}
			return false;
		}
		return true;
	}
	
	/**
	 * Checks if the String consists only of "." and "_".
	 * Section Names like these are not valid.
	 * @param str
	 * @return true when this String consists only of "." and "_"
	 */ 
	public static boolean isPeriodAndUnderbar(String str) {
		if (str.length() == 0) {
			return false;
		}
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if (c == 0x5F || c == 0x2E) {
				continue;
			}
			return false;
			
		}
		return true;
		
	}
	
	/**
	 * Checks if the String starts with a decimal number(0-9).
	 * If the string is empty, returns false.
	 * @param str
	 * @return true when this String starts with a decimal number.
	 * @since 1.2.0
	 */
	public static boolean startsWithDecimalNum(String str) {
		if (str.length() == 0) {
			return false;
		}
		char c = str.charAt(0);
		if (c >= 0x30 && c <= 0x39) {
			return true;
		}
		return false;
		
	}
	

	/**
	 * Returns a String converted to a cygwin format path.
	 * @param winPath the windows path String
	 * @return the converted cygwin format path
	 */
	public static String parseCygPath(String winPath) {
		String cygPath = ResourceProvider.getResourceString("cygwin.pathPrefix"); //$NON-NLS-1$
		
		// remove apostrophe
		winPath = removeDoubleQuotes(winPath);
		
		// convert \ to /
		IPath path = new Path(winPath);
		path = new Path(path.toString());
 
		String device = path.getDevice();
		if (device != null) {
			// conver device(c:) to drive(c)
			String[] segments = path.toString().split(device);
			cygPath += device.replaceFirst(":", "") + segments[1];	//$NON-NLS-1$ //$NON-NLS-2$
		} else {
			cygPath = path.toString();
		}
		
		return cygPath;
	
	}
	
	/**
	 * Removes the double quotations if the String is quoted.
	 * @param path
	 * @return the path with double quotations removed
	 */
	public static String removeDoubleQuotes(String path) {
		if (path.startsWith("\"") && path.endsWith("\"")) {  //$NON-NLS-1$ //$NON-NLS-2$
			path = path.substring(1, path.length() - 1);
		}
		return path;
	}
	
	/**
	 * 
	 * @param address address to be converted to hex String 
	 * @return the hex representation of the given address
	 */
	public static String getAddressString(long address) {
		String retStr = Long.toHexString(address).toUpperCase();
		while (retStr.length() < MAX_ADDR_CHARS) {
			retStr = "0" + retStr;	 //$NON-NLS-1$
		}
		return retStr;
	}

	
	/**
	 * Checks the bit masks of the vector address and return an error message.
	 * @param cpuType the current CPU type
	 * @param lVecAddr the vector address
	 * @return an error message or null if none
	 * @since 1.3.0
	 * @deprecated use checkVecAddrBits(IAddrBitChecker checker, long lVecAddr)
	 */
	public static String checkVecAddrBits(int cpuType, long lVecAddr) {
		String errorMessage = null;

		// check vector address bit masks
		if (cpuType == GeneralInfo.CPU_STD) {
			if ((lVecAddr & 0x7FF) != 0 || (lVecAddr & 0xF0000000L) != 0) {
				errorMessage = ResourceProvider.getResourceString("msg.vec.addr.bit.std");;
			}
		} else {
			if ((lVecAddr & 0x3FF) != 0) {
				errorMessage = ResourceProvider.getResourceString("msg.vec.addr.bit");
			}
		}

		return errorMessage;
	}
	
	
	/**
	 * Checks the bit masks of the vector address and return an error message.
	 * @param checker the checker object that implemnts {@link IAddrBitChecker}
	 * @param lVecAddr the vector address
	 * @return an error message or null if none
	 * @since 1.4.0
	 */
	public static String checkVecAddrBits(IAddrBitChecker checker, long lVecAddr) {
		checker.check(lVecAddr);
		return checker.getMessage();
	}
	
	/**
	 * Returns the number of chars that are allowed for address strings
	 * @return char counts
	 */
	public static int getAddressMaxChars() {
		return MAX_ADDR_CHARS;
	}
	
}
