/*
 * Created on  2005/07/21
 * Modified on 2006/01/25 by D.Fujimoto changed interval.build value to 2000.
 * Modified on 2006/02/16 by D.Fujimoto added kanji filter preference.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.pref;


import java.util.Locale;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * This class sets the defaults to the Preference Store. 
 * 
 * These settings can be overridden in the plugin_customization.ini file by specifying
 * PLUGIN_ID/KEY=VALUE.
 * 
 * @author D.Fujimoto
 *
 */
public class Gnu33PreferenceInitializer extends AbstractPreferenceInitializer {

	/* ( Javadoc)
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 * 
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = Gnu33CorePlugin.getDefault().getPreferenceStore();
		
		// The path to gnu33 tools dir from eclipse.exe
		store.setDefault("gnu33tools.relativePath", "..");
		
		// auto update flag for Sections  
		store.setDefault("sections.autoupdate", true);
		
		// wait interval to prevent clock skew in make.exe
		store.setDefault("interval.build", 2000);

		// default address in the wizard		
		store.setDefault("wizard.vecAddr", 0L);
		// allocate object to vector section(wizard)
		store.setDefault("allocVecSection", true);
		store.setDefault("vectorFileName", "vector");
		
		// compiler kanji filter switch(when nl is ja)
		String nl = Platform.getNL();
		store.setDefault("ccompiler.knjfilt.value", nl.startsWith(Locale.JAPANESE.getLanguage()));
		
		// enable 2pass making
		store.setDefault("perform2pass", true);

		// enable simIO lib linking
		store.setDefault("addSimIOLib", true);

		// __END_section label outside section 
		store.setDefault("writeEndOfSectionLabelOutside", true);

		// default goal
		store.setDefault("defaultGoal", BuildSetting.GOAL_ELF);
		
		// set cpu specific settings
		CpuSettingsManager.initializeStoreWithDefault(store);
		
	}

}
