/*
 * Created on 2004/09/29
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0) for Eclipse3.2
 *
 */
package jp.co.epson.ide.eclipse.gnu33.ui;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;

import org.eclipse.cdt.ui.CUIPlugin;
import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * The PerspectiveFactory for a gnu33 specific perspective. 
 * @author D.Fujimoto
 *
 */
public class Gnu33PerspectiveFactory implements IPerspectiveFactory {
	
	private static final String ID_CONSOLE_VIEW = "org.eclipse.ui.console.ConsoleView"; //$NON-NLS-1$
	private static final String ID_MAKE_TARGET_VIEW = "org.eclipse.cdt.make.ui.views.MakeView"; //$NON-NLS-1$
	
	/**
	 * Creates this perspective's layout.
	 * Similar as the c perspective.
	 * @see org.eclipse.ui.IPerspectiveFactory#createInitialLayout(org.eclipse.ui.IPageLayout)
	 */
	public void createInitialLayout(IPageLayout layout) {
		String editorArea = layout.getEditorArea();
		
		IFolderLayout topLeft = layout.createFolder("topLeft", IPageLayout.LEFT, 0.25f, editorArea); //$NON-NLS-1$
		topLeft.addView(CUIPlugin.CVIEW_ID);
		topLeft.addView(IPageLayout.ID_RES_NAV);
		
		IFolderLayout bottom = layout.createFolder("bottom", IPageLayout.BOTTOM, 0.75f, editorArea); //$NON-NLS-1$
		bottom.addView(ID_CONSOLE_VIEW);
		bottom.addView(IPageLayout.ID_PROBLEM_VIEW);
		bottom.addView(IPageLayout.ID_PROP_SHEET);
		bottom.addPlaceholder(IPageLayout.ID_BOOKMARKS);
		bottom.addPlaceholder(IPageLayout.ID_TASK_LIST);
		
		
		IFolderLayout topRight = layout.createFolder("topRight", IPageLayout.RIGHT, 0.75f, editorArea); //$NON-NLS-1$
		topRight.addView(IPageLayout.ID_OUTLINE);
		topRight.addView(ID_MAKE_TARGET_VIEW);
		
		layout.addActionSet(CUIPlugin.ID_CELEMENT_CREATION_ACTION_SET);
		
		
		// Show view actions
		layout.addShowViewShortcut(IPageLayout.ID_OUTLINE);			// Outline
		layout.addShowViewShortcut(IPageLayout.ID_PROBLEM_VIEW);	// Problems
		layout.addShowViewShortcut(CUIPlugin.CVIEW_ID);				// C/C++ Projects
		layout.addShowViewShortcut(IPageLayout.ID_RES_NAV);			// Navigator
		layout.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);		// Properties
		layout.addShowViewShortcut(ID_CONSOLE_VIEW);				// Console
		layout.addShowViewShortcut(IPageLayout.ID_BOOKMARKS);		// Bookmarks
		layout.addShowViewShortcut(IPageLayout.ID_TASK_LIST);		// Tasks
//		layout.addShowViewShortcut(SearchUI.SEARCH_RESULT_VIEW_ID);	// Search
		layout.addShowViewShortcut(ID_MAKE_TARGET_VIEW);			// Make Targets
		
		// Show In actions
		layout.addShowInPart(CUIPlugin.CVIEW_ID);
		layout.addShowInPart(IPageLayout.ID_RES_NAV);
		
		// New actions
		layout.addNewWizardShortcut(Gnu33CorePlugin.PROJECT_WIZARD_ID);
		layout.addNewWizardShortcut("org.eclipse.cdt.ui.wizards.NewSourceFileCreationWizard");
		layout.addNewWizardShortcut("org.eclipse.cdt.ui.wizards.NewHeaderFileCreationWizard");
		layout.addNewWizardShortcut("org.eclipse.cdt.ui.wizards.NewSourceFolderCreationWizard");
		layout.addNewWizardShortcut("org.eclipse.cdt.ui.wizards.NewFileCreationWizard");
		
		
		// Open Perspective actions
		layout.addPerspectiveShortcut(Gnu33CorePlugin.PERSPECTIVE_ID);

		
	}
	
}
