/*
 * Created on 2007/03/26
 * 
 * This class must be common to GNU33 and GNU17.
 */
package jp.co.epson.ide.eclipse.gnu33.ui;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.progress.WorkbenchJob;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * This class is a Job that if the given file is opened in an editor,
 * it will refresh(revert) the file in that editor.
 * 
 * This is designed for forcibly reopening the files(.mak/.lds etc) generated by
 * the FilesBuilder.
 * 
 * This job must be called in the UI thread as the builder may invoke this.
 * 
 * @author D.Fujimoto
 * @since 1.4.0
 * 
 */
public class RefreshFileInEditorJob extends WorkbenchJob {

	public static final String JOB_NAME = "reopen_file_in_editor";
	
	/*
	 * The file that should be opened in the editor 
	 */
	private IFile targetFile;
	
	/**
	 *
	 */
	public RefreshFileInEditorJob(IFile targetFile) {
		super(JOB_NAME);
		this.targetFile = targetFile;
		
		setSystem(true);
	}

	/**
	 * Reverts the open file to its saved state.
	 */
	public IStatus runInUIThread(IProgressMonitor monitor) {

		boolean open = false;
		
		IEditorInput input = new FileEditorInput(targetFile);
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			IWorkbenchPage[] pages = windows[i].getPages();
			for (int j = 0; j < pages.length; j++) {
				IEditorPart editor = pages[j].findEditor(input);
				if (editor != null) {
					// this file is open in an editor
					open = true;

					if (editor instanceof ITextEditor) {
						ITextEditor textEditor = (ITextEditor) editor;

//						// close and open
//						ed.close(false);
//						IDE.openEditor(pages[j], input, editor.getEditorSite().getId(), true);

						// try to revert to the saved file
						textEditor.doRevertToSaved();

						// debug
//						System.out.println("refreshed " + targetFile.getName());
						
					}
				}
			}
		}
		
		return Status.OK_STATUS;
	}

	
	public boolean shouldRun() {
		if (targetFile == null || !targetFile.exists()) {
			return false;
		}
		return super.shouldRun();
	}
	
}
