/*
 * Created on 2004/08/18
 * Modified on 2005/08/10 by D.Fujimoto modified createFieldEditors() not to display () for
 * 											options that do not specify a command.
 * Modified on 2006/11/27 by D.Fujimoto modified createFieldEditors() not to display optinos
 * 											that are invisible.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.build;

import java.util.Iterator;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.build.IBuildCategory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;

import org.eclipse.cdt.managedbuilder.ui.properties.BuildOptionComboFieldEditor;
import org.eclipse.cdt.managedbuilder.ui.properties.MultiLineTextFieldEditor;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Text;

/**
 * A FieldEditorPage for each build option's category.
 * The page will be shown when a category is selected in the TreeViewer.
 * @author D.Fujimoto
 *
 */
public class BuildSettingsOptionPage extends BuildSettingsInputPage {

	/**
	 * The category of this page. 
	 */
	private IBuildCategory category;
	
	/**
	 * Constructor.
	 */
	public BuildSettingsOptionPage(IBuildCategory category, int cpuType) {
		super(cpuType);
		this.category = category;
	}


	/**
	 * Create the field editors for this page.
	 */
	protected void createFieldEditors() {
		// call parent implementation to obtain the PreferenceStore
		super.createFieldEditors();

		// iterate over options
		List options = category.getOptions();
		for (Iterator iter = options.iterator(); iter.hasNext();) {
			IBuildOption option = (IBuildOption) iter.next();
			String optString = new String();
			
			// don't show invisible options
			if (option.isInvisible()) {
				continue;
			}
			
			if (option.getCommand() != null && option.getCommand().length() != 0) {
				optString = " (" + option.getCommand() + ")";
			}
			
			// Note:Each FieldEditor's first argument must be identical
			switch (option.getType()) {
				case IBuildOption.TYPE_BOOLEAN :
					BooleanFieldEditor booleanField = 
						new BooleanFieldEditor(option.getId(), 
								option.getLabel() + optString, getFieldEditorParent());
					addField(booleanField);
					fieldList.add(booleanField);
					break;
				case IBuildOption.TYPE_STRING :
					StringFieldEditor strField = 
						new StringFieldEditor(option.getId(), 
								option.getLabel(), getFieldEditorParent());
					addField(strField);
					fieldList.add(strField);
					break;
				case IBuildOption.TYPE_ENUM :
					BuildOptionComboFieldEditor comboField = 
						new BuildOptionComboFieldEditor(option.getId(), 
								option.getLabel(), option.getEnumOptions(), "",
								getFieldEditorParent());
					addField(comboField);
					fieldList.add(comboField);
					break;
				case IBuildOption.TYPE_LIST :
					FileListControlFieldEditor2 listField = 
						new FileListControlFieldEditor2(option.getId(), 
								option.getLabel() + optString, getFieldEditorParent(), 
								option.getBrowseType());
					addField(listField);
					fieldList.add(listField);
					break;
					
				case IBuildOption.TYPE_STRING_MULTI :
					MultiLineTextFieldEditor multiStringField = new MultiLineTextFieldEditor(option.getId(), 
							option.getLabel(), getFieldEditorParent());
					// supress cr/lf input
					Text text = multiStringField.getTextControl(getFieldEditorParent());
					text.addKeyListener(new KeyAdapter() {
						public void keyPressed(KeyEvent e) {
							char c = e.character;
							if (c == '\n' || c == '\r' || c== '\f') {
								e.doit = false;
							}
						}
					});
					addField(multiStringField);
					fieldList.add(multiStringField);
					break;
				default :
					break;

			}
		}
		
	}
	
	
	/**
	 * Gather the current values from the PreferenceStore and 
	 * write it in the BuildSetting.
	 */
	public boolean performOk() {
		// write values to the PreferenceStore
		boolean ok = super.performOk();
		
		List options = category.getOptions();
		for (Iterator iter = options.iterator(); iter.hasNext();) {
			IBuildOption option = (IBuildOption) iter.next();
			String id = option.getId();
			switch (option.getType()) {
				case IBuildOption.TYPE_BOOLEAN :
					boolean boolValue = getPreferenceStore().getBoolean(id);
					option.setBooleanValue(boolValue);
					break;
				case IBuildOption.TYPE_STRING :
				case IBuildOption.TYPE_STRING_MULTI :
					String stringValue = getPreferenceStore().getString(id);
					option.setStringValue(stringValue);
					break;
				case IBuildOption.TYPE_ENUM :
					int enumValue = 0;
					String[] enums = option.getEnumOptions();
					String selectedEnum = getPreferenceStore().getString(id);
					for (int i = 0; i < enums.length; i++) {
						if (selectedEnum.equals(enums[i])) {
							enumValue = i;
							break;
						}
					}
					option.setEnumValue(enumValue);
					break;
				case IBuildOption.TYPE_LIST :
					String[] listValue = null;
					String stringList = getPreferenceStore().getString(id);
					if (stringList == null || stringList.equals("")) {
						listValue = new String[0];
					} else {
						listValue = stringList.split(IBuildOption.LIST_DELIMITER);
					}
					option.setListValue(listValue);
					break;
			}
		}
		
		return ok;
	}
}
