/*
 * Created on 2006/05/24 (refactored from GDBLaunchPropertyPage)
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/07/19 by D.Fujimoto icd6 supported in various methods 
 * 											especially makeDebuggerTypeCombo() + populateDebuggerType() etc.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.debugger;

import jp.co.epson.ide.eclipse.gnu33.debug.AddrBitChecker;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog for creating the commands from a template.
 * This dialog contains the conventional UIs.
 * The code contained herein are separeted from GDBLaunchPropertyPage. 
 * 
 * @author D.Fujimoto
 * @since 1.3.0
 */
public class CommandTemplateDialog extends TitleAreaDialog {

	public static final String DLG_TEMPLATE_TITLE = DebuggerSettingDialogResources.getString("gdbPage.tempDlg.title");
	public static final String DLG_TEMPLATE_DESC = DebuggerSettingDialogResources.getString("gdbPage.tempDlg.description");
	public static final String LABEL_OK_BTN = DebuggerSettingDialogResources.getString("gdbPage.tempDlg.ok.label");	//$NON-NLS-1$
	
	public static final String LABEL_DEBUGGER = DebuggerSettingDialogResources.getString("gdbPage.debugger.label");	 //$NON-NLS-1$
	public static final String LABEL_VECTOR_ADDR = DebuggerSettingDialogResources.getString("gdbPage.vecAddr.label");	 //$NON-NLS-1$
	public static final String LABEL_PORTS = DebuggerSettingDialogResources.getString("gdbPage.ports.label");	 //$NON-NLS-1$
	public static final String LABEL_CONNECT_PORT = DebuggerSettingDialogResources.getString("gdbPage.connectPorts.label");	 //$NON-NLS-1$
	public static final String LABEL_PORT_WAITS = DebuggerSettingDialogResources.getString("gdbPage.portWaits.label");	 //$NON-NLS-1$
	public static final String LABEL_BAUD_RATES = DebuggerSettingDialogResources.getString("gdbPage.baudRates.label");	 //$NON-NLS-1$
	public static final String LABEL_PARALLEL_PORTS = DebuggerSettingDialogResources.getString("gdbPage.parallelPorts.label");	 //$NON-NLS-1$
	public static final String LABEL_DEBUG_RAM_ADDR = DebuggerSettingDialogResources.getString("gdbPage.debugRamAddr.label");	 

	public static final String ERR_DLG_TITLE_BAUD_MON = DebuggerSettingDialogResources.getString("err.dlg.title.baud.mon");	 //$NON-NLS-1$
	public static final String ERR_DLG_MSG_BAUD_MON = DebuggerSettingDialogResources.getString("err.dlg.msg.baud.mon");	 //$NON-NLS-1$
	public static final String MSG_VEC_ADDR_BLANK = DebuggerSettingDialogResources.getString("msg.vec.addr.blank");	 //$NON-NLS-1$
	public static final String MSG_RAM_ADDR_BLANK = DebuggerSettingDialogResources.getString("msg.ram.addr.blank");	 //$NON-NLS-1$
	public static final String MSG_NOT_HEX = DebuggerSettingDialogResources.getString("msg.not.hex");	 //$NON-NLS-1$

	public static final String LABEL_CONFIRM_TITLE = DebuggerSettingDialogResources.getString("gdbPage.tempDlg.confirm.title");	//$NON-NLS-1$
	public static final String APPLY_CONFIRM = DebuggerSettingDialogResources.getString("gdbPage.templateApply.confirm");	//$NON-NLS-1$

	// the current CPU type
	private int cpuType;

	// the current debugger setting
	private DebuggerSetting setting;

	// the controls for CommandTemplateDialog
	private Combo comboDebuggerType;
	private Text textVecAddr;
	private Combo comboConnectPort;
	private Combo comboPortWait;
	private Combo comboBaudRate;
	private Combo comboParallel;
	private Text textDebugRAMAddr;

	/**
	 * Constructor.
	 * Codes copied from GDBLaunchPropertyPage.getProjectSettings().
	 * 
	 * @param parentShell	shell for this dialog
	 * @param	cpuType	current CPU type	
	 * @param	setting debugger setting for the UI
	 */
	protected CommandTemplateDialog(Shell parentShell, int cpuType,
			DebuggerSetting setting) {
		super(parentShell);

		this.cpuType = cpuType;
		this.setting = setting;
	}
	
	/**
	 * Access to the setting which results after
	 * OK button gets pressed.
	 */
	public DebuggerSetting getSetting() {
		return setting;
	}

	/**
	 * Sets the title
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(DLG_TEMPLATE_TITLE);
	}

	/**
	 * Controls created here have been moved from GDBLaunchPropertyPage.createContents().
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {

		Composite composite = new Composite(parent, SWT.NONE);

		GridData gridData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_CENTER
				| GridData.VERTICAL_ALIGN_BEGINNING);
		gridData.widthHint = 407;
		composite.setLayoutData(gridData);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		composite.setLayout(gridLayout);

		// debugger type
		Label labelDebuggerType = new Label(composite, SWT.NONE);
		labelDebuggerType.setLayoutData(new GridData());
		labelDebuggerType.setText(LABEL_DEBUGGER);
		comboDebuggerType = new Combo(composite, SWT.READ_ONLY);
		GridData gridDebugType = new GridData();
		gridDebugType.widthHint = 76;
		gridDebugType.horizontalSpan = 3;
		comboDebuggerType.setLayoutData(gridDebugType);
		comboDebuggerType.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDebuggerTypeSelection();
			}
		});

		// vector address
		Label labelVecAddr = new Label(composite, SWT.NONE);
		labelVecAddr.setText(LABEL_VECTOR_ADDR);
		GridData gdLabelVec = new GridData();
		gdLabelVec.horizontalSpan = 2;
		labelVecAddr.setLayoutData(gdLabelVec);

		textVecAddr = new Text(composite, SWT.BORDER);
		textVecAddr.setTextLimit(UIUtil.getAddressMaxChars());
		GridData gdTextVec = new GridData();
		gdTextVec.widthHint = 89;
		gdTextVec.horizontalSpan = 2;
		textVecAddr.setLayoutData(gdTextVec);
		textVecAddr.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableHexInput(e);
			}
		});
		textVecAddr.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});


		Group groupPort = new Group(composite, SWT.NONE);
		groupPort.setText(LABEL_PORTS);
		GridLayout layoutGroup = new GridLayout();
		layoutGroup.numColumns = 4;
		groupPort.setLayout(layoutGroup);
		GridData gridGroup = new GridData();
		gridGroup.horizontalSpan = 4;
		gridGroup.grabExcessHorizontalSpace = true;
		gridGroup.horizontalAlignment = GridData.FILL;
		groupPort.setLayoutData(gridGroup);

		// connect port
		Label labelConnectPort = new Label(groupPort, SWT.NONE);
		labelConnectPort.setText(LABEL_CONNECT_PORT);
		comboConnectPort = new Combo(groupPort, SWT.READ_ONLY);
		GridData gridConnectPort = new GridData();
		gridConnectPort.widthHint = 65;
		comboConnectPort.setLayoutData(gridConnectPort);

		// port wait
		Label labelPortWait = new Label(groupPort, SWT.NONE);
		labelPortWait.setText(LABEL_PORT_WAITS);
		comboPortWait = new Combo(groupPort, SWT.READ_ONLY);
		GridData gridPortWait = new GridData();
		gridPortWait.widthHint = 70;
		comboPortWait.setLayoutData(gridPortWait);

		Label labelDummy1 = new Label(groupPort, SWT.NONE);
		GridData gridDummy1 = new GridData();
		gridDummy1.horizontalSpan = 2;
		labelDummy1.setLayoutData(gridDummy1);

		// baud rate
		Label labelBaudRate = new Label(groupPort, SWT.NONE);
		labelBaudRate.setText(LABEL_BAUD_RATES);
		comboBaudRate = new Combo(groupPort, SWT.READ_ONLY);
		GridData gridBaudRate = new GridData();
		gridBaudRate.widthHint = 70;
		comboBaudRate.setLayoutData(gridBaudRate);
		comboBaudRate.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBaudRateSelection();
			}
		});

		Label labelDummy2 = new Label(groupPort, SWT.NONE);
		GridData gridDummy2 = new GridData();
		gridDummy2.horizontalSpan = 2;
		labelDummy2.setLayoutData(gridDummy2);

		// parallel port
		Label labelParallel = new Label(groupPort, SWT.NONE);
		labelParallel.setText(LABEL_PARALLEL_PORTS);
		comboParallel = new Combo(groupPort, SWT.READ_ONLY);
		GridData gridParallel = new GridData();
		gridParallel.widthHint = 70;
		comboParallel.setLayoutData(gridParallel);

		// debug RAM address
		Label labelRAMAddr = new Label(groupPort, SWT.NONE);
		GridData gridRAMAddr1 = new GridData();
		gridRAMAddr1.horizontalSpan = 2;
		labelRAMAddr.setLayoutData(gridRAMAddr1);
		labelRAMAddr.setText(LABEL_DEBUG_RAM_ADDR);
		textDebugRAMAddr = new Text(groupPort, SWT.BORDER);
		textDebugRAMAddr.setTextLimit(UIUtil.getAddressMaxChars());
		GridData gridRAMAddr2 = new GridData();
		gridRAMAddr2.horizontalSpan = 2;
		gridRAMAddr2.widthHint = 89;
		textDebugRAMAddr.setLayoutData(gridRAMAddr2);
		textDebugRAMAddr.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableHexInput(e);
			}
		});
		textDebugRAMAddr.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});


		// set the title area
		setTitle(DLG_TEMPLATE_TITLE);
		setMessage(DLG_TEMPLATE_DESC);

		// initialize the control values
		initialize();

		applyDialogFont(composite);
		return composite;
	}


	/* ( Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {

		// create Overwrite and Close buttons
		// Overwrite is considered to be an OK button(see okPressed())
		// Close is considered to be a Cancel button
		createButton(parent, IDialogConstants.OK_ID, LABEL_OK_BTN, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL, false);

	}
	
	/**
	 * Initializes the values for controls.
	 * This facade is moved from GDBLaunchPropertyPage.refreshPage()
	 */
	private void initialize() {

		// set defaults for the template dialog
		makeDebuggerTypeCombo();
		populateDebuggerType(setting.getDebuggerType());
		makeConnectPortCombo();
		populateConnectPortCombo(setting.getConnectPort(), setting.getComPortNo());
		makeOtherCombos();
		populateOtherCombos();

		textVecAddr.setText(UIUtil.getAddressString(setting.getVectorAddress()));
		textDebugRAMAddr.setText(UIUtil.getAddressString(setting.getDebugRamAddr()));
	}


	/**
	 * Shows a confirm dialog.
	 * If confirmed, gathers all input values into a DebuggerSetting object, 
	 * which this dialog's parent will use later. 
	 * 
	 * The codes for value gathering have been moved from GDBLaunchPropertyPage.
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {

		// open a dialog to overwriting commands
		if (!MessageDialog.openConfirm(getShell(), LABEL_CONFIRM_TITLE, APPLY_CONFIRM)) {
			// if aborted, do nothing
			return;  
		}
		
		
		// debugger type
		int debuggerType = 0;
		String selectedType = comboDebuggerType.getItem(comboDebuggerType
				.getSelectionIndex());

		if (selectedType.equals(DebuggerSetting.TYPE_ICD2_STR)) {
			debuggerType = DebuggerSetting.TYPE_ICD2;
		} else if (selectedType.equals(DebuggerSetting.TYPE_ICD3_STR)) {
			debuggerType = DebuggerSetting.TYPE_ICD3;
/* >>>>> ADDED by D.Fujimoto 2007/07/19 icd 6 */
		} else if (selectedType.equals(DebuggerSetting.TYPE_ICD6_STR)) {
			debuggerType = DebuggerSetting.TYPE_ICD6;
/* <<<<< ADDED by D.Fujimoto 2007/07/19 icd 6 */
		} else if (selectedType.equals(DebuggerSetting.TYPE_MON_STR)) {
			debuggerType = DebuggerSetting.TYPE_MON;
		} else if (selectedType.equals(DebuggerSetting.TYPE_SIM_STR)) {
			debuggerType = DebuggerSetting.TYPE_SIM;
		}

		DebuggerSetting newSetting = new DebuggerSetting(debuggerType);
		newSetting.setName(setting.getName());

		// boot vector 
		newSetting.setVectorAddress(Long.parseLong(textVecAddr.getText().trim(), 16));

		// connect port
		String selectedPort = comboConnectPort.getItem(comboConnectPort
				.getSelectionIndex());
		if (selectedPort.equals(DebuggerSetting.PORT_NONE_STR)) {
			newSetting.setConnectPort(DebuggerSetting.PORT_NONE);
		} else if (selectedPort.equals(DebuggerSetting.PORT_USB_STR)) {
			newSetting.setConnectPort(DebuggerSetting.PORT_USB);
		} else if (selectedPort.startsWith(DebuggerSetting.PORT_COM_STR)) {
			newSetting.setConnectPort(DebuggerSetting.PORT_COM);
			String portNoStr = selectedPort
					.substring(DebuggerSetting.PORT_COM_STR.length());
			newSetting.setComPortNo(Integer.parseInt(portNoStr));

			// port wait
			newSetting.setPortWait(comboPortWait.getSelectionIndex());
			// baud rate
			newSetting.setBaudRate(DebuggerSetting.BAUD_RATES[comboBaudRate.getSelectionIndex()]);
			// parallel port
			newSetting.setParallelPort(comboParallel.getSelectionIndex());
		}

		// debug ram address
		if (textDebugRAMAddr.isEnabled()) {
			newSetting.setDebugRamAddr(Long.parseLong(textDebugRAMAddr.getText().trim(), 16));
		}

		// replace the setting
		this.setting = newSetting;

		super.okPressed();
	}


	/**
	 * Makes the debugger type combo's items.
	 * The items vary opon the CPU type.
	 * The items' order is related to that in performOk().
	 */
	private void makeDebuggerTypeCombo() {
		comboDebuggerType.removeAll();

		switch (cpuType) {
			case GeneralInfo.CPU_STD:
				comboDebuggerType.setItems(new String[] {
						DebuggerSetting.TYPE_ICD2_STR,
						DebuggerSetting.TYPE_ICD3_STR,
						DebuggerSetting.TYPE_ICD6_STR,
						DebuggerSetting.TYPE_MON_STR,
						DebuggerSetting.TYPE_SIM_STR
				});
				break;
			case GeneralInfo.CPU_PE:
				comboDebuggerType.setItems(new String[] {
						DebuggerSetting.TYPE_ICD3_STR,
						DebuggerSetting.TYPE_ICD6_STR,
						DebuggerSetting.TYPE_SIM_STR
				});
				break;
			case GeneralInfo.CPU_ADV:
				comboDebuggerType.setItems(new String[] {
						DebuggerSetting.TYPE_ICD3_STR,
						DebuggerSetting.TYPE_ICD6_STR,
						DebuggerSetting.TYPE_SIM_STR
				});
				break;
			default:
				// error
				break;

		}
	}

	/**
	 * Makes the connect port combo's items
	 * The items vary upon the selected debugger type
	 *
	 */
	private void makeConnectPortCombo() {
		comboConnectPort.removeAll();

		String selectedDebugger = comboDebuggerType.getItem(comboDebuggerType
				.getSelectionIndex());
		if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD2_STR)
				|| selectedDebugger.equals(DebuggerSetting.TYPE_MON_STR)) {
			String[] comItems = new String[DebuggerSetting.COM_MAX];
			for (int i = 0; i < comItems.length; i++) {
				comItems[i] = DebuggerSetting.PORT_COM_STR + (i + 1);
			}
			comboConnectPort.setItems(comItems);

		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD3_STR)) {
			comboConnectPort.setItems(new String[] {
				DebuggerSetting.PORT_USB_STR
			});
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD6_STR)) {
			comboConnectPort.setItems(new String[] {
				DebuggerSetting.PORT_USB_STR
			});
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_SIM_STR)) {
			comboConnectPort.setItems(new String[] {
				DebuggerSetting.PORT_NONE_STR
			});
		}

		comboConnectPort.select(0);
	}

	/**
	 * Makes the port-wait,baud-rate,parallel-port combos' items
	 * The items vary upon the selected connect port type.
	 *
	 */
	private void makeOtherCombos() {

		// Port waits
		comboPortWait.removeAll();
		String port = comboConnectPort.getItem(comboConnectPort
				.getSelectionIndex());
		if (port.startsWith(DebuggerSetting.PORT_COM_STR)) {
			String[] waits = new String[DebuggerSetting.WAIT_MAX + 1];
			for (int i = 0; i < waits.length; i++) {
				waits[i] = String.valueOf(i);
			}
			comboPortWait.setItems(waits);
		} else {
			comboPortWait.setItems(new String[] {
				DebuggerSetting.WAIT_NONE_STR
			});
		}
		comboPortWait.select(0);

		// Baud rates
		if (port.startsWith(DebuggerSetting.PORT_COM_STR)) {
			String[] baudRates = new String[DebuggerSetting.BAUD_RATES.length];
			for (int i = 0; i < baudRates.length; i++) {
				baudRates[i] = String.valueOf(DebuggerSetting.BAUD_RATES[i]);
			}
			comboBaudRate.setItems(baudRates);
		} else {
			comboBaudRate.setItems(new String[] {
				DebuggerSetting.BAUD_NONE_STR
			});
		}
		comboBaudRate.select(0);

		// Parallel ports
		comboParallel.setItems(new String[] {
				DebuggerSetting.LPT_NONE_STR, DebuggerSetting.LPT_1_STR,
				DebuggerSetting.LPT_2_STR
		});
		comboParallel.select(0);

	}

	/**
	 * Fills in the default setting for debugger type
	 * directly from the project descriptor.
	 * 
	 * These must be modified in association with makeDebuggerTypeCombo().
	 * @param debuggerType debugger type defined in DebuggerSetting
	 */
	private void populateDebuggerType(int debuggerType) {

		int selectedIndex = 0;

		switch (cpuType) {
			case GeneralInfo.CPU_STD:
				switch (debuggerType) {
					case DebuggerSetting.TYPE_ICD2:
						selectedIndex = 0;
						break;
					case DebuggerSetting.TYPE_ICD3:
						selectedIndex = 1;
						break;
					case DebuggerSetting.TYPE_ICD6:
						selectedIndex = 2;
						break;
					case DebuggerSetting.TYPE_MON:
						selectedIndex = 3;
						break;
					case DebuggerSetting.TYPE_SIM:
						selectedIndex = 4;
						break;
				}
				break;
			case GeneralInfo.CPU_PE:
				switch (debuggerType) {
/* >>>>> MODIFIED by D.Fujimoto 2005/08/19 pe +icd2 unsupported */
					case DebuggerSetting.TYPE_ICD3:
						selectedIndex = 0;
						break;
					case DebuggerSetting.TYPE_ICD6:
						selectedIndex = 1;
						break;
					case DebuggerSetting.TYPE_SIM:
						selectedIndex = 2;
						break;
				}
				break;
/* <<<<< MODIFIED by D.Fujimoto 2005/08/19 pe +icd2 unsupported */
			case GeneralInfo.CPU_ADV:
				switch (debuggerType) {
					case DebuggerSetting.TYPE_ICD3:
						selectedIndex = 0;
						break;
					case DebuggerSetting.TYPE_ICD6:
						selectedIndex = 1;
						break;
					case DebuggerSetting.TYPE_SIM:
						selectedIndex = 2;
						break;
				}
				break;
			default:
				// error
				break;
		}
		comboDebuggerType.select(selectedIndex);
		handleDebuggerTypeSelection();// needed
	}

	/**
	 * Fills in the default settings to connect port
	 * directly from the project descriptor.
	 * @param connectPort port type in DebuggerSetting
	 * @param comPortNo port number when connectPort is COM
	 */
	private void populateConnectPortCombo(int connectPort, int comPortNo) {

		switch (connectPort) {
			case DebuggerSetting.PORT_COM:
				comboConnectPort.select(comPortNo - 1);
				break;
			case DebuggerSetting.PORT_USB:
				comboConnectPort.select(0);
				break;
			case DebuggerSetting.PORT_NONE:
				comboConnectPort.select(0);
				break;
		}
	}

	/**
	 * Fills in the default settings to port-wait, baud-rate, parallel port
	 * directly from the project descriptor.
	 */
	private void populateOtherCombos() {

		comboPortWait.select(setting.getPortWait());

		comboBaudRate.select(DebuggerSetting.BAUD_RATES[1]);
		for (int i = 0; i < DebuggerSetting.BAUD_RATES.length; i++) {
			if (setting.getBaudRate() == DebuggerSetting.BAUD_RATES[i]) {
				comboBaudRate.select(i);
			}
		}

		comboParallel.select(setting.getParallelPort());
	}

	/**
	 * Event handler for changing debugger type.
	 */
	private void handleDebuggerTypeSelection() {
		int selectedIndex = comboDebuggerType.getSelectionIndex();
		String selectedDebugger = comboDebuggerType.getItem(selectedIndex);

		// remake the combos
		makeConnectPortCombo();
		makeOtherCombos();

		// controls are enabled/disabled by the selected debugger type
		if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD2_STR)) {
			comboPortWait.setEnabled(true);
			comboBaudRate.setEnabled(true);
			comboParallel.setEnabled(true);
			comboBaudRate.select(1);
			textDebugRAMAddr.setEnabled(false);
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD3_STR)) {
			comboPortWait.setEnabled(false);
			comboBaudRate.setEnabled(false);
			comboParallel.setEnabled(false);
			if (cpuType == GeneralInfo.CPU_PE) {
				textDebugRAMAddr.setEnabled(true);
			} else {
				textDebugRAMAddr.setEnabled(false);
			}
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_ICD6_STR)) {
			// same as ICD3
			comboPortWait.setEnabled(false);
			comboBaudRate.setEnabled(false);
			comboParallel.setEnabled(false);
			if (cpuType == GeneralInfo.CPU_PE) {
				textDebugRAMAddr.setEnabled(true);
			} else {
				textDebugRAMAddr.setEnabled(false);
			}
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_MON_STR)) {
			comboPortWait.setEnabled(true);
			comboBaudRate.setEnabled(true);
			comboParallel.setEnabled(false);
			comboBaudRate.select(1);
			textDebugRAMAddr.setEnabled(false);
		} else if (selectedDebugger.equals(DebuggerSetting.TYPE_SIM_STR)) {
			comboPortWait.setEnabled(false);
			comboBaudRate.setEnabled(false);
			comboParallel.setEnabled(false);
			textDebugRAMAddr.setEnabled(false);
		}

		// reset the debug RAM address if necessary
		String addr = textDebugRAMAddr.getText().trim();
		if (addr.length() == 0 || !UIUtil.isHexString(addr)) {
			textDebugRAMAddr.setText(UIUtil.getAddressString(setting.getDebugRamAddr()));
		}
	}


	/**
	 * Event handler for changing baud rate.
	 * Checks if baudrate=115200 when debugger type is MON
	 */
	private void handleBaudRateSelection() {
		int selectedTypeIndex = comboDebuggerType.getSelectionIndex();
		String selectedDebugger = comboDebuggerType.getItem(selectedTypeIndex);
		if (selectedDebugger.equals(DebuggerSetting.TYPE_MON_STR)) {
			int selectedRateIndex = comboBaudRate.getSelectionIndex();
			String selectedRate = comboBaudRate.getItem(selectedRateIndex);
			String expectedRate = Integer
					.toString(DebuggerSetting.BAUD_RATES[1]);
			if (!selectedRate.equals(expectedRate)) {
				MessageDialog.openWarning(getShell(),
						ERR_DLG_TITLE_BAUD_MON,
						ERR_DLG_MSG_BAUD_MON + DebuggerSetting.BAUD_RATES[1] + "");
				comboBaudRate.select(1);
			}
		}
	}

	/**
	 * Event handler for modifying text boxes.
	 * Checks if address strings are input correctly as a hex string.
	 * Sets an error message to the page.
	 */
	private void verifyInputs() {

		String errorMessage = null;
		String ramAddr = textDebugRAMAddr.getText();
		String vecAddr = textVecAddr.getText();
		long lRamAddr = 0;
		long lVecAddr = 0;

		try {

			// check hexadecimal
			if (ramAddr.length() != 0 && vecAddr.length() != 0) {
				lRamAddr = Long.parseLong(ramAddr, 16);
				lVecAddr = Long.parseLong(vecAddr, 16);
			}
			
			// check blank and bits
			if (vecAddr.length() == 0) {
				errorMessage = MSG_VEC_ADDR_BLANK;
			} else {
				errorMessage = UIUtil.checkVecAddrBits(new AddrBitChecker(cpuType), lVecAddr);
			}
		
			// check blank
			if (ramAddr.length() == 0) {
				errorMessage = MSG_RAM_ADDR_BLANK;
			}

		} catch (NumberFormatException e) {
			errorMessage = MSG_NOT_HEX;
		}

		if (errorMessage != null) {
			setMessage(errorMessage, IMessageProvider.ERROR);	
		} else {
			setMessage(DLG_TEMPLATE_DESC);	// restore the description
		}
		
		// update button state
		Button okButton = getButton(IDialogConstants.OK_ID);
		if (okButton != null) {
			okButton.setEnabled(errorMessage == null);
		}

	}

}
