/*
 * Created on 2004/08/30
 * Modified on 2005/08/19 by D.Fujimoto modified makeDebuggerTypeCombo() and
 * 											populateDebuggerType() deleted icd2 selection.
 * Modified on 2006/03/13 by D.Fujimoto modified createContents(), refreshPage(), performOK()
 * 											for adding user command UI.
 * Modified on 2006/03/17 by D.Fujimoto created gatherInputs(), CommandFileViewDialog.
 * Modified on 2006/03/17 by D.Fujimoto deleted checkUseVecAddr, modified refreshPage(), gatherInputs(), 
 * 											deprecated handleUseVecAddrSelection() and 
 * 											replaced vector address UI,
 * 											overrided setValid(),
 * Modified on 2006/03/17 by D.Fujimoto deprecated checkVecAddrBits().
 * Modified on 2006/05/24 by D.Fujimoto completely modified to show a editor area.
 * Modified on 2006/06/27 by D.Fujimoto modified gatherInputs() adding a comment when no commands are
 * 											specified.
 * Modified on 2006/06/29 by D.Fujimoto modified refreshPage() to show target file.
 * Modified on 2006/07/21 by.D.Fujimoto modified refreshPage() to show cmd file contents.
 * Modified on 2006/07/25 by D.Fujimoto modified performOk() and gatherInputs() for cmd file output.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 * Modified on 2007/08/03 by D.Fujimoto modified performOk() to show error when pj file is read only.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.debugger;

import java.text.MessageFormat;

import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.file.CommandFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.others.ProjectAccessor;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.BufferedRuleBasedScanner;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.FastPartitioner;
import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.PatternRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

/**
 * The Property Page for Debugger Setting in a Gnu33 project.
 * @author D.Fujimoto
 */
public class GDBLaunchPropertyPage extends AbstractGnu33PropertyPage {

	public static final String LABEL_EDIT_CMD = DebuggerSettingDialogResources.getString("gdbPage.editCmd.label");	//$NON-NLS-1$
	public static final String LABEL_DLG_BTN = DebuggerSettingDialogResources.getString("gdbPage.dlgBtn.label");	//$NON-NLS-1$
	
	public static final String DLG_CMD_CHANGED_TITLE = DebuggerSettingDialogResources.getString("gdbPage.cmdChanged.title");	 //$NON-NLS-1$
	public static final String DLG_CMD_CHANGED_MSG = DebuggerSettingDialogResources.getString("gdbPage.cmdChanged.msg");	 //$NON-NLS-1$
	

	/**
	 * DebuggerSetting object from the project descriptor.
	 */
	private DebuggerSetting setting;

	/**
	 * DebuggerSetting to pass/get to/from the template dialog
	 */
	private DebuggerSetting dlgSetting;
	
	/**
	 * The currently set CPU type. Obtained from the project descriptor.
	 */
	private int cpuType;

	/**
	 * Flag indicating whether the command (text)
	 * has changed.
	 */
	private boolean isCmdDirty;

	/**
	 * Flag for command file read state
	 */
	private boolean cmdReadFailed;
	
	/**
	 * The command file editor
	 */
	private SourceViewer textViewer;

	/**
	 * Template button
	 */
	private Button openDlgBtn;
	
	/**
	 * The template dialog instance.
	 * Every debugger setting UI have moved to this dialog. 
	 */
	private CommandTemplateDialog tempDlg;
	
	// the label control
	private Label editCmdLabel;
	private Label targetFileLabel;
	
	/**
	 * Constructor. Called by the framework.
	 */
	public GDBLaunchPropertyPage() {
		super();
	}
	
	
	/**
	 * Create the UI here.
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {

		Composite composite = new Composite(parent, SWT.NONE);

		// from here
		GridLayout gridLayout = new GridLayout();
		composite.setLayout(gridLayout);
		GridData gridData = new GridData(SWT.FILL,
				SWT.FILL, true, true);
		composite.setLayoutData(gridData);
		
		editCmdLabel = new Label(composite, SWT.WRAP);		// for big fonts
		GridData gdShowCmdLbl = new GridData();
		gdShowCmdLbl.grabExcessHorizontalSpace = true;		// for big fonts
		editCmdLabel.setLayoutData(gdShowCmdLbl);
		
		// label showing target file
		targetFileLabel = new Label(composite, SWT.WRAP);
		GridData gdTargetLabel = new GridData();
		gdTargetLabel.grabExcessHorizontalSpace = true;		// for big fonts
		targetFileLabel.setLayoutData(gdTargetLabel);
		
		// create a SourceViewer to show/edit the commands
		textViewer = new SourceViewer(composite, null, SWT.MULTI | SWT.BORDER
				| SWT.H_SCROLL | SWT.V_SCROLL);

		// set Document and Partitioner
		IDocument doc = new Document();
		doc.addDocumentListener(new IDocumentListener() {
			public void documentAboutToBeChanged(DocumentEvent event) {
			}

			/**
			 * Turns on the dirty state when the document changes.
			 * Systematic document changes should reset the dirty flag 
			 */
			public void documentChanged(DocumentEvent event) {
				isCmdDirty = true;
			}
		});
		IDocumentPartitioner partitioner = new FastPartitioner(
				new CommandFilePartitionScanner(), new String[] {CommandFilePartitionScanner.TOK});
		doc.setDocumentPartitioner(partitioner);
		partitioner.connect(doc);

		// configure() before setting the document
		textViewer.configure(new CommandFileViewerConfiguration());
		textViewer.setDocument(doc);

		GridData gdText = new GridData();
		gdText.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		gdText.heightHint = 300;
		gdText.grabExcessVerticalSpace = true;
		gdText.horizontalAlignment = SWT.FILL;
		gdText.verticalAlignment = SWT.FILL;
		textViewer.getTextWidget().setLayoutData(gdText);
		
		
		// template dialog and button
		// dialog opener button
		openDlgBtn = new Button(composite, SWT.PUSH);
		GridData gdButton = new GridData();
		gdButton.widthHint = SWT.DEFAULT;
		openDlgBtn.setLayoutData(gdButton);
		openDlgBtn.setText(LABEL_DLG_BTN);
		openDlgBtn.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				handleOpenTemplatePressed();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		//to here
		
		
		// read descriptor file and refresh controls
		resetToDefaults();

		return composite;
	}


	/**
	 * Get the DebuggerSettings from the Project Description file
	 */
	public void getProjectSettings() {
		super.getProjectSettings();
		cpuType = descManager.getCPUType();
		setting = descManager.getGDBSetting();
		dlgSetting = descManager.getGDBSetting();
	}

	/**
	 * Refreshes the whole displays of this page.
	 * If the project descriptor/command file does not contain any commands,
	 * or the command file is missing,
	 * they will be created here.
	 */
	public void refreshPage() {

		String cmdFileName = setting.getName() + CommandFileGenerator.EXT_CMD;
		
		// label showing target file
		editCmdLabel.setText(MessageFormat.format(LABEL_EDIT_CMD, new Object[] {cmdFileName}));
		
		String targetFile = MessageFormat.format(DebuggerSettingDialogResources.getString("gdbPage.targetFile.label"), 
				new Object[] {cmdFileName});
		targetFileLabel.setText(targetFile);
		

		try {
			// try to refresh project in case the command file is not in sync
			project.refreshLocal(IResource.DEPTH_ONE, null);
		} catch (CoreException ignore) {
		}
		
		// read cmd file in project (if exists)
		String fileContents = "";
		ProjectAccessor accessor = new ProjectAccessor(project);
		if (accessor.fileExistsInProject(cmdFileName)) {
			fileContents = readContentsFromFile(cmdFileName);
		} else {
			// create commands from the descriptor setting
			CommandFileGenerator gen = new CommandFileGenerator(project, descManager, setting);
			fileContents = gen.makeContents();
		}

		// show an error dialog when contents are null
		if (fileContents == null) {
			MessageDialog.openError(getShell(), DebuggerSettingDialogResources.getString("gdbPage.fileErr.dlg.title"),
					DebuggerSettingDialogResources.getString("gdbPage.fileErr.dlg.msg"));

			// set failed just to show an error every time
			cmdReadFailed = true;
			textViewer.setEditable(false);
			openDlgBtn.setEnabled(false);
			
		} else {
			textViewer.getDocument().set(fileContents);
			isCmdDirty = false;
			
			cmdReadFailed = false;
			textViewer.setEditable(true);
			openDlgBtn.setEnabled(true);
			
			// only for updating the OK button
			if (isContentsCreated()) {
				setErrorMessage(null);
				getContainer().updateButtons();
			}
			
		}
		
	}


	/**
	 * Event handler when the template dialog button gets pressed.
	 * @since 1.3.0
	 */
	private void handleOpenTemplatePressed() {
		// template dialog
		tempDlg = new CommandTemplateDialog(getShell(), cpuType, setting);
		int result = tempDlg.open();
		
		if (result == CommandTemplateDialog.OK) {
			
			// replace the commands
			dlgSetting = tempDlg.getSetting();
			CommandFileGenerator gen = new CommandFileGenerator(project, descManager, dlgSetting);
			textViewer.getDocument().set(gen.makeContents());
			isCmdDirty = true;
			
		}
	}


	/**
	 * Checks whether it is alright to leave this page.
	 * @return false to abort page flipping page, true to permit
	 * @see org.eclipse.jface.preference.IPreferencePage#okToLeave()
	 * @since 1.3.0
	 */
	public boolean okToLeave() {

		// check if the command text area has been modified
		if (isCmdDirty) {
			
			// show a dialog
			boolean updateNow = MessageDialog.openQuestion(getShell(), DLG_CMD_CHANGED_TITLE, DLG_CMD_CHANGED_MSG);
			
			if (updateNow) {
				  // immediately update the descriptor file
				  performOk();
			} else {
				// set the commands to its default
				refreshPage();
			}
			
		}

		// page flipping is always ok
		return true;

	}
	
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage#isValid()
	 */
	public boolean isValid() {
		if (cmdReadFailed) {
			return false;		// not good when the file was unable to read
		}
		
		return super.isValid();
	}
	
	/**
	 * Handles ok button events.
	 * Check for errors, gathers the input values, and updates the description file.
	 */
	public boolean performOk() {

		// write the settings to the description file
		// (only settings from this page)

		if (!isValid()) {
			return false;
		}

		try {

			DebuggerSetting newSetting = gatherInputs();

			// write it to the file
			descManager.updateDebuggerSetting(newSetting);

			// immediately write commands to the command file
			String cmdFileName = newSetting.getName() + CommandFileGenerator.EXT_CMD;
			ProjectAccessor accessor = new ProjectAccessor(project);
			if (isCmdDirty || !accessor.fileExistsInProject(cmdFileName) || cmdReadFailed) {
				String commands = textViewer.getDocument().get().replaceAll("\r", "");

				// write failed and do not close
				boolean writeOk = writeContentsToFile(cmdFileName, commands);
				if (!writeOk) {
					MessageDialog.openError(getShell(), DebuggerSettingDialogResources.getString("gdbPage.fileWriteErr.dlg.title"),
							DebuggerSettingDialogResources.getString("gdbPage.fileWriteErr.dlg.msg"));
					return false;
				}
				
			}
			
			isCmdDirty = false;

		} catch (CoreException e) {
/* >>>>> MODIFIED by D.Fujimoto 2007/08/03 display error */
			String title = MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName();
			String message = e.getLocalizedMessage();
			Logger.log(message);
			displayErrorDialog(title, message);
/* <<<<< MODIFIED by D.Fujimoto 2007/08/03 display error */
			return false;
		}
		return super.performOk();
	}

	
	/**
	 * Gather the inputs into a DebuggerSetting object.
	 * @return Setting with all of the fields set.
	 * 
	 * Refactored from performOk()
	 */
	private DebuggerSetting gatherInputs() {
		
		// Add the editor's text to the current setting
		DebuggerSetting newSetting = dlgSetting;
		// restore default vector address (value when this page was opened)
		newSetting.setVectorAddress(setting.getVectorAddress());
		
		return newSetting;
	}


	
	
	// classes for SourceViewer use
	
	/**
	 * PartitionScanner for comment lines
	 */
	class CommandFilePartitionScanner extends RuleBasedPartitionScanner {
		static final String TOK = "COMMENT";

		public CommandFilePartitionScanner() {
			// # lines are comments(when on the first column)
			IToken token = new Token(TOK);

			// EndOfLineRules does not commentize as intended but use anyway
			EndOfLineRule rule = new EndOfLineRule("#", token);
			rule.setColumnConstraint(0);
//			CommentRule rule = new CommentRule('#', token);
			
			
			setPredicateRules(new IPredicateRule[] {
				rule
			});
		}
	}
	
	/**
	 * Partitioning comment lines 
	 * Comment lines should start with a certain character
	 * with no leading characters.
	 * e.g) target icd usb  # comment  <- these are not considered as comments
	 * Unused for 1.3.0
	 */
	static class CommentRule extends PatternRule {

		
		public CommentRule(char startChar, IToken defaultToken) {
			super(Character.toString(startChar), System.getProperty("line.separator"), defaultToken, (char) 0, true, true);
		}

		/*
		 * Commentize from the starting char to the next line separator.
		 * @see IRule#evaluate
		 */
		public IToken evaluate(ICharacterScanner scanner, boolean resume) {
			int c;
			int nCharsToRollback = 0;
			boolean detected = false;
			
			if (scanner.getColumn() > 0) {
				return Token.UNDEFINED;	
			}

			do {
				c = scanner.read();
				System.out.println(Character.toString((char) c));
				nCharsToRollback++;
			} while (Character.isWhitespace((char) c));
			
			
			// check the start char
			if (c == fStartSequence[0]) {
				detected = true;
			}

			if (detected) {

				do {
					c = scanner.read();
				} while (!endSequenceDetected(scanner));
				scanner.unread();

				return getSuccessToken();

			} else { // Doesn't start with '#', roll back scanner

				for (int i = 0; i < nCharsToRollback; i++) {
					scanner.unread();
				}
			}

			return Token.UNDEFINED;
		}

	}

	/**
	 * ViewerConfiguration for the SourceViewer
	 */
	class CommandFileViewerConfiguration extends SourceViewerConfiguration {

		/**
		 * content color
		 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(org.eclipse.jface.text.source.ISourceViewer)
		 */
		public IPresentationReconciler getPresentationReconciler(
				ISourceViewer sourceViewer) {
			PresentationReconciler rec = new PresentationReconciler();

			// color for comments
			Color c = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);
			DefaultDamagerRepairer dr = new DefaultDamagerRepairer(
					new SimpleScanner(new TextAttribute(c)));
			rec.setDamager(dr, CommandFilePartitionScanner.TOK);
			rec.setRepairer(dr, CommandFilePartitionScanner.TOK);

			// color for non comments
			Color def = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
			dr = new DefaultDamagerRepairer(new SimpleScanner(new TextAttribute(def)));
			rec.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
			rec.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

			return rec;
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(org.eclipse.jface.text.source.ISourceViewer)
		 */
		public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
			return new String[] {
					IDocument.DEFAULT_CONTENT_TYPE,
					CommandFilePartitionScanner.TOK
			};
		}
		
	}

	
	/**
	 * dummy Scanner for coloring command lines
	 */
	class SimpleScanner extends BufferedRuleBasedScanner {
		public SimpleScanner(TextAttribute attr) {
			setDefaultReturnToken(new Token(attr));
		}
	}
}
