/*
 * Created on 2004/08/24
 * Modified on 2005/04/07 by D.Fujimoto added getAnsiLibPathDefault()
 * 										changed performOk() for ansilib paths
 * Modified on 2005/07/12 by D.Fujimoto changed performOk() to use GeneralInfo.getLibPath() 
 * 										changed getAnsiLibPathDefault() and getEmulibPathDefault()
 * 										added methods getCppLibPathDefault() and getLongLongLibPathDefault()
 * Modified on 2005/07/25 by D.Fujimoto changed performOk() and getXXXLibPathDefault() 
 * 											made tool dir for libs use variable instead of specifying absolute paths. 
 * Modified on 2005/08/08 by D.Fujimoto changed createContents() made useHighPrecision checkbox invisible and 
 * 											deleted selection listener.
 * Modified on 2006/03/17 by D.Fujimoto refactored in performOk() -> refreshAllPages()
 * Modified on 2006/05/16 by D.Fujimoto modified performOk() for resetting command file commands.
 * Modified on 2006/06/07 by D.Fujimoto modified performOk() for not resetting command file commands.
 * Modified on 2006/07/25 by D.Fujimoto modified performOk() as a result of raw commands deleted.
 * Modified on 2007/03/12 by D.Fujimoto modified performOk() for clean build when something changes.
 * Migrated for GNU33 on 2007/03/13 using GNU17(1.0.0)
 * Modified on 2007/03/16 by D.Fujimoto added various CpuSettingsManager modifications.
 * Modified on 2007/06/29 by D.Fujimoto added getSimIOLibPathDefault() and 
 * 										changed performOk()
 * Modified on 2007/07/30 by D.Fujimoto changed performOk() to close dialog even when not to clean.
 * Modified on 2007/08/03 by D.Fujimoto modified performOk() to show error when pj file is read only.
 */
package jp.co.epson.ide.eclipse.gnu33.ui.dialogs.general;

import java.io.IOException;
import java.util.List;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.Gnu33ProjectDescriptorManager;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSetting;
import jp.co.epson.ide.eclipse.gnu33.build.BuildSettingFactory;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildOption;
import jp.co.epson.ide.eclipse.gnu33.build.IBuildTool;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;
import jp.co.epson.ide.eclipse.gnu33.debug.DebuggerSetting;
import jp.co.epson.ide.eclipse.gnu33.file.MakeFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.CleanJob;
import jp.co.epson.ide.eclipse.gnu33.others.Logger;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.AbstractGnu33PropertyPage;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/**
 * The Property Page for General Setting in a Gnu33 project. 
 * @author D.Fujimoto
 *
 */
public class GeneralPropertyPage extends AbstractGnu33PropertyPage {

	public static final String LABEL_TARGET_CPU_TYPE = GeneralSettingDialogResources.getString("generalPage.targetCPUType.label");
	public static final String LABEL_TARGET_CPU_NAME = GeneralSettingDialogResources.getString("generalPage.targetCPUName.label");
	public static final String LABEL_USE_HIGH_PRECISION_LIB =GeneralSettingDialogResources.getString("generalPage.useHighPrecLib.label"); 
	public static final String DLG_CPU_CHANGED_TITLE = GeneralSettingDialogResources.getString("generalPage.cpuChanged.title");
	public static final String DLG_CPU_CHANGED_MSG1 = GeneralSettingDialogResources.getString("generalPage.cpuChanged.msg");
	
	/**
	 * GeneralInfo object from the project descriptor.
	 */
	private GeneralInfo generalInfo;
	
	/**
	 * The original cpu type defined in the project descriptor.
	 * This is not the selection index.
	 */
	private int defaultCPUtype;
	
	/**
	 * The original cpu name written in the project descriptor.
	 */
	private String defaultCPUname;
	
	/**
	 * The original useHighPrecisionLib flag defined in the project descriptor.
	 */
	private boolean defaultUseHigh;
	
	/**
	 * Flag indicating that CPU type combo has been changed from the default.
	 */
	private boolean isCpuDirty;
	
	/**
	 * Flag indicating that useHighPrecisionLib check has been changed from the default.
	 */
	private boolean isUseHighDirty;
	
	/**
	 * Replacement string for tool dir.
	 */
	protected String toolDirVar;
	
	/**
	 * The Settings Manager
	 */
	private CpuSettingsManager cpuManager;
	
	/**
	 * Constructor. Called by the framework.
	 */
	public GeneralPropertyPage() {
		super();
		toolDirVar = MakeFileGenerator.expandMacro(MakeFileGenerator.MACRO_TOOL_DIR);
		
		// get the CPUManager
		cpuManager = CpuSettingsManager.getManager();
	}

	// the controls
	private Combo comboCPU;
	private Button checkHighPrecision;
	private Label labelCPU;
	
	/**
	 * Create the UI here.
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		
		Composite composite = new Composite(parent, SWT.NONE);
		
		GridData gridData = new GridData(GridData.GRAB_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
		composite.setLayoutData(gridData);
		composite.setLayout(new GridLayout());

		Group groupCPU = new Group(composite, SWT.NONE);
		GridData gridGroup = new GridData(GridData.GRAB_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING
				| GridData.HORIZONTAL_ALIGN_FILL);
		gridGroup.widthHint = 220;
		groupCPU.setLayoutData(gridGroup);
		groupCPU.setText(LABEL_TARGET_CPU_NAME);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		groupCPU.setLayout(gridLayout);

		comboCPU = new Combo(groupCPU, SWT.READ_ONLY);
		GridData gridComboCPU = new GridData(GridData.VERTICAL_ALIGN_FILL);
		gridComboCPU.horizontalSpan = 2;
		gridComboCPU.widthHint = 100;
		comboCPU.setLayoutData(gridComboCPU);

/* >>>>> DELETED by D.Fujimoto 2007/03/16 items are set by the CpuSettingsManager */		
//		comboCPU.setItems(new String[] {
//				GeneralInfo.CPU_STD_STR, 
//				GeneralInfo.CPU_PE_STR, 
//				GeneralInfo.CPU_ADV_33401_STR});
/* <<<<< DELETED by D.Fujimoto 2007/03/16 items are set by the CpuSettingsManager */
		comboCPU.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleCPUSelection();
			}		   
		});
		comboCPU.select(0);

		// CPU label
		labelCPU = new Label(groupCPU, SWT.WRAP);
		GridData gdLabel = new GridData();
		gdLabel.horizontalAlignment = SWT.BEGINNING;
		gdLabel.horizontalIndent = 10;
		gdLabel.widthHint = 350;
		labelCPU.setLayoutData(gdLabel);
		
/** >>>>> MODIFIED by D.Fujimoto 2005/08/08 checkbox invisible **/
		checkHighPrecision = new Button(composite, SWT.CHECK);
		checkHighPrecision.setVisible(false);
		checkHighPrecision.setEnabled(false);
		checkHighPrecision.setText(LABEL_USE_HIGH_PRECISION_LIB);
/** <<<<< MODIFIED by D.Fujimoto 2005/08/08 checkbox invisible **/
		
		// read descriptor file and refresh controls
		resetToDefaults();
		
		return composite;
		
	}

	/**
	 * Returns the CPU type selected in this page.
	 * CPU type is not the String displayed in the combo.
	 * It is the CPU core type associated with the CPU name
	 * @return	the CPU type as int
	 * @since 1.4.0
	 */
	public int getSelectedCpuType() {
		
		String cpuName = getSelectedCpuName();
		GeneralInfo generalInfo = (GeneralInfo) cpuManager.getCpuInfoMap().get(cpuName);
		return generalInfo.getCpuType();
	}

	/**
	 * Returns the CPU name
	 * @return	the CPU name selected in the combo
	 * @since 1.4.0
	 * 
	 * Changed in 1.4.1 to return CPU display name
	 */
	public String getSelectedCpuName() {
/* >>>>> MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */				
//		return comboCPU.getItem(comboCPU.getSelectionIndex()); 
		String cpuDisplayName = comboCPU.getItem(comboCPU.getSelectionIndex()); 
		return cpuManager.getCpuName(cpuDisplayName);
/* <<<<< MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */
	}

	
	/**
	 * Event handler for CPU combo.
	 * Marks the flag dirty if CPU type(name) has changed from the
	 * default.
	 * 
	 * As of 1.4.0, the selection is determined by the selected NAME
	 * and not the index.
	 */
	private void handleCPUSelection() {
		String selectedName = getSelectedCpuName();
		
		if (selectedName.equals(defaultCPUname)) {
			isCpuDirty = false;
		} else {
			isCpuDirty = true;	  
		}
		
		
		// label for the CPU
		labelCPU.setText(GeneralSettingDialogResources.getString("generalPage.cpuType.desc") + "" +
				GeneralInfo.parseCPUString(getSelectedCpuType()));
	}
	
	/**
	 * Event handler for Use High Precision Lib check.
	 * Marks the flag dirty if the check state has changed from the 
	 * default.
	 * 
	 * @deprecated 1.1.1 because of checkbox made invisible.
	 */
	private void handleUseHighLibSelection() {
		boolean selected = checkHighPrecision.getSelection();
		if (selected == defaultUseHigh) {
			isUseHighDirty = false;
		} else {
			isUseHighDirty = true;
		}
	}

	/**
	 * Get the GeneralInfo from the Project Description file.
	 */
	public void getProjectSettings() {
		super.getProjectSettings();
		generalInfo = descManager.getGeneralInfo();
		defaultCPUtype = generalInfo.getCpuType();
		defaultCPUname = generalInfo.getCpuName();
		defaultUseHigh = generalInfo.isUseHighPrecisionLib();
		
	}
	
	/**
	 * Refreshes the whole displays of this page.
	 */
	public void refreshPage() {
/* >>>>> MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */				
		// initial CPU combo
//		String[] cpuNames = cpuManager.getCpuNames();
//		if (cpuNames.length == 0) {
		String[] cpuDisplayNames = cpuManager.getCpuDisplayNames();
		if (cpuDisplayNames.length == 0) {
/* <<<<< MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */
			MessageDialog.openError(null, ResourceProvider.getResourceString("propertyPage.cpuinfo.failed.title"), 
					ResourceProvider.getResourceString("propertyPage.cpuinfo.failed"));
		}

/* >>>>> MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */		
//		comboCPU.setItems(cpuManager.getCpuNames());
//		comboCPU.select(0);
//		for (int i = 0; i < comboCPU.getItemCount(); i++) {
//			if (defaultCPUname.equals(comboCPU.getItem(i))) {
//				comboCPU.select(i);
//			}
//		}
		
		comboCPU.setItems(cpuDisplayNames);
		comboCPU.select(0);
		for (int i = 0; i < comboCPU.getItemCount(); i++) {
			String displayName = comboCPU.getItem(i);
			if (defaultCPUname.equals(cpuManager.getCpuName(displayName))) {
				comboCPU.select(i);
			}
		}
/* <<<<< MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */
		handleCPUSelection();
		
		
		checkHighPrecision.setSelection(defaultUseHigh);

		// reset dirty flags
		isCpuDirty = false;
		isUseHighDirty = false;
	}

	
	/**
	 * Checks whether it is alright to leave this page.
	 * @return false to abort page flipping page, true to permit
	 * @see org.eclipse.jface.preference.IPreferencePage#okToLeave() 
	 */
	public boolean okToLeave() {
		boolean canFlip = true;
		
		if (isCpuDirty || isUseHighDirty) {
			// the CPU has changed, so show a dialog
			
			boolean updateNow = MessageDialog.openQuestion(getShell(), 
					DLG_CPU_CHANGED_TITLE, DLG_CPU_CHANGED_MSG1);
			if (updateNow) {
				// immediately update the descriptor file
				performOk();
			} else {
				// set CPU combo to default
/* >>>>> MODIFIED by D.Fujimoto 2007/03/16 reset combo CPU by name */				
				for (int i = 0; i < comboCPU.getItemCount(); i++) {
/* >>>>> MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */				
//					if (defaultCPUname.equals(comboCPU.getItem(i))) {
//						comboCPU.select(i);
//					}
					// refresh the CPU display to CPU display name
					String displayName = comboCPU.getItem(i);
					if (defaultCPUname.equals(cpuManager.getCpuName(displayName))) {
						comboCPU.select(i);
					}
/* <<<<< MODIFIED by D.Fujimoto 2008/05/31 use CPU display name */
				}
				handleCPUSelection();
				isCpuDirty = false;
/* <<<<< MODIFIED by D.Fujimoto 2007/03/16 reset combo CPU by name */
				
				// set useHigh check to default
				checkHighPrecision.setSelection(defaultUseHigh);
				isUseHighDirty = false;
			}
		}

		return canFlip;
	}
	
	
	/**
	 * Handles ok button events.
	 * Gather the inputs, if CPU type has changed then make default settings,
	 * update the settings to the descriptor file. 
	 */
	public boolean performOk() {
		
		// write the settings to the description file
		// (if CPU type has changed, also rewrite other Gnu33 settings)
/* >>>>> MODIFIED by D.Fujimoto 2007/03/16 selected CPU is not the selected index */
		generalInfo.setCpuType(getSelectedCpuType());
		generalInfo.setCpuName(getSelectedCpuName());
		generalInfo.setUseHighPrecisionLib(checkHighPrecision.getSelection());
		generalInfo.setInitialTtbrValue(cpuManager.getInitialTtbrValueForCpu(getSelectedCpuName()));
		// memory model is not updated here (always as default)
		
		
		try {
			
			// write general setting to descriptor file
			descManager.updateGeneralSetting(generalInfo);
			
			// do when something has changed
			if (isCpuDirty || isUseHighDirty) {
/* >>>>> ADDED by D.Fujimoto 2007/03/22 initialize with the selected CPU */
				cpuManager.initializeStoreByCpu(Gnu33CorePlugin.getDefault().getPreferenceStore(), generalInfo.getCpuName());
/* <<<<< ADDED by D.Fujimoto 2007/03/22 initialize with the selected CPU */
				
				if (isCpuDirty) {
					// write default debugger setting to descriptor file
					
/* >>>>> ADDED by D.Fujimoto 2007/06/29 prevent _gnu33IDEsuffix from being added twice */					
					// erase suffix from setting name
					String settingName = descManager.getGDBSetting().getName();
					settingName = settingName.substring(0, settingName.lastIndexOf(ResourceProvider.getResourceString("common.suffix")));
/* <<<<< ADDED by D.Fujimoto 2007/06/29 prevent _gnu33IDEsuffix from being added twice */					
					DebuggerSetting newGDBsetting = CpuSettingsManager.createDefaultDebuggerSetting(settingName);
					newGDBsetting.setVectorAddress(generalInfo.getInitialTtbrValue());	// new CPU's vector address
					
					descManager.updateDebuggerSetting(newGDBsetting);
				} 
					
				// write default parameter setting to descriptor file
				;// no chages made to parameter setting
					
				BuildSetting buildSetting = (BuildSetting) descManager.getBuildList().get(0);
				if (isCpuDirty) {
					// change (-mc33pe, -mc33adv) compiler settings to descriptor file
/* >>>>> MODIFIED by D.Fujimoto 2007/03/13 improved CPU option switching */					
					BuildSettingFactory.switchCpuOptions(buildSetting, generalInfo.getCpuType());
					
					IBuildTool compiler = buildSetting.getCompiler();
//					IBuildOption userOption = compiler.getOption(BuildSettingFactory.C_USER_OPT_ID);
//					String optionString = userOption.getStringValue();
//					optionString = BuildSettingFactory.alterUserOps(optionString, selectedCPU);
//					userOption.setStringValue(optionString);
//									
//					// change (-mc33pe, -mc33adv) assembler settings to descriptor file
//					IBuildTool assembler = buildSetting.getAssembler();
//					userOption = assembler.getOption(BuildSettingFactory.ASM_USER_OPT_ID);
//					optionString = userOption.getStringValue();
//					optionString = BuildSettingFactory.alterUserOps(optionString, selectedCPU);
//					userOption.setStringValue(optionString);
/* <<<<< MODIFIED by D.Fujimoto 2007/03/13 improved CPU option switching */
					
					// disable the compiler's -medda32 option if the CPU has changed to ADV 
					if (generalInfo.getCpuType() == GeneralInfo.CPU_ADV) {
						IBuildOption medda32 = compiler.getOption(BuildSettingFactory.C_MEDDA_OPT_ID);
						medda32.setBooleanValue(false);
					}
					
				}
				
				// update the linker's ansi c and emulation and other library paths
				IBuildTool linker = buildSetting.getLinker();
				IBuildOption libs = linker.getOption(BuildSettingFactory.LD_LIB_OPT_ID);
				String[] libString = libs.getListValue();
				for (int i = 0; i < libString.length; i++) {

					IPath newLibPath = generalInfo.getLibPath();
					if (libString[i].equalsIgnoreCase(getEmuLibPathDefault())) {
						newLibPath = newLibPath.append(generalInfo.getEmulibName());
						libString[i] = toolDirVar + newLibPath.toString();
					} else if (libString[i].equalsIgnoreCase(getAnsiLibPathDefault())) {
						newLibPath = newLibPath.append(generalInfo.getAnsiLibName());
						libString[i] = toolDirVar + newLibPath.toString();
					} else if (libString[i].equalsIgnoreCase(getCppLibPathDefault())) {
						newLibPath = newLibPath.append(generalInfo.getCppLibName());
						libString[i] = toolDirVar + newLibPath.toString();
					} else if (libString[i].equalsIgnoreCase(getLongLongLibPathDefault())) {
						newLibPath = newLibPath.append(generalInfo.getLongLongLibName());
						libString[i] = toolDirVar + newLibPath.toString();
/* >>>>> ADDED by D.Fujimoto 2007/06/29 for simio lib */
					} else if (libString[i].equalsIgnoreCase(getSimIOLibPathDefault())) {
						newLibPath = newLibPath.append(generalInfo.getSimIOLibName());
						libString[i] = toolDirVar + newLibPath.toString();
/* <<<<< ADDED by D.Fujimoto 2007/06/29 for simio lib */
					}
				}
				libs.setListValue(libString);
				
				descManager.updateBuildSetting(descManager.getBuildList());
				
				// update the linkerscript's ansi c and emulation and other library paths
				List sections = descManager.getLdsSetting().getSections();
				for (int i = 0; i < sections.size(); i++) {
					SectionData section = (SectionData) sections.get(i);
					List objList = section.getObjectList();
					for (int j = 0; j < objList.size(); j++) {
						String obj = (String) objList.get(j);

						IPath newLibPath = generalInfo.getLibPath();
						if (obj.equalsIgnoreCase(getEmuLibPathDefault())) {
							newLibPath = newLibPath.append(generalInfo.getEmulibName());
							objList.set(j, toolDirVar + newLibPath.toString());
						} else if (obj.equalsIgnoreCase(getAnsiLibPathDefault())) {
							newLibPath = newLibPath.append(generalInfo.getAnsiLibName());
							objList.set(j, toolDirVar + newLibPath.toString());
						} else if (obj.equalsIgnoreCase(getCppLibPathDefault())) {
							newLibPath = newLibPath.append(generalInfo.getCppLibName());
							objList.set(j, toolDirVar + newLibPath.toString());
						} else if (obj.equalsIgnoreCase(getLongLongLibPathDefault())) {
							newLibPath = newLibPath.append(generalInfo.getLongLongLibName());
							objList.set(j, toolDirVar + newLibPath.toString());
/* >>>>> ADDED by D.Fujimoto 2007/06/29 for simio lib */
						} else if (obj.equalsIgnoreCase(getSimIOLibPathDefault())) {
							newLibPath = newLibPath.append(generalInfo.getSimIOLibName());
							objList.set(j, toolDirVar + newLibPath.toString());
/* <<<<< ADDED by D.Fujimoto 2007/06/29 for simio lib */
						}
					}
				}
				
				descManager.updateLinkerScriptSetting(descManager.getLdsSetting());

				
				// refresh other Property Pages(that are related to Gnu33 project)
				refreshAllPages();
				
/* >>>>> ADDED by D.Fujimoto 2007/04/13 clean to let the next build be a rebuild */    
				// perform a clean build here
				boolean doClean = false;
				String title = GeneralSettingDialogResources.getString("needsbuild_title");
				String msg = GeneralSettingDialogResources.getString("needsbuild_message");
				MessageDialog dialog = new MessageDialog(getShell(), title, null, msg, MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 1);
				int res = dialog.open();
				if (res == Dialog.OK) {
					CleanJob cleanJob = new CleanJob(project);
					cleanJob.schedule();
				} else {
					; // cancel pressed, no clean
/* >>>>> MODIFIED by D.Fujimoto 2007/07/30 close dialog when OK is pressed regardless of clean prompt */
//					return false;
/* <<<<< MODIFIED by D.Fujimoto 2007/07/30 close dialog when OK is pressed regardless of clean prompt */
				}
/* <<<<< ADDED by D.Fujimoto 2007/04/13 clean to let the next build be a rebuild */

			}
		   
			
		} catch (CoreException e) {
/* >>>>> MODIFIED by D.Fujimoto 2007/08/03 display error */
			String title = MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName();
			String message = e.getLocalizedMessage();
			Logger.log(message);
			displayErrorDialog(title, message);
/* <<<<< MODIFIED by D.Fujimoto 2007/08/03 display error */
			return false;
		} catch (IOException e) {
			Logger.log(MSG_UPDATE_FAILED + Gnu33ProjectDescriptorManager.getDescriptorFileName());
			return false;
		} finally {
			isCpuDirty = false;
			isUseHighDirty = false;
		}
		
		return super.performOk();
	}
	
	/**
	 * Returns the ansi c library path that was initially set before
	 * the CPU or the library use checkbox has changed.
	 * @return the original ansi c library path
	 * @since 1.1.0
	 */
	private String getAnsiLibPathDefault() {
		IPath path = GeneralInfo.getLibPathByCpuType(defaultCPUtype);
		
		path = path.append(generalInfo.getAnsiLibName());
		return toolDirVar + path.toString();
	}
  
	/**
	 * Returns the emulation library path that was initially set before
	 * the CPU or the library use checkbox has changed. 
	 * 
	 * @return the original emulation library path
	 */
	private String getEmuLibPathDefault() {
		IPath path = GeneralInfo.getLibPathByCpuType(defaultCPUtype);
		
		if (defaultUseHigh) {
			path = path.append(ResourceProvider.getResourceString("gnu33tools.emulibNameP"));
		} else {
			path = path.append(ResourceProvider.getResourceString("gnu33tools.emulibName"));
		}
		
		return toolDirVar + path.toString();
		
	}
	
	/**
	 * Returns the cpp library path that was initially set before 
	 * the CPU or the library use checkbox has changed.
	 * 
	 * @return the original cpp library path
	 * @since 1.1.1
	 */
	private String getCppLibPathDefault() {
		IPath path = GeneralInfo.getLibPathByCpuType(defaultCPUtype);
		
		path = path.append(generalInfo.getCppLibName());
		return toolDirVar + path.toString();
	}
	
	/**
	 * Returns the longlong library path that was initially set before 
	 * the CPU or the library use checkbox has changed.
	 * 
	 * @return the original longlong library path
	 * @since 1.1.1
	 */
	private String getLongLongLibPathDefault() {
		IPath path = GeneralInfo.getLibPathByCpuType(defaultCPUtype);
		
		path = path.append(generalInfo.getLongLongLibName());
		return toolDirVar + path.toString();
	}  
	
	/**
	 * Returns the simio library path that was initially set before
	 * the memory model list has changed.
	 * @return the original simio library path
	 * @since 1.4.0
	 */
	private String getSimIOLibPathDefault() {
		IPath path = GeneralInfo.getLibPathByCpuType(defaultCPUtype);
		
		path = path.append(generalInfo.getSimIOLibName());
		return toolDirVar + path.toString();
	}
	
}
