/*
 * Created on 2004/07/08
 * Modified on 2004/04/07 by D.Fujimoto in createControl()
 * Modified on 2005/07/28 by D.Fujimoto added selectInitialCPU() selectUseHighPrecision() for debugging purposes.
 * Modified on 2005/08/08 by D.Fujimoto changed createControl() made useHighPrecision checkbox invisible.
 * Modified on 2005/11/18 by D.Fujimoto added UI for vector section settings and methods
 * 											handleVecCheck(), verifyInputs(), getVectorSectionData().
 * Modified in 2006/03/22 by D.Fujimoto modified createControl(), handleCPUSelection(),
 * 											getVectorSectionData() for TTBR inputs.
 * Migrated for GNU33 on 2007/01/19 using GNU17(1.0.0)
 */
package jp.co.epson.ide.eclipse.gnu33.ui.wizards;

import java.io.IOException;
import java.text.MessageFormat;

import jp.co.epson.ide.eclipse.gnu33.Gnu33CorePlugin;
import jp.co.epson.ide.eclipse.gnu33.ResourceProvider;
import jp.co.epson.ide.eclipse.gnu33.cpuinfo.CpuSettingsManager;
import jp.co.epson.ide.eclipse.gnu33.debug.AddrBitChecker;
import jp.co.epson.ide.eclipse.gnu33.file.AbstractFileGenerator;
import jp.co.epson.ide.eclipse.gnu33.general.GeneralInfo;
import jp.co.epson.ide.eclipse.gnu33.linkerscript.SectionData;
import jp.co.epson.ide.eclipse.gnu33.others.UIUtil;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.general.GeneralSettingDialogResources;
import jp.co.epson.ide.eclipse.gnu33.ui.dialogs.linkerscript.LinkerScriptSettingDialogResources;

import org.eclipse.cdt.make.ui.wizards.MakeProjectWizardOptionPage;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

/**
 * 2nd Wizard Page for Gnu33 C Project.
 *
 * @author D.Fujimoto
 *	
 */
public class NewGnu33CDTProjectWizardPage extends WizardPage {

	// label strings
	public static final String LABEL_TAB1 = NewWizardResources.getString("NewWizard.label.tab1");
	public static final String LABEL_TARGET_CPU_NAME = GeneralSettingDialogResources.getString("generalPage.targetCPUName.label"); 
	public static final String LABEL_USE_HIGH_PRECISION_LIB = GeneralSettingDialogResources.getString("generalPage.useHighPrecLib.label");
	public static final String LABEL_VEC_GROUP = NewWizardResources.getString("NewWizard.label.vecGroup");
	public static final String LABEL_VEC_CHECK = NewWizardResources.getString("NewWizard.label.vecCheck");
	public static final String LABEL_VEC_DESC = NewWizardResources.getString("NewWizard.label.vecDescription");

	public static final String VEC_SECTION_NAME = SectionData.VEC_SECTION_NAME;
	
	// title & description
	public static final String PAGE_TITLE = "NewWizard.pageTitle";
	public static final String PAGE_DESCRIPTION = "NewWizard.pageDescription";
	
	public static final String PAGE_NAME = "NewGnu33CDTProjectWizardPage";
	
	// initialization flag
	private boolean firstTime = true;

/* >>>>> DELETED by D.Fujimoto 2007/03/15 CPU is not the selected index */	
//	// The selected CPU type
//	private int selectedCPU;
/* <<<<< DELETED by D.Fujimoto 2007/03/15 CPU is not the selected index */
	
	// the controls
	private Combo comboCPU;
	private Label labelCPU;
	private Button checkHighPrecision;
	private Button checkUseVec;
	private Combo comboVecFile;
	private Text textVecAddr;

	// default file names to show in the combo
	String[] defaultVecFiles = {"vector", "boot"};
	
	private IPreferenceStore store;
	private CpuSettingsManager cpuManager;
	
	
	/**
	 * Constructor for SampleNewWizardPage.
	 */
	public NewGnu33CDTProjectWizardPage() {
		super(PAGE_NAME);
		setTitle(NewWizardResources.getString(PAGE_TITLE));
		setDescription(NewWizardResources.getString(PAGE_DESCRIPTION));
		
		// get the CPUManager
		cpuManager = CpuSettingsManager.getManager();
		
		store = Gnu33CorePlugin.getDefault().getPreferenceStore();
	}
	

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new FillLayout());
		//
		setControl(container);

		// TabFolder
		TabFolder tabFolder = new TabFolder(container, SWT.NONE);
//		  tabFolder.setBounds(0, 0, 485, 270);

		// TabItem1
		TabItem tabItem1 = new TabItem(tabFolder, SWT.NONE);
		tabItem1.setText(LABEL_TAB1);
		Composite composite = new Composite(tabFolder, SWT.NONE);
		tabItem1.setControl(composite);
		
		// Copied from GeneralPropertyPage --begin
		GridData gridData = new GridData(GridData.GRAB_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
		composite.setLayoutData(gridData);
		composite.setLayout(new GridLayout());


		Group groupCPU = new Group(composite, SWT.NONE);
		GridData gridGroup = new GridData(GridData.GRAB_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING
				| GridData.HORIZONTAL_ALIGN_FILL);
		gridGroup.widthHint = 220;
		groupCPU.setLayoutData(gridGroup);
		groupCPU.setText(LABEL_TARGET_CPU_NAME);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		groupCPU.setLayout(gridLayout);

		comboCPU = new Combo(groupCPU, SWT.READ_ONLY);
		GridData gridComboCPU = new GridData(GridData.VERTICAL_ALIGN_FILL);
		gridComboCPU.horizontalSpan = 2;
		gridComboCPU.widthHint = 100;
		comboCPU.setLayoutData(gridComboCPU);
/* >>>>> DELETED by D.Fujimoto 2007/03/15 moved to fillSettings() */ 
//		comboCPU.setItems(new String[] {
//				GeneralInfo.CPU_STD_STR, 
//				GeneralInfo.CPU_PE_STR, 
//				GeneralInfo.CPU_ADV_33401_STR});
/* <<<<< DELETED by D.Fujimoto 2007/03/15 moved to fillSettings() */
		comboCPU.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleCPUSelection();
			}		   
		});
		
		// CPU label
		labelCPU = new Label(groupCPU, SWT.WRAP);
		GridData gdLabel = new GridData();
		gdLabel.horizontalAlignment = SWT.BEGINNING;
		gdLabel.horizontalIndent = 10;
		gdLabel.widthHint = 350;
		labelCPU.setLayoutData(gdLabel);
		
		// checkbox invisible
		checkHighPrecision = new Button(composite, SWT.CHECK);
		checkHighPrecision.setVisible(false);
		checkHighPrecision.setEnabled(false);
		checkHighPrecision.setText(LABEL_USE_HIGH_PRECISION_LIB);
		// Copied from GeneralPropertyPage --over

		// vector section controls
		Group vecComp = new Group(composite, SWT.NONE);
		String groupText = MessageFormat.format(LABEL_VEC_GROUP, new String[] {VEC_SECTION_NAME});
		vecComp.setText(groupText);
		GridData gdVec = new GridData(GridData.GRAB_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING 
				| GridData.HORIZONTAL_ALIGN_FILL);
		gdVec.widthHint = 400;
		vecComp.setLayoutData(gdVec);
		GridLayout glVec = new GridLayout();
		glVec.numColumns = 2;
		vecComp.setLayout(glVec);

		// checkbox
		checkUseVec = new Button(vecComp, SWT.CHECK);
		checkUseVec.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleVecCheck();
			}
		});
		GridData gdCheckVec = new GridData();
		gdCheckVec.horizontalSpan = 2;
		checkUseVec.setLayoutData(gdCheckVec);
		String checkText = MessageFormat.format(LABEL_VEC_CHECK, new String[] {VEC_SECTION_NAME});
		checkUseVec.setText(checkText);

		Text labelVec = new Text(vecComp, SWT.MULTI | SWT.READ_ONLY | SWT.WRAP);
		labelVec.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		GridData gdLabelVec = new GridData();
		gdLabelVec.horizontalIndent = 20;
		gdLabelVec.horizontalSpan = 2;
		labelVec.setLayoutData(gdLabelVec);
		labelVec.setText(LABEL_VEC_DESC);

		// file name combo
		comboVecFile = new Combo(vecComp, SWT.NONE);
		GridData gdComboVec = new GridData();
		gdComboVec.widthHint = 100;
		gdComboVec.horizontalIndent = 20;
		comboVecFile.setLayoutData(gdComboVec);
		comboVecFile.setTextLimit(32);
		comboVecFile.setItems(defaultVecFiles);
		comboVecFile.addKeyListener(new KeyAdapter() {
			 public void keyPressed(KeyEvent e) {
				UIUtil.enableAlphNumInput(e);
			}
		});
		comboVecFile.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();
			}
		});

		Label labelExt = new Label(vecComp, SWT.NONE);
		labelExt.setText(AbstractFileGenerator.EXT_OBJ);

		Group vecAddrComp = new Group(composite, SWT.NONE);
		vecAddrComp.setText(NewWizardResources.getString("NewWizard.label.vecAddrGroup"));
		GridData gdVecAddrComp = new GridData(GridData.VERTICAL_ALIGN_BEGINNING 
				| GridData.HORIZONTAL_ALIGN_FILL);
		gdVecAddrComp.widthHint = 400;
		vecAddrComp.setLayoutData(gdVecAddrComp);
		GridLayout glVecAddr = new GridLayout();
		glVecAddr.numColumns = 2;
		vecAddrComp.setLayout(glVecAddr);

		Label labelVecAddr = new Label(vecAddrComp, SWT.NONE);
		labelVecAddr.setText(LinkerScriptSettingDialogResources.getString("linkerscript.vecAddrLabel"));	//$NON-NLS-1$
		
		textVecAddr = new Text(vecAddrComp, SWT.BORDER | SWT.SINGLE);
		textVecAddr.setTextLimit(UIUtil.getAddressMaxChars());
		GridData gdTextVec = new GridData();
		gdTextVec.widthHint = 90;
		textVecAddr.setLayoutData(gdTextVec);
		textVecAddr.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				UIUtil.enableHexInput(e);
			}
		});
		textVecAddr.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				verifyInputs();

				// set value to PreferenceStore
				long addr = 0;
				try {
					addr = Long.parseLong(textVecAddr.getText(), 16);	
				} catch (NumberFormatException nfe) {
					// ignore
				}
				store.setValue("wizard.vecAddr", addr);
			}
		});
		
		Label labelVecAddrDesc = new Label(vecAddrComp, SWT.NONE);
		labelVecAddrDesc.setText(NewWizardResources.getString("NewWizard.label.vecAddrDesc"));
		GridData gdVecAddrDesc = new GridData();
		gdVecAddrDesc.horizontalSpan = 2;
		labelVecAddrDesc.setLayoutData(gdVecAddrDesc);

		// 
		
/* >>>>> DELETED by D.Fujimoto 2007/03/15 moved to fillSettnigs() */
//		// initial settings
//		selectInitialCPU(0);
//		handleCPUSelection();		// fills the address
//		checkUseVec.setSelection(true);
//		comboVecFile.setText(defaultVecFiles[0]);
//		verifyInputs();
/* <<<<< DELETED by D.Fujimoto 2007/03/15 moved to fillSettnigs() */
		
		setControl(container);
		
		fillSettings();
	
	}
	
	/**
	 * Fill the controls with their dialog settings.
	 * Only sets if a setting exists.
	 * 
	 * For QA purposes
	 * @since 1.2.0
	 */
	private void fillSettings() {

/* >>>>> ADDED by D.Fujimoto 2007/03/15 for filling the UI from CpuSettingsManager */		
		// initial settings
/* >>>>> MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */		
//		String[] cpuNames = cpuManager.getCpuNames();
//		if (cpuNames.length == 0) {
		String[] cpuDisplayNames = cpuManager.getCpuDisplayNames();
		if (cpuDisplayNames.length == 0) {
/* <<<<< MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */
			MessageDialog.openError(null, ResourceProvider.getResourceString("propertyPage.cpuinfo.failed.title"), 
					ResourceProvider.getResourceString("propertyPage.cpuinfo.failed"));
			return;
		}
/* >>>>> MODIFIED by D.Fujimoto 2008/05/30 display names for combo */
//		comboCPU.setItems(cpuNames);
		comboCPU.setItems(cpuDisplayNames);
/* <<<<< MODIFIED by D.Fujimoto 2008/05/30 display names for combo */
		selectInitialCPU(0);
		handleCPUSelection();		// fills the address

		// no memory model selection for GNU33
//		comboMemModel.select(0);	// select NORMAL
//		handleMemoryModelSelection();	// force an event
		
		checkUseVec.setSelection(store.getBoolean("allocVecSection"));
		comboVecFile.setText(store.getString("vectorFileName"));
		verifyInputs();
		
/* <<<<< ADDED by D.Fujimoto 2007/03/15 for filling the UI from CpuSettingsManager */

		// below are only for QA settings 
		IDialogSettings section = getDialogSettings().getSection("wizpage2");
		if (section != null) {
			selectInitialCPU(section.get("default.cpuName"));
			checkUseVec.setSelection(section.getBoolean("default.useVector"));
			comboVecFile.setText(section.get("default.vectorFileName"));
		}

		section = getDialogSettings().getSection("wizpage2.vecAddr");
		if (section != null) {
			textVecAddr.setText(section.get("default.vecAddr"));
		}

	}
	
	
	/**
	 * The Next button will be disabled if the next page is 
	 * MakeProjectWizardOptionPage.
	 */
	public boolean canFlipToNextPage() {
		IWizardPage nextPage = getNextPage();
		if (nextPage instanceof MakeProjectWizardOptionPage) {
			return false;
		}
		
		return super.canFlipToNextPage();
	}
	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if (visible && firstTime) {
			initialize();
			firstTime = false;
		}
		
	}
	
	/**
	 * Does things for the first time this dialog has become visible.
	 */
	private void initialize() {

		String defaultName = "";

		IWizardPage startingPage = getWizard().getStartingPage();
		if (startingPage instanceof WizardNewProjectCreationPage) {
			WizardNewProjectCreationPage newProjPage = (WizardNewProjectCreationPage) startingPage;
			defaultName = newProjPage.getProjectName();
		}

	}

	/**
	 * Sets the initial comboCPU by index.
	 * 
	 * @param index	the index of the combo to be selected.
	 * @since 1.1.1
	 * @deprecated 
	 */
	public void selectInitialCPU(int index) {
		comboCPU.select(index);
		handleCPUSelection();	// force a selection event
	}

	/**
	 * Sets the initial comboCPU by cpu name.
	 * 
	 * @param cpuName	the name of the combo to be selected.
	 * @since 1.4.0
	 */
	public void selectInitialCPU(String cpuName) {
/* >>>>> MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */		
//		comboCPU.select(comboCPU.indexOf(cpuName));
		comboCPU.select(comboCPU.indexOf(cpuManager.getCpuDisplayName(cpuName)));
/* <<<<< MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */
		handleCPUSelection();	// force a selection event
	}
	
	/**
	 * Returns the CPU type selected in this page. 
	 * @deprecated		this method only returns the index of the selected combo
	 * 	use getSelectedCpuType() instead
	 */
	public int getSelectedCPU() {
		return comboCPU.getSelectionIndex();
	}
	
	/**
	 * Returns the CPU type selected in this page.
	 * CPU type is not the String displayed in the combo.
	 * It is the CPU core type associated with the CPU name
	 * @return	the CPU type as int
	 * @since 1.4.0
	 */
	public int getSelectedCpuType() {
		
		String cpuName = getSelectedCpuName();
		GeneralInfo generalInfo = (GeneralInfo) cpuManager.getCpuInfoMap().get(cpuName);
		return generalInfo.getCpuType();
	}

	/**
	 * Returns the CPU name
	 * @return	the CPU name selected in the combo
	 * @since 1.4.0
	 */
	public String getSelectedCpuName() {
/* >>>>> MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */
//		return comboCPU.getItem(comboCPU.getSelectionIndex()); 
		String cpuDisplayName = comboCPU.getItem(comboCPU.getSelectionIndex()); 
		return cpuManager.getCpuName(cpuDisplayName);
/* <<<<< MODIFIED by D.Fujimoto 2008/05/30 convert cpuName to cpuDisplayName */
	}
	
	/**
	 * Returns if the high precision check is checked in this page. 
	 * 
	 */
	public boolean isUseHighPrecision() {
		return checkHighPrecision.getSelection();
	}
	
	/**
	 * Sets the checkbox on/off
	 * @param use the on/off state of the check box
	 * @since 1.1.1
	 * @deprecated Not in use since 1.2.0
	 */
	public void selectUseHighPrecision(boolean use) {
		checkHighPrecision.setSelection(use);
	}
	
	
	/**
	 * Handles CPU selection events.
	 * This is independant from the selected index of the combo.
	 * 
	 * @since 1.4.0	Completely modified.
	 */
	private void handleCPUSelection() {
		
		// fill the vector address
		String cpuName = getSelectedCpuName();
		GeneralInfo generalInfo = (GeneralInfo) cpuManager.getCpuInfoMap().get(cpuName);
		long address = generalInfo.getInitialTtbrValue();
		
		textVecAddr.setText(UIUtil.getAddressString(address));
		
		selectUseHighPrecision(generalInfo.isUseHighPrecisionLib());
		
		// label for the CPU
		labelCPU.setText(GeneralSettingDialogResources.getString("generalPage.cpuType.desc") + "" +
				GeneralInfo.parseCPUString(getSelectedCpuType()));

	}
	
/* >>>>> DELETED by D.Fujimoto 2007/03/15 CPU selection must not rely on selected index */
//	/**
//	 * Handles CPU selection events.
//	 */
//	private void handleCPUSelection() {
//		selectedCPU = comboCPU.getSelectionIndex();
//		
//		// fill the vector address
//		int cpuType = GeneralInfo.parseCPUInt(comboCPU.getItem(selectedCPU));
//		long address = 0L;
//		switch (cpuType) {
//			case GeneralInfo.CPU_STD:
//				address = store.getLong("vectorAddress.std");
//				break;
//			case GeneralInfo.CPU_PE:
//				address = store.getLong("vectorAddress.pe");
//				break;
//			case GeneralInfo.CPU_ADV:
//				address = store.getLong("vectorAddress.adv");
//				break;
//		}
//		textVecAddr.setText(UIUtil.getAddressString(address));
//		
//	}
/* <<<<< DELETED by D.Fujimoto 2007/03/15 CPU selection must not rely on selected index */
	
	/**
	 * Handles Use Vector check selection events.
	 * @since 1.2.0
	 */
	private void handleVecCheck() {
		
		boolean checked = checkUseVec.getSelection(); 
		
		comboVecFile.setEnabled(checked);

   		verifyInputs();

	}
	
	/**
	 * Input checks for this page.
	 * This method is also responsible for setting the 
	 * page complete status.
	 * 
	 * Checks:
	 * 	vector file name check.
	 *
	 * @since 1.2.0
	 */
	private void verifyInputs() {
		String errorMessage = null;
		
		
		if (checkUseVec.getSelection()) {
			String text = comboVecFile.getText();
			if (text.length() == 0) {
				errorMessage = NewWizardResources.getString("NewWizard.msg.vecBlank");
			} else if (!UIUtil.isAlphNumString(text)) {
				errorMessage = NewWizardResources.getString("NewWizard.msg.vecInvalidChar");
			} else if (UIUtil.startsWithDecimalNum(text)) {
				errorMessage = NewWizardResources.getString("NewWizard.msg.vecNumStart");
			}
			
		}
		
		// vector address checks
		String vecAddr = textVecAddr.getText();
		long lVecAddr = 0;
		if (vecAddr.length() != 0) {
			
			if (!UIUtil.isHexString(vecAddr)) {
				errorMessage = ResourceProvider.getResourceString("msg.not.hex");	 //$NON-NLS-1$
			} else {
				lVecAddr = Long.parseLong(vecAddr, 16);
/* >>>>> MODIFIED by D.Fujimoto 2007/03/15 CPU type is not the selected index */
//				String msg = UIUtil.checkVecAddrBits(new AddrBitChecker(GeneralInfo.parseCPUInt(comboCPU.getItem(selectedCPU))), lVecAddr);
				String msg = UIUtil.checkVecAddrBits(new AddrBitChecker(getSelectedCpuType()), lVecAddr);
/* <<<<< MODIFIED by D.Fujimoto 2007/03/15 CPU type is not the selected index */
				if (msg != null) {
					// do not overwrite because an error may already exist  
					errorMessage = msg;	
				}
				 
			}
		} else {
			errorMessage = ResourceProvider.getResourceString("msg.vec.addr.blank");	 //$NON-NLS-1$
		}
		
		setErrorMessage(errorMessage);
		setPageComplete(errorMessage == null);
	}
	
	/**
	 * Returns a GeneralInfo based on the settings in this wizard.
	 * This must be called when the page is completed.
	 * 
	 * @return	the info full with settings in the wizard 
	 * @since 1.4.0
	 */
	public GeneralInfo getGeneralInfo() {
		
		if (!isPageComplete()) {
			return null;
		}
		
		//get the selected CPU type and name 
		String cpuName = getSelectedCpuName();
		int cpuType = getSelectedCpuType();
		boolean use = isUseHighPrecision();

		// set the parameters for project creation
		GeneralInfo general = new GeneralInfo();
		general.setCpuType(cpuType);
		general.setCpuName(cpuName);
		general.setUseHighPrecisionLib(use);
		// memory model is left to default
		
		
		// configure the IPreferenceStore with the selected CPU
		try {
			cpuManager.initializeStoreByCpu(store, cpuName);
		} catch (IOException e) {
			// open an error dialog
			MessageDialog.openError(getShell(), 
					NewWizardResources.getString("NewWizard.msg.project.failed"), e.getMessage());
			return null;
		}
		
		return general;
	}
	
	/**
	 * Returns the vector section setting.
	 * This must be called when the page is completed.
	 *	
	 * @return the SectionData based on the inputs in this page. <br>
	 * 		Will return a SectionData regardless of the checkbox selection.
	 * @since 1.2.0
	 */
	public SectionData getVectorSectionData() {
		
		if (!isPageComplete()) {
			return null;
		}
		
		// get the specified file name
		String file = null;
		if (checkUseVec.getSelection()) {
			file = comboVecFile.getText();
		}
		
   		// create a vector section
		SectionData vector = SectionData.createVectorSection(Long.parseLong(textVecAddr.getText(), 16), file);
   	
		return vector;
	
	}

	
}
