/*
 * 쐬: 2005/10/26
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommand;


/**
 * Interface for monitor controllers. Subclasses should have
 * a default constuctor which will not throw exceptions.
 * 
 * @author D.Fujimoto
 *
 */
public interface IOSMonitorController {

	/**
	 * Returns the target OS name.
	 * @return	target OS name this controller works for. 
	 */
	public String getTargetOSName();
	
	/**
	 * Initializes the controller.
	 */
	public void init();

	/**
	 * Initializes the controller with the given session key
	 * @param sessionKey	a value
	 */
	public void init(long sessionKey);
	
	/**
	 * Checks whether init() has been done properly.
	 * @return	true if init() is done properly.
	 */
	public boolean isInitialized();

	/**
	 * Returns the input object which stores the data.
	 * @return	input object. This may be null.
	 */
	public Object getMonitorInput();
	
	/**
	 * Called when the controller is in invalidate state.
	 */
	public void invalidate();
	
	/**
	 * Handles a UI request
	 * @param tabIndex	index to check the counter for each tab
	 * @param command	the command to be executed
	 */
	public void handleRequest(int tabIndex, MonitorCommand command);
	
	/**
	 * Returns the session key which this controller is registered to. 
	 * 
	 * @return	the session key value
	 */
	public long getSessionKey();
}
