/*
 * 쐬: 2005/09/09
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;


/**
 * Concrete class which represents the info of the connected OS.<br>
 * This class has data concerning the OS. 
 * Also has the parser to fill the data herein.  
 * 
 * @author D.Fujimoto
 *
 */
public class RTOSInfo implements IOSInfo, IParser {

	// sizes
	private static final int nameLength = Integer.parseInt(
			OSMonitorPlugin.getResourceString("osinfo.nameLength"));	//$NON-NLS-1$
	private static final int versionLength = Integer.parseInt(
			OSMonitorPlugin.getResourceString("osinfo.versionLength"));	//$NON-NLS-1$
	
	/**
	 * OS name.
	 */
	private String name;
	
	/**
	 * OS version.
	 */
	private String version;


	/**
	 * Constructor.
	 *
	 */
	protected RTOSInfo() {};
	
	public RTOSInfo(char[] data) {
		parseData(0, data);
	}
	
	
	/**
	 * Returns the name of the OS.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the version of the OS.
	 */
	public String getVersion() {
		return version;
	}
	
	/**
	 * Sets this instances fields from the given array.
	 * 
	 * The operations in this method should be ordered properly.
	 * @param parseType	the parse type switch(not used)
	 * @param readData		the data array
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = null;
		 
		
		// parse the name
		data = new char[nameLength];
		buf.get(data);
		name = new String(data).trim();
		
		// parse the version
		data = new char[versionLength];
		buf.get(data);
		version = new String(data).trim();
		
		data = null;
	}
	

}
