/*
 * 쐬: 2005/09/26
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.DataWithParser;

/**
 * Data model with a parser.<br>
 * 
 * @author D.Fujimoto
 *
 */
public abstract class AbstractDataModel extends DataWithParser  {


	/**
	 * The id of this data model.
	 */
	private int id;

	/**
	 * The control block address for this data.
	 */
	protected long controlBlockAddress;
	
	/**
	 * The initial block address for this data.
	 */
	protected long initBlockAddress;
	
	
	/**
	 * Empty Constructor.
	 */
	protected AbstractDataModel() {}
	
	/**
	 * Constructor with an id set.
	 * @param id	the id to set.
	 */
	protected AbstractDataModel(int id) {
		this.id = id;
	}

	/**
	 * Common skeleton of parsing data by the given type.
	 * Types are defined in ToppersObjectTable.
	 * 
	 * @param parseType	the parse type mode used as a switch
	 * @param readData		the data to be parsed
	 */
	public abstract void parseData(int parseType, char[] readData);
	
	/**
	 * Get access to the control block address
	 * @return	address
	 */
	public long getControlBlockAddress() {
		return controlBlockAddress;
	}

	/**
	 * Set access to the control block address
	 */

	public void setControlBlockAddress(long controlBlockAddress) {
		this.controlBlockAddress = controlBlockAddress;
	}

	/**
	 * Access to the ID
	 * @return	the ID of this data
	 */
	public int getId() {
		return id;
	}
}
