/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for event flag info.
 * 
 * @author D.Fujimoto
 *
 */
public class EventFlagData extends AbstractDataModel implements IQueue {


	/**
	 * Special type for parseData().
	 */
	public static final int TYPE_WAIT_INFO = 100;
	
	
	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;
	
	/**
	 * The flag pattern of this event.
	 */
	private long flagPattern;
	
	/**
	 * Attribute for this event flag.
	 * 
	 */
	private int eventFlagAttribute;
	
	/**
	 * Wait pattern if this task is in event flag queue.
	 */
	private long waitPattern;
	
	/**
	 * Wait mode if this task is in event flag queue.
	 */
	private int waitMode;
	
	/**
	 * Indices for wait queue.
	 */
	private int[] waitQueue;
	
	/**
	 * Constructor. 
	 * @param id
	 */
	public EventFlagData(int id) {
		super(id);
	}
	
	public int getEventFlagAttribute() {
		return eventFlagAttribute;
	}
	public long getFlagPattern() {
		return flagPattern;
	}
	public long getWaitPattern() {
		return waitPattern;
	}
	public int getWaitMode() {
		return waitMode;
	}
		
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {

		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// event flag control block
			case ToppersObjectTable.TYPE_EVENT_CTRL :
				
				// QUEUE next
				buf.get(data);
				nextQueue = DataParser.parse(data);
				
				// QUEUE previous
				buf.get(data);
				previousQueue = DataParser.parse(data);
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
			
				// flag pattern
				buf.get(data);
				flagPattern = DataParser.parse(data);
				
				break;
			
			// event flag init block
			case ToppersObjectTable.TYPE_EVENT_INIT :
				
				// attribute
				buf.get(data);
				eventFlagAttribute = (int) DataParser.parse(data);
				
				// initial flag pattern	(ignore)
				buf.get(data);
				
				
				break;
				
				// event flag wait info
			case TYPE_WAIT_INFO :
				
				// winfo	(ignore)
				buf.get(data);
				
				// wobjcb	(ignore)
				buf.get(data);
				
				// wait pattern
				buf.get(data);
				waitPattern = DataParser.parse(data);
				
				// wait mode
				buf.get(data);
				waitMode = (int) DataParser.parse(data);
				
				break;
		
		}
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue(int)
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue(int)
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		waitQueue = indices;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return waitQueue;
	}
}
