/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for fixed memory pool info.
 * 
 * @author D.Fujimoto
 *
 */
public class FixedMemoryPoolData extends AbstractDataModel implements IQueue {


	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;
	
	/**
	 * Address of unused area.
	 */
	private long unusedAreaTopAddress;	
	
	/**
	 * Address of released area.
	 */
	private long releasedBlockTopAddress;
	
	/**
	 * The attribute for this memory pool.
	 */
	private int memoryPoolAttribute;
	
	/**
	 * The size of memory block in bytes.
	 */
	private long memoryBlockSize;
	
	/**
	 * Top address of pool area. 
	 */
	private long memoryPoolTopAddress;
	
	/**
	 * Tail address of pool area.
	 */
	private long memoryPoolTailAddress;
	
	/**
	 * Indices for wait queue.
	 */
	private int[] waitQueue;
	
	/**
	 * Constructor. 
	 * @param id
	 */
	public FixedMemoryPoolData(int id) {
		super(id);
	}
	
	public long getMemoryBlockSize() {
		return memoryBlockSize;
	}
	public int getMemoryPoolAttribute() {
		return memoryPoolAttribute;
	}
	public long getMemoryPoolTailAddress() {
		return memoryPoolTailAddress;
	}
	public long getMemoryPoolTopAddress() {
		return memoryPoolTopAddress;
	}
	public long getReleasedBlockTopAddress() {
		return releasedBlockTopAddress;
	}
	public long getUnusedAreaTopAddress() {
		return unusedAreaTopAddress;
	}
	
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// mem pool control block
			case ToppersObjectTable.TYPE_FIX_MEM_CTRL :
				
				// QUEUE next
				buf.get(data);
				nextQueue = DataParser.parse(data);
				
				// QUEUE previous
				buf.get(data);
				previousQueue = DataParser.parse(data);
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
				
				// unused area address
				buf.get(data);
				unusedAreaTopAddress = DataParser.parse(data);
				
				// released area address
				buf.get(data);
				releasedBlockTopAddress = DataParser.parse(data);
				
				
				break;
			
			// mem pool init block
			case ToppersObjectTable.TYPE_FIX_MEM_INIT :
				
				// attribute
				buf.get(data);
				memoryPoolAttribute = (int) DataParser.parse(data);
				
				// memory block size
				buf.get(data);
				memoryBlockSize = DataParser.parse(data);
				
				// top address
				buf.get(data);
				memoryPoolTopAddress = DataParser.parse(data);
				
				// tail address
				buf.get(data);
				memoryPoolTailAddress = DataParser.parse(data);
				
				break;
		
		}
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue(int)
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue(int)
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		waitQueue = indices;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return waitQueue;
	}
}
