/*
 * 쐬: 2005/10/05
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for interrupt handler info.
 * 
 * @author D.Fujimoto
 *
 */
public class InterruptHandlerData extends AbstractDataModel implements INamedProcess {

		
	/**
	 * The name of this handler.
	 */
	private String handlerName;
	
	/**
	 * The handler number.
	 */
	private int handlerNo;
	
	/**
	 * Address of initial Program counter.
	 */
	private long initialPcAddress;
	
	/**
	 * Attribute of this interrupt handler.
	 */
	private int handlerAttribute;
	
	
	/**
	 * Constructor. 
	 * @param id
	 */
	public InterruptHandlerData(int id) {
		super(id);
	}
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// init block
			case ToppersObjectTable.TYPE_INTERRUPT_INIT :
				
				// handler number
				buf.get(data);
				handlerNo = (int) DataParser.parse(data);
				
				// attribute
				buf.get(data);
				handlerAttribute = (int) DataParser.parse(data);
				
				// pcAddress
				buf.get(data);
				initialPcAddress = DataParser.parse(data);
				
				
				break;
		
		}
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getAddress()
	 */
	public long getAddress() {
		return initialPcAddress;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#getProcessName()
	 */
	public String getProcessName() {
		return handlerName;
	}

	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.INamedProcess#setProcessName(java.lang.String)
	 */
	public void setProcessName(String name) {
		handlerName = name;
	}

}
