/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for mailbox info.
 * 
 * @author D.Fujimoto
 *
 */
public class MailBoxData extends AbstractDataModel implements IQueue {


	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;
	
	/**
	 * The address of this mailbox's head message.
	 */
	private long headMsgAddress;
	
	/**
	 * The attribute for this mailbox.
	 */
	private int mbxAttribute;
	
	/**
	 * Max message priority.
	 */
	private int maxPriority;
	
	/**
	 * Indices for wait queue.
	 */
	private int[] waitQueue;
	
	
	/**
	 * Constructor.
	 * @param id	id to set.
	 */
	public MailBoxData(int id) {
		super(id);
	}
	
	public int getMaxPriority() {
		return maxPriority;
	}
	public int getMbxAttribute() {
		return mbxAttribute;
	}
	public boolean msgExists() {
		return headMsgAddress != 0 ? true : false; 
	}
	
	
	/**
	 * Parses the given data by the aforeset parse type.
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
		
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			
			// mail control block
			case ToppersObjectTable.TYPE_MAILBOX_CTRL :
				
				// QUEUE next
				buf.get(data);
				nextQueue = DataParser.parse(data);
			
				// QUEUE previous
				buf.get(data);
				previousQueue = DataParser.parse(data);			
				
				// init block address
				buf.get(data);
				initBlockAddress = DataParser.parse(data);
				
				// head address
				buf.get(data);
				headMsgAddress = DataParser.parse(data);
				
				// last address	(ignore)
				buf.get(data);
				
				break;
			
			// mail init block
			case ToppersObjectTable.TYPE_MAILBOX_INIT :
				
				// attribute
				buf.get(data);
				mbxAttribute = (int) DataParser.parse(data);
				
				// priority
				buf.get(data);
				maxPriority = (int) DataParser.parse(data);
				
				break;
		
		
		}
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue(int)
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue(int)
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		waitQueue = indices;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return waitQueue;
	}
}
