/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;


/**
 * Data model for cyclic handler info.
 * 
 * @author D.Fujimoto
 *
 */
public class ReadyQueueData extends AbstractDataModel implements IQueue {


	/**
	 * Address for previous queue.
	 */
	private long previousQueue;
	
	/**
	 * Address for next queue.
	 */
	private long nextQueue;
	
	/**
	 * This ready queue's priority.
	 */
	private int priority;
	
	/**
	 * Indices for ready task queue.
	 */
	private int[] readyQueue;

	
	/**
	 * @param id
	 */
	public ReadyQueueData(int id) {
		super(id);
	}
	
	public int getPriority() {
		return priority;
	}
	/**
	 * Sets the priority. 
	 * 
	 * @param priority	Begins with <= 1
	 */
	public void setPriority(int priority) {
		this.priority = priority;
	}
	
	
	/**
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
	
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
	
		switch (parseType) {
			
			// ready queue
			case ToppersObjectTable.TYPE_READY_QUEUE :
			
				// next
				buf.get(data);
				nextQueue = DataParser.parse(data);
				
				// previous
				buf.get(data);
				previousQueue = DataParser.parse(data);
				
				break;
			
			
		}
	
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getNextQueue()
	 */
	public long getNextQueue(int type) {
		return nextQueue;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getPrevQueue()
	 */
	public long getPrevQueue(int type) {
		return previousQueue;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#setQueueIndices(int[], int)
	 */
	public void setQueueIndices(int[] indices, int type) {
		readyQueue = indices;
	}
	/* ( Javadoc)
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.IQueue#getQueueIndices(int)
	 */
	public int[] getQueueIndices(int type) {
		return readyQueue;
	}

	
}
