/*
 * 쐬: 2005/06/14
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data;

import java.nio.CharBuffer;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.AbstractDataModel;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersObjectTable;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.DataParser;



/**
 * Data model for task management info.
 * 
 * @author D.Fujimoto
 *
 */
public class TaskManagementData extends AbstractDataModel {
	

	/**
	 * Interrupt nest counter.<br>
	 * 
	 * If 0, indicates task context status.
	 * If 1 or over, indicates non-task context status.
	 */
	private int interruptNestCount;
	
	/**
	 * Indicates if task dispatching is allowed.
	 * 
	 */
	private boolean dispatchAllowed;
	
	/**
	 * Indicates the address to the info on the 
	 * current running task.<br>
	 * 
	 * If 0, no running task available.
	 */
	private long runningTaskAddress;
	
	/**
	 * Indicates the address to the info on the 
	 * task in ready state.<br>
	 * 
	 * If 0, no ready task available.
	 */
	private long readyTaskAddress;
	
	/**
	 * The system time of the OS.<br>
	 * Elapsed time from the boot of OS.
	 */
	private long systemTime;
	
	/**
	 * Indicates the running task's index of the TaskData array. 
	 * If -1, no task is running.
	 * If -2, index is invalid.
	 */
	private int runTaskIndex;
	
	/**
	 * Indicates the ready task's index of the TaskData array.
	 * If -1, no task is ready.
	 * If -2, index is invalid.
	 */
	private int readyTaskIndex;
	
	/**
	 * The Lockecd status.
	 * This will be set externally(not by parseData()).
	 */
	private boolean isLocked;
	
	/**
	 * Constructor.
	 */
	public TaskManagementData() {
	}
	
	public boolean isDispatchAllowed() {
		return dispatchAllowed;
	}
	public long getReadyTaskAddress() {
		return readyTaskAddress;
	}
	public long getRunningTaskAddress() {
		return runningTaskAddress;
	}
	public int getInterruptNestCount() {
		return interruptNestCount;
	}
	public long getSystemTime() {
		return systemTime;
	}
	/**
	 * Get the index of the TaskData which would run next.
	 * If no task will run next, the return value should be -1.
	 * @return
	 */
	public int getReadyTaskIndex() {
		return readyTaskIndex;
	}
	public void setReadyTaskIndex(int readyTaskIndex) {
		this.readyTaskIndex = readyTaskIndex;
	}
	/**
	 * Get the index of the TaskData which is currently running.
	 * If no task running, the return value should be -1.
	 * @return
	 */
	public int getRunTaskIndex() {
		return runTaskIndex;
	}
	public void setRunTaskIndex(int runTaskIndex) {
		this.runTaskIndex = runTaskIndex;
	}
	public boolean isLocked() {
		return isLocked;
	}
	public void setLocked(boolean isLocked) {
		this.isLocked = isLocked;
	}

	/**
	 * Parses the given data by the aforeset parse type.
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.IParser#parseData(int, char[])
	 */
	public void parseData(int parseType, char[] readData) {
	
		CharBuffer buf = CharBuffer.wrap(readData);
		
		// buffer to get from the CharBuffer
		char[] data = new char[FOUR_BYTES_LENGTH];
		
		switch (parseType) {
			// interrupt nest count
			case ToppersObjectTable.TYPE_INTERRUPT_NEST :
				buf.get(data);
				interruptNestCount = (int) DataParser.parse(data);
				break;

			// dispatch flag	
			case ToppersObjectTable.TYPE_DISPATCH_FLAG : 
				buf.get(data);
				dispatchAllowed = (DataParser.parse(data) != 0) ? true : false;
				break;

			// running task address
			case ToppersObjectTable.TYPE_RUN_TASK_ADDR :
				buf.get(data);
				runningTaskAddress = DataParser.parse(data);
				break;
				
			// ready task address
			case ToppersObjectTable.TYPE_RDY_TASK_ADDR :
				buf.get(data);
				readyTaskAddress = DataParser.parse(data);
				break;
				
			// system time
			case ToppersObjectTable.TYPE_SYSTIME :
				buf.get(data);
				systemTime = DataParser.parse(data); 
				break;
		}
		
	}
	
}