/*
 * 쐬: 2005/09/21
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util;

/**
 * Utility class which converts arrayed byte data to
 * a numeric data.
 *  
 * @author D.Fujimoto
 *
 */
public class DataParser {

	
	/**
	 * Flag if the data should be flipped.<br>
	 * If the data is stored by Little Endian, set to true.
	 * If the data is stored by Big Endian, set to false.
	 */
	public static boolean flip = true;
	
	
	/**
	 * Parses the data stored in the array to a long.
	 * If the filp flag is true, flips the data stored in the array.
	 * If the flip flag is false, does not flip the data.<br>
	 * 
	 * For example, if the data is {0x01, 0x00}, 
	 * the results are:<br>
	 * flip == true		0x00000001.<br>
	 * flip == false	0x00000100.<br>
	 * 
	 * @param data		the data to be parsed
	 * @return	parsed data in long format
	 */
	public static long parse(char[] data) {
		
		if (flip) {
			return charToLongFlip(data);
		} else {
			return charToLong(data);
		}
		
	}
	
	
	/**
	 * Returns char[] data into long.
	 * Used at big endian.
	 */
	private static long charToLong(char[] data) {
		
		long ret = 0;
		
		for (int i = data.length - 1; i >= 0; i--) {
			// cast char data to long and left-shift
			ret |= ((long) data[i]) << (8 * (data.length - 1 - i));
		}
		
		return ret;
		
	}
	
	
	/**
	 * Returns char[] data into long.
	 * Used at little endian.(x86 compatible)
	 */
	private static long charToLongFlip(char[] data) {

		long ret = 0;

		for (int i = data.length - 1; i >= 0; i--) {
			// cast char data to long and left-shift
			ret |= ((long) data[i]) << (8 * i);
		}
		
		return ret;

	}

}
