/*
 * 쐬: 2005/10/24
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util;

/**
 * Converts a numeric data to a Hex String 
 * in a specified format for displaying in the UI.
 * 
 * @author D.Fujimoto
 *
 */
public class HexFormatter {

	/**
	 * The returned String length will be filled up zeros to this length. (without prefix)
	 */
	public static final int DEFAULT_CHARS = 8;
	
	/**
	 * The returned String will have the prefix specified here.
	 */
	public static final String DEFAULT_PREFIX = "0x";
	
	/**
	 * Formats the given parameter to a Hex string.
	 * @param number	the data 
	 * @return		hex String for display 
	 */
	public static String format(long number) {
		return format(number, DEFAULT_CHARS, false);
	}
	
	/**
	 * Formats the given parameter to a Hex string.
	 * @param number		the data to be converted
	 * @param chars		max chars
	 * @param upperCase	convert to upper case when true 
	 * @return		hex String for display
	 */
	public static String format(long number, int chars, boolean upperCase) {
	
		StringBuffer sb = new StringBuffer(Long.toHexString(number));
		
		for (int i = sb.length(); i < chars; i++) {
			sb.insert(0, "0");
		}
		
		sb.insert(0, DEFAULT_PREFIX);
		
		String ret = new String(sb); 
		if (upperCase) {
			ret = ret.toUpperCase();
		}
		
		return ret;
	
	}
	
}
