/*
 * 쐬: 2005/05/16
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSInfo;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.IOSMonitorController;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.OSInfoManager;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.ToppersMonitorController;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommand;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.command.MonitorCommandFactory;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content.IMonitorViewContent;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceColors;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;



/**
 * View for Toppers Task Monitor
 * 
 * @author D.Fujimoto
 *
 */
public class ToppersMonitorView extends ViewPart {

	// the view's target OS. this should be matched with the controller
	public static final String TARGET_OS_NAME = "TOPPERS/JSP"; //$NON-NLS-1$
	
	// labels for the view
	public static final String REFRESH_BTN = OSMonitorPlugin.getResourceString("taskview.refreshButton"); //$NON-NLS-1$
	public static final String CONNECT_LBL = OSMonitorPlugin.getResourceString("taskview.connectLabel"); //$NON-NLS-1$
	
	// error messages
	public static final String NO_OS_MSG = OSMonitorPlugin.getResourceString("taskview.msg.osNotConnected"); //$NON-NLS-1$
	public static final String CONN_FAILED = OSMonitorPlugin.getResourceString("taskview.msg.connectFailed"); //$NON-NLS-1$
	public static final String CONTROLLER_ERR = OSMonitorPlugin.getResourceString("taskview.msg.controllerError"); //$NON-NLS-1$
	public static final String RUNNING_ERR = OSMonitorPlugin.getResourceString("taskview.msg.debuggerRunning"); //$NON-NLS-1$
	
	public static final String RUNNING_DLG_TITLE = OSMonitorPlugin.getResourceString("taskview.dlg.title.running"); //$NON-NLS-1$
	
	
	// The refresh button
	private Button refreshButton;
	// Label for the OS name
	private Label labelOs;
	// display area composite
	private Composite displayArea;	
	// The layout for the TabFolder display area.
	private StackLayout stackLayout;
	
	/**
	 * Label showing error message
	 */ 
	private Label errorLabel;

	/**
	 * TabFolder shown in this view
	 */
	private TabFolder folder;

	/**
	 * Viewers shown in this view.
	 * One viewer for each Tab.
	 */
	private TableViewer[] viewers;

	/**
	 * The OS information.
	 */
	private IOSInfo osInfo;
	
	/**
	 * Previously selected tab index.
	 */
	private int previousIndex;

	/**
	 * Create the UI.
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
	
		// the parent composite
		Composite comp = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		comp.setLayout(layout);
	
		// label
		Label label = new Label(comp, SWT.NONE);
		label.setText(CONNECT_LBL);
		
		// label for OS info
		labelOs = new Label(comp, SWT.BORDER);
		GridData gdOsName = new GridData();
		gdOsName.grabExcessHorizontalSpace = true;
		gdOsName.horizontalAlignment = GridData.FILL;
		labelOs.setLayoutData(gdOsName);
		
		// refresh button
		refreshButton = new Button(comp, SWT.PUSH);
		refreshButton.setText(REFRESH_BTN);
		GridData gdButton = new GridData();
		gdButton.horizontalAlignment = GridData.END;
		gdButton.verticalAlignment = GridData.BEGINNING;
		refreshButton.setLayoutData(gdButton);
		refreshButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleRefresh(folder.getSelectionIndex(), true);
			}
		});

		// create the display area
		displayArea = new Composite(comp, SWT.NONE);
		GridData gdDComp = new GridData(GridData.FILL_BOTH);
		gdDComp.horizontalSpan = layout.numColumns;
		displayArea.setLayoutData(gdDComp);
		stackLayout = new StackLayout();
		displayArea.setLayout(stackLayout);

		// create the error label for the display area
		errorLabel = new Label(displayArea, SWT.NONE);
		errorLabel.setForeground(JFaceColors.getErrorText(parent.getDisplay()));
		
		stackLayout.topControl = errorLabel;
		
		// create the TabFolder for the display area
		folder = new TabFolder(displayArea, SWT.TOP);

		// create the viewers and attach to TabFolder
		IMonitorViewContent[] contents  = ToppersMonitorViewerFactory.createToppersMonitorContents(folder);
		viewers = new TableViewer[contents.length];
		for (int i = 0; i < contents.length; i++) {
			TabItem tab = new TabItem(folder, SWT.NONE);
			tab.setText(contents[i].getTableName());

			viewers[i] = contents[i].getTableViewer();
			tab.setControl(viewers[i].getControl());
		}
		
		// TabFolder's Listeners
		folder.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// refresh the tab if the selection has actually changed 
				if (folder.getSelectionIndex() != previousIndex) {
					handleRefresh(folder.getSelectionIndex(), false);
				}
			}
		});
		folder.addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				// prevent the SelectionListener to perform refresh twice
				previousIndex = folder.getSelectionIndex();
				handleRefresh(folder.getSelectionIndex(), false);
			}
		});
		
		// initial refresh of the TabFolder contents
		handleRefresh(0, false);
//		if (OSMonitorPlugin.getDefault().isConnectionOk()) {
//			handleRefresh(0, false);
//		}

	}
	
	
	/**
	 * Check if the OS info has been retreived and
	 * get the info.<br>
	 * Also checks if the OS name matches this view.
	 */
	private boolean isValidOS() {
		
		boolean osOK = false;
		
		// check if OS info is available
		OSInfoManager osMan = OSMonitorPlugin.getDefault().getOSManager();
		if (osMan.isOsRead()) {

			// get the name and version 			
			osInfo = osMan.getOSInfo();
			
			// check the OS name ()
			if (osInfo.getName().equals(TARGET_OS_NAME)) {
				osOK = true;	
			}
			
		} else {
			osOK = false;
		}	
	
		return osOK;
	}

	/**
	 * Checks if the given parameter is a valid controller.
	 * @param controller
	 * @return	true if the controller is valid, false otherwise
	 */
	private boolean isValidController(IOSMonitorController controller) {
		
		if (controller != null && controller.isInitialized()) {
			if (controller.getTargetOSName().equals(TARGET_OS_NAME)) {
				return true;
			}
		}
		
		return false;
	}
	
			
	/**
	 * Updates the views contents. Mainly invoked when the refresh button 
	 * is pressed.<br>
	 * 
	 * @param  index		the tab index (this should not be -1)
	 * @param	forceInit 	when set to true, forces initialization of controller
	 */
	private void handleRefresh(int index, boolean forceInit) {
	
		// get the busy flag
		boolean isBusy = OSMonitorPlugin.getDefault().isConnectionBusy();
		
		
		// create the controller if necessary
		OSMonitorPlugin.getDefault().prepareController(forceInit);
		
		if (isValidOS()) {

			// get the controller
			IOSMonitorController cont = OSMonitorPlugin.getDefault().getMonitorController();
			
			if (isValidController(cont)) {
				
				// set the OS name:version
				labelOs.setText(osInfo.getName() + ":" + osInfo.getVersion());
				
				// get the data only if the debugger is not running
				if (!isBusy) {

					// get the currently selected tab index
					index = index == -1 ? 0 : index;
					
					// create the command and pass to the controller
					MonitorCommand command = MonitorCommandFactory.createCommand(index, (ToppersMonitorController) cont);
					cont.handleRequest(index, command);
					
					IToppersMonitorInput input = (IToppersMonitorInput) cont.getMonitorInput();
					
					// set the data to the associated Tab
					if (input != null) {
						viewers[index].setInput(input);
						
						
						// pack the column widths
						viewers[index].getTable().setVisible(false);				// prevent flickers
						TableColumn[] cols = viewers[index].getTable().getColumns();
						for (int j = 0; j < cols.length; j++) {
							cols[j].pack();
						}
						viewers[index].getTable().setVisible(true);
						
					}

				}
				
				// set the display area to show the TabFolder
				stackLayout.topControl = folder;
			
			} else {
				// controller is not available
				setErrorPane(NO_OS_MSG, CONTROLLER_ERR);
			}
			
		} else {
			
			// OS was not retreived
			setErrorPane(NO_OS_MSG, CONN_FAILED);
			
		}
		
		// show the display area
		displayArea.layout();
		
		// if the debugger is running, show a dialog
		if (isBusy) {
			MessageDialog.openWarning(getViewSite().getShell(), RUNNING_DLG_TITLE, RUNNING_ERR);
		}

	}
	
	/**
	 * Sets the stack layout to the errorLabel.
	 * Should call layout() after this method is finished.
	 * @param osLabelStr			the String to display in the OS label
	 * @param errorLabelMessage	the String to display in the error label
	 */
	private void setErrorPane(String osLabelStr, String errorLabelMessage) {
		
		// set the OS label
		labelOs.setText(osLabelStr);
		// set the error label
		errorLabel.setText(errorLabelMessage);
		
		// set the stack layout
		stackLayout.topControl = errorLabel;
	
	}
	
	/**
	 * Do things when this view gets disposed.
	 * Here, the controller gets invalidated.
	 */
	public void dispose() {
		super.dispose();
		
		// invalidate the controller(initialized at next open)
		OSMonitorPlugin.getDefault().invalidateController();
	}
	
	
	/* ( Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
	}

}
