/*
 * 쐬: 2005/06/15
 *
 */
package jp.co.epson.ide.eclipse.gnu33.osmonitor.ui.views.content;

import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.OSMonitorPlugin;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.IToppersMonitorInput;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.os.tps.input.data.TaskData;
import jp.co.epson.ide.eclipse.gnu33.osmonitor.core.util.HexFormatter;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;



/**
 * @author D.Fujimoto
 *  
 */
public class TaskMonitorContent extends AbstractMonitorViewContent {

	/**
	 * minimum priority to calculate the priorities of each task.
	 */
	private int minPriority;
	
	/**
	 * Address of current running task. 
	 */
	private long runTaskAddr;
	
	
	/**
	 * @param id	prefix to the properties 
	 * @param columnCount
	 */
	public TaskMonitorContent(String id, int columnCount) {
		super(id, columnCount);
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see jp.co.epson.ide.eclipse.gnu33.osmonitor.core.views.IMonitorViewContent#createTableViewer(org.eclipse.swt.widgets.Composite)
	 */
	public TableViewer createTableViewer(Composite parent) {

		Table table = new Table(parent, defaultTableStyle);
		table.setHeaderVisible(headerVisible);
		table.setLinesVisible(linesVisible);

		TableColumn[] cols = new TableColumn[getColumnCount()];
		for (int i = 0; i < cols.length; i++) {
			cols[i] = new TableColumn(table, defaultColumnStyle, i);
			cols[i].setText(columnLabels[i]);
			cols[i].setWidth(columnWidth[i]);
		}

		viewer = new TableViewer(table);

		viewer.setContentProvider(new TaskControlContentProvider());
		viewer.setLabelProvider(new TaskControlLabelProvider());

		return viewer;
	}

	
	public class TaskControlContentProvider implements IStructuredContentProvider {

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IToppersMonitorInput) {
				IToppersMonitorInput element = (IToppersMonitorInput) inputElement;
				
				// save the min priority
				TaskMonitorContent.this.minPriority = element.getMinPriority();

//				TaskManagementData mng = (TaskManagementData) element.getTaskManagementData();
				// save the control address of the running task
				TaskMonitorContent.this.runTaskAddr = element.getRunTaskAddress();
				
			
				return element.getTaskData();

			
			}
			return null;		
			
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

	}
	
	public class TaskControlLabelProvider extends AbstractToppersMonitorLabelProvider 
		implements ITableLabelProvider {
		
		
		String STATUS_DMT = OSMonitorPlugin.getResourceString("tps.taskStatus.dormant");
		String STATUS_RUN = OSMonitorPlugin.getResourceString("tps.taskStatus.run");
		String STATUS_RDY = OSMonitorPlugin.getResourceString("tps.taskStatus.ready");
		String STATUS_WAIT = OSMonitorPlugin.getResourceString("tps.taskStatus.wait");
		String STATUS_SUS = OSMonitorPlugin.getResourceString("tps.taskStatus.suspend");
		String STATUS_WAS = OSMonitorPlugin.getResourceString("tps.taskStatus.was");
		
		
		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* ( Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			String ret = "";
			TaskData data = (TaskData) element;
			
			boolean isRunningTask = data.getControlBlockAddress() == TaskMonitorContent.this.runTaskAddr;
			int reqCount = 0;
			
			switch (columnIndex) {
				case 0 :
					ret = Integer.toString(data.getId());
					break;
				case 1 :
					ret = createProcessNameString(data.getProcessName());
					break;
				case 2 :
					switch (data.getTaskStatus()) {
						case TaskData.STATUS_DMT :
							ret = STATUS_DMT;
							break;
						case TaskData.STATUS_RUNNABLE :
							if (isRunningTask) {
								ret = STATUS_RUN;
							} else {
								ret = STATUS_RDY;
							}
							break;
						case TaskData.STATUS_WAIT :
							ret = STATUS_WAIT;
							break;
						case TaskData.STATUS_SUS :
							ret = STATUS_SUS;
							break;
						case TaskData.STATUS_WAS :
							ret = STATUS_WAS;
							break;
						default :
							ret = UNKNOWN_VAL;
							break;
					}
					break;
				case 3 :
					if (isRunningTask || data.getTaskStatus() == TaskData.STATUS_DMT) {
						ret = NOTSPEC_VAL;
					} else {
						ret = HexFormatter.format(data.getPcAddress());	
					}				
					break;
				case 4 :
					ret = HexFormatter.format(data.getInitPcAddress());
					break;
				case 5 :
					if (isRunningTask || data.getTaskStatus() == TaskData.STATUS_DMT) {
						ret = NOTSPEC_VAL;
					} else {
						ret = HexFormatter.format(data.getSpAddress());
					}
					break;
				case 6 :
					ret = HexFormatter.format(data.getInitSpAddress());
					break;
				case 7 :
					ret = Integer.toString(data.getPriority(TaskMonitorContent.this.minPriority));
					break;
				case 8 :
					ret = Integer.toString(data.getInitPriority(TaskMonitorContent.this.minPriority));
					break;
				case 9 :
					reqCount = data.getStartReq();
					if (reqCount > 0) {
						ret = REQ_VAL;
					} else {
						ret = NONE_VAL;
					}
					break;
				case 10 :
					reqCount = data.getWupReq();
					if (reqCount > 0) {
						ret = REQ_VAL;
					} else {
						ret = NONE_VAL;
					}
					break;
				case 11 :
					reqCount = data.getSuspendReq();
					if (reqCount > 0) {
						ret = REQ_VAL;
					} else {
						ret = NONE_VAL;
					}
					break;
				default :
					break;
					
			}

			return ret;
		
		}
		
	}

}