/***********************************************************

 Revision :
		2002/01/21	C.Nakayama Add Cnrl-C function.(GNU33cygwin)
		2002/01/22	C.Nakayama Add cycle display function.(GNU33cygwin)
		2002/01/24	C.Nakayama Advanced instruction bug fix.(GNU33cygwin)
		2005/03/29	T.Tazaki   Create for insight gdb.

**********************************************************/


/* Generic remote debugging interface for simulators.
   Copyright 1993, 1994, 1996, 1997, 2000 Free Software Foundation, Inc.
   Contributed by Cygnus Support.
   Steve Chamberlain (sac@cygnus.com).

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.	*/


#include "defs.h"
#include "inferior.h"
#include "gdb_wait.h"
#include "value.h"
#include "gdb_string.h"
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <setjmp.h>
#include <errno.h>
#include "terminal.h"
#include "target.h"
#include "gdbcore.h"
#include "callback.h"
#include "c33common.h"
#include "remote-sim.h"
#include "remote-utils.h"
#include "command.h"

#include <tcl.h>
#include <stdio.h>
#include "sysdep.h"
#include "opcode/c33.h" 
#include "dis-asm.h"

#include "Sim33.h"
#include "def_api.h"

//#define SOFT_BREAK_INST 0x0400		/* del T.Tazaki 2005/05/06 */

/**********************************************/
/* Extern functions */
/**********************************************/
extern void _initialize_remote_sim PARAMS ((void));
extern int (*ui_loop_hook) PARAMS ((int signo));
extern struct c33_target_info c33TargetInfo; /* target information */
/* >>>>> change T.Tazaki 2003/02/27 */
/*
extern void sim_map_set( struct c33_target_info org_c33TargetInfo );
extern void sim_map_set2( struct c33_target_info org_c33TargetInfo );
extern void sim_map_set3( struct c33_target_info org_c33TargetInfo ); // sim-memopt.c //
extern void sim_map_set4( struct c33_target_info org_c33TargetInfo ); // sim-hload.c //
*/
extern void sim_map_set( struct c33_target_info *org_c33TargetInfo );
extern void sim_map_set2( struct c33_target_info *org_c33TargetInfo );
extern void sim_map_set3( struct c33_target_info *org_c33TargetInfo ); /* sim-memopt.c */
extern void sim_map_set4( struct c33_target_info *org_c33TargetInfo ); /* sim-hload.c */
/* <<<<< change T.Tazaki 2003/02/27 */
extern void sim_map_set5( struct c33_target_info *org_c33TargetInfo ); /* sim-load.c */ /* add T.Tazaki 2003/07/17 */
extern int g_resume_flag;				/* add 2003/06/30 T.Tazaki */
extern int g_iWatchBreakpointDisableFlag;	/* add 2003/08/06 T.Tazaki */
extern int g_temp_brk;					/* add 2003/03/31 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern unsigned long g_ulTemp_brk_addr;  /* add 2003/10/14 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern int g_SoftPcBreakTable[ 510 ];	/* add 2003/04/01 T.Tazaki : Temprary Soft PC break --> Hard PC break */
extern int g_SoftPcBreakTable_Index;	/* add 2003/04/01 T.Tazaki : Temprary Soft PC break --> Hard PC break */

extern int	g_iSamePCMode;	/*   add T.Tazaki 2003/11/07 */
extern int	c33_GetBreakNumber( int iType, unsigned long ulCurrentPC ); /* add 2004/01/28 T.Tazaki */

/**********************************************/
/* Prototypes Static */
/**********************************************/

static void dump_mem PARAMS ((char *buf, int len));
static void init_callbacks PARAMS ((void));
static void end_callbacks PARAMS ((void));
static int gdb_os_write_stdout PARAMS ((host_callback *, const char *, int));
static void gdb_os_flush_stdout PARAMS ((host_callback *));
static int gdb_os_write_stderr PARAMS ((host_callback *, const char *, int));
static void gdb_os_flush_stderr PARAMS ((host_callback *));
static int gdb_os_poll_quit PARAMS ((host_callback *));

/* printf_filtered is depreciated */
static void gdb_os_printf_filtered PARAMS ((host_callback *, const char *,...));
static void gdb_os_vprintf_filtered PARAMS ((host_callback *, const char *, va_list));
static void gdb_os_evprintf_filtered PARAMS ((host_callback *, const char *, va_list));
static void gdb_os_error PARAMS ((host_callback *, const char *,...));
unsigned long gdbsim_fetch_register PARAMS ((int regno));
static void gdbsim_store_register PARAMS ((int regno, unsigned long ulData));

static void gdbsim_store_register_cmd PARAMS ((int regno));
static void gdbsim_kill PARAMS ((void));
static void gdbsim_load PARAMS ((char *prog, int fromtty));
static void gdbsim_create_inferior PARAMS ((char *exec_file, char *args, char **env));
static void gdbsim_open PARAMS ((char *args, int from_tty));
static void gdbsim_close PARAMS ((int quitting));
static void gdbsim_detach PARAMS ((char *args, int from_tty));
static void gdbsim_resume (ptid_t ptid, int step, enum target_signal siggnal);
static ptid_t gdbsim_wait (ptid_t ptid, struct target_waitstatus *status);
static void gdbsim_prepare_to_store PARAMS ((void));
static void vfnAddRegDump(void);
static void vfnAddRegDump_Save_ext(char *szStatusBuf);		/* add T.Tazaki 2005/04/14 */
static void vfnFormatPSR (unsigned long ulPSR, char *pszTmpBuf);
static void vfnFormatPSR_adv (unsigned long ulPSR, char *pszTmpBuf);
static void vfnAddMemAccessInfo (unsigned long ulAddr, int iAccessType, int iDataSize, unsigned long ulData, int blNoData);
/* >>>>> change iruma m.takeishi '03.12.22 */
/*
static void c33_SimTraceGetSorceCode(unsigned long ulAddr, char* pOut );
*/
static void c33_SimTraceGetSorceCode(unsigned long ulAddr, char* pOut, int iSize );
/* <<<<< change iruma m.takeishi '03.12.20 */
static int fnGo( int iretd_flag );
static int fnStdio( void );

/* >>> del 2002.06.05 T.Tazaki C33 */
/*
static int gdbsim_xfer_inferior_memory PARAMS ((CORE_ADDR memaddr,
						unsigned long *myaddr, int len,
						int write,
						struct target_ops * target));
*/

/* add 2002.06.05 T.Tazaki C33 */
static int gdbsim_xfer_inferior_memory (CORE_ADDR memaddr, char *myaddr,	
					int len, int write,
					struct mem_attrib *attrib,
					struct target_ops *target);

static void gdbsim_files_info PARAMS ((struct target_ops * target));
static void gdbsim_mourn_inferior PARAMS ((void));
static void gdbsim_stop PARAMS ((void));
static void simulator_command PARAMS ((char *args, int from_tty));
static short gdb_InitSim(void);
static int	fnExeEachInst (unsigned short uwCode);
static int	fnExeEachInst_trace (unsigned short uwCode);	/* add 2003/05/06 T.Tazaki */
static void c33_PutsTraceString( char *szString );
static void c33_SimReset (int iResetMode );
static int	c33_SimDataFill(int iType, unsigned long ulStartAddr, unsigned long ulSize, unsigned long ulData);
static int	c33_SimMemoryMove(int iType, unsigned long ulStartAddr, unsigned long ulDstAddr, unsigned long ulSize);
static void c33_SimMemoryWrite(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr);
static void c33_SimMemoryRead(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr);

static void gdb_MemoryMove(int iParamNum, int iType, unsigned long ulStartAddr, unsigned long ulEndAddr, unsigned long ulData);
static void gdb_TraceMode(int iParamNum, char *cParam1, char *cParam2);
static void gdbsim_cntrl_c PARAMS((int signo));/* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */

/*	cycle display function. */
#define MAP_NUM 32
#define MEM_ACCESS_DEFAULT 7
/* #define gdb_AddCycle(x) (gulCycle += x) */
static void gdb_AddCycle( unsigned long cycle);
static unsigned char fnCheckAdvancedMode( void );
static int fnGetEndian( CORE_ADDR ulAddr);
static int fnCheckRomArea( CORE_ADDR ulAddr);
static int fnCheckExistAdvArea( void );
static unsigned long gdb_GetMemoryAccessCycle(unsigned long ulAddr, int iwrite, int iSize);
unsigned long gulCycle = 0;
static struct c33_interlock c33InterLockInfo;	/* 2002/08/01 T.Tazaki */
static void fnInterlock(int iInstMode, unsigned short uwRegno, unsigned short uwRegBase, int iCycle1, int iCycle2 );
/* >>>>> add iruma m.takeishi '03.09.02 */
static int fnGetExtType( unsigned short usCode );
static int fnCheckInstCombi( int iLoop, int iRepeat, int iExt );
/* <<<<< add iruma m.takeishi '03.09.02 */
/* >>>>> add iruma m.takeishi '04.03.25 */
static struct c33_map_info * c33_GetMapInfo( CORE_ADDR ulAddr);
/* <<<<< add iruma m.takeishi '04.03.25 */
static unsigned long g_iSaveSkipAddr = C33_INVALID_ADDR;	/* add T.Tazaki 2003/10/14 */


static int g_iExtEachInst = FALSE;
static int g_iResume = FALSE;

static int fnIlleAddr( void );
static int fnZeroDiv( void );

static int fnExtException(unsigned long ulExeptPC);			/* add T.Tazaki 2005/02/21 */
static int fnPush_ExtExceptPC (unsigned long ulExtExeptPC); /* add T.Tazaki 2005/02/21 */

/* 錾ړB T.Tazaki 2007/07/12 */
int 			fnPush (unsigned short uwRegNum);
int 			fnPop (unsigned short uwRegNum);

/* Naming convention:

   sim_* are the interface to the simulator (see remote-sim.h).
   gdbsim_* are stuff which is internal to gdb.  */

/* Forward data declarations */
extern struct target_ops gdbsim_ops;

static int program_loaded = 0;

static int g_iStop_button = 0;			/* add T.Tazaki 2008/05/27 */

static int giKeybreak = 0;			  /*  Cnrl-C function. */

struct BreakData stBreakData;		  /* watch point */

#if 0  /* del T.Tazaki 2003/12/02 >>> */
/* macclr execute timming */
static int iMacclrFlag = 0;
static unsigned long ulMacclrCurrentInst = 0;
/* >>>>> add iruma m.takeishi '03.04.28 */
#endif

/* frame update flag */
int giFrameClear = 0;				  /* if giFramteClear =1 and gdbtk_update is caleed, current	
										 and selected frame is cleared. */
/* <<<<< add iruma m.takeishi '03.04.28 */

/* >>>>> add iruma m.takeishi '04.03.16 */
#define RESERVELOOP_INSTCNT 	   5
static int	iRsvLoop_instCnt = 0;
static int	iIsLoop_int = 0;				/* when loop mode, 1 is set. */
/* <<<<< add iruma m.takeishi '04.03.16 */

int g_iSim_open = 0;			/* add T.Tazaki 2004/06/11 */

void	 vfnIncPC (long lByte);

void	 vfnInitExtension ();	/* add 2004/01/28 T.Tazaki */

/* >>>>> add iruma m.takeishi '04.03.25 */
static struct c33_CachedCode {
	unsigned long	   ulPc;
	unsigned short int usCode;
	int 			   iCached; 			   /* if code is cashed, 1 */
} stC33_CachedCode;
/* <<<<< add iruma m.takeishi '04.03.25 */

/**********************************************/
/* file CPU mode							  */
/**********************************************/
extern unsigned char g_ucAdvFlag;  /* =0x00:STD,=0x41:ADV add T.Tazaki 2003/11/05 */

/**********************************************/
/* Continue [Count],  ignore				  */
/**********************************************/
extern int g_iContinue_count;			/* add 2003/11/06 T.Tazaki */
extern int g_iIgnore_number;			/* add 2003/11/07 T.Tazaki */
extern int g_iIgnore_count;				/* add 2003/11/07 T.Tazaki */

/************************************************************/
/* c33 log command : target command illegal message skip flag */
/************************************************************/
int g_gdbsim_open = 0;

/* gdbsim_resume() 1:step = 0(cont) 	   add T.Tazaki 2003/08/08 */
int g_iContFlag;

/**********************************************/
/* PC Break  : original code save			  */ /* add T.Tazaki 2003/09/19 */
/**********************************************/
unsigned long  g_ulSoftBreak_Addr[C33_SOFT_PC_BREAK_MAX];
unsigned short g_usSoftBreak_code[C33_SOFT_PC_BREAK_MAX];
unsigned short g_HardBreak1_code;
unsigned short g_HardBreak2_code;

/**********************************************/
/* WATCH Break								   */ /* add T.Tazaki 2003/09/19 */
/**********************************************/
int g_iStepLoopBreak = 0;		/* infcmd.c : step_i() count loop break flag */

/**********************************************/
/* Trace									  */
/**********************************************/

#define STATUS_BUFFER_MAX		500 	/* trace static buffer max size */
#define STATUS_BUFFER_CNT_MAX	256 	/* trace static buffer max count */
#define STATUS_WORK_BUFFER_MAX	5000	/* add T.Tazaki 2005/03/14 : use szStatusBuf[] at c33_TraceDump()  */

static int g_iTraceDump   = TM_OFF; 		/* 1 = tm on,0 = tm off */
static int g_iTraceOutput = TRACE_WINDOW;	/* 0 = display output, 1 = file output */
static FILE *Trace_fp = NULL;			/* trace output file pointer */
static int g_iTraceFormat = 0;			/* trace format number (1-4) */
static unsigned long g_ulInstNum = 0;	/* Trace Instrubtion Number. */

static char g_szStatusBuf[STATUS_BUFFER_CNT_MAX][STATUS_BUFFER_MAX];	/* tarce register output mode : access address,r/w,data */
static int g_iStatusBufCnt = 0;
static char g_szStatusBufForExtExcept[STATUS_BUFFER_CNT_MAX][STATUS_BUFFER_MAX];	/* add T.Tazaki 2005/03/16 */
static int g_iExtExceptCnt = 0;														/* add T.Tazaki 2005/03/16 */
int g_iTraceMode = 0;				 /* 1=Trace mode ,c33_Disasm():c33-dis.c */ /* 2003/05/19 T.Tazaki */

static int g_iExtStartPC_Flag = 0;					/* add T.Tazaki 2005/03/29 */
static unsigned long g_ulExtStartPC = 0;			/* add T.Tazaki 2005/03/29 */

/* >>>>> add iruma m.takeishi '03.12.19 */
/* move a big array from local to static. */
static char 	   szSourceLine[ C33_TRACE_DATA_LINE_MAX + 1 ];
/* <<<<< add iruma m.takeishi '03.12.19 */

static void c33_TraceDump( unsigned long ulAddr, char *pszStatusBuf );			/* add T.Tazaki 2005/03/14 */
static void c33_TraceDump_Save_ext( unsigned long ulAddr, char *pszStatusBuf );	/* add T.Tazaki 2005/04/14 */
static void c33_TraceDump_Output_ext( void );									/* add T.Tazaki 2005/04/14 */

/**********************************************/
/* Simulated I/O							  */
/**********************************************/

#if 0	/* del T.Tazaki 2005/06/27 */
unsigned short g_sStdin_code;			/* stdin  orijinal code */
unsigned short g_sStdout_code;			/* stdout orijinal code */
#endif

extern int g_c33_std_in;   /* 1:already key input */
extern int g_c33_Index;
extern char g_c33_stdin_string[1000];

extern FILE *fpStdinfile;
extern FILE *fpStdoutfile;
extern int g_use_windows;

/**********************************************/
/* finish command							  */
/**********************************************/
extern void c33_GetBreakPoint( void );	/* add 2003/05/06 T.Tazaki */
extern void c33_GetBreakPoint2( void ); /* add 2003/08/06 T.Tazaki */
extern int g_SoftPcBreakTable[ 510 ];	/* add 2003/05/06 T.Tazaki */
extern int g_SoftPcBreakTable_Index;	/* add 2003/05/06 T.Tazaki */
/* add 2003/05/07 T.Tazaki for finish commnad Temprary break >>> */
extern int g_WatchNum;					/* add 2003/06/30 T.Tazaki */
extern char g_What[300];				/* add 2003/06/30 T.Tazaki */
extern int g_HardPcBreakTable[ 510 ];
extern int g_HardPcBreakTable_Index;
extern int g_WriteWatchBreakAddr;
extern int g_WriteWatchBreakAddrFlag;
extern int g_ReadWatchBreakAddr;
extern int g_ReadWatchBreakAddrFlag;
extern int g_WriteReadWatchBreakAddr;
extern int g_WriteReadWatchBreakAddrFlag;
/* add 2003/05/07 T.Tazaki for finish commnad Temprary break <<< */

extern void c33_delete_temp_breakpoint (unsigned long ulPC);	/* add 2003/07/08 T.Tazaki */

/**********************************************/
/* repeat instruction						  */
/**********************************************/
static unsigned int iRepeatFirst = 0;	/* 1 = first insn after repeat insn */

/* >>>>> add iruma m.takeishi '03.12.19 */
/***************************************
  *  reset 33 core	  *
 ***************************************/
int 			iExceptionType; /* exception type */
int 			nTrapAfterNMI;	/* counter of traps after NMI */
int 			blStartNMI; /* NO : no NMI, YES : NMI */

struct StartINTR stStartINTR;	/* start INTR ? */
struct DelaySlot stDelaySlot;	/* delay slot */
struct Extension stExtension;	/* extension */

/* <<<<< add iruma m.takeishi '03.12.19 */

/**********************************************/
/* Log file output control					  */
/**********************************************/
extern int g_Gdbtk_Update;				/* Do not log file output flag 1=ON */

/* Focus to Console window */
extern int g_iTargetOpen_end;			/* 1=Target command end (top.c): add T.Tazaki 2003/10/07 */

/**********************************************/
/* SIM Next command 						  */
/**********************************************/
extern int g_iSimNextCommand;			/* =1:SIM next command	add T.Tazaki 2004/08/02 */
extern CORE_ADDR step_range_end;   		/* infcmd.c NEXT Exclusive	 add T.Tazaki 2004/08/02 */

/**********************************************/
/* SIM set $pc=xxxxx						  */
/**********************************************/
extern int g_command_loop;				/* add 2004/08/09 T.Tazaki : log command ,1=command loop running */
extern int g_iSet_Command;				/* add T.Tazaki 2004/08/09 : set_command() in printcmd.c set $pc=xxxxx -> source window update flag */
extern int g_reg_update_now;			/* add T.Tazaki 2004/08/09 : 1=update now (c33_reg_update) */



extern int g_iDebugRun;					/* add T.Tazaki 2005/04/27 */

extern int g_iStdin_EOF;			/* =1:stdin file inout EOF */

/**********************************************/
/* gnu33ide.dll 							  */
/**********************************************/
#ifdef __CYGWIN__				/* add T.Tazaki 2005/09/08 */
#include <windows.h>

extern WORD ( __stdcall *dll33_ClearCommandUpdate) ( void );/* add T.Tazaki 2006/01/12 */
extern WORD ( __stdcall *dll33_GdbEnable) ( void );
extern WORD ( __stdcall *dll33_GdbDisable) ( void );
extern WORD ( __stdcall *dll33_GdbRun) ( void );		/* add T.Tazaki 2005/12/08 */
extern WORD ( __stdcall *dll33_GdbStop) ( void );		/* add T.Tazaki 2005/12/08 */
extern WORD ( __stdcall *dll33_addGdbGoCounter) ( void );
extern WORD ( __stdcall *dll33_checkCommand) ( int *iCommand, unsigned char *pucAddress, unsigned long *pulAddress,
								  unsigned long *pulSize );
extern WORD ( __stdcall *dll33_setMem) ( unsigned long ulSize, unsigned char *pucData );
extern WORD ( __stdcall *dll33_getMem) ( unsigned long ulSize, unsigned char *pucData, int iParent );
extern WORD ( __stdcall *dll33_setSymbol) ( unsigned char *pucSymbol, int iResult );
extern WORD ( __stdcall *dll33_setIL) ( unsigned char ucIL );

extern HINSTANCE hIdeDll;
extern int g_GdbEnable;
extern unsigned long c33_CnvSymbolToAddress (char *exp);

#endif

/**********************************************/
/* Add cycle.  */
/**********************************************/
static void
gdb_AddCycle( unsigned long ulCycle )
{
	
	gulCycle += ulCycle;
}
/*******************************************************************************/
/* We must keep track of whether the simulator has been opened or not because
   GDB can call a target's close routine twice, but sim_close doesn't allow
   this.  We also need to record the result of sim_open so we can pass it
   back to the other sim_foo routines.	*/
/*******************************************************************************/
static SIM_DESC gdbsim_desc = 0;

static void
dump_mem (buf, len)
	 char *buf;
	 int len;
{
  if (len <= 8)
	{
	  if (len == 8 || len == 4)
	{
	  long l[2];
	  memcpy (l, buf, len);
	  printf_filtered ("\t0x%lx", l[0]);
	  printf_filtered (len == 8 ? " 0x%x\n" : "\n", l[1]);
	}
	  else
	{
	  int i;
	  printf_filtered ("\t");
	  for (i = 0; i < len; i++)
		printf_filtered ("0x%x ", buf[i]);
	  printf_filtered ("\n");
	}
	}
}

static host_callback gdb_callback;
static int callbacks_initialized = 0;

/**********************************************/
/* Initialize gdb_callback.  */
/**********************************************/

static void
init_callbacks ()
{
  if (!callbacks_initialized)
	{
	  gdb_callback = default_callback;
	  gdb_callback.init (&gdb_callback);
	  gdb_callback.write_stdout = gdb_os_write_stdout;
	  gdb_callback.flush_stdout = gdb_os_flush_stdout;
	  gdb_callback.write_stderr = gdb_os_write_stderr;
	  gdb_callback.flush_stderr = gdb_os_flush_stderr;
	  gdb_callback.printf_filtered = gdb_os_printf_filtered;
	  gdb_callback.vprintf_filtered = gdb_os_vprintf_filtered;
	  gdb_callback.evprintf_filtered = gdb_os_evprintf_filtered;
	  gdb_callback.error = gdb_os_error;
	  gdb_callback.poll_quit = gdb_os_poll_quit;
	  gdb_callback.magic = HOST_CALLBACK_MAGIC;
	  callbacks_initialized = 1;
	}
}

/**********************************************/
/* Release callbacks (free resources used by them).  */
/**********************************************/

static void
end_callbacks ()
{
  if (callbacks_initialized)
	{
	  gdb_callback.shutdown (&gdb_callback);
	  callbacks_initialized = 0;
	}
}

/**********************************************/
/* GDB version of os_write_stdout callback.  */
/**********************************************/

static int
gdb_os_write_stdout (p, buf, len)
	 host_callback *p;
	 const char *buf;
	 int len;
{
  int i;
  char b[2];

  ui_file_write (gdb_stdtarg, buf, len);
  return len;
}

/**********************************************/
/* GDB version of os_flush_stdout callback.  */
/**********************************************/

static void
gdb_os_flush_stdout (p)
	 host_callback *p;
{
  gdb_flush (gdb_stdtarg);
}

/**********************************************/
/* GDB version of os_write_stderr callback.  */
/**********************************************/

static int
gdb_os_write_stderr (p, buf, len)
	 host_callback *p;
	 const char *buf;
	 int len;
{
  int i;
  char b[2];

  for (i = 0; i < len; i++)
	{
	  b[0] = buf[i];
	  b[1] = 0;
	  fputs_unfiltered (b, gdb_stdtarg);
	}
  return len;
}

/**********************************************/
/* GDB version of os_flush_stderr callback.  */
/**********************************************/

static void
gdb_os_flush_stderr (p)
	 host_callback *p;
{
  gdb_flush (gdb_stderr);
}

/**********************************************/
/* GDB version of printf_filtered callback.  */
/**********************************************/

static void
gdb_os_printf_filtered (host_callback * p, const char *format,...)
{
  va_list args;
  va_start (args, format);

  vfprintf_filtered (gdb_stdout, format, args);

  va_end (args);
}

/**********************************************/
/* GDB version of error vprintf_filtered.  */
/**********************************************/

static void
gdb_os_vprintf_filtered (host_callback * p, const char *format, va_list ap)
{
  vfprintf_filtered (gdb_stdout, format, ap);
}

/**********************************************/
/* GDB version of error evprintf_filtered.	*/
/**********************************************/

static void
gdb_os_evprintf_filtered (host_callback * p, const char *format, va_list ap)
{
  vfprintf_filtered (gdb_stderr, format, ap);
}

/**********************************************/
/* GDB version of error callback.  */
/**********************************************/

static void
gdb_os_error (host_callback * p, const char *format,...)
{
  if (error_hook)
	(*error_hook) ();
  else
	{
	  va_list args;
	  va_start (args, format);
	  verror (format, args);
	  va_end (args);
	}
}

#ifndef REGISTER_SIM_REGNO
#define REGISTER_SIM_REGNO(N) (N)
#endif


/* static */  /* del T.Tazaki 2005/09/16 */
unsigned long	ulReg[31];	/* 33 core register model */ /* 29 --> 31 change T.Tazaki 2003/11/28 */

/************************************************
 * Register read *
 ************************************************/

unsigned long
gdbsim_fetch_register (regno)
	 int regno;
{
//	extern unsigned long ulReg[];
	unsigned long ulPSR;


	if (regno == -1)
	{
		for (regno = 0; regno < NUM_REGS; regno++)
			gdbsim_fetch_register (regno);
	}
	else
	{
		supply_register(regno, (char *)&ulReg[regno]);
	}

	if (regno == SP) {
		/* add T.Tazaki 2004/01/28 >>> */
		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			ulPSR = ulReg[PSR];
			if( ( ulPSR & SV_FLAG ) == 0 ){ /* Super Visor Mode ? */
				ulReg[SP] = ulReg[SSP];
			}
			else
			{
				ulReg[SP] = ulReg[USP];
			}
		}
		else
		{
			/*@ set data word boundary */
			ulReg[SP] &= C33_SP_MASK;
		}
	}
	else if (regno == DP) {
		/*@ set data word boundary */
		ulReg[DP] &= C33_DP_MASK;
	}

	return (ulReg[regno]);
}

unsigned long ulData;
unsigned long ulTmpData;
unsigned long ulVectorAddr;   /* vector table address */

/************************************************
 * Register write *
 ************************************************/
static void
gdbsim_store_register (regno, ulData)
	 int regno;
	 unsigned long ulData;
{
//	extern unsigned long ulReg[];
	unsigned long ulPSR;
	unsigned long ulTmp1;
	unsigned long ulTmp2;

	ulPSR = ulReg[PSR];

	/*@ check if you write PC */
	if (regno == PC) {
		/*@ set data half word boundary */
		ulReg[PC] = (ulData & C33_PC_MASK);
		
		/* >>>>> add iruma m.takeishi '04.03.15 */
		iRsvLoop_instCnt = 0;
		/* <<<<< add iruma m.takeishi '04.03.15 */
		
	}
	/*@ check if you write SP */
	else if (regno == SP) {
		/*@ set data word boundary */
		ulReg[SP] = (ulData & C33_SP_MASK);
		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			if( ( ulPSR & SV_FLAG ) == 0 ){ /* Super Visor Mode ? */
				ulReg[SSP] = (ulData & C33_SP_MASK);
			}
			else
			{
				ulReg[USP] = (ulData & C33_SP_MASK);
			}
		}
	}
	else if( regno == SSP ){
		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			/* >>>>> delete iruma m.takeishi '04.03.29 */
			/*
			ulReg[SSP] = (ulData & C33_SP_MASK);
			*/
			/* <<<<< delete iruma m.takeishi '04.03.29 */
			if( ( ulPSR & SV_FLAG ) == 0 ){ /* Super Visor Mode ? */
				/* >>>>> add iruma m.takeishi '04.03.29 */
				ulReg[SSP] = (ulData & C33_SP_MASK);
				/* <<<<< add iruma m.takeishi '04.03.29 */
				ulReg[SP]  = (ulData & C33_SP_MASK); /* copy %ssp to %sp */
			}
		}
	}
	else if( regno == USP ){
		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			ulReg[USP] = (ulData & C33_SP_MASK);
			if( ( ulPSR & SV_FLAG ) != 0 ){ /* User Mode ? */
				ulReg[SP]  = (ulData & C33_SP_MASK); /* copy %usp to %sp */
			}
		}
	}
	/*@ check if you write PSR */
	else if (regno == PSR) {
		/*@ write data to PSR */
		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			ulPSR = ulReg[PSR];
			ulTmp1 = ulPSR	& SV_FLAG;
			ulTmp2 = ulData & SV_FLAG;
			/* PSR:SV change ? */
			if( ulTmp1 != ulTmp2 ){
				if( ( ulTmp2 & SV_FLAG ) == 0 ){ /* Super Visor Mode ? */
					ulReg[SP]  = ulReg[SSP]; /* copy %ssp to %sp */
				}
				else
				{
					ulReg[SP]  = ulReg[USP]; /* copy %usp to %sp */
				}
			}
			
			

			ulReg[PSR] = ulData & 0xff73ffdfUL;   /* mask */
		}
		else  if( fnCheckAdvancedMode() == C33_PRODUCT_PE )		/* add T.Tazaki 2005/03/14 */
		{
			/* PE */
			ulReg[PSR] = ulData & 0x00000f1fUL;   /* mask */
		}
		else
		{
			/* standard macro */
			ulReg[PSR] = ulData & 0x00000fdfUL;   /* mask */
		}
	}
	else if (regno == TTBR) {
	/*@ set data 1KB boundary */
		/* >>>>> add iruma m.takeishi '03.12.19 */
		/*
		ulReg[TTBR] = ulData & C33_TTBR_MASK;  // bit9-0 = 0 fix //
		*/
		if( ( ulPSR & SV_FLAG ) == 0 ){ /* Super Visor Mode ? */
			ulReg[TTBR] = ulData & C33_TTBR_MASK;  /* bit9-0 = 0 fix */
		}
		/* <<<<< add iruma m.takeishi '03.12.19 */
	}
	/*@ check if you write DP */
	else if (regno == DP) {
		/*@ set data word boundary */
		ulReg[DP] = (ulData & C33_DP_MASK);
	}
	else if (regno == LSA) {
		/*@ set data half word boundary */
		ulReg[LSA] = (ulData & C33_LSA_MASK);
	}
	else if (regno == LEA) {
		/*@ set data half word boundary */
		ulReg[LEA] = (ulData & C33_LEA_MASK);
	}
	else {
	/*@ write data to register */
		ulReg[regno] = ulData;
	}

	/* update registers information  */
	if( g_iResume != TRUE )
	{
		flush_cached_frames ();
//		  sim_Gdbtk_Update ();	/* when Reg Window change -> hangu up ,del 2002/09/05 T.Tazaki */
								/* but, "set $r0 = 0x55" -> not change Reg window */
		registers_changed ();	/* add T.Tazaki 2003/09/10 cui bug fix */
		
	}

}


/**********************************************/
/* gdbsim_store_register_cmd */
/**********************************************/
static void
gdbsim_store_register_cmd (regno)
	 int regno;
{
	extern Tcl_Interp *gdbtk_interp;	/*add T.Tazaki 2004/06/03 */
	unsigned long ulVal;
	/* >>>>> add iruma m.takeishi '04.03.16 */
	unsigned long ulBefore;
	/* <<<<< add iruma m.takeishi '04.03.16 */
	/* add T.Tazaki 2004/08/09 >>> */
	int iConsoleCommand = 0;
	char szbuf[100];
	char *pszbuf;

	pszbuf = szbuf;
	/* add T.Tazaki 2004/08/09 <<< */

	/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 >>> */
	if( regno == PC && g_iSet_Command == 1)		/* set $pc=xxxx ? */	
	{
		c33_Cmd_Reset2( C33_ICD_HOT_RESET );	/* hot reset , but no message */
	}
	/* set $pc=xxxx + next bug fix  add T.Tazaki 2008/06/17 <<< */


	ulVal = read_register(regno);
	
	/* >>>>> add iruma m.takeishi '04.03.16 */
	if ( regno == PSR ) {
		ulBefore = gdbsim_fetch_register( PSR );
		if ( (ulBefore & LM_FLAG) != 0 &&	
			 (ulVal & LM_FLAG) == 0 ) {
			 iIsLoop_int = 0;
		}
	}
	else if ( regno == PC ) {
		vfnInitExtension();
		
		/* >>>>> add irumam.takeishi '04.03.25 */
		stC33_CachedCode.iCached = 0;
		/* <<<<< add irumam.takeishi '04.03.25 */
	}
	/* <<<<< add iruma m.takeishi '04.03.16 */
	
	gdbsim_store_register(regno,ulVal);

	/* add update reg window	T.Tazaki 2004/06/04 >>> */
	if( g_use_windows == 1 )  /* GUI Mode ? */
	{

		if( g_command_loop == 0 )	/* if Not command file ?  add T.Tazaki 2005/09/15 */
		{
			g_reg_update_now = 1;	/* add T.Tazaki 2004/06/04 */
//			Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */
		}

//		if( regno == PC )		/* set %PC ? */	/* del T.Tazaki 2005/09/21 */
//		{
			if( g_command_loop == 0 && g_iSet_Command == 1)	/* Not command file && set $pc command ? */
			{
				g_iSet_Command = 0;
				/* source window update */
				giFrameClear = 1;
				sim_Gdbtk_Update();
				Tcl_SetVar( gdbtk_interp,"c33_set_pc_command","ON",TCL_GLOBAL_ONLY );
				if( g_iTargetOpen_end == 1 )	/* Target command end ? */
				{
					pszbuf = Tcl_GetVar(gdbtk_interp, "c33_console_cmd", TCL_GLOBAL_ONLY);	 
					iConsoleCommand = atoi( pszbuf );

					if( iConsoleCommand == 1 )
					{
						/* focus chenge to console window */
						Tcl_Eval( gdbtk_interp, "c33_focus_console" ); /* gdbtcl/srcbar.itcl */

						/* clear flag */
						Tcl_SetVar( gdbtk_interp,"c33_console_cmd","0",TCL_GLOBAL_ONLY ); /* gdbtcl/coonsole.itb */

					}
				}
				
				/* set $pc=boot -> next -> set $pc=boot -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 >>> */ 
				init_wait_for_inferior ();
				clear_proceed_status ();
				/* set $pc=boot -> next -> set $pc=boot -> next bug fix add T.Tazaki 2008/06/17 : DDTS gnu33v32-00025 <<< */ 

				return_to_top_level (RETURN_QUIT);	/* return to console */
			}
//		}
		Tcl_SetVar( gdbtk_interp,"c33_set_pc_command","OFF",TCL_GLOBAL_ONLY );
		g_reg_update_now = 0;
	}
	/* add update reg window	T.Tazaki 2004/06/04 <<< */
}

/****************************************************************************/
/* Format		 : void sim_Gdbtk_Update (void) 							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : The window is re-displayed.								*/
/****************************************************************************/
void
sim_Gdbtk_Update (void)
{
	extern Tcl_Interp *gdbtk_interp;

	g_Gdbtk_Update = 1; 	/* Do not log file output */

	registers_changed ();
	if (gdbtk_interp != NULL)
	{
		/* >>>>> change iruma m.takeishi '03.04.28 */
		/*
		flush_cached_frames ();
		*/
		if ( giFrameClear == 1 ) {
		  flush_cached_frames ();
		  giFrameClear = 0;
		}
		/* <<<<< change iruma m.takeishi '03.04.28 */
		Tcl_Eval (gdbtk_interp, "gdbtk_update");
	}

	g_Gdbtk_Update = 0;

}

/****************************************************************
 *	gdbsim_reset_reg : reset register (called for initialize) *
 ****************************************************************/

void
gdbsim_reset_reg (iMode)
int iMode;	/* 1:COLD, 0:HOT */
{
	int 			nRegNum;
	char			cTmpBuf[4];
	unsigned long	ulTmpData;
	unsigned long	ulData;
	unsigned long	ulTTBR = 0x20000000UL;

	extern unsigned long ulVectorAddr;
//	extern unsigned long ulReg[];

	g_iResume = TRUE;	/* for Not refresh windows by Register stor */
	
	/*@ read PC from trap table */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* ADVENCED MACRO */
		if( iMode == 1 )
		{
			/* COLD RESET */
			ulTTBR = 0x20000000UL;
		}
		else
		{
			/* HOT RESET */
			ulTTBR = gdbsim_fetch_register( TTBR );
		}
		
		/* Check advanced map area from 0x20000000 minimum 1 byte exist */
		if( fnCheckExistAdvArea() == 0 && ulTTBR == 0x20000000UL )	  /* Not Exsit ? */
		{
			/* mirror vector address set */
			(void)gdbsim_xfer_inferior_memory(0xc00000, (char *)&ulTmpData, 4, 0, NULL, NULL);
		}
		else
		{
			(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
		}
	}
	else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	/* add PE mode	T.Tazaki 2004/07/07 */
	{
		/* PE */
		if( iMode == 1 )
		{
			/* COLD RESET */
//			ulTTBR = 0x20000000UL;
			ulTTBR = 0x00c00000UL;	/* bug fix 2004/08/18 T.Tazaki */
		}
		else
		{
			/* HOT RESET */
			ulTTBR = gdbsim_fetch_register( TTBR );
		}
		
		(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	else
	{
		/* STANDARD MACRO */
		(void)gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulVectorAddr, 4, 0, NULL, NULL);	
		(void)gdbsim_xfer_inferior_memory(ulVectorAddr, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}

	for (nRegNum = R0 ; nRegNum < NUM_REGS ; nRegNum++) {	

		switch( nRegNum )
		{
		case R0:
		case R1:
		case R2:
		case R3:
		case R4:
		case R5:
		case R6:
		case R7:
		case R8:
		case R9:
		case R10:
		case R11:
		case R12:
		case R13:
		case R14:
		case R15:
		case AHR:
		case ALR:
			gdbsim_store_register ((unsigned short)nRegNum, 0xaaaaaaaaUL);
			break;
		case PSR:
		case LCO:
		case LSA:
		case LEA:
		case SOR:
		case DP:
		case USP:
		case SSP:
			gdbsim_store_register ((unsigned short)nRegNum, 0x0UL);
			break;
		case SP:
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV || fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdbsim_store_register ((unsigned short)nRegNum, 0x0UL);
			}
			else
			{
				gdbsim_store_register ((unsigned short)nRegNum, 0xaaaaaa8UL);
			}
			break;
		case TTBR:
			gdbsim_store_register ((unsigned short)nRegNum, ulTTBR);
			break;
		case IDIR:	/* add T.Tazaki 2003/11/28 */
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )						/* add T.Tazaki 2005/02/28 */
			{
				gdbsim_store_register ((unsigned short)nRegNum, 0x04000000UL);
			}
			else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )					/* add T.Tazaki 2005/02/28 */
			{
				gdbsim_store_register ((unsigned short)nRegNum, 0x06000000UL);	/* add T.Tazaki 2005/02/28 */
			}
			
			break;
		case DBBR:	/* add T.Tazaki 2003/11/28 */
			gdbsim_store_register ((unsigned short)nRegNum, 0x60000UL);
			break;
		case PC:
			gdbsim_store_register (PC, ulTmpData);
			break;
	   default:
			gdbsim_store_register ((unsigned short)nRegNum, 0x0UL);
			break;
	   }
	}

	g_iResume = FALSE;
}

/**********************************************************************/
/* Load an executable file into the target process.  This is expected to
   not only bring new code into the target process, but also to update
   GDB's symbol tables to match.  */
/**********************************************************************/

static void
gdbsim_load (prog, fromtty)
	 char *prog;
	 int fromtty;
{
  if (sr_get_debug ())
	printf_filtered ("gdbsim_load: prog \"%s\"\n", prog);

  inferior_ptid = null_ptid;
  program_loaded = 1;

  /* FIXME: We will print two messages on error.
	 Need error to either not print anything if passed NULL or need
	 another routine that doesn't take any arguments.  */
  if (sim_load (gdbsim_desc, prog, NULL, fromtty) == SIM_RC_FAIL)
  {
	c33_error ("unable to load program");
	return;
  }
//	generic_load (prog, fromtty);
	
	/* sɎgpXV */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

  /* FIXME: If a load command should reset the targets registers then
	 a call to sim_create_inferior() should go here. */

//	program_loaded = 1;
}


/**********************************************************************/
/* Start an inferior process and set inferior_pid to its pid.
   EXEC_FILE is the file to run.
   ARGS is a string containing the arguments to the program.
   ENV is the environment vector to pass.  Errors reported with error ().
   On VxWorks and various standalone systems, we ignore exec_file.	*/
/* This is called not only when we first attach, but also when the
   user types "run" after having attached.	*/
/**********************************************************************/

static void
gdbsim_create_inferior (exec_file, args, env)
	 char *exec_file;
	 char *args;
	 char **env;
{
  int len;
  char *arg_buf, **argv;

  if (exec_file == 0 || exec_bfd == 0)
	warning ("No executable file specified.");
//	if (!program_loaded)
//	  warning ("No program loaded.");

  if (sr_get_debug ())
	printf_filtered ("gdbsim_create_inferior: exec_file \"%s\", args \"%s\"\n",
			 (exec_file ? exec_file : "(NULL)"),
			 args);

  gdbsim_kill ();
  remove_breakpoints ();
  init_wait_for_inferior ();

  if (exec_file != NULL)
	{
	  len = strlen (exec_file) + 1 + strlen (args) + 1 + /*slop */ 10;
	  arg_buf = (char *) alloca (len);
	  arg_buf[0] = '\0';
	  strcat (arg_buf, exec_file);
	  strcat (arg_buf, " ");
	  strcat (arg_buf, args);
	  argv = buildargv (arg_buf);
	  make_cleanup_freeargv (argv);
	}
  else
	argv = NULL;
  sim_create_inferior (gdbsim_desc, exec_bfd, argv, env);

  inferior_ptid = pid_to_ptid (42);
  insert_breakpoints ();	/* Needed to get correct instruction in cache */

  clear_proceed_status ();

  /* NB: Entry point already set by sim_create_inferior. */
  proceed ((CORE_ADDR) -1, TARGET_SIGNAL_DEFAULT, 0);
}

/****************************************************************************/
/* Format		 : static void gdbsim_open (char *name, int from_tty)		*/
/* Arguments																*/
/*	char *name	 : Device name												*/
/*	int from_tty : 1 to indicate this is from the console.					*/
/* Return value  : void 													*/
/* Explanation	 : It connects with SIM and SIM is initialized. 			*/
/****************************************************************************/
static void
gdbsim_open (args, from_tty)
	 char *args;
	 int from_tty;
{
  int len;
  char *arg_buf;
  char **argv;
  int  i;
  extern struct BreakData stBreakData;
  extern Tcl_Interp *gdbtk_interp;		/* add T.Tazaki 2004/01/08 */


	if( c33TargetInfo.iSim_open == 1 )
	{
		c33_error("The simulator is already connected.\n");
		return;
	}

  if (sr_get_debug ())
	printf_filtered ("gdbsim_open: args \"%s\"\n", args ? args : "(null)");

  /* Remove current simulator if one exists.  Only do this if the simulator
	 has been opened because sim_close requires it.
	 This is important because the call to push_target below will cause
	 sim_close to be called if the simulator is already open, but push_target
	 is called after sim_open!	We can't move the call to push_target before
	 the call to sim_open because sim_open may invoke `error'.	*/
		
	
	
  if (gdbsim_desc != NULL)
	unpush_target (&gdbsim_ops);


  len = (7 + 1			/* gdbsim */
	 + strlen (" -E little")
	 + strlen (" --architecture=xxxxxxxxxx")
	 + (args ? strlen (args) : 0)
	 + 50) /* slack */ ;
  arg_buf = (char *) alloca (len);
  strcpy (arg_buf, "gdbsim");	/* 7 */
  /* Specify the byte order for the target when it is both selectable
	 and explicitly specified by the user (not auto detected). */
	 
/* Del 2002/07/10 T.Tazaki for make
  if (TARGET_BYTE_ORDER_SELECTABLE_P
*/
  if (1 			/* Modify 2002/07/10 T.Tazaki for make */
	  && !TARGET_BYTE_ORDER_AUTO)
	{
	  switch (TARGET_BYTE_ORDER)
	{
	case BIG_ENDIAN:
	  strcat (arg_buf, " -E big");
	  break;
	case LITTLE_ENDIAN:
	  strcat (arg_buf, " -E little");
	  break;
	default:
	internal_error (__FILE__, __LINE__, "Value of TARGET_BYTE_ORDER unknown");
	}
	}
  /* Specify the architecture of the target when it has been
	 explicitly specified */
  if (!TARGET_ARCHITECTURE_AUTO)
	{
	  strcat (arg_buf, " --architecture=");
	  strcat (arg_buf, TARGET_ARCHITECTURE->printable_name);
	}
  /* finally, any explicit args */
  if (args)
	{
	  strcat (arg_buf, " ");	/* 1 */
	  strcat (arg_buf, args);
	}
  argv = buildargv (arg_buf);
  if (argv == NULL)
  {
	error ("Insufficient memory available to allocate simulator arg list.");
  }
  make_cleanup_freeargv (argv);


  /* MAP Information set */
  /* >>>>> change T.Tazaki 2003/02/27 */
  /*
  sim_map_set( c33TargetInfo ); 	// for sim memory allocation //
  sim_map_set2( c33TargetInfo );	// for sim memory access endian //
  sim_map_set3( c33TargetInfo );	// for sim memory allocation : sim-memopt.c //
  sim_map_set4( c33TargetInfo );	// for sim memory allocation : sim-hload.c //
  */
  sim_map_set( &c33TargetInfo );	 /* for sim memory allocation */
  sim_map_set2( &c33TargetInfo );	 /* for sim memory access endian */
  sim_map_set3( &c33TargetInfo );	 /* for sim memory allocation : sim-memopt.c */
  sim_map_set4( &c33TargetInfo );	 /* for sim memory allocation : sim-hload.c */
  /* <<<<< change change T.Tazaki 2003/02/27 */
  sim_map_set5( &c33TargetInfo );	 /* for sim memory allocation : sim-load.c */ /* add T.Tazaki 2003/07/17 */

  /* add T.Tazaki 2003/06/09 >>> */
  if( c33TargetInfo.pMapInfo != NULL )
  {
	  if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV ) /* Parameter file is advanced mode ? */
	  {
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? add T.Tazaki 2003/11/05 */
		{
		  if( g_ucAdvFlag != 'A' )	/* check if ELF file is not Advanced macro. */
		  {
				printf_filtered ("Warning: target file is not advanced macro program.\n");
		  }
		}
	  }
	  /* add T.Tazaki 2004/07/07   add PE SIM Mode */
	  else if( c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )   /* Parameter file is PE mode ? */
	  {
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? */
		{
			if( g_ucAdvFlag != 'P' )	/* check if ELF file is not PE macro. */
			{
				printf_filtered ("Warning: target file is not PE program.\n");
			}
		}
	  }
	  else
	  {
		if( g_ucAdvFlag != 0xff )	/* "file"command executed ? add T.Tazaki 2003/11/05 */
		{
		  if( g_ucAdvFlag != 0 )	/* check if ELF file is not Standard macro. */
		  {
				printf_filtered ("Warning: target file is not standard macro program.\n");
		  }
		}
	  }
  }
  else
  {
	  if( g_ucAdvFlag != 0 )	/* check if ELF file is not Standard macro. */
	  {
			printf_filtered ("Warning: target file is not standard macro program.\n");
	  }
  }
  /* add T.Tazaki 2003/06/09 <<< */

  init_callbacks ();
  gdbsim_desc = sim_open (SIM_OPEN_DEBUG, &gdb_callback, exec_bfd, argv);

  if (gdbsim_desc == 0)
  {
	c33_error ("unable to create simulator instance");
	return;
  }
  push_target (&gdbsim_ops);

  /* default vector set  */
  if (gdb_InitSim()){
	c33_error ("Initialize error");
	return;
  }


  start_remote();
//	normal_stop();

	/* A setup of a simulation was completed. */
	c33TargetInfo.iSim_open = 1;
	g_iSim_open = 1;				/* add T.Tazaki for gdbtk.ini reg windows size	 T.Tazaki 2004/06/11 */


	/* A target is reset. */ /* add T.Tazaki 2003/09/10 */
	if (c33_Cmd_Reset(C33_ICD_COLD_RESET) != C33_NO_ERR)
	{
		return;
	}

	/* call command mode initialize */
	c33TargetInfo.iCallMode = 1;

	/* Interlock cycle initialize */
	c33InterLockInfo.iInterlockMode = 0;
	c33InterLockInfo.iCycle1   = 0;
	c33InterLockInfo.iCycle2   = 0;
	c33InterLockInfo.iNextInst = 0;
	c33InterLockInfo.uwRegno   = 0;
	
	for (i = 0 ; i < C33_STDOUT_BREAK_NUM ; i++)
	{
		c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
	}
	/* Clear hard pc break */
	for (i = 0 ; i < C33_HARD_PC_BREAK_MAX ; i++)
	{
		c33TargetInfo.ulHardPcBreak[i] = C33_BREAK_DISABLE;
	}
	/* Clear data break */
	c33TargetInfo.ulDataBreak = C33_BREAK_DISABLE;
	stBreakData.ulAddr = C33_BREAK_DISABLE;
	stBreakData.iType = NO_ACCESS;

	/* step temporary break initialize */
	
	for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
	{
		g_ulSoftBreak_Addr[i] = C33_BREAK_DISABLE;
		g_usSoftBreak_code[i] = 0;
	}
	/* Reset counters */
	c33TargetInfo.iClockMode = 2;
	c33TargetInfo.iClockFunction = 1;

	gulCycle = 0;
	
	printf_filtered ("Connected to the simulator.\n");

	/* >>>>> add iruma m.takeishi '03.04.28 */
	giFrameClear = 0;
	/* <<<<< add iruma m.takeishi '03.04.28 */
	
	g_iTargetOpen_end = 1;	/* already "target XXX" for focus to console window add T.Tazaki 2003/10/07 */

	/* >>>>> add irumam.takeishi '04.03.25 */
	stC33_CachedCode.iCached = 0;
	/* <<<<< add irumam.takeishi '04.03.25 */

	/* Registers window update & re-display  add T.Tazaki 2004/01/08 >>> */ 
	/* CPU mode set for Registers window */ 
	if( g_use_windows == 1 )	/* GUI Mode ? */
	{
		if( c33TargetInfo.ucAdvanced == C33_PRODUCT_ADV )
		{
			Tcl_SetVar( gdbtk_interp,"c33_reg_mode","ADV",TCL_GLOBAL_ONLY );
			/* Registers window update */
			Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */

			/* Display "Target/CPU" : srcbar.itcl	 2004/05/24 >>> */
			Tcl_SetVar( gdbtk_interp,"c33_target_name","SIM:ADV",TCL_GLOBAL_ONLY );
			Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
			/* Display "Target/CPU" : srcbar.itcl	 2004/05/24 <<< */
		}
		else if( c33TargetInfo.ucAdvanced == C33_PRODUCT_PE )	/* add T.Tazaki 2004/07/07 */	
		{
			Tcl_SetVar( gdbtk_interp,"c33_reg_mode","PE",TCL_GLOBAL_ONLY );
			/* Registers window update */
			Tcl_Eval( gdbtk_interp, "c33_reg_update" ); /* gdbtcl/srcbar.itcl */

			Tcl_SetVar( gdbtk_interp,"c33_target_name","SIM:PE ",TCL_GLOBAL_ONLY );
			Tcl_Eval( gdbtk_interp, "c33_cpumode_set" );	
		}
		else
		{
			/* Display "Target/CPU" : srcbar.itcl	 2004/05/24 >>> */
			Tcl_SetVar( gdbtk_interp,"c33_target_name","SIM:STD",TCL_GLOBAL_ONLY );
			Tcl_Eval( gdbtk_interp, "c33_cpumode_set" ); 
			/* Display "Target/CPU" : srcbar.itcl	 2004/05/24 <<< */
		}

	}
	/* Registers window update & re-display  add T.Tazaki 2004/01/08 <<< */ 

#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		Tcl_Eval( gdbtk_interp, "c33_ide_event_start" ); /* gdbtcl/srcbar.itcl */

		/* gnu33ide.dll initialize  */
		hIdeDll = LoadLibraryEx( "gnu33ide", NULL, 0 );
		if( hIdeDll != NULL )
		{
			dll33_ClearCommandUpdate  = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"ClearCommandUpdate" ); /* add T.Tazaki 2006/01/12 */
			dll33_GdbEnable       = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbEnable" );
			dll33_GdbDisable      = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbDisable" );
			dll33_GdbRun          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbRun" );	/* add T.Tazaki 2005/12/08 */
			dll33_GdbStop         = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"GdbStop" );	/* add T.Tazaki 2005/12/08 */
			dll33_addGdbGoCounter = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"addGdbGoCounter" );
			dll33_checkCommand    = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"checkCommand" );
			dll33_setMem          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setMem" );
			dll33_getMem          = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"getMem" );	
			dll33_setSymbol       = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setSymbol" );
			dll33_setIL           = (void ( _stdcall *) (WORD))GetProcAddress( hIdeDll,"setIL" );
			dll33_ClearCommandUpdate();/* add T.Tazaki 2006/01/12 */
			dll33_GdbEnable();
			g_GdbEnable = 1;
		}
	}
#endif
	
	sim_Gdbtk_Update ();


}

/****************************************************************************/
/* Format		 : static void gdbsim_close (int quitting)					*/
/* Arguments																*/
/*	int quitting : 0-Connection close 1-Quit debugger						*/
/* Return value  : void 													*/
/* Explanation	 : Connection with SIM is closed.							*/
/****************************************************************************/
static void
gdbsim_close (quitting)
	 int quitting;
{
	if( c33TargetInfo.iSim_open == 0 )
	{
		return;
	}

  if (sr_get_debug ())
	printf_filtered ("gdbsim_close: quitting %d\n", quitting);

  program_loaded = 0;

  if (gdbsim_desc != NULL)
	{
	  sim_close (gdbsim_desc, quitting);
	  gdbsim_desc = NULL;
	}

  end_callbacks ();
  generic_mourn_inferior ();

	/* Does it connect with SIM? */
	if (c33TargetInfo.iSim_open)
	{
		/* Connection with SIM is closed. */
		c33TargetInfo.iSim_open = 0;
	}
#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_GdbDisable();
			g_GdbEnable = 0;
			FreeLibrary( hIdeDll );
			hIdeDll = NULL;
		}
	}
#endif

}

/**********************************************************************/
/* Kill the running program.  This may involve closing any open files
   and releasing other resources acquired by the simulated program.  */
/**********************************************************************/

static void
gdbsim_kill ()
{
  if (sr_get_debug ())
	printf_filtered ("gdbsim_kill\n");

  /* There is no need to `kill' running simulator - the simulator is
	 not running */
  inferior_ptid = null_ptid;

#ifdef __CYGWIN__	/* add T.Tazaki 2005/09/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_GdbDisable();
			g_GdbEnable = 0;
			FreeLibrary( hIdeDll );
			hIdeDll = NULL;
		}
	}
#endif

}


/* Takes a program previously attached to and detaches it.
   The program may resume execution (some targets do, some don't) and will
   no longer stop on signals, etc.	We better not have left any breakpoints
   in the program or it'll die when it hits one.  ARGS is arguments
   typed by the user (e.g. a signal to send the process).  FROM_TTY
   says whether to be verbose or not.  */
/* Terminate the open connection to the remote debugger.
   Use this when you want to detach and do something else with your gdb.  */

static void
gdbsim_detach (args, from_tty)
	 char *args;
	 int from_tty;
{

	if( c33TargetInfo.iSim_open == 0 )
	{
		return;
	}

  if (sr_get_debug ())
	printf_filtered ("gdbsim_detach: args \"%s\"\n", args);

	pop_target ();		  /* calls gdbsim_close to do the real work */

  if (from_tty)
	printf_filtered ("Ending simulator %s debugging\n", target_shortname);
}

/* Resume execution of the target process.	STEP says whether to single-step
   or to run free; SIGGNAL is the signal value (e.g. SIGINT) to be given
   to the target, or zero for no signal.  */

static enum target_signal resume_siggnal;
static int resume_step;

/****************************************************************************/
/* Format		 : static void gdbsim_resume  (ptid_t ptid, int step, enum target_signal siggnal) */
/* Explanation	 : step or cont execute 									*/
/*																			*/
/****************************************************************************/

static void
gdbsim_resume (ptid_t ptid, int step, enum target_signal siggnal)
{
  unsigned long ulData;
  unsigned long ulAddr;
  unsigned short ulCode;
  unsigned long  ulPSR;
  unsigned long  ulPC;
  /* >>>>> delete iruma m.takeishi '03.08.29 */
  /*
  unsigned long  ulPC_temp;
  */
  /* >>>>> delete iruma m.takeishi '03.08.29 */
  int			 iStatus;
  int			 iExtInsn = 0;
  int			 iSaveStep; 				/* for Delayed slot */
  int			 iLoop;
  extern struct DelaySlot stDelaySlot;		/* for Delayed slot */
  char			 szDisasm[256]; 			/* for mnemonic 	*/
  char			 szStatusBuf[STATUS_WORK_BUFFER_MAX];	/* 5000->define change T.Tazaki 2005/03/14 */
  /* >>>>> delete iruma m.takeishi '03.12.19 */
  /*
  char			 szSourceLine[256]; 		// for Line & Source String //
  */
  /* <<<<< delete iruma m.takeishi '03.12.19 */
  char			 szTmp[100];
  int			 i_sim_Gdbtk_Update_cnt;
  int			 iRet;
  int			 iMacFlag;
  int 			 iSoftBrkMatch = 0;			/* for Soft pc break  add T.Tazaki 2003/11/07 */
  int			 i;							/* for Soft pc break  add T.Tazaki 2003/11/07 */
  unsigned long  ulCurrentAddr;				/* for Soft pc break  add T.Tazaki 2003/11/07 */
  int 			 iType;						/* for add T.Tazaki 2003/11/07 */
  int 			 iNum;						/* for add T.Tazaki 2003/11/07 */
  /* >>>>> add iruma m.takeishi '03.08.29 */
  int			 iLSA_Hit = 0;
  int			 iRepeatInst = 0;
  /* >>>>> delete iruma m.takeishi '03.12.19 */
  /*
  extern struct Extension stExtension;
  */
  /* <<<<< delete iruma m.takeishi '03.12.19 */

  /* <<<<< add iruma m.takeishi '03.08.29 */
  /* >>>>> add iruma m.takeishi '03.12.19 */
  int iLSA_HitForLoop = 0;
  /* <<<<< add iruma m.takeishi '03.12.19 */
  /* >>>>> add iruma m.takeishi '04.03.25 */
  int iIsLcoDown_Loop;
  unsigned long ulTmpPsr;
  struct c33_map_info *stpMapInfo = NULL;
  /* <<<<< add iruma m.takeishi '04.03.25 */
  unsigned short usCode;					/* PE code check  add T.Tazaki 2004/08/05 */
  unsigned long ulExtExceptPC = 0;			/* PE except address		  add T.Tazaki 2005/02/21 */
  int			iExtExceptCnt = 0;			/* PE except ext inst counter add T.Tazaki 2005/02/21 */
  int			iExtExceptFlg = 0;			/* PE ext exception flag	  add T.Tazaki 2005/02/22 */
  int  			iStatusForTrace;			/* add T.Tazaki 2005/03/14 */
  unsigned long ulAddrForTrace;				/* add T.Tazaki 2005/03/14 */

  static RETSIGTYPE (*prev_sigint_bk) ();/* 2002/01/21	C.Nakayama Add Cnrl-C function.(GNU33cygwin) */

  extern int g_iGdbtk_Update_cnt;	/* bug fix : while() cannot stop : add T.Tazaki 2008/05/26 */

  g_iResume = TRUE;
	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		if( step )
		{
			printf_filtered( "<gdbsim_resume> STEP \n");
		}else{
			printf_filtered( "<gdbsim_resume> RUN \n");
		}
	}

  if( step == 0 )	/* add 2003/08/08 T.Tazaki */
  {
	  g_iContFlag = 1;
  }
  else
  {
	  g_iContFlag = 0;
  }

  prev_sigint_bk = signal (SIGINT, gdbsim_cntrl_c);/* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */

	
  iSaveStep = step;   /* 2002/06/06 T.Tazaki step save */

	
  inferior_ptid = pid_to_ptid (42);
  if (PIDGET (inferior_ptid) != 42)
  {
	c33_error ("The program is not being run.");
	return;
  }
  i_sim_Gdbtk_Update_cnt = 0;


	/* "c33 clockmd" Command */
	if( c33TargetInfo.iClockMode == 1 ) /* reset ? */
	{
		gulCycle = 0;	/* Counter Clear */
	}

	/* add T.Tazaki 2004/11/10 >>> */
	giKeybreak = 0;
	c33TargetInfo.iKeyBreak = 0;
	/* add T.Tazaki 2004/11/10 <<< */


    g_resume_flag = 1; /* add T.Tazaki 2005/12/08 */

#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
	if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
	{
		if( hIdeDll != NULL )
		{
			dll33_addGdbGoCounter();		/* gnu33ide.dll : GO counter +1    */
			dll33_GdbRun();					/* gnu33ide.dll : GO flag ON   */
		}
	}
#endif

	
  do
  {
	/* bug fix : while() cannot stop : add T.Tazaki 2008/05/27 >>> */
	if( step )
	{
       Sleep( 3 ); 			/* 100 --> 3 change T.Tazaki 2008/06/06 */
       c33_Gdbtk_Idle ();
	}
	/* bug fix : while() cannot stop : add T.Tazaki 2008/05/27 <<< */

	if(giKeybreak == 1){/* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */
	  giKeybreak = 0;

	  /* >>>>> add iruma m.takeishi '04.03.24 */
	  /* if below code is not exist,	
	   *  ld.w	   %r0, 0
	   *  jpr	   %r0
	   *  if you do "step" at jpr inst, SIM mode goes into endless loop.
	   */
	  { 	
		extern int stop_after_trap;
		stop_after_trap = 1;
	  }
	  /* <<<<< add iruma m.takeishi '04.03.24 */
	  break;/* Ctrl-C*/
	}

	
	/* add 2002/08/22 T.Tazaki for Stop button */
	if( step != 1 )
	{
		if( i_sim_Gdbtk_Update_cnt == 0 )
		{
			c33_Gdbtk_Idle ();	/* add 2003/02/24 T.Tazaki */
			
			i_sim_Gdbtk_Update_cnt = 2000;
		}
		else
		{
			--i_sim_Gdbtk_Update_cnt;
		}
	}

	/*@ check if instruction is delayed */
	if (stDelaySlot.blUsed == YES) {
		/*@ code is in delayed slot */
		ulAddr = stDelaySlot.ulAddr;

		/*@ you execute delayed instruction */
		stDelaySlot.blRunDelayed = YES;
		
		step = iSaveStep;	/* restore step */
	}
	else {
		/*@ code is in memory */
		ulAddr = gdbsim_fetch_register(PC);

		/*@ you don't execute delayed instruction */
		stDelaySlot.blRunDelayed = NO;
	}

	if ( g_iContFlag == 1 )	/* continue ? */
	{
		/***********************************************************************/
		/* check if ulAddr is soft pc break 		add T.Tazaki 2004/01/13 >>> */
		/***********************************************************************/
		c33_GetBreakPoint();

		for( i = 0; i < g_SoftPcBreakTable_Index; i++)
		{
			if( c33TargetInfo.ulSoftPcBreak[i] != C33_BREAK_DISABLE )
			{
				if( c33TargetInfo.ulSoftPcBreak[i] == ulAddr )
				{
					/* check if continue parameter(break counter) exist. */
					if ( g_iContinue_count > 0 )
					{
						--g_iContinue_count;
						if( g_iContinue_count == 0 )
						{
							g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
#endif				
							break;	/* exit for i */
						}
					}
					/* "ignore" command */
					else if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
					{
						iType = 1;
						iNum = c33_GetBreakNumber( iType, ulAddr );
						if( iNum == g_iIgnore_number )
						{
							if( g_iIgnore_count > 0 )
							{
								--g_iIgnore_count;
							}
							if( g_iIgnore_count == 0 )
							{
								g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
								if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
								{
									if( hIdeDll != NULL )
										dll33_GdbStop();
								}
#endif				
								break;		/* exit for i */
							}
						}
						/* bug fix add T.Tazaki 2008/06/20 >>> [DDTS gnu33v32-00027] */
						else
						{
							g_resume_flag = 0;
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
							break;	/* exit for i */
						}
						/* bug fix add T.Tazaki 2008/06/20 <<< [DDTS gnu33v32-00027] */
					}
					else
					{
						g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
						if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
						{
							if( hIdeDll != NULL )
								dll33_GdbStop();
						}
#endif				
						break;	/* exit for i */
					}
				}
			}
		}
		if( g_resume_flag == 0 )
		{
			break;	/* exit while() */
		}
		/* check if ulAddr is soft pc break 		add T.Tazaki 2004/01/13 <<< */

		/***********************************************************************/
		/* check if ulAddr is hard pc break1 									*/
		/***********************************************************************/
		if( c33TargetInfo.ulHardPcBreak[0] != C33_BREAK_DISABLE ){
			if( c33TargetInfo.ulHardPcBreak[0] == ulAddr ){
				if( g_resume_flag == 1 )	/* Not display	2'nd calling */ /* add T.Tazaki 2003/09/19 */
				{
					/* continueu[N񐔃`FbN@add T.Tazaki 2003/11/07 >>> */
					if ( g_iContinue_count > 0 )
					{
						--g_iContinue_count;
						if( g_iContinue_count == 0 )
						{
							puts_filtered( "Break by hard pc break1.\n" );
							g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
#endif				
							break;
						}
					}
					/* "ignore" command */
					else if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
					{
						iType = 2;
						iNum = c33_GetBreakNumber( iType, ulAddr );
						if( iNum == g_iIgnore_number )
						{
							--g_iIgnore_count;
							if( g_iIgnore_count == 0 )
							{
								puts_filtered( "Break by hard pc break1.\n" );
								g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
								if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
								{
									if( hIdeDll != NULL )
										dll33_GdbStop();
								}
#endif				
								break;
							}
						}
						/* bug fix add T.Tazaki 2008/06/20 >>> [DDTS gnu33v32-00027] */
						else
						{
							g_resume_flag = 0;
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
							break;	/* exit for i */
						}
						/* bug fix add T.Tazaki 2008/06/20 <<< [DDTS gnu33v32-00027] */
					}
					else
					{
						puts_filtered( "Break by hard pc break1.\n" );
						g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
						if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
						{
							if( hIdeDll != NULL )
								dll33_GdbStop();
						}
#endif				
						break;
					}
					/* add T.Tazaki 2003/11/07 >>> */
				}
			}
		}
		/***********************************************************************/
		/* check if ulAddr is hard pc break2 									*/
		/***********************************************************************/
		if( c33TargetInfo.ulHardPcBreak[1] != C33_BREAK_DISABLE ){
			if( c33TargetInfo.ulHardPcBreak[1] == ulAddr ){
				if( g_resume_flag == 1 )	/* Not display	2'nd calling */ /* add T.Tazaki 2003/09/19 */
				{
					/* continue break count check add T.Tazaki 2003/11/07 >>> */
					if ( g_iContinue_count > 0 )
					{
						--g_iContinue_count;
						if( g_iContinue_count == 0 )
						{
							puts_filtered( "Break by hard pc break2.\n" );
							g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
#endif				
							break;
						}
					}
					/* "ignore" command */
					else if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
					{
						iType = 2;
						iNum = c33_GetBreakNumber( iType, ulAddr );
						if( iNum == g_iIgnore_number )
						{
							--g_iIgnore_count;
							if( g_iIgnore_count == 0 )
							{
								puts_filtered( "Break by hard pc break2.\n" );
								g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
								if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
								{
									if( hIdeDll != NULL )
										dll33_GdbStop();
								}
#endif				
								break;
							}
						}
						/* bug fix add T.Tazaki 2008/06/20 >>> [DDTS gnu33v32-00027] */
						else
						{
							g_resume_flag = 0;
							if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
							{
								if( hIdeDll != NULL )
									dll33_GdbStop();
							}
							break;	/* exit for i */
						}
						/* bug fix add T.Tazaki 2008/06/20 <<< [DDTS gnu33v32-00027] */
					}
					else
					{
						puts_filtered( "Break by hard pc break2.\n" );
						g_resume_flag = 0;
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
						if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
						{
							if( hIdeDll != NULL )
								dll33_GdbStop();
						}
#endif				
						break;
					}
					/* add T.Tazaki 2003/11/07 >>> */
				}
			}
		}
	}

	stDelaySlot.ulAddr = ulAddr + 2;		/* next address save */

	g_iExtEachInst = TRUE;


	/* >>>>> change iruma m.takeishi '04.03.25 */
	/*
	gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulCode, 2, 0, NULL, NULL);
	*/
	/****	reads two instruction if bus-size is 32bits, ****/
	/****	and word geometry.							 ****/

	/* is word geometry? */
	iRet = 0;
	if ( stC33_CachedCode.iCached == 1 ) {
		if ( stC33_CachedCode.ulPc == ulAddr ) {
			ulCode = (unsigned long)(stC33_CachedCode.usCode);
			iRet = -1;
		}
		stC33_CachedCode.iCached = 0;
	}

	if ( iRet != -1 ) {
		iRet = 0;
		if ( ulAddr % 4 == 0 ) {
			/* is 32bit bus? */
			stpMapInfo = c33_GetMapInfo( ulAddr );
			if ( stpMapInfo != NULL ) {
				
				if ( stpMapInfo->ulDeviceSize == C33_WORD ) {
					iRet = 1;
				}
			}
		}
		if ( iRet == 0 ) {
			/* normal fetch */
			gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulCode, 2, 0, NULL, NULL);
		} else {
			
			/* read 2 inst */
			gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulData, 4, 0, NULL, NULL);
			
			/* change about Endian */
			iRet = 0; /* litle */
			if ( stpMapInfo != NULL ) {
				if ( stpMapInfo->iEndian == C33_BIG ) {
					iRet = 1; /* Big */
				}
			}
			
			if ( iRet == 0 ) {
				stC33_CachedCode.usCode = (unsigned short)(ulData >> 16);
				ulCode = (unsigned short)(0xffff & ulData);
			} else {
				stC33_CachedCode.usCode = (unsigned short)(0xffff & ulData);
				ulCode = (unsigned short)( 0xffff & (ulData >> 16) );
			}
			stC33_CachedCode.ulPc = ulAddr + 2;
			stC33_CachedCode.iCached = 1;
		}
	}
	/* <<<<< change iruma m.takeishi '04.03.25 */

	/* disp exe code */

	/*@ disassemble */
	g_iTraceMode = 1;		 /* ld.w %r0,0x2a (xld.w %r0,0xffffffea) --> (ld.w %r0,0x2a):Not extend */
	c33_Disasm ((unsigned short)ulCode, szDisasm);
	g_iTraceMode = 0;
	
	(void) strcat (szDisasm, "                                   ");
	szDisasm[28] = 0;

	(void) sprintf (szStatusBuf, "%08ld %08lX %04X  %s",
				g_ulInstNum , ulAddr, (int) ulCode, szDisasm);

	g_iStatusBufCnt = 0;		/* for r/w trace */
	ulAddrForTrace = ulAddr;	/* for trace  add T.Tazaki 2005/03/14 */

	/* >>>>> add iruma m.takeishi '04.03.16 */
	if ( iRsvLoop_instCnt != 0 ) {
	
	
		/* checks PC=LEA. If current is ext-inst, ignored */
		/* Below is copy of LM decrement routine. */
		iRet = 1;
		if ( iLSA_HitForLoop == 1 && stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_NONE ) {
			iRet = 0;
		} else {
			/*@ current PC is equal to LEA */
			if (ulAddr == ulReg[LEA]) {
				
				if ( ( stExtension.iType == EXT_NONE ) &&
					 ( stExtension.iTypeExt == EXT_NONE ) ) {
					iRet = 0;
				}
			}
		}
		if ( iRet == 0 ) {
			/* error */
			puts_filtered( "Too near between LM set and LEA.\n" );
			{ extern int stop_after_trap;
			 stop_after_trap = 1;
			}
		  break;
		}

		/* if current inst, which is next to the inst setting LM, is ext-inst,	
		   ext inst is not counted. */
		if ( iRsvLoop_instCnt != RESERVELOOP_INSTCNT || 
			 fnGetExtType( (unsigned short)ulCode ) == EXT_NONE ) { 
			 iRsvLoop_instCnt--;
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.16 */

	/*==================================================================*/
	/* PE undefined instruction interrupt	add T.Tazaki 2004/08/05 >>> */
	/*==================================================================*/
	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		usCode = ulCode & 0xff00;
		if( usCode == 0x8a00 || usCode == 0x8e00 || usCode == 0x9600 || usCode == 0xb200 ) /* scan0,scan1,mirror,mac ? */
		{
			iStatus = fnPush (PC);
			if( iStatus == 0 )
			{
				iStatus = fnPush (PSR);
				if( iStatus == 0 )
				{
					/* %psr IE = 0 */
					ulPSR = gdbsim_fetch_register (PSR);
					ulPSR &= 0xffffffef;
				   	gdbsim_store_register( PSR, ulPSR );
			
					/* %idir udi set */
					ulData = gdbsim_fetch_register (IDIR);
					ulData = (ulData & 0xffff0000) + (ulCode & 0x0000ffff);
					gdbsim_store_register( IDIR, ulData );

					/* Jump to undefined instruction exception interrupt */
					ulAddr = gdbsim_fetch_register (TTBR);
					ulAddr += 12;
					gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulData, 4, 0, NULL, NULL);
					gdbsim_store_register( PC, ulData );
					/* add T.Tazaki 2005/03/25 >>> */
					/* trace process at undefined instrunction interrupt */
					if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */
						(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulAddrForTrace);
						c33_TraceDump( ulAddrForTrace, szStatusBuf );
						vfnAddRegDump ();
					}
					g_ulInstNum++;
					/* add T.Tazaki 2005/03/25 <<< */

					continue;
				}
			}
			if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
				puts_filtered( "Break by accessing no map.\n" );
				break;
			}
			if ((iStatus & ROM_DATA_BREAK) != 0) {
				puts_filtered( "Break by writing ROM area.\n" );
				break;
			}
			if ((iStatus & STACK_OVERFLOW ) != 0) {
				puts_filtered( "Break by stack overflow.\n" );
				break;
			}
		}

		/* add T.Tazaki 2005/02/21 >>> */
		
		/****************************/
		/* ext Exception process	*/
		/****************************/
		
		usCode = ulCode & 0xe000;
		if( usCode == 0xc000 )	/* ext ? */
		{
			if( iExtExceptCnt == 2 )
			{
				iExtExceptCnt = 0;
				iExtExceptFlg = 1;
			}
			else
			{
				if( iExtExceptCnt == 0 )
				{
					ulExtExceptPC = gdbsim_fetch_register( PC ); /* update ext start address */
					g_ulExtStartPC = ulExtExceptPC;				 /* save first ext address add T.Tazaki 2005/03/29 */
					g_iExtStartPC_Flag = 1;						 /* it use at fnZeroDiv(),fnIlleAddr() */
				}
				++iExtExceptCnt;
			}
		}
		else
		{
			iExtExceptCnt = 0;
			iExtExceptFlg = 0;
		}

		/* add T.Tazaki 2005/02/21 <<< */

	}
	/* PE undefined instruction interrupt	add T.Tazaki 2004/08/05 <<< */
	/* STD/ADV undefined instruction interrupt	 add T.Tazaki 2005/03/29 >>> */
	else
	{
		/****************************/
		/* ext exception process	*/
		/****************************/
		
		usCode = ulCode & 0xe000;
		if( usCode == 0xc000 )	/* ext ? */
		{
			if( iExtExceptCnt == 2 )
			{
				iExtExceptCnt = 0;
				iExtExceptFlg = 1;
			}
			else
			{
				if( iExtExceptCnt == 0 )
				{
					ulExtExceptPC = gdbsim_fetch_register( PC ); /* update ext start address */
					g_ulExtStartPC = ulExtExceptPC;				 /* save first ext address add T.Tazaki 2005/03/29 */
					g_iExtStartPC_Flag = 1;						 /* it use at fnZeroDiv(),fnIlleAddr() */
				}
				++iExtExceptCnt;
			}
		}
		else
		{
			iExtExceptCnt = 0;
			iExtExceptFlg = 0;
		}
	}
	/* STD/ADV undefined instruction interrupt	 add T.Tazaki 2005/03/29 <<< */


	/* check if PE ext exception? 	add T.Tazaki 2005/02/22 */

	if( iExtExceptFlg == 0 )
	{
		/*==========================================================*/
		/* Instruction execute										*/
		/*==========================================================*/

		iStatus = fnExeEachInst( (unsigned short)ulCode);

	}
	else
	{
		/* PE only */
		iStatus = fnExtException(ulExtExceptPC);  /* jump to ext exception vector address : add T.Tazaki 2005/02/22 */
		iExtExceptFlg = 0;
		iExtExceptCnt = 0;		/* add T.Tazaki 2005/03/16 */
		g_iExtExceptCnt = 0;	/*  add T.Tazaki 2005/03/16 */
		
		/* trace process at undefined instrunction interrupt  add T.Tazaki 2005/03/14 >>> */
		if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */
			(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulExtExceptPC);
			c33_TraceDump( ulExtExceptPC, szStatusBuf );
		}
		/*@ count up instruction number */
		g_ulInstNum++;
		if( g_iTraceDump == TM_ON ){
			vfnAddRegDump ();
		}
		/* add T.Tazaki 2005/03/14 <<< */
		ulCode = 0;	/* add T.Tazaki 2005/03/16 */

		if( g_iContFlag == 0 )	/* STEP ? */
		{
			break;
		}

		continue;								/* add T.Tazaki 2005/03/16 */
	}
	
	iStatusForTrace = iStatus;									/* add T.Tazaki 2005/03/14 */
	
	/*==========================================================*/
	/* Error status check										*/
	/*==========================================================*/

	/* >>>>> add iruma m.takeishi '04.03.18 */
	/* changes LM_FLAG before exception occurs. */
	/* this routine is copy of LM=1 */
	if ( stExtension.iTypeExtCond != EXTCOND_SKIP ) {

		if ( (ulReg[ PSR ] & LM_FLAG) != 0 ) {
			iRet = 1;
			/* was PC hit i past by ext-inst ? */
			if ( iLSA_HitForLoop == 1 && stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_NONE ) {
				iRet = 0;
				iLSA_HitForLoop = 0;
			} else {
			/*@ current PC is equal to LEA */
				if (ulAddr == ulReg[LEA]) {
				
					if ( ( stExtension.iType != EXT_NONE ) ||
						 ( stExtension.iTypeExt != EXT_NONE ) ) {
						iLSA_HitForLoop = 1;
					} else {
						iRet = 0;
					}
				}
			}
			iIsLcoDown_Loop = iRet;
			if ( iRet == 0 ) {
				if ( ulReg[LCO] == 0 ) {
					/* down LM_FLAG */
					ulTmpPsr = gdbsim_fetch_register( PSR );
					ulTmpPsr &= ~LM_FLAG;
					gdbsim_store_register( PSR, ulTmpPsr );
				}
			}
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.18 */



	if((iStatus & S_ZERO_EX ) != 0) 		/* Zero division exception ? */
	{

		/* add T.Tazaki 2005/03/29 >>> */
		c33_TraceDump_Output_ext();	
		g_iExtExceptCnt = 0;
		/* add T.Tazaki 2005/03/29 <<< */

		/* >>>>> add iruma m.takeishi '04.03.18 */
		ulTmpPsr = gdbsim_fetch_register( PSR );
		if ( ulTmpPsr & LM_FLAG ) {
			iIsLoop_int = 1;
		}
		/* <<<<< add iruma m.takeishi '04.03.18 */
		iStatus = fnZeroDiv();	/* jump zero division exception vector address */
	}
	if((iStatus & S_ADDR_EX ) != 0) 		/* illigal address exception ? */
	{
		/* add T.Tazaki 2005/03/29 >>> */
		c33_TraceDump_Output_ext();
		g_iExtExceptCnt = 0;
		/* add T.Tazaki 2005/03/29 <<< */

		/* >>>>> add iruma m.takeishi '04.03.18 */
		ulTmpPsr = gdbsim_fetch_register( PSR );
		if ( ulTmpPsr & LM_FLAG ) {
			iIsLoop_int = 1;
		}
		/* <<<<< add iruma m.takeishi '04.03.18 */
		iStatus = fnIlleAddr(); /* jump Illegal address exception vector address */
	}

	g_iExtStartPC_Flag = 0;		/*   add T.Tazaki 2005/04/15 */
	
	g_iExtEachInst = FALSE;


	if((iStatus & DATA_BREAK) != 0 )
	{
		/* add T.Tazaki 2003/06/30 >>> */
		
		/* Watch point display */
		
		if( g_resume_flag == 1 )	/* Not display c33icd_wait 2'nd calling */ /* add T.Tazaki 2003/08/01 */
		{
			/* continue break count check add T.Tazaki 2003/11/07 >>> */

			/* "ignore" command */
			if ( g_iIgnore_number > 0 && g_iIgnore_count > 0 )
			{
				if ( g_iIgnore_count > 0 )
				{
					--g_iIgnore_count;
					if( g_iIgnore_count > 0 )
					{
						continue;
					}
				}
			}
			/* "cont [count]" command " */
			else if ( g_iContinue_count > 0 )
			{
				--g_iContinue_count;
				if( g_iContinue_count > 0 )
				{
					continue;
				}
			}
			/* add T.Tazaki 2003/11/07 >>> */

			if( g_iWatchBreakpointDisableFlag == 1 )	/* SIM watch + step or next mode ? */
			{
				c33_GetBreakPoint2();	 /* get all breakpoint infomation */ /* add 2003/08/06 T.Tazaki */
			}
			else
			{
				c33_GetBreakPoint();	/* get all breakpoint infomation */
			}
			if( g_WriteWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
				g_iStepLoopBreak = 1;		/* infcmd.c step_1() count loop break add 2003/09/19 T.Tazaki */
			}
			else if( g_ReadWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
				g_iStepLoopBreak = 1;		/* infcmd.c step_1()count count loop brea add 2003/09/19 T.Tazaki */
			}
			else if( g_WriteReadWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
				g_iStepLoopBreak = 1;		/* infcmd.c step_1()count count loop brea add 2003/09/19 T.Tazaki */
			}
		}
		/* add T.Tazaki 2003/06/30 >>> */
		
		g_resume_flag = 0;	/* add T.Tazaki 2003/08/01 */
#ifdef __CYGWIN__	/* add T.Tazaki 2005/12/08 */
		if( g_use_windows == 1 )  /* GUI Mode and Not command file ? */	
		{
			if( hIdeDll != NULL )
				dll33_GdbStop();
		}
#endif				

		break;
	}
	/* >>>>> add iruma m.takeishi '03.08.19 */
	if ((iStatus & PUSHPOP_ILLEGAL) != 0) {
		puts_filtered( "illegal RC number of PSR.\n" );
		g_iExtExceptCnt = 0;			/*  2005/04/15 */
		break;
	}
	/* <<<<< add iruma m.takeishi '03.08.19 */
	if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
		puts_filtered( "Break by accessing no map.\n" );
		g_iExtExceptCnt = 0;			/*  2005/04/15 */
		/* add T.Tazaki 2003/09/03 >>> */
		if( g_iContFlag == 0 )
		{
			vfnIncPC (CODE);	/* PC += CODE */	
			vfnInitExtension ();
		}
		/* add T.Tazaki 2003/09/03 <<< */
		break;
	}
	if ((iStatus & ROM_DATA_BREAK) != 0) {
		puts_filtered( "Break by writing ROM area.\n" );
		g_iExtExceptCnt = 0;			/*  2005/04/15 */
		/* add T.Tazaki 2003/09/03 >>> */
		if( g_iContFlag == 0 )
		{
			vfnIncPC (CODE);	/* PC += CODE */	
			vfnInitExtension ();
		}
		/* add T.Tazaki 2003/09/03 <<< */
		break;
	}
	if ((iStatus & STACK_OVERFLOW ) != 0) {
		puts_filtered( "Break by stack overflow.\n" );
		g_iExtExceptCnt = 0;			/*  2005/04/15 */
		/* add T.Tazaki 2003/09/03 >>> */
		if( g_iContFlag == 0 )
		{
			vfnIncPC (CODE);	/* PC += CODE */	
			vfnInitExtension ();
		}
		/* add T.Tazaki 2003/09/03 <<< */
		break;
	}
	if ((iStatus & SLP_HALT_BREAK) != 0) {
		puts_filtered( "Break by sleep or halt.\n" );
		break;
	}
	
	
	
	if((iStatus & ILL_INST_BREAK ) != 0)	/* illegal instruction ? */
	{
		if( ulCode != SOFT_BREAK_INST ){			/* Not Soft Break ? "brk" */
			puts_filtered( "Illegal instruction.\n" );
			g_iExtExceptCnt = 0;			/*  2005/04/15 */
			/* >>>>> add iruma m.takeishi '04.03.29 */
			{	
			  extern int stop_after_trap;
			  stop_after_trap = 1;
			}
			/* <<<<< add iruma m.takeishi '04.03.29 */
			break;
		}
		else
		{
			/* simulated i/o process */
			iRet = fnStdio();
			if( iRet == 1 )
			{
				/* add T.Tazaki 2005/06/29 >>> */
				ulPC =  read_pc();
				
			    if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == ulPC )
		        {
			        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1];
				}
				else
				{
			        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1];
				}
				gdbsim_xfer_inferior_memory(ulPC, (char *)&usCode, 2, 1, NULL, NULL);

				/* STEP */
				do
				{
					iStatus = fnExeEachInst( usCode);
					if ( fnGetExtType( usCode ) == EXT_NONE ) // not all ext
					{
						break;
					}
					ulAddr = gdbsim_fetch_register(PC);
					gdbsim_xfer_inferior_memory(ulAddr, (char *)&usCode, 2, 0, NULL, NULL);
				}while( 1 );

		        usCode = SOFT_BREAK_INST;
				gdbsim_xfer_inferior_memory(ulPC, (char *)&usCode, 2, 1, NULL, NULL);
				continue;
				/* add T.Tazaki 2005/06/29 <<< */

//				vfnIncPC (CODE);		/* Stdio soft break : PC += CODE */ /* del T.Tazaki 2005/06/27 */
			}
			else
			{
				break;					/* Normal soft break */
			}
		}
	}
	if ((iStatus & ILL_DELAY_BREAK ) != 0)	/* illegal delay instruction ? */
	{
		puts_filtered( "Illegal delayed instruction.\n" );
		g_iExtExceptCnt = 0;			/* 2005/04/15 */
		(void) gdbsim_store_register (PC, stDelaySlot.ulAddr - 4);

		stDelaySlot.blUsed = NO;
		stDelaySlot.blRunDelayed = NO;
		break;
	}
	
	
	/* >>>>> add iruma m.takeishi '03.09.02 */
	if ( stExtension.iTypeExtCond != EXTCOND_SKIP ) {
	/* <<<<< change iruma m.takeishi '03.09.02 */
	
		if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */

			/* 		add T.Tazaki 2005/03/14 >>> */
			if( (iStatusForTrace & S_ZERO_EX ) != 0 ||		   /* Zero division exception ? */
				(iStatusForTrace & S_ADDR_EX ) != 0 )		   /* illigal address exception ? */
			{
				(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulAddr);
			}
			
			if( ( ulCode & 0xe000 ) == 0xc000 )	/* ext imm13 ? */
			{				
				c33_TraceDump_Save_ext( ulAddr, szStatusBuf );
				strcpy( g_szStatusBufForExtExcept[g_iExtExceptCnt] , szStatusBuf );
				++g_iExtExceptCnt;
			}
			else
			{
				/* add T.Tazaki 2005/03/29 >>> */
				if ((iStatusForTrace & 0x3ffff) == 0 || (iStatusForTrace & DELAY_FLAG) == DELAY_FLAG)	/* No error break ? or DELAYED */
				{
					c33_TraceDump_Output_ext();	
					g_iExtExceptCnt = 0;		
				}
				/* add T.Tazaki 2005/03/29 <<< */

				c33_TraceDump( ulAddr, szStatusBuf );
			}

		}
	
		/*@ count up instruction number */
		g_ulInstNum++;
		
		/*==========================================================*/
		/* Delayed Instruntion										*/
		/*==========================================================*/
		
		if ((iStatus & DELAY_FLAG) == DELAY_FLAG)
		{
			/*@ record that next code is read from delayed slot */
			stDelaySlot.blUsed = YES;
	
			/*@ mask delay flag */
			iStatus &= ~DELAY_FLAG;
	
			/*@ record next address of ".d" instruction */
			stDelaySlot.ulNewPC = gdbsim_fetch_register(PC);
			
			step = 2;		/* one more loop */
		}
		else
		{
			/*@ record that next code is not read from delayed slot */
			stDelaySlot.blUsed = NO;
		}
	
		/*@ check if this instruction is delayed */
		if (stDelaySlot.blRunDelayed == YES) {
			/*@ restore address that previous branch instruction set */
			(void) gdbsim_store_register (PC, stDelaySlot.ulNewPC);
			
		}
	
		/*==========================================================*/
		/* PSR : LM Flag check										*/
		/*==========================================================*/
	
		
		/* >>>>> change iruma m.takeishi '04.03.18 */
		if ((ulReg[PSR] & LM_FLAG) != 0 || iIsLcoDown_Loop == 0 ) {
			if ( iIsLcoDown_Loop == 0 ) {
				if (ulReg[LCO] > 0) {
				
					/* >>>>> add iruma m.takeishi '04.03.18 */
					if ( ((iStatus & S_ZERO_EX		  ) ==0) && 
						 ((iStatus & S_ADDR_EX		  ) ==0) && 
						 ((ulCode  & 0xfffc) != 0x480 ) /* int imm2 */ ) {
						/*@ if LCO > 0, load LSA to PC */
						(void) gdbsim_store_register (PC, ulReg[LSA]);
					}
	
					/*@ decrement of LCO */
					(void) gdbsim_store_register (LCO, --ulReg[LCO]);
				}
				else {
					/*@ LM flag clear */
					ulReg[PSR] &= ~LM_FLAG;
					(void) gdbsim_store_register (PSR, ulReg[PSR]);
					
					iIsLoop_int = 0;
				}
			}
		}
	
		/*==========================================================*/
		/* PSR : RM Flag check										*/
		/*==========================================================*/
	
		else if ((ulReg[PSR] & RM_FLAG) != 0) {
	
			ulPC = gdbsim_fetch_register( PC );
	
			/* check first insn after "repeat" insn ? */
			if( iRepeatFirst == 1 )
			{
				// This is not Reserve-repeat mode.
				iRepeatInst = 1;
				iRet = 0;
				/* <<<<< change iruma m.takeishi '03.09.01 */
				iRepeatFirst = 0;
			}
			else
			{
				/* >>>>> change iruma m.takeishi '03.08.29 */
				/* Fixed for reserved repeat oparation */
				/*
				ulPC_temp = ulPC - CODE;
				*/
				iRet = 1;
				if ( iLSA_Hit == 0 ) {
					if ( ulPC - CODE == ulReg[LSA] ) {
					
						if ( iRepeatInst != 1 &&
							( ( stExtension.iType != EXT_NONE ) ||
							 ( stExtension.iTypeExt != EXT_NONE ) ) ) {
							 // if not repeat inst, and 
							 //  now reserve-repeat mode, executing "ext" inst.
							iLSA_Hit = 1;  // LSA hit, ext inst!
							iRet = 0;	   // do next inst
						}
						
					} else {
						// repeat mode is not started.
						iRet = 0;
					}
				} else {
					// Now Reserve-repeat mode
					if ( ( stExtension.iType != EXT_NONE ) ||
						 ( stExtension.iTypeExt != EXT_NONE ) ) {
						iRet = 0;
					} else {
						iLSA_Hit = 0;
					}
				}
				/* <<<<< change iruma m.takeishi '03.08.29 */
			}
	
			/* >>>>> change iruma m.takeishi '03.08.29 */
			/*
			if( ulPC_temp == ulReg[LSA] )
			*/
			if ( iRet == 1 )
			/* <<<<< change iruma m.takeishi '03.08.29 */
			{
				if (ulReg[LCO] > 0) {
					
					/*@ load LSA to PC for repeat */
					(void) gdbsim_store_register (PC, ulReg[LSA]);
	
					/*@ decrement of LCO */
					(void) gdbsim_store_register (LCO, --ulReg[LCO]);
				}
				else
				{
					/*@ RM flag clear */
					ulReg[PSR] &= ~RM_FLAG;
					(void) gdbsim_store_register (PSR, ulReg[PSR]);
					/* >>>>> add iruma m.takeishi '03.08.29 */
					iRepeatInst = 0;
					/* <<<<< add iruma m.takeishi '03.08.29 */
				}
			}
		}
	
		/*@ register dump, if necessary */
		if( g_iTraceDump == TM_ON )
		{
			if( ( ulCode & 0xe000 ) == 0xc000 )	/* ext imm13 ? */
			{
				if( g_iTraceFormat == TRACE_REGISTER || g_iTraceFormat == TRACE_REGISTER_SOURCE )
				{
					vfnAddRegDump_Save_ext ( szStatusBuf );	/*  2005/04/14 */
					strcpy( g_szStatusBufForExtExcept[g_iExtExceptCnt] , szStatusBuf );
					++g_iExtExceptCnt;
				}
			}
			else
			{
				vfnAddRegDump ();
			}
		}

	}

	iExtInsn = 0;
	/* >>>>> change iruma m.takeishi '03.09.01 */
	/*
	if( (ulCode & 0xe000 ) == 0xc000 ){ // ext ? //
		iExtInsn = 1;	// Not Break : next instruntion step //
	}
	*/
	if ( fnGetExtType( (unsigned short)ulCode ) != EXT_NONE ) { // not all ext
		iExtInsn = 1;
		if ( stExtension.iTypeExtCond == EXTCOND_GO ) {
			/* output ext-cond inst for trace, and next-inst's trace outputing is */
			/*	not done. */
			stExtension.iTypeExtCond = EXTCOND_SKIP;
		}
	} else {
		
		if ( stExtension.iTypeExtCond == EXTCOND_SKIP ||	
			 stExtension.iTypeExtCond == EXTCOND_EXEC ) {

			/* >>>>> add iruma m.takeishi '04.03.25 */
			/* if below code doesn't exist, 
			 * 2 inst fetch cycle is not added when 2 inst is	
			   prefetched in skipping. */
			if ( stExtension.iTypeExtCond == EXTCOND_SKIP ) {
				stC33_CachedCode.iCached = 0;
			}
			/* <<<<< add iruma m.takeishi '04.03.25 */

			 // off ext cond mode.
			stExtension.iTypeExtCond = EXTCOND_NONE;
		}
		
	}
	/* <<<<< change iruma m.takeishi '03.09.02 */

	/* Temporary break ? add T.Tazaki 2003/11/10 >>> */
	ulCurrentAddr = gdbsim_fetch_register(PC);
	if ( g_temp_brk == 1 && g_ulTemp_brk_addr == ulCurrentAddr )
	{
		/*  add T.Tazaki 2005/05/13 >>> */
		extern int stop_after_trap;			
		if( g_iSimNextCommand == 1 )
		{
			stop_after_trap = 1;				
		}
		/* bug fix  add T.Tazaki 2005/05/13 <<< */
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
		   printf_filtered( "<gdbsim_resume> break by temprary. address=[%x]\n",(int)ulCurrentAddr);
		}
//		/* >>>>> add iruma m.takeishi '04.04.13 */
//		{
//		  extern int stop_after_trap;
//		  stop_after_trap = 1;
//		}
		break;
	}
	/* Temporary break ? add T.Tazaki 2003/11/10 <<< */

	/* check if ulAddr is soft pc break		add T.Tazaki 2003/11/07 <<< */


  }
  while(step != 1 || iExtInsn == 1 );


  c33_Gdbtk_Idle ();	/* add 2003/02/24 T.Tazaki */

  signal (SIGINT, prev_sigint_bk);/* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */

  g_iResume = FALSE;

  resume_siggnal = siggnal;
  resume_step = step;
  
}
/* add T.Tazaki 2005/03/14 >>> */
/****************************************************************************/
/* Format		 : static void c33_TraceDump( unsigned long ulAddr, char *pszStatusBuf ) */
/* Arguments																*/
/*	unsigned long ulAddr	 : Target trace address 						*/
/*	char *pszStatusBuf		 : trace output buffer pointer					*/
/* Return value 															*/
/* Explanation	 : The trace dump for exception interrupt.					*/
/****************************************************************************/
static void
c33_TraceDump( unsigned long ulAddr, char *pszStatusBuf  )
{
	int		iLoop;
	char	szTmp[100];

	c33_Gdbtk_Idle ();

	if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
	{
		c33_SimTraceGetSorceCode( ulAddr, szSourceLine, sizeof(szSourceLine)  );
	}
	
	if (g_iStatusBufCnt == 0)
	{
		(void) strcat (pszStatusBuf, "-------- -- --------");
		sprintf( szTmp," %08u",(unsigned int)gulCycle );
		(void) strcat (pszStatusBuf, szTmp);

		if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
		{
			(void) strcat (pszStatusBuf, "   " );
			if ( STATUS_WORK_BUFFER_MAX -1 >= strlen( pszStatusBuf ) + strlen( szSourceLine ) ) {
				strcat( pszStatusBuf, szSourceLine );
			} else {
				strncat( pszStatusBuf, szSourceLine, STATUS_WORK_BUFFER_MAX-1 - strlen( pszStatusBuf ) );
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
			}
		}
				
		if( Trace_fp != NULL )	
		{
			if ( STATUS_WORK_BUFFER_MAX-2 >= strlen( pszStatusBuf )) {
				strcat( pszStatusBuf, "\n" );
			} else {
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-2 ] = '\n';
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
			}
		}
		c33_PutsTraceString( pszStatusBuf );
	}
	else
	{
		for( iLoop = 0; iLoop < g_iStatusBufCnt; ++iLoop )
		{
			if ( iLoop == 0 ) {
				(void) strcat (pszStatusBuf, g_szStatusBuf[0]);
			} else {
				(void) strcpy (pszStatusBuf, g_szStatusBuf[iLoop]);
			}
			if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
			{
				if( iLoop == 0 )
				{
					(void) strcat (pszStatusBuf, "   " );
					if ( STATUS_WORK_BUFFER_MAX -1 >= strlen( pszStatusBuf ) + strlen( szSourceLine ) ) {
						strcat( pszStatusBuf, szSourceLine );
					} else {
						strncat( pszStatusBuf, szSourceLine, STATUS_WORK_BUFFER_MAX-1 - strlen(pszStatusBuf) );
						pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
					}
				}
			}
			if( Trace_fp != NULL )
			{
				if ( STATUS_WORK_BUFFER_MAX-2 >= strlen( pszStatusBuf ) ) {
					strcat( pszStatusBuf, "\n" );
				} else {
					pszStatusBuf[ STATUS_WORK_BUFFER_MAX-2 ] = '\n';
					pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
				}
			}
			c33_PutsTraceString( pszStatusBuf );
		}
		g_iStatusBufCnt = 0;		/* clear buffer memory access(r/w) count */
	}
}
/* add T.Tazaki 2005/03/14 <<< */

/* add T.Tazaki 2005/03/16 >>> */
/****************************************************************************/
/* Format		 : static void c33_TraceDump_Save_ext( unsigned long ulAddr, char *pszStatusBuf ) */
/* Arguments																*/
/*	unsigned long ulAddr	 : Target trace address 						*/
/*	char *pszStatusBuf		 : trace output buffer pointer					*/
/* Return value 															*/
/* Explanation	 : The trace dump data save for ext .						*/
/****************************************************************************/
static void
c33_TraceDump_Save_ext( unsigned long ulAddr, char *pszStatusBuf  )
{
	int		iLoop = 0;
	int		iLen = 0;
	char	szTmp[100];


	if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
	{
		c33_SimTraceGetSorceCode( ulAddr, szSourceLine, sizeof(szSourceLine)  );
	}
	
	if (g_iStatusBufCnt == 0)	/* Memory Access None ? */
	{
		(void) strcat (pszStatusBuf, "-------- -- --------");
		sprintf( szTmp," %08u",(unsigned int)gulCycle );
		(void) strcat (pszStatusBuf, szTmp);

		if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
		{
			(void) strcat (pszStatusBuf, "   " );
			if ( STATUS_WORK_BUFFER_MAX -1 >= strlen( pszStatusBuf ) + strlen( szSourceLine ) ) {
				strcat( pszStatusBuf, szSourceLine );
			} else {
				strncat( pszStatusBuf, szSourceLine, STATUS_WORK_BUFFER_MAX-1 - strlen( pszStatusBuf ) );
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
			}
		}
				
		if( Trace_fp != NULL )	
		{
			if ( STATUS_WORK_BUFFER_MAX-2 >= strlen( pszStatusBuf )) {
				strcat( pszStatusBuf, "\n" );
			} else {
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-2 ] = '\n';
				pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
			}
		}
	}
	else
	{
		for( iLoop = 0; iLoop < g_iStatusBufCnt; ++iLoop )
		{
			if ( iLoop == 0 ) {
				(void) strcat (pszStatusBuf, g_szStatusBuf[0]);
			} else {
				(void) strcpy (pszStatusBuf, g_szStatusBuf[iLoop]);
			}
			if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
			{
				if( iLoop == 0 )
				{
					(void) strcat (pszStatusBuf, "   " );
					if ( STATUS_WORK_BUFFER_MAX -1 >= strlen( pszStatusBuf ) + strlen( szSourceLine ) ) {
						strcat( pszStatusBuf, szSourceLine );
					} else {
						strncat( pszStatusBuf, szSourceLine, STATUS_WORK_BUFFER_MAX-1 - strlen(pszStatusBuf) );
						pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
					}
				}
			}
			if( Trace_fp != NULL )
			{
				if ( STATUS_WORK_BUFFER_MAX-2 >= strlen( pszStatusBuf ) ) {
					strcat( pszStatusBuf, "\n" );
				} else {
					pszStatusBuf[ STATUS_WORK_BUFFER_MAX-2 ] = '\n';
					pszStatusBuf[ STATUS_WORK_BUFFER_MAX-1 ] = '\0';
				}
			}
		}
		g_iStatusBufCnt = 0;		/* clear buffer memory access(r/w) count */
	}
}

/****************************************************************************/
/* Format		 : static void c33_TraceDump_Output_ext( void )				*/
/* Arguments																*/
/* Return value 															*/
/* Explanation	 : The trace dump output ext.								*/
/****************************************************************************/
static void
c33_TraceDump_Output_ext()
{
	int 	iLoop;
	char	szStatusBuf[STATUS_WORK_BUFFER_MAX];
	char 	*pszStatusBuf;

	
	if( g_iExtExceptCnt > 0 )
	{
		if( g_iTraceFormat == TRACE_NORMAL || g_iTraceFormat == TRACE_SOURCE )
		{
			pszStatusBuf = szStatusBuf;
			strcpy( pszStatusBuf,g_szStatusBufForExtExcept[0] );
			
			if( g_iExtExceptCnt > 1 )
			{
				if( Trace_fp == NULL )
				{
					pszStatusBuf[ strlen(g_szStatusBufForExtExcept[0]) ]	 = '\n';
					pszStatusBuf[ strlen(g_szStatusBufForExtExcept[0]) + 1 ] = '\0';
				}
				strcat( pszStatusBuf,g_szStatusBufForExtExcept[1] );
			}
			c33_PutsTraceString( pszStatusBuf );
		}
		else
		{
			for( iLoop = 0; iLoop < g_iExtExceptCnt; ++iLoop )
			{
				c33_PutsTraceString( g_szStatusBufForExtExcept[iLoop] );
			}
		}
	}
}

/* add T.Tazaki 2005/03/16 <<< */



/* Notify the simulator of an asynchronous request to stop.

   The simulator shall ensure that the stop request is eventually
   delivered to the simulator.	If the call is made while the
   simulator is not running then the stop request is processed when
   the simulator is next resumed.

   For simulators that do not support this operation, just abort */

static void
gdbsim_stop ()
{
  giKeybreak = 1;	/* 2002/08/22 T.Tazaki add */
/* 2002/08/22 T.Tazaki del	
  if (!sim_stop (gdbsim_desc))
	{
	  quit ();
	}
*/
/* add T.Tazaki 2003/03/24 >>> */
	/* Set ESC break request */
	c33TargetInfo.iKeyBreak = 1;
/* add T.Tazaki 2003/03/24 <<< */

}

/* GDB version of os_poll_quit callback.
   Taken from gdb/util.c - should be in a library */

static int
gdb_os_poll_quit (host_callback *p)
{
  if (ui_loop_hook != NULL)
	ui_loop_hook (0);

  if (quit_flag)		/* gdb's idea of quit */
	{
	  quit_flag = 0;		/* we've stolen it */
	  return 1;
	}
  else if (immediate_quit)
	{
	  return 1;
	}
  return 0;
}

/* Wait for inferior process to do something.  Return pid of child,
   or -1 in case of error; store status through argument pointer STATUS,
   just as `wait' would. */

static void
gdbsim_cntrl_c (signo)
	 int signo;
{
  /* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */
  /* gdbsim_stop (); */
  giKeybreak = 1;
  /* 2002/01/21  C.Nakayama Add Cnrl-C function.(GNU33cygwin) */
}

static ptid_t
gdbsim_wait (ptid_t ptid, struct target_waitstatus *status)
{

  static RETSIGTYPE (*prev_sigint) ();
  int sigrc = 0;
  enum sim_stop reason = sim_running;

  if (sr_get_debug ())
	printf_filtered ("gdbsim_wait\n");

//#if defined (HAVE_SIGACTION) && defined (SA_RESTART)
//	{
//	  struct sigaction sa, osa;
//	  sa.sa_handler = gdbsim_cntrl_c;
//	  sigemptyset (&sa.sa_mask);
//	  sa.sa_flags = 0;
//	  sigaction (SIGINT, &sa, &osa);
//	  prev_sigint = osa.sa_handler;
//	}
//#else
  prev_sigint = signal (SIGINT, gdbsim_cntrl_c);
//#endif

  status->kind = TARGET_WAITKIND_STOPPED;
  status->value.sig = TARGET_SIGNAL_TRAP;

  signal (SIGINT, prev_sigint);
  resume_step = 0;
  
#if 0

  sim_stop_reason (gdbsim_desc, &reason, &sigrc);

  switch (reason)
	{
	case sim_exited:
	  status->kind = TARGET_WAITKIND_EXITED;
	  status->value.integer = sigrc;
	  break;
	case sim_stopped:
	  switch (sigrc)
	  {
	  case SIGABRT:
		quit ();
		break;
	  case SIGINT:
	  case SIGTRAP:
	  default:
		status->kind = TARGET_WAITKIND_STOPPED;
		/* The signal in sigrc is a host signal.  That probably
		   should be fixed.  */
		status->value.sig = target_signal_from_host (sigrc);
		break;
	  }
	  break;
	case sim_signalled:
	  status->kind = TARGET_WAITKIND_SIGNALLED;
	  /* The signal in sigrc is a host signal.	That probably
		 should be fixed.  */
	  status->value.sig = target_signal_from_host (sigrc);
	  break;
	case sim_running:
	case sim_polling:
	  /* FIXME: Is this correct? */
	  break;
	}
	
#endif

  return inferior_ptid;
}

/* Get ready to modify the registers array.  On machines which store
   individual registers, this doesn't need to do anything.	On machines
   which store all the registers in one fell swoop, this makes sure
   that registers contains all the registers from the program being
   debugged.  */

static void
gdbsim_prepare_to_store ()
{
  /* Do nothing, since we can store individual regs */
}



/********************************************************************
 * simulation memory read or write *
 ********************************************************************/

static int
gdbsim_xfer_inferior_memory (CORE_ADDR memaddr, char *myaddr, int len,
				 int write, 
				 struct mem_attrib *attrib ATTRIBUTE_UNUSED,
				 struct target_ops *target ATTRIBUTE_UNUSED)
{

	unsigned long ulData = 0;
	int iStatus = 0;
	int iStatus2 = 0;
	int iType;
	unsigned char *pucMyAddr;
	unsigned char ucChar;
	unsigned char *pucChar = &ucChar;
	
	extern struct BreakData stBreakData;
	unsigned char ucData1,ucData2,ucData3,ucData4;
	int iEndian = C33_LITTLE;
	

  if (sr_get_debug ())
	{
	  /* FIXME: Send to something other than STDOUT? */
	  printf_filtered ("gdbsim_xfer_inferior_memory: myaddr 0x");
	  gdb_print_host_address (myaddr, gdb_stdout);
	  printf_filtered (", memaddr 0x%s, len %d, write %d\n",
			   paddr_nz (memaddr), len, write);
	  if (sr_get_debug () && write)
	dump_mem (myaddr, len);
	}


	/* check if program execute */
	if( g_iExtEachInst == TRUE )
	{
		gdb_AddCycle(gdb_GetMemoryAccessCycle(memaddr, write, len));
	}

	iType = BYTE_DATA;
	if( len == 2 ) iType = HALF_DATA;
	if( len % 4 == 0 ) iType = WORD_DATA;

	if ((memaddr % 4 == 0 && len % 4 == 0) || (memaddr % 2 == 0 && len == 2) || (len != 2 && len != 4))
	{
	   if (write)
	   {
		  /*************************************/
		  /* Memory Write					   */
		  /*************************************/

		  /* add T.Tazaki 2003/10/07 >>> */
		  /* check if 0x4812d is 0x59 and StandardMacro and Word Write to 0x48134 */
		  if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		  { 	
			  if( memaddr >= C33_ADDR_TTBR && memaddr <= (C33_ADDR_TTBR+3) )
			  {
				   sim_read (gdbsim_desc, 0x4812d, (unsigned char *)pucChar, 1);
				   if( *pucChar != 0x59 )
				   {
					   return iStatus;	 /* skip */
				   }
				   else
				   {
					  /* if 0x48137 is written, then ttbr write mask. */
					  if( ( (C33_ADDR_TTBR+3) >= memaddr ) && ( (C33_ADDR_TTBR+3) <= (memaddr+len-1) ) )
					  {
						  /* 0x4812d initialize */
						  *pucChar = 0x00;
						  sim_write (gdbsim_desc, 0x4812d, (unsigned char *)pucChar, 1);
					  }
				   }
			  }
		  }
		  /* add T.Tazaki 2003/10/07 <<< */

		  /* add T.Tazaki 2004/07/08 >>> */
		  else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
		  { 	
		  	  /* 0x48134 emulation TTBR */
			  if( memaddr == C33_ADDR_TTBR && iType == WORD_DATA )
			  {
				  pucMyAddr = myaddr;
				  ulData = *( pucMyAddr + 3 );
				  ulData <<= 8;
				  ulData += *( pucMyAddr + 2 );
				  ulData <<= 8;
				  ulData += *( pucMyAddr + 1 );
				  ulData <<= 8;
				  ulData += *pucMyAddr;
				  (void) gdbsim_store_register (TTBR, ulData);
			  }
		  }
		  /* add T.Tazaki 2004/07/08 <<< */

		  len = sim_write (gdbsim_desc, memaddr, (unsigned char *)myaddr, len);

		  if((stBreakData.iType == WRITE_ACCESS) || (stBreakData.iType == BOTH_ACCESS)){
			  if(memaddr == stBreakData.ulAddr){
				  iStatus2 |= DATA_BREAK;
			  }
		  }

		  /* trace information make : data access */
		  iEndian = fnGetEndian( memaddr );
		  if( g_iTraceDump == TM_ON ){

			  pucMyAddr = myaddr;
			  switch( iType )
			  {
				case BYTE_DATA:
					ulData = *pucMyAddr;
					break;
				case HALF_DATA:
					if( iEndian == C33_BIG ){
						ulData = *pucMyAddr;
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
					}
					else
					{
						ulData = *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *pucMyAddr;
					}
					break;
				case WORD_DATA:
					if( iEndian == C33_BIG ){
						ulData = *pucMyAddr;
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 2 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 3 );
					}
					else
					{
						ulData = *( pucMyAddr + 3 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 2 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *pucMyAddr;
					}
					break;
			  }
			  if( g_iExtEachInst == TRUE )
			  {
				  vfnAddMemAccessInfo (memaddr, WRITE_ACCESS, iType, ulData, NO );
			  }
		  }

		  /* check if address is in map area */
		  iStatus = c33_CheckMapArea( memaddr );
		  if( iStatus != 0 )
		  {
			iStatus |= iStatus2;
			return iStatus;
		  }

		  /* check if address is in ROM area */
		  iStatus = fnCheckRomArea( memaddr );
		  iStatus |= iStatus2;
		  return iStatus;

	   }
	   else
	   {

		  /*************************************/
		  /* Memory Read					   */
		  /*************************************/

		  len = sim_read (gdbsim_desc, memaddr, (unsigned char *)myaddr, len);

		  if((stBreakData.iType == READ_ACCESS) || (stBreakData.iType == BOTH_ACCESS)){
			  if(memaddr == stBreakData.ulAddr){
				  iStatus2 |= DATA_BREAK;
			  }
		  }

		  /* trace information make : data access */
		  iEndian = fnGetEndian( memaddr );
		  if( g_iTraceDump == TM_ON ){

			  pucMyAddr = myaddr;

			  switch( iType )
			  {
				case BYTE_DATA:
					ulData = *pucMyAddr;
					break;
				case HALF_DATA:
					if( iEndian == C33_BIG ){
						ulData = *pucMyAddr;
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
					}
					else
					{
						ulData = *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *pucMyAddr;
					}
					break;
				case WORD_DATA:
					if( iEndian == C33_BIG ){
						ulData = *pucMyAddr;
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 2 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 3 );
					}
					else
					{
						ulData = *( pucMyAddr + 3 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 2 );
						ulData <<= 8;
						ulData += *( pucMyAddr + 1 );
						ulData <<= 8;
						ulData += *pucMyAddr;
					}
					break;
			  }
			
			  if( g_iExtEachInst == TRUE )
			  {
				  vfnAddMemAccessInfo (memaddr, READ_ACCESS, iType, ulData, NO );
			  }
		  }
			
		  /* check if address is in map area */
		  iStatus = c33_CheckMapArea( memaddr );
		  iStatus |= iStatus2;
		  return iStatus;

		}
		
		return iStatus;
	}
	else
	{
	   return S_ADDR_EX;
	}
}

static void
gdbsim_files_info (target)
	 struct target_ops *target;
{
  char *file = "nothing";

  if (exec_bfd)
	file = bfd_get_filename (exec_bfd);

  if (sr_get_debug ())
	printf_filtered ("gdbsim_files_info: file \"%s\"\n", file);

  if (exec_bfd)
	{
	  printf_filtered ("\tAttached to %s running program %s\n",
			   target_shortname, file);
	  sim_info (gdbsim_desc, 0);
	}
}

/* Clear the simulator's notion of what the break points are.  */

static void
gdbsim_mourn_inferior ()
{
  if (sr_get_debug ())
	printf_filtered ("gdbsim_mourn_inferior:\n");

  remove_breakpoints ();
  generic_mourn_inferior ();
}

/********************************************************************
 * simulation insert pc break point *
 ********************************************************************/

static int
gdbsim_insert_breakpoint (addr, contents_cache)
	 CORE_ADDR addr;
	 char *contents_cache;
{
	int i;
	int iResult;


	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "1:<gdbsim_insert_breakpoint> address=[%x] g_ulTemp_brk_addr=[%x] g_temp_brk=[%d]\n",(int)addr,(int)g_ulTemp_brk_addr,g_temp_brk);
	}
	if ( g_temp_brk == 1 )
	{
		c33_GetBreakPoint();
		
		/* search soft pc break same address */
		for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
		{
			if( addr == g_SoftPcBreakTable[ i ] )
			{
				break;
			}
		}
		if (  i == g_SoftPcBreakTable_Index )
		{
			/* check if temp break is already setted. */
			if( g_ulTemp_brk_addr != C33_INVALID_ADDR )
			{
				return 0;	/* skip */
			}


			if( g_iSimNextCommand == 1 )
			{
				if( step_range_end != 1 )
				{
					/* check if argument of function is function.  example: sub (sub2(),sub3() );  add T.Tazaki 2005/04/27 >>> */
					if( c33_SimCheckManyCall( addr, step_range_end) == 1 )
					{
						g_ulTemp_brk_addr = step_range_end;    /* save temporary break address */
					}
					else
					{
						g_ulTemp_brk_addr = addr;			   /* save temporary break address */
					}
					/* chack if argument of function is function.  example: sub (sub2(),sub3() );  add T.Tazaki 2005/04/27 <<< */

					if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
					{
			   			printf_filtered( "2:<gdbsim_insert_breakpoint> g_ulTemp_brk_addr=[%x] g_temp_brk=[%d]\n",(int)g_ulTemp_brk_addr,g_temp_brk);
					}
				}
		   		return 0;
		    }
		    else
		    {
				g_ulTemp_brk_addr = addr;	 /* save temporary break address */
				if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
				{
		   			printf_filtered( "3:<gdbsim_insert_breakpoint> g_ulTemp_brk_addr=[%x] g_temp_brk=[%d]\n",(int)g_ulTemp_brk_addr,g_temp_brk);
				}
		   		return 0;
			}

		}
	}

	if( g_iSamePCMode == 1 )	/*   add T.Tazaki 2003/11/07 */
	{
		return 0;
	}

	/* if address is invalid address, skip this function.  add T.Tazaki 2003/10/14 */
	if ( addr == C33_INVALID_ADDR )
	{
		return 0;
	}

	/* Check same simulated i/o address */
	if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == addr )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return 1;
	}
	if( c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] == addr )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return 1;
	}

	/* Search a free break */
	for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
	{
		if ( c33TargetInfo.ulSoftPcBreak[i] == C33_BREAK_DISABLE )
		{
			break;
		}
	}
	/* Check break address*/
	if ( c33_BreakAddrCheck(addr, C33_SOFT_PC_BREAK) == C33_ERR )
	{
		return 1;
	}
	else
	{
		if ( i == C33_SOFT_PC_BREAK_MAX )
		{
			c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
			return 1;
		}
		else
		{
			c33TargetInfo.ulSoftPcBreak[i] = addr;
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				 printf_filtered( "4:<gdbsim_insert_breakpoint> SOFT PC Break SET addr=[%x] \n",(int)addr);
			}
		}
	}

/*  del T.Tazaki 2003/11/07 >>> */

/*
#ifdef SIM_HAS_BREAKPOINTS
  SIM_RC retcode;

  retcode = sim_set_breakpoint (gdbsim_desc, addr);

  switch (retcode)
	{
	case SIM_RC_OK:
	  return 0;
	case SIM_RC_INSUFFICIENT_RESOURCES:
	  return ENOMEM;
	default:
	  return EIO;
	}
#else
  return memory_insert_breakpoint (addr, contents_cache);
#endif
*/

/* del T.Tazaki 2003/11/07 <<< */

/* add T.Tazaki 2003/11/07 >>> */
	return 0;
/* add T.Tazaki 2003/11/07 <<< */
}

/********************************************************************
 * simulation remove pc break point *
 ********************************************************************/

static int
gdbsim_remove_breakpoint (addr, contents_cache)
	 CORE_ADDR addr;
	 char *contents_cache;
{

	int i;
#if 0 /* del T.Tazaki 2003/10/14 */
	addr &= 0xfffffffe;
#endif

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "1:<gdbsim_remove_breakpoint> address=[%x] g_temp_brk=[%d]\n",(int)addr,g_temp_brk);
	}

	if ( g_temp_brk == 1 && g_ulTemp_brk_addr == addr )
	{

		/* Next for C++ source  del T.Tazaki 2005/06/01 >>> */
//			g_temp_brk = 0; 		/* clear flag */
		/* Next for C++   del T.Tazaki 2005/06/01 <<< */
		
		g_ulTemp_brk_addr = C33_INVALID_ADDR;	/* clear temporary break address  2003/09/01 T.Tazaki */
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "3:<gdbsim_remove_breakpoint> g_ulTemp_brk_addr=[%x]\n",(int)g_ulTemp_brk_addr);
		}
		return 0;	/* skip */
	}

	if( g_iSamePCMode == 1 )	/*  add T.Tazaki 2003/11/07 */
	{
		return 0;
	}

	/* if address is invalid address, skip this function.  add T.Tazaki 2003/10/14 */
	if ( addr == C33_INVALID_ADDR )
	{
		return 0;
	}

#if 0
/* add T.Tazaki 2003/10/14 >>>>>> */
	/*  add T.Tazaki 2003/10/14 */
	if ( addr == g_iSaveSkipAddr )
	{
		g_iSaveSkipAddr = C33_INVALID_ADDR;
		return 0;
	}

	if ( g_temp_brk == 1 && g_ulTemp_brk_addr == addr )
	{
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "2:<gdbsim_remove_breakpoint> address=[%x]\n",(int)addr);
		}
		g_temp_brk = 0; 		/* clear flag */
		g_ulTemp_brk_addr = C33_INVALID_ADDR;	/* clear temporary break address  2003/09/01 T.Tazaki */
		return 0;	/* skip */
	}
/* add T.Tazaki 2003/10/14 <<<<<< */
#endif

	/* Search same address */
	for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
	{
		if ( c33TargetInfo.ulSoftPcBreak[i] == addr )
		{
			break;
		}
	}

	if ( i == C33_SOFT_PC_BREAK_MAX )
	{
		/*  add T.Tazaki 2004/08/02 >>> */
		if( g_iSimNextCommand == 1 )
		{
			/* Next for C++ source  del T.Tazaki 2005/06/01 >>> */
//			g_temp_brk = 0; 		/* clear flag */
			/* Next for C++   del T.Tazaki 2005/06/01 <<< */
			g_ulTemp_brk_addr = C33_INVALID_ADDR;	/* clear temporary break address  2003/09/01 T.Tazaki */
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
				printf_filtered( "2:<gdbsim_remove_breakpoint> g_ulTemp_brk_addr=[%x] g_temp_brk=[%d]\n",(int)g_ulTemp_brk_addr,g_temp_brk);
			}
			return 0;
		}
		/* add T.Tazaki 2004/08/02 <<< */
		c33_error (C33_ERR_MSG_BRK_SOFT_CLR, (int)addr);
		return 1;
	}
	else
	{
		c33TargetInfo.ulSoftPcBreak[i] = C33_BREAK_DISABLE;
		if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
		{
			printf_filtered( "3:<gdbsim_remove_breakpoint> SOFT PC Break DISABLE address=[%x]\n",(int)addr);
		}
	}

/*  del T.Tazaki 2003/11/07 >>> */
/*
#ifdef SIM_HAS_BREAKPOINTS
  SIM_RC retcode;

  retcode = sim_clear_breakpoint (gdbsim_desc, addr);

  switch (retcode)
	{
	case SIM_RC_OK:
	case SIM_RC_UNKNOWN_BREAKPOINT:
	  return 0;
	case SIM_RC_INSUFFICIENT_RESOURCES:
	  return ENOMEM;
	default:
	  return EIO;
	}
#else
  return memory_remove_breakpoint (addr, contents_cache);
#endif
*/
/* del T.Tazaki 2003/11/07 <<< */

	return 0;		/* add T.Tazaki 2003/11/07 */
}

/********************************************************************
 * simulation insert data break point *
 ********************************************************************/

int gdbsim_insert_watchpoint(addr, len, type)
  CORE_ADDR addr;
  int len;
  int type;
{
  extern struct BreakData stBreakData;

  stBreakData.ulAddr = addr;
  switch( type )
  {
	case 0:  stBreakData.iType = WRITE_ACCESS;
			 break;
	case 1:  stBreakData.iType = READ_ACCESS;
			 break;
	case 2:  stBreakData.iType = BOTH_ACCESS;
			 break;
	default:
			stBreakData.iType = NO_ACCESS;
			break;
  }
  return 0;
}

/********************************************************************
 * simulation remove data break point *
 ********************************************************************/

int gdbsim_remove_watchpoint(addr, len, type)
  CORE_ADDR addr;
  int len;
  int type;
{
  extern struct BreakData stBreakData;


  if( addr == stBreakData.ulAddr )
  {

	  stBreakData.iType = NO_ACCESS;

  }

  return 0;
}


/* Pass the command argument through to the simulator verbatim.  The
   simulator must do any command interpretation work.  */

void
simulator_command (args, from_tty)
	 char *args;
	 int from_tty;
{

/* del Do not used 2002/10/10 T.Tazaki >>> */
#if 0

	int iParamNum;
	char cCommand[C33_COMMAND_MAX + 1];
	char cParam1[C33_COMMAND_MAX + 1];
	char cParam2[C33_COMMAND_MAX + 1];
	char cParam3[C33_COMMAND_MAX + 1];
	int iParam1;
	int iParam2;
	int iParam3;
	int iParam4;
	char szBuf[256];

	iParamNum = sscanf(args, "%s %x %x %x %x", &cCommand[0], &iParam1, &iParam2, &iParam3, &iParam4);

  if (gdbsim_desc == NULL)
	{

	  c33_error ("Not connected to the simulator target");
	  return;
	}

	if (strcmp(args, "rstc") == 0){
		gdbsim_reset(1);
		gdbsim_fetch_register(-1);
		stop_pc = read_pc ();
	}
	else if (strcmp(args, "rsth") == 0){
		gdbsim_reset(0);
		gdbsim_fetch_register(-1);
		stop_pc = read_pc ();
	}
	else if (strcmp(cCommand,"tm") == 0)
	{
		iParamNum = sscanf(args, "%s %s %s", &cCommand[0], &cParam1[0], &cParam2[0]);
		gdb_TraceMode(iParamNum, cParam1, cParam2);
	}
	else if (strcmp(args,"cycle") == 0){
		sprintf(szBuf,"Cycle %u\n", (unsigned int)gulCycle);
		puts_filtered (szBuf);
	}
	else if (strcmp(args,"cycle0") == 0){
		gulCycle = 0;
		sprintf(szBuf,"Cycle %u\n", (unsigned int)gulCycle);
		puts_filtered (szBuf);
	}
	else {
		c33_error (C33_ERR_COMMAND_INVALID);
		return;
  }


  registers_changed ();

#endif
/* del Do not used 2002/10/10 T.Tazaki <<< */

}


/* Define the target subroutine names */

struct target_ops gdbsim_ops;

static void
init_gdbsim_ops (void)
{

  gdbsim_ops.to_shortname = "sim";
  gdbsim_ops.to_longname = "simulator";
  gdbsim_ops.to_doc = "Use the compiled-in simulator.";
  gdbsim_ops.to_open = gdbsim_open;
  gdbsim_ops.to_close = gdbsim_close;
  gdbsim_ops.to_attach = NULL;
  gdbsim_ops.to_post_attach = NULL;
  gdbsim_ops.to_require_attach = NULL;
  gdbsim_ops.to_detach = gdbsim_detach;
  gdbsim_ops.to_require_detach = NULL;
  gdbsim_ops.to_resume = gdbsim_resume;
  gdbsim_ops.to_wait = gdbsim_wait;
  gdbsim_ops.to_post_wait = NULL;
  gdbsim_ops.to_fetch_registers = (void *)gdbsim_fetch_register;

/*		  2002/01/31  C.Nakayama Add special registers.(GNU33cygwin) */
  gdbsim_ops.to_store_registers = gdbsim_store_register_cmd;

  gdbsim_ops.to_prepare_to_store = gdbsim_prepare_to_store;
  gdbsim_ops.to_xfer_memory = gdbsim_xfer_inferior_memory;
  gdbsim_ops.to_files_info = gdbsim_files_info;
  gdbsim_ops.to_insert_breakpoint = gdbsim_insert_breakpoint;
  gdbsim_ops.to_remove_breakpoint = gdbsim_remove_breakpoint;
  gdbsim_ops.to_terminal_init = NULL;
  gdbsim_ops.to_terminal_inferior = NULL;
  gdbsim_ops.to_terminal_ours_for_output = NULL;
  gdbsim_ops.to_terminal_ours = NULL;
  gdbsim_ops.to_terminal_info = NULL;
  gdbsim_ops.to_kill = gdbsim_kill;
  gdbsim_ops.to_load = gdbsim_load;
  gdbsim_ops.to_lookup_symbol = NULL;
  gdbsim_ops.to_create_inferior = gdbsim_create_inferior;
  gdbsim_ops.to_post_startup_inferior = NULL;
  gdbsim_ops.to_acknowledge_created_inferior = NULL;
  gdbsim_ops.to_clone_and_follow_inferior = NULL;
  gdbsim_ops.to_post_follow_inferior_by_clone = NULL;
  gdbsim_ops.to_insert_fork_catchpoint = NULL;
  gdbsim_ops.to_remove_fork_catchpoint = NULL;
  gdbsim_ops.to_insert_vfork_catchpoint = NULL;
  gdbsim_ops.to_remove_vfork_catchpoint = NULL;
  gdbsim_ops.to_has_forked = NULL;
  gdbsim_ops.to_has_vforked = NULL;
  gdbsim_ops.to_can_follow_vfork_prior_to_exec = NULL;
  gdbsim_ops.to_post_follow_vfork = NULL;
  gdbsim_ops.to_insert_exec_catchpoint = NULL;
  gdbsim_ops.to_remove_exec_catchpoint = NULL;
  gdbsim_ops.to_has_execd = NULL;
  gdbsim_ops.to_reported_exec_events_per_exec_call = NULL;
  gdbsim_ops.to_has_exited = NULL;
  gdbsim_ops.to_mourn_inferior = gdbsim_mourn_inferior;
  gdbsim_ops.to_can_run = 0;
  gdbsim_ops.to_notice_signals = 0;
  gdbsim_ops.to_thread_alive = 0;
  gdbsim_ops.to_stop = gdbsim_stop;
  gdbsim_ops.to_pid_to_exec_file = NULL;
  gdbsim_ops.to_stratum = process_stratum;
  gdbsim_ops.DONT_USE = NULL;
  gdbsim_ops.to_has_all_memory = 1;
  gdbsim_ops.to_has_memory = 1;
  gdbsim_ops.to_has_stack = 1;
  gdbsim_ops.to_has_registers = 1;
  gdbsim_ops.to_has_execution = 1;
  gdbsim_ops.to_sections = NULL;
  gdbsim_ops.to_sections_end = NULL;
  gdbsim_ops.to_magic = OPS_MAGIC;

#ifdef TARGET_REDEFINE_DEFAULT_OPS
  TARGET_REDEFINE_DEFAULT_OPS (&gdbsim_ops);
#endif

}


void
_initialize_remote_sim ()
{
  init_gdbsim_ops ();
  add_target (&gdbsim_ops);

  add_com ("sim <command>", class_obscure, simulator_command,
	   "Send a command to the simulator.");
}
/****************************************************************************/
/* Format		 : int c33_SimSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5) */
/* Arguments																*/
/*	int iID 	 : Command index											*/
/*	int iParam1  : Command parameter1										*/
/*	int iParam2  : Command parameter2										*/
/*	int iParam3  : Command parameter3										*/
/*	int iParam4  : Command parameter4										*/
/*	void *pvPraram5: Command parameter5 									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The specified command is transmitted.					*/
/****************************************************************************/
int
c33_SimSendMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5)
{
	int iResult = C33_NO_ERR;

	/* Target open? */
	if(c33TargetInfo.iSim_open == 0)
	{
		/* Target closed */
		return C33_ERR;
	}
	else
	{
		c33TargetInfo.iErrorCode = C33_NO_ERR;
	}

	switch (iID)
	{
		case C33_ID_RESET:
			c33_SimReset(iParam1);
			break;
		case C33_ID_DATA_FILL:
			iResult = c33_SimDataFill(iParam1, iParam2, iParam3, iParam4);
			break;
		case C33_ID_DATA_MOVE:
			iResult = c33_SimMemoryMove(iParam1, iParam2, iParam3, iParam4);
			break;
		case C33_ID_DATA_WRITE:
			c33_SimMemoryWrite(iParam1, iParam2, iParam3, pvParam5);
			break;
		case C33_ID_DATA_READ:
			c33_SimMemoryRead(iParam1, iParam2, iParam3, pvParam5);
			break;
		case C33_ID_DATA_BREAK_SET:
			gdbsim_insert_watchpoint(iParam1, 0, iParam2);
			break;
		case C33_ID_DATA_BREAK_CLR:
			gdbsim_remove_watchpoint(iParam1, 0, iParam2);
			break;
		case C33_ID_HARD_PC_BREAK_SET:
		case C33_ID_HARD_PC_BREAK2:
		case C33_ID_HARD_PC_BREAK_CLR:
		case C33_ID_CAN:
			break;
			

		default:
			c33_error ( C33_ERR_COMMAND_INVALID );
			return C33_ERR;
	}

	return iResult;
}

/****************************************************************************/
/* Format		 : int c33_SimRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5) */
/* Arguments																*/
/*	int iID 	 : Command index											*/
/*	int iParam1  : Command parameter1										*/
/*	int iParam2  : Command parameter2										*/
/*	int iParam3  : Command parameter3										*/
/*	int iParam4  : Command parameter4										*/
/*	void *pvPraram5: Command parameter5 									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The specified command received.							*/
/****************************************************************************/
int
c33_SimRecvMessage (int iID, int iParam1, int iParam2, int iParam3, int iParam4, void *pvParam5)
{
	/* Target open? */
	if(c33TargetInfo.iSim_open == 0)
	{
		/* Target closed */
		return C33_ERR;
	}

	switch (iID)
	{
		case C33_ID_RESET:
		case C33_ID_DATA_FILL:
		case C33_ID_DATA_MOVE:
		case C33_ID_DATA_WRITE:
		case C33_ID_DATA_READ:
		case C33_ID_DATA_BREAK_SET:
		case C33_ID_DATA_BREAK_CLR:
		case C33_ID_CAN:
			if (c33TargetInfo.iErrorCode != C33_NO_ERR)
			{
				return C33_ERR;
			}
			break;
	}

	return C33_NO_ERR;
}

/****************************************************************************/
/* Format		 : void c33_SimReset (int iResetMode )						*/
/* Arguments																*/
/*	int iResetMode	: Reset Mode  COLD / HOT								*/
/* Return value  : void 													*/
/* Explanation	 : Simmulator reset.										*/
/****************************************************************************/
static void
c33_SimReset (int iResetMode )
{

	gdbsim_reset(iResetMode);
	
	gdbsim_fetch_register(-1);
	stop_pc = read_pc ();
	registers_changed ();

}
/****************************************************************************/
/* Format		 : void gdbsim_WindowUpdate ( void )						*/
/* Arguments																*/
/* Return value  : void 													*/
/* Explanation	 : window update after RESET. called by c33common			*/
/****************************************************************************/
void
gdbsim_WindowUpdate ()
{
	gdbsim_reset(0);
	
	gdbsim_fetch_register(-1);
	stop_pc = read_pc ();
	registers_changed ();
}

/* >>>>> delete iruma m.takeishi '03.12.19 */
#if 0
/***************************************
  *  gdbsim_reset : reset 33 core	 *
 ***************************************/
int 			iExceptionType; /* exception type */
int 			nTrapAfterNMI;	/* counter of traps after NMI */
int 			blStartNMI; /* NO : no NMI, YES : NMI */

struct StartINTR stStartINTR;	/* start INTR ? */
struct DelaySlot stDelaySlot;	/* delay slot */
struct Extension stExtension;	/* extension */
#endif
/* <<<<< delete iruma m.takeishi '03.12.19 */


void
gdbsim_reset (iMode)
int iMode;	/* 1:COLD, 0:HOT */
{				/* return void */
	extern int		iExceptionType; /* exexception type of previous inst */
	extern int		nTrapAfterNMI;
	extern int		blStartNMI;
	extern struct StartINTR stStartINTR;
	extern struct DelaySlot stDelaySlot;	/* for Delayed slot */
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	extern unsigned long ulVectorAddr;

	char		  ucPData;
	unsigned long ulData;

	if( fnCheckAdvancedMode() == C33_PRODUCT_STD )	/* add T.Tazaki 2003/09/10 */
	{
		/* in cold reset, reset TTBR & Vector address */
		if (iMode == 1)
		{
			/* Default vector set */
			/* TTBR write enabled */
			ucPData = 0x59;
			gdbsim_xfer_inferior_memory(0x4812d, &ucPData, 1, 1, NULL, NULL);
			
			/* TTBR set */
			ulData = VECTOR_ADDR_MPU;			/* 0x0 ---> 0xc00000 change T.Tazaki 2003/09/10 */
			gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulData, 4, 1, NULL, NULL);
			
			ucPData = 0x0;
			gdbsim_xfer_inferior_memory(0x4812d, &ucPData, 1, 1, NULL, NULL);
		}
	}
	
	gdbsim_reset_reg(iMode);

	/*@ exception type is "not happen" */
	iExceptionType = 0;

	/*@ no traps happened after NMI */
	nTrapAfterNMI = 0;

	/*@ don't start NMI */
	blStartNMI = NO;

	/*@ don't start maskable interrupt */
	stStartINTR.iType = NO_INTR;

	/*@ no delayed slot used */
	stDelaySlot.blUsed = NO;
	stDelaySlot.blRunDelayed = NO;
	
	/*@ initialize extension */
	stExtension.iType = EXT_NONE;
	stExtension.iTypeExt = EXT_NONE;/*		  2002/01/31  C.Nakayama Add special registers.(GNU33cygwin) */
	/* >>>>> add iruma m.takeishi '03.09.02 */
	stExtension.iTypeExtCond = EXTCOND_NONE;
	/* >>>>> add iruma m.takeishi '03.09.02 */

	/* Clock Counter Reset */
	gulCycle = 0;

	/*@ leave slp/halt mode */
	(void) fnDoSlpHalt (RESET);

	/* >>>>> add iruma m.takeishi '03.04.28 */
	giFrameClear = 1;
	/* <<<<< add iruma m.takeishi '03.04.28 */

	/* >>>>> add iruma m.takeishi '04.03.16 */
	iIsLoop_int = 0;
	/* <<<<< add iruma m.takeishi '04.03.16 */

	/* >>>>> add irumam.takeishi '04.03.25 */
	stC33_CachedCode.iCached = 0;
	/* <<<<< add irumam.takeishi '04.03.25 */

	sim_Gdbtk_Update ();

	return;
}

/************************************************
 *	fnDoSlpHalt : manage slp/halt mode	*
 ************************************************/
int blSlpHalt;

int
fnDoSlpHalt (iCommand)
	 int			 iCommand;
{				/* return 0: not slp/halt mode, 1: slp/halt mode */
	int 			iResult = 0;

	extern int		blSlpHalt;	/* 0: not slp/halt mode, 1: slp/halt mode */

	switch (iCommand) { 	/* command */

	case SET:			/* set slp/halt mode */

	/*@ enter slp/halt mode */
	blSlpHalt = iResult = YES;
	break;

	case CHECK_AND_RESET:	/* check slp/halt mode */

	/*@ prepare to return mode */
	iResult = blSlpHalt;

	/*@ leave slp/halt mode */
	blSlpHalt = NO;
	break;

	case RESET: 	/* reset slp/halt mode */

	/*@ leave slp/halt mode */
	blSlpHalt = iResult = NO;
	break;
	}

	return (iResult);		/* return reslut */
}

/************************************************
 *	 gdb sim initialize called by gdbsim_open *
 ************************************************/
static
short gdb_InitSim()
{
	unsigned long		 ulData;
	char				 ulPData;

#if 0
	gdbsim_reset(1);
#endif
	gdbsim_fetch_register(-1);

	/* Default vector set */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		gdbsim_store_register (TTBR,0x20000000UL );
	}
	else
	{
		ulPData = 0x59; /* TTBR write enable */
		gdbsim_xfer_inferior_memory(0x4812d, (char *)&ulPData, 1, 1, NULL, NULL);

		ulData = VECTOR_ADDR_MPU;
		gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulData, 4, 1, NULL, NULL);

		ulPData = 0x00; /* TTBR write disable */
		gdbsim_xfer_inferior_memory(0x4812d, (char *)&ulPData, 1, 1, NULL, NULL);

	}

   
#if 0
	gdbsim_reset(1);

	puts_filtered (C33_MSG_TARGET_COLD_RESET_INFO);
	puts_filtered (C33_MSG_TARGET_DONE);
#endif

	return(0);
}


/****************************************************************************/
/* Format		 : static int c33_SimDataFill(int iType, unsigned long ulStartAddr, unsigned long ulSize, unsigned long ulData) */
/* Arguments																*/
/*	int iType				  : Byte,half word,word 						*/
/*	unsigned long ulStartAddr : Start address								*/
/*	unsigned long ulEndAddr   : Size										*/
/*	unsigned long ulData	  : Fill data									*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Memory fill execution.									*/
/****************************************************************************/
static int
c33_SimDataFill(int iType, unsigned long ulStartAddr, unsigned long ulSize, unsigned long ulData)
{
	unsigned long	ulAccessSize = 1;
	unsigned long	ulEndAddr;

	/* Check boundary */
	if ( iType == C33_BYTE)
	{
		ulAccessSize = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		ulAccessSize = 2;
	}
	else if ( iType == C33_WORD)
	{
		ulAccessSize = 4;
	}

	ulEndAddr = ulStartAddr + ulSize;

	
	c33_InitCancel();	/* add 2003/03/19 T.Tazaki : Check Escape Key ON  */

	for( ; ulStartAddr < ulEndAddr; ulStartAddr += ulAccessSize){
		gdbsim_xfer_inferior_memory(ulStartAddr, (char *)&ulData, ulAccessSize, 1, NULL, NULL);

		/* add 2003/03/24 T.Tazaki : Check STOP button or ESC key >>>  */
		if( c33TargetInfo.iKeyBreak == 1 )
		{
			/* Memory fill cancel */
			c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
			return C33_ERR;
		}
		if( c33_CheckCancel( 0x040000 ) == 1 )		/* change 0x1000 ==> 0x040000 T.Tazaki 2003/10/15 */
		{
			/* Memory fill cancel */
			c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
			return C33_ERR;
		}
		/* add 2003/03/24 T.Tazaki : Check STOP button or ESC key <<<  */

	}

	c33_EndCancel();	/* Cancel Check End  add 2003/03/19 T.Tazaki : Check Escape Key ON	*/

	return C33_NO_ERR;

}

/****************************************************************************/
/* Format		 : static int c33_SimMemoryMove(int iType, unsigned long ulStartAddr, unsigned long ulDstAddr, unsigned long ulSize ) */
/* Arguments																*/
/*	int iType				  : Byte,half word,word 						*/
/*	unsigned long ulStartAddr : Start address								*/
/*	unsigned long ulDstAddr   : Destination address 						*/
/*	unsigned long ulSize	  : Size										*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : Memory move execution.									*/
/****************************************************************************/
static int
c33_SimMemoryMove(int iType, unsigned long ulStartAddr, unsigned long ulDstAddr, unsigned long ulSize)
{
	unsigned long ulAccessSize = 1;
	unsigned long ulData;
	unsigned long ulSrcAddr;

	/* Check boundary */
	if ( iType == C33_BYTE)
	{
		ulAccessSize = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		ulAccessSize = 2;
	}
	else if ( iType == C33_WORD)
	{
		ulAccessSize = 4;
	}

	c33_InitCancel();	/* add 2003/03/19 T.Tazaki : Check Escape Key ON  */

	for( ulSrcAddr = ulStartAddr; ulSrcAddr < (ulStartAddr + ulSize); )
	{
		gdbsim_xfer_inferior_memory(ulSrcAddr,	(char *)&ulData, ulAccessSize, 0, NULL, NULL);
		gdbsim_xfer_inferior_memory(ulDstAddr,	(char *)&ulData, ulAccessSize, 1, NULL, NULL);
		ulSrcAddr += ulAccessSize;
		ulDstAddr += ulAccessSize;
		
		/* add 2003/03/24 T.Tazaki : Check STOP button or ESC key >>>  */
		if( c33TargetInfo.iKeyBreak == 1 )
		{
			/* Memory fill cancel */
			c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
			return C33_ERR;
		}
		if( c33_CheckCancel( 0x040000 ) == 1 )		/* change 0x1000 ==> 0x040000 T.Tazaki 2003/10/15 */
		{
			/* Memory move cancel */
			c33TargetInfo.iErrorCode = C33_ERROR_RS_KEY_BREAK;
			return C33_ERR;
		}
		/* add 2003/03/24 T.Tazaki : Check STOP button or ESC key <<<  */

	}
	
	c33_EndCancel();	/* Cancel Check End  add 2003/03/19 T.Tazaki*/

	return C33_NO_ERR;

}

/****************************************************************************/
/* Format		 : static void c33_SimMemoryWrite(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr) */
/* Arguments																*/
/*	unsigned long ulAddr : Target address									*/
/*	unsigned long ulLen  : Data Length										*/
/*	int iUnit			 : C33_BYTE,C33_HALF_WORD,C33_WORD					*/
/*	char* pcMyAddr		 : Debugger address 								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The memory write is performed.							*/
/****************************************************************************/
static void
c33_SimMemoryWrite(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr)
{
	unsigned long ulAccessSize = 1;
	unsigned long ulEndAddr;

	/* Check boundary */
	if ( iType == C33_BYTE)
	{
		ulAccessSize = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		ulAccessSize = 2;
	}
	else if ( iType == C33_WORD)
	{
		ulAccessSize = 4;
	}

	ulEndAddr = ulStartAddr + ulLen;

	for( ; ulStartAddr < ulEndAddr; ulStartAddr += ulAccessSize){
		gdbsim_xfer_inferior_memory(ulStartAddr, pcMyAddr, ulAccessSize, 1, NULL, NULL);
		pcMyAddr += ulAccessSize;
	}
}
/****************************************************************************/
/* Format		 : static void c33_SimMemoryRead(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr) */
/* Arguments																*/
/*	unsigned long ulAddr : Target address									*/
/*	unsigned long ulLen  : Data Length										*/
/*	int iType			 : C33_BYTE,C33_HALF_WORD,C33_WORD					*/
/*	char* pcMyAddr		 : Debugger address 								*/
/* Return value 															*/
/*	C33_NO_ERR	 : Success return											*/
/*	C33_ERR 	 : Error return 											*/
/* Explanation	 : The memory read is performed.							*/
/****************************************************************************/
static void
c33_SimMemoryRead(unsigned long ulStartAddr, unsigned long ulLen, int iType, char *pcMyAddr)
{
	unsigned long ulAccessSize = 1;
	unsigned long ulEndAddr;

	/* Check boundary */
	if ( iType == C33_BYTE)
	{
		ulAccessSize = 1;
	}
	else if ( iType == C33_HALF_WORD)
	{
		ulAccessSize = 2;
	}
	else if ( iType == C33_WORD)
	{
		ulAccessSize = 4;
	}

	ulEndAddr = ulStartAddr + ulLen;

	for( ; ulStartAddr < ulEndAddr; ulStartAddr += ulAccessSize){
		gdbsim_xfer_inferior_memory(ulStartAddr, pcMyAddr, ulAccessSize, 0, NULL, NULL);
		pcMyAddr += ulAccessSize;
	}

}

/****************************************************************************/
/* Format		 : void c33_SimSoftPcBreakSet(unsigned long ulBreakAddr)	*/
/* Arguments																*/
/*	unsigned long ulBreakAddr : break target address						*/
/* Return value  : void 													*/
/* Explanation	 : original code save and soft break set.					*/
/****************************************************************************/
void
c33_SimSoftPcBreakSet(unsigned long ulBreakAddr)
{
	unsigned short usCode;
	int i;

	/* Search a free break */
	for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
	{
		if ( g_ulSoftBreak_Addr[i] == C33_BREAK_DISABLE )
		{
			break;
		}
	}

	/* original address save */
	g_ulSoftBreak_Addr[i] = ulBreakAddr;

#if 0	/* del T.Tazaki 2003/11/07 */
	/* original code save */
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&g_usSoftBreak_code[i], 2, 0, NULL, NULL);

	/* set Soft Pc break code */
	usCode = SOFT_BREAK_INST;
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&usCode, 2, 1, NULL, NULL);
#endif
}
/****************************************************************************/
/* Format		 : void c33_SimSoftPcBreakClear( unsigned long ulBreakAddr) */
/* Arguments																*/
/*	unsigned long ulBreakAddr : break target address						*/
/* Return value  : void 													*/
/* Explanation	 : soft break clear and original code set.					*/
/****************************************************************************/
void
c33_SimSoftPcBreakClear( unsigned long ulBreakAddr)
{
	int i;

	/* Search same address */
	
	for (i = 0 ; i < C33_SOFT_PC_BREAK_MAX ; i++)
	{
		if ( g_ulSoftBreak_Addr[i] == ulBreakAddr )
		{
			break;
		}
	}

	if ( g_ulSoftBreak_Addr[i] != C33_BREAK_DISABLE )
	{
#if 0	/* del T.Tazaki 2003/11/07 */
		/* original code load */
		gdbsim_xfer_inferior_memory(g_ulSoftBreak_Addr[i], (char *)&g_usSoftBreak_code[i], 2, 1, NULL, NULL);
#endif
		g_ulSoftBreak_Addr[i] = C33_BREAK_DISABLE;
		g_usSoftBreak_code[i] = 0;

	}
}


/****************************************************************************/
/* Format		 : void c33_SimStdoutSet(unsigned long ulBreakAddr) 		*/
/* Arguments																*/
/*	unsigned long ulBreakAddr : break target address						*/
/* Return value 															*/
/* Explanation	 : simulated out.											*/
/****************************************************************************/
int
c33_SimStdoutSet(unsigned long ulBreakAddr)
{
	unsigned short usCode;
	int iStatus = C33_NO_ERR;

	/* original code save */
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&usCode, 2, 0, NULL, NULL);

    c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1] = usCode;

	/* set Soft Pc break code */
	usCode = SOFT_BREAK_INST;
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&usCode, 2, 1, NULL, NULL);

	return iStatus;
}
/****************************************************************************/
/* Format		 : void c33_SimStdoutClear(void)							*/
/* Arguments																*/
/* Return value 															*/
/* Explanation	 : simulated i/o clear. 									*/
/****************************************************************************/
void
c33_SimStdoutClear()
{
	unsigned short usCode;

	/* original code load */
	usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1];
	gdbsim_xfer_inferior_memory(c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1], (char *)&usCode, 2, 1, NULL, NULL);

	c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] = C33_BREAK_DISABLE;	/*add T.Tazaki 2005/06/27 */

}
/****************************************************************************/
/* Format		 : void c33_SimStdinSet(unsigned long ulBreakAddr)			*/
/* Arguments																*/
/*	unsigned long ulBreakAddr : break target address						*/
/* Return value  : 0=NO ERROR,-1=ERROR										*/
/* Explanation	 : simulated i/o set.										*/
/****************************************************************************/
int
c33_SimStdinSet(unsigned long ulBreakAddr)
{
	unsigned short usCode;
	int iStatus = C33_NO_ERR;
	
	
	/* original code save */
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&usCode, 2, 0, NULL, NULL);

    c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1] = usCode;

	/* set Soft Pc break code */
	usCode = SOFT_BREAK_INST;
	gdbsim_xfer_inferior_memory(ulBreakAddr, (char *)&usCode, 2, 1, NULL, NULL);

	/* flag & read buffer index clear */
	g_c33_std_in = 0;
	g_c33_Index = 0;

	return iStatus;

}
/****************************************************************************/
/* Format		 : void c33_SimStdinClear(void) 							*/
/* Arguments																*/
/* Return value 															*/
/* Explanation	 : simulated i/o clear. 									*/
/****************************************************************************/
void
c33_SimStdinClear()
{
	unsigned short usCode;
	
	/* original code load */
	usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1];
	gdbsim_xfer_inferior_memory(c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] , (char *)&usCode, 2, 1, NULL, NULL);
	g_c33_std_in = 0;
	g_c33_Index = 0;

}

/****************************************************************************/
/* Format		 : static int fnStdio(void) 								*/
/* Arguments																*/
/* Return value  : 1=Stdio break, 0=normal soft break						*/
/* Explanation	 : simulated i/o break process. 							*/
/****************************************************************************/
static int
fnStdio()
{
	unsigned long ulAddr;
	unsigned char ucSize;
	unsigned char ucChar;
	unsigned char szString[100];
	char	 szbuf[1000];
	char	 *pszbuf;
	int 	 iChar;
	int 	 iCnt;
	int 	 iRet = 0;
	int 	 i;
	int 	 iLen;

	extern Tcl_Interp *gdbtk_interp;

	pszbuf = szbuf;

	ulAddr = gdbsim_fetch_register(PC);

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/05/17 */
	{
		printf_filtered( "fnStdio  PC=[%08x] STDIN_ADDR=[%08x] STDOUT_ADDR=[%08x]\n",(int)ulAddr,(int)c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1],
		(int)c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] );
	}

	/*===================*/
	/* check Stdin break */
	/*===================*/
	if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == ulAddr )
	{
		if( fpStdinfile != NULL )
		{
			/*-----------------------*/
			/* read buffer from file */
			/*-----------------------*/

			ulAddr = c33TargetInfo.ulReadBufAddr + 1;
			iCnt = 0;
			do
			{
				iChar = fgetc( fpStdinfile );
				if( iChar == EOF )
				{
/* del T.Tazaki 2005/06/27 >>> */
//		            c33_SimStdinClear();
//			        c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] = C33_BREAK_DISABLE;
/* del T.Tazaki 2005/06/27 <<< */
/* del T.Tazaki 2005/06/15 >>> */
//					fclose( fpStdinfile );
//					fpStdinfile = NULL;
/* del T.Tazaki 2005/06/15 <<< */
					break;
				}
				gdbsim_xfer_inferior_memory(ulAddr, (char *)&iChar, 1, 1, NULL, NULL); /* read buffer set */
				++iCnt;
				++ulAddr;
				if( iCnt >= 64 )
				{
					break;	/* read buffer full */
				}
			}while( 1 );

			/* size set */
			gdbsim_xfer_inferior_memory(c33TargetInfo.ulReadBufAddr, (char *)&iCnt, 1, 1, NULL, NULL); /* read buffer set */

		}
		else
		{
			/*--------------------------------------------------*/
			/* read buffer from Simulated I/O window key input	*/	/*	add 2003/03/07 T.Tazaki */
			/*--------------------------------------------------*/

			if( g_use_windows == 1 )  /* GUI Mode ? */
			{

				/* wait key input "STRING" + ENTER in Simulated I/O window */

				if( g_c33_std_in == 0 )
				{
					/* add T.Tazaki 2003/09/25 >>> */
					/* Dummy : One space output for first Stdin(read_str) */
					Tcl_SetVar( gdbtk_interp,"c33_stdout_string"," ",TCL_GLOBAL_ONLY );
					Tcl_Eval( gdbtk_interp, "c33_stdout" ); /* gdbtcl/srcbar.itcl */

					/* Clear before TCL input string */
					Tcl_SetVar( gdbtk_interp,"c33_stdin_string","",TCL_GLOBAL_ONLY );	
					/* add T.Tazaki 2003/09/25 <<< */


					c33TargetInfo.iKeyBreak = 0;	/* bug fix T.Tazaki 2004/11/10 */

					/* Get Simutaled I/O Window Key input String */
					do
					{
						if(c33TargetInfo.iKeyBreak == 1)	/* add T.Tazaki 2004/06/22 */
						{
							printf_filtered( C33_MSG_KEY_BREAK );
							printf_filtered( "\n" );
							break;
						}

						*pszbuf = (char)NULL; /* string initilaize */
								
						Tcl_Eval( gdbtk_interp, "c33_stdin" ); /* gdbtcl/srcbar.itcl */

						pszbuf = Tcl_GetVar(gdbtk_interp, "c33_stdin_string", TCL_GLOBAL_ONLY);

						c33_Gdbtk_Idle ();
										
					}while( *pszbuf == (char)NULL );

					g_c33_std_in = 1;	/* already keyinput */
					g_c33_Index = 0;
					strcpy( g_c33_stdin_string, pszbuf );

					/* Clear before TCL input string */
					Tcl_SetVar( gdbtk_interp,"c33_stdin_string","",TCL_GLOBAL_ONLY );

				}

				ulAddr = c33TargetInfo.ulReadBufAddr + 1;
				iCnt = 0;
				do
				{
					iChar = g_c33_stdin_string[g_c33_Index];
					if( iChar == (char)NULL )
					{
						/* Initialize flag & read buffer index */
						g_c33_std_in = 0;
						g_c33_Index = 0;
						break;
					}
					gdbsim_xfer_inferior_memory(ulAddr, (char *)&iChar, 1, 1, NULL, NULL); /* read buffer set */

					++g_c33_Index;
					++iCnt;
					if( iCnt >= 64 )
					{
						break;	/* read buffer full */
					}
					++ulAddr;
					++pszbuf;
				}while( 1 );

				/* size set */
				gdbsim_xfer_inferior_memory(c33TargetInfo.ulReadBufAddr, (char *)&iCnt, 1, 1, NULL, NULL); /* read buffer set */

			}
		}
		
		
		iRet = 1;
	}
	
	/*====================*/
	/* check Stdout break */
	/*====================*/
	if( c33TargetInfo.ulSoftPcBreak[C33_STDOUT_BREAK_NUM-1] == ulAddr )
	{
		if( fpStdoutfile != NULL )
		{
			/*----------------------*/
			/* write buffer to file */
			/*----------------------*/

			/* get buffer size */
			gdbsim_xfer_inferior_memory(c33TargetInfo.ulWriteBufAddr, &ucSize, 1, 0, NULL, NULL);
			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{
				for( ulAddr = c33TargetInfo.ulWriteBufAddr + 1; ulAddr <= (c33TargetInfo.ulWriteBufAddr+ucSize); ++ulAddr )
				{
					gdbsim_xfer_inferior_memory(ulAddr, (char *)&ucChar, 1, 0, NULL, NULL); /* write buffer read */

					if( fputc( ucChar, fpStdoutfile ) == EOF )
					{
						fclose( fpStdoutfile );
						fpStdoutfile = NULL;
						c33_error (C33_ERR_FILE_WRITE);
						break;
					}
				}
				
			}
		}
		else
		{
			/*--------------------------------------*/
			/* write buffer to simulated I/O Window */	/*	add 2003/03/06 T.Tazaki */
			/*--------------------------------------*/

			/* get buffer size */
			gdbsim_xfer_inferior_memory(c33TargetInfo.ulWriteBufAddr, &ucSize, 1, 0, NULL, NULL);

			if( ucSize > 0 && ucSize < 65 ) /* size = max 64 */
			{
				i = 0;
				for( ulAddr = c33TargetInfo.ulWriteBufAddr + 1; ulAddr <= (c33TargetInfo.ulWriteBufAddr+ucSize); ++ulAddr )
				{
					gdbsim_xfer_inferior_memory(ulAddr, (char *)&ucChar, 1, 0, NULL, NULL); /* write buffer read */

					szString[ i ] = ucChar;
					++i;
				}
				szString[ i ] = 0x0;	/* NULL */
				
				if( g_use_windows == 1 )  /* GUI Mode ? */
				{
					/* change binary to "." : add T.Tazaki 2004/05/26 >>> */
					if( szString[0] != 0 )
					{
						iLen = strlen( szString );
						for( i = 0; i < iLen; ++i )
						{
							if( szString[i] <= 0x1f || (szString[i] >= 0x80 && szString[i] <= 0x9f) || szString[i] >= 0xe0)
							{
								if( szString[i] != 0x0a && szString[i] != 0x0d )
								{
									szString[i] = '.';
								}
							}
						}
						/* change binary to "." : add T.Tazaki 2004/05/26 <<< */

						/* Set TCL Global data and Call simulated I/O output pref */
						Tcl_SetVar( gdbtk_interp,"c33_stdout_string",szString,TCL_GLOBAL_ONLY );
						Tcl_Eval( gdbtk_interp, "c33_stdout" ); /* gdbtcl/srcbar.itcl */
					}
				}
				
			}
		}
		iRet = 1;
	}
	
	return iRet;
}

/****************************************************************************/
/* Format		 : static void	gdb_TraceMode(iParamNum, cParam1, cParam2)	*/
/* Arguments																*/
/*	int iParamNum			  : Parameter number							*/
/*	char *					  : on / off									*/
/*	char *					  : trace output file name						*/
/* Return value  : void 													*/
/* Explanation	 : Memory move execution.									*/
/****************************************************************************/


static void
gdb_TraceMode(int iParamNum, char *cParam1, char *cParam2)
{
	char cParBuf[256];

	if(strcmp(cParam1,"off")==0 ){
		g_iTraceDump = TM_OFF;
		if( Trace_fp != NULL ){
			fclose( Trace_fp );
		}
		puts_filtered ("trace mode off\n");
	}
	else
	{
		if(strcmp(cParam1,"on")==0 )
		{
			if( g_iTraceDump == TM_ON ){ /* Already ON ? */
				c33_error ("trace mode already on.");
				return;
			}
			else
			{
				if( cParam2[0] == 0 ){
					c33_error (C33_ERR_COMMAND_INVALID);	/* file name not exist */
					return;
				}
				else
				{
					if ((Trace_fp = fopen(cParam2, "w")) == NULL){
						c33_error ("file open error.\n");
						return;
					}
					else{
						g_iTraceDump = TM_ON;
						g_ulInstNum = 1;
						puts_filtered ("trace mode on\n");
					}
				}
			}
		}
		else
		{
			c33_error (C33_ERR_COMMAND_INVALID);
			return;
		}
	}
}

#if 0		/* 錾ֈړB T.Tazaki 2007/07/12 */
int 			fnPush (unsigned short uwRegNum);
int 			fnPop (unsigned short uwRegNum);
#endif
struct DoubleWord fnAddDoubleWord (struct DoubleWord stDest, struct DoubleWord stSrc);
struct DoubleWord fnSubDoubleWord (struct DoubleWord stDest, struct DoubleWord stSrc);
unsigned long	fnChangeCV_ADD (int blCarry, unsigned long ulDataSrc, unsigned long ulDataDest, unsigned long ulFlag);
unsigned long	fnChangeCV_SUB (int blCarry, unsigned long ulDataSrc, unsigned long ulDataDest, unsigned long ulFlag);
unsigned long	fnChangeNZ (unsigned long ulData, unsigned long ulFlag);
int 			fnExeSpecial (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwImm2);
int 			fnExePushnPopn (unsigned short uwOp2, unsigned short uwBit4_5, unsigned short uwRegNum);
int 			fnExeControl (unsigned short uwOp2, unsigned short uwRegNum, int blDelayed);
int 			fnExeReturn (unsigned short uwOp2, int blDelayed);
int 			fnExeReturnMMU (int blDelayed);
int 			fnExeBranch (unsigned short uwOp1, unsigned short uwImm, int blDelayed);
int 			fnExeTrans (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegBase, unsigned short uwRegNum);
int 			fnExeTransSpecial (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeTransReg (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeStack (unsigned short uwOp1, unsigned short uwImm, unsigned short uwRegNum);
int 			fnExeDataPoint (unsigned short uwOp1, unsigned short uwImm, unsigned short uwRegNum);
int 			fnExeArith (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeArithImm (unsigned short uwOp1, unsigned short uwImm6, unsigned short uwRegNum);
int 			fnExeArithSP (unsigned short uwOp1, unsigned short uwImm10);
int 			fnExeArithC (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeArithDP (unsigned short uwRegDest);
int 			fnExeShiftRot (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwShift, unsigned short uwRegNum);
int 			fnExeDiv (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegNum);
int 			fnDiv0s (unsigned short uwRegNum);
int 			fnDiv0u (unsigned short uwRegNum);
void			vfnDiv1 (unsigned short uwRegNum);
void			vfnDiv2s (unsigned short uwRegNum);
void			vfnDiv3s ();
void			vfnDivu (unsigned short uwRegNum);
void			vfnDivs (unsigned short uwRegNum);
int 			fnExeScan (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeSwapMirror (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeMacW (unsigned short uwOp2,unsigned short uwRegSrc);
unsigned long	fnMlt32_32 (struct DoubleWord *stAns, long lSrc1, long lSrc2);
int 			fnExeMac1 (unsigned short uwOp1, unsigned short uwRegSrc1, unsigned short uwRegSrc2);
int 			fnExeMac (unsigned short uwRegSrc);
int 			fnExeMacClr (void);
int 			fnExeSat (unsigned short uwClass, unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegSrc, unsigned short uwRegDest);
int 			fnExeMlt (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegSrc, unsigned short uwRegDest);
struct DoubleWord fnMltDoubleWord (long lDataDest, long lDataSrc);
struct DoubleWord fnMltUnsignedDoubleWord (unsigned long ulDataDest, unsigned long ulDataSrc);
int 			fnExeExt (unsigned short uwImm13);

int 			fnExeExtReg (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwOp3);
int 			fnExeCond(unsigned short uwCode);

int 			fnExePsrSetClr (unsigned short uwOp3, unsigned short uwImm5);
int 			fnExeLoop (unsigned short uwRegAddr, unsigned short uwRegCount);
int 			fnExeLoopRegImm (unsigned short uwImmAddr, unsigned short uwRegCount);
int 			fnExeLoopImmImm (unsigned short uwImmAddr, unsigned short uwImmCount);
int 			fnExeRepeat (unsigned short uwOp2, unsigned short uwRepeatCount);
int 			fnExeCoproFlag ();
int 			fnExeCoproTransReg (unsigned short uwImm4, unsigned short uwRegDest);
int 			fnExeCoproTransImm (unsigned short uwRegSrc, unsigned short uwImm4);
int 			fnExeCoproCom (unsigned short uwImm6);


extern void vfnCountUpRealClockInstruction(unsigned long);/* 04/25/2000 C.Nakayama		  Modify API spec and clock function*/

/*		  2002/01/31  C.Nakayama Add special registers.(GNU33cygwin) */
#define SPECIAL_REG(x)	(unsigned short)(x + (unsigned short)PSR)

/************************************************
 *	vfnIncPC : increase PC register 	*
 ************************************************/

#ifndef SPARC

void 
vfnIncPC (long lByte)
#else /* SPARC */

void
vfnIncPC (lByte)
	 long			 lByte; /* bytes to increase */

#endif /* SPARC */

{				/* return void */

//	extern unsigned long ulReg[];

	ulReg[PC] += lByte; 	/* increase PC */

	return;
}

/************************************************
 *	fnPush : push register to stack 	*
 ************************************************/
#ifndef SPARC

int 
fnPush (unsigned short uwRegNum)
#else /* SPARC */

int
fnPush (uwRegNum)
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	RegData;
	unsigned long	ulAddr;
	/* >>>>> add iruma m.takeishi '04.04.01 */
	/* In ver 1.2, below code is vanished, why??? */
	/* >>>>> add iruma m.takeishi '03.09.09 */
	int 			iRet = 1;	 // default is NG.
	/* <<<<< add iruma m.takeishi '03.09.09 */
	/* <<<<< add iruma m.takeishi '04.04.01 */

	struct c33_map_info *pstMapInfo;
	
	/* Stack overflow check */
	ulAddr = gdbsim_fetch_register (SP) - 4;
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			if( pstMapInfo->iMapAttr == C33_STACK )
			{
				/* check if address renge */
				/* >>>>> add iruma m.takeishi '04.04.01 */
				/* In ver 1.2, below code is vanished, why??? */
				/* >>>>> change iruma m.takeishi '03.09.18 */
				/*
				if(( ulAddr < pstMapInfo->ulStartAddr ) || ( ulAddr > pstMapInfo->ulEndAddr ))
				{
					return STACK_OVERFLOW;
				*/
				iRet = 2;
				if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
				{
					iRet = 0;
					break;
				/* <<<<< change iruma m.takeishi '03.09.18 */
				/* <<<<< add iruma m.takeishi '04.04.01 */
				}
			}
			
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
		
		/* >>>>> add iruma m.takeishi '04.04.01 */
		/* In ver 1.2, below code is vanished, why??? */
		/* >>>>> add iruma m.takeishi '03.09.18 */
		if ( iRet == 2 ) {
			return STACK_OVERFLOW;
		}
		/* <<<<< add iruma m.takeishi '03.09.18 */
		/* <<<<< add iruma m.takeishi '04.04.01 */
	}

	/*@ change stack pointer */
	(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) - 4));

	/*@ write data of specified register to stack */
	if ( uwRegNum == 999 )	/* data 0x00000000 push ? 		T.Tazaki 2004/01/30 */
	{
		RegData = 0x0;
	}
	else
	{
		RegData = gdbsim_fetch_register(uwRegNum);
	}
	iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (SP),	
			   (char *)&RegData, 4, 1, NULL, NULL);

	return (iStatus);		/* return status */
}

/************************************************
 *	fnPop : pop register from stack 	*
 ************************************************/

#ifndef SPARC

int 
fnPop (unsigned short uwRegNum)
#else /* SPARC */

int
fnPop (uwRegNum)
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData;
	unsigned long	ulAddr;
	unsigned long	ulTemp;
	unsigned long	ulTemp2;
	unsigned long	ulFlag;

	/* >>>>> add iruma m.takeishi '04.04.01 */
	/* why velow codes are vanished in GNU ver1.2??? */
	/* >>>>> add iruma m.takeishi '03.09.09 */
	int 			iRet = 1;	 // default is NG.
	/* <<<<< add iruma m.takeishi '03.09.09 */
	/* <<<<< add iruma m.takeishi '04.04.01 */

	struct c33_map_info *pstMapInfo;
	
	/* Stack overflow check */
	ulAddr = gdbsim_fetch_register (SP);
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			if( pstMapInfo->iMapAttr == C33_STACK )
			{
				/* check if address renge */

				/* >>>>> add iruma m.takeishi '04.04.01 */
				/* why velow codes are vanished in GNU ver1.2??? */
				/* >>>>> change iruma m.takeishi '03.09.18 */
				/*
				if(( ulAddr < pstMapInfo->ulStartAddr ) || ( ulAddr > pstMapInfo->ulEndAddr ))
				{
					return STACK_OVERFLOW;
				*/
				iRet = 2;
				if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
				{
					iRet = 0;
					break;
				/* <<<<< change iruma m.takeishi '03.09.18 */
				/* <<<<< add iruma m.takeishi '04.04.01 */
				}
			}
			
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}

		/* >>>>> add iruma m.takeishi '04.04.01 */
		/* why velow codes are vanished in GNU ver1.2??? */
		/* >>>>> add iruma m.takeishi '03.09.18 */
		if ( iRet == 2 ) {
			return STACK_OVERFLOW;
		}
		/* <<<<< add iruma m.takeishi '03.09.18 */
		/* <<<<< add iruma m.takeishi '04.04.01 */
	}



	/*@ read data of the specified register from stack */
	iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (SP), (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	ulFlag = gdbsim_fetch_register (PSR);

	/*@ check not exception, not error break */
	if (iStatus == C33_NO_ERR) {
		
		/* check if pop register is not update register or Not pop( ret instruction ). 		add T.Tazaki 2004/02/03 */
		if( uwRegNum == 999 || uwRegNum == SP || uwRegNum == USP || uwRegNum == SSP || (uwRegNum == PC && (ulFlag & PM_FLAG) ) )
		{
			/* ONLY SP+=4 */
			(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) + 4));
		}
		else if( uwRegNum == PSR )
		{
			/* DE(bit14),ME(bit13) flag : Not Change */
			ulTemp = gdbsim_fetch_register( PSR );
			ulTemp &= ( DE_FLAG | ME_FLAG );
			ulData &= ~DE_FLAG;
			ulData &= ~ME_FLAG;
			ulData |= ulTemp;
				
			/* User mode : HE/SV/IL/IE = Read ONLY Bit */
			ulTemp = gdbsim_fetch_register( PSR );
			ulTemp2 = gdbsim_fetch_register( PSR );
			if( ulTemp & SV_FLAG )	/* user mode ? */
			{
				ulTemp &= ( HE_FLAG | SV_FLAG | IL_BITS | IE_FLAG );
				ulData &= ~HE_FLAG;
				ulData &= ~SV_FLAG;
				ulData &= ~IL_BITS;
				ulData &= ~IE_FLAG;
				ulData |= ulTemp;
			}
			/*@ change register with the read data */
			(void) gdbsim_store_register (uwRegNum, ulData);

			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
			{
				if( ulTemp2 & SV_FLAG )  /* user mode ? */
				{
					(void) gdbsim_store_register (USP, (gdbsim_fetch_register (USP) + 4));
				}
				else
				{
					(void) gdbsim_store_register (SSP, (gdbsim_fetch_register (SSP) + 4));
				}
			}
			else
			{
				(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) + 4));
			}
		}
		else
		{
			(void) gdbsim_store_register (uwRegNum, ulData);
			(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) + 4));
		}
	}

	/*@ clear rom write break flag */
	iStatus &= ~ROM_DATA_BREAK;

	return (iStatus);		/* return status */
}

/************************************************
 *	fnAddDoubleWord : execute 64Bit_Add * 
 ************************************************/

#ifndef SPARC

struct DoubleWord 
fnAddDoubleWord (struct DoubleWord stDest, struct DoubleWord stSrc)
#else /* SPARC */

struct DoubleWord
fnAddDoubleWord (stDest, stSrc)
	 struct DoubleWord stDest;
	 struct DoubleWord stSrc;

#endif /* SPARC */

{				/* return stDest (= stDest + stSrc) */
	unsigned long	ulCarry;

	/*@ check carry */
	if ((ULONG_MAX - stSrc.ulLow) < stDest.ulLow) {
	ulCarry = 1;		/* carry */
	}
	else {
	ulCarry = 0;		/* no carry */
	}

	/*@ calculate */
	stDest.ulHigh += stSrc.ulHigh + ulCarry;
	stDest.ulLow += stSrc.ulLow;

	/*@ return calculated data */
	return (stDest);
}

/************************************************
 *	fnSubDoubleWord : execute 64bit_sub *
 ************************************************/

#ifndef SPARC

struct DoubleWord 
fnSubDoubleWord (struct DoubleWord stDest, struct DoubleWord stSrc)
#else /* SPARC */

struct DoubleWord
fnSubDoubleWord (stDest, stSrc)
	 struct DoubleWord stDest;
	 struct DoubleWord stSrc;

#endif /* SPARC */

{				/* return stDest (= stDest - stSrc) */
	unsigned long	ulBorrow;

	/*@ check borrow */
	if (stDest.ulLow < stSrc.ulLow) {
	ulBorrow = 1;		/* borrow */
	}
	else {
	ulBorrow = 0;		/* no borrow */
	}

	/*@ calculate */
	stDest.ulHigh -= (stSrc.ulHigh + ulBorrow);
	stDest.ulLow += (ULONG_MAX - stSrc.ulLow) + 1;

	/*@ return calculated data */
	return (stDest);
}

/********************************************************
 *	fnChangeCV_ADD : check CV flags for add/adc *
 ********************************************************/

#ifndef SPARC

unsigned long 
fnChangeCV_ADD (int blCarry, unsigned long ulDataSrc, unsigned long ulDataDest, unsigned long ulFlag)
#else /* SPARC */

unsigned long
fnChangeCV_ADD (blCarry, ulDataSrc, ulDataDest, ulFlag)
	 int			 blCarry;	/* WITH_CARRY / WITHOUT_CARRY */
	 unsigned long	 ulDataSrc;
	 unsigned long	 ulDataDest;
	 unsigned long	 ulFlag;	/* PSR */

#endif /* SPARC */

{				/* return PSR */
	unsigned long	ulCarry;

	/*@ check executing ADC instruction */
	if (blCarry == WITH_CARRY) {
	/*@ executing ADC, check carry */
	if ((ulFlag & C_FLAG) != 0) {
		ulCarry = 1;	/* carry */
	}
	else {
		ulCarry = 0;	/* no carry */
	}
	}
	else {
	/*@ not executing ADC, assuming no carry */
	ulCarry = 0;
	}

	if( ulCarry == 1 )
	{
		if ( ULONG_MAX == ulDataSrc || ULONG_MAX == ulDataDest )
		{
			ulFlag |= C_FLAG;	/* carry */
		}
		else if (((ULONG_MAX - ulCarry) - ulDataSrc) < ulDataDest) {
			ulFlag |= C_FLAG;	/* carry */
		}
		else {
			ulFlag &= ~C_FLAG;	/* no carry */
		}
	}
	else
	{
		if (((ULONG_MAX - ulCarry) - ulDataSrc) < ulDataDest) {
			ulFlag |= C_FLAG;	/* carry */
		}
		else {
			ulFlag &= ~C_FLAG;	/* no carry */
		}
	}

	/*@ check plus + plus */
	if ((ulDataSrc <= LONG_MAX) && (ulDataDest <= LONG_MAX)) {
	/*@ plus + plus, check overflow */
	if (ulDataSrc + ulDataDest + ulCarry > LONG_MAX) {
		ulFlag |= V_FLAG;	/* overflow */
	}
	else {
		ulFlag &= ~V_FLAG;	/* no overflow */
	}
	}
	/*@ check minus + minus */
	else if ((ulDataSrc > LONG_MAX) && (ulDataDest > LONG_MAX)) {
	/*@ minus + minus, check overflow */
	if (ulDataSrc + ulDataDest + ulCarry <= LONG_MAX) {
		ulFlag |= V_FLAG;	/* overflow */
	}
	else {
		ulFlag &= ~V_FLAG;	/* no overflow */
	}
	}
	else {
	/*@ when plus + minus, no overflow */

	ulFlag &= ~V_FLAG;	/* no overflow */
	}

	/*@ return PSR */
	return (ulFlag);
}

/********************************************************
 *	fnChangeCV_SUB : check CV flag for sub/sbc	*
 ********************************************************/

#ifndef SPARC

unsigned long 
fnChangeCV_SUB (int blCarry, unsigned long ulDataSrc, unsigned long ulDataDest, unsigned long ulFlag)
#else /* SPARC */

unsigned long
fnChangeCV_SUB (blCarry, ulDataSrc, ulDataDest, ulFlag)
	 int			 blCarry;	/* WITH_CARRY / WITHOUT_CAARY */
	 unsigned long	 ulDataSrc;
	 unsigned long	 ulDataDest;
	 unsigned long	 ulFlag;	/* PSR */

#endif /* SPARC */

{				/* return PSR */
	unsigned long	ulBorrow;

	/*@ check executing SBC instruction */
	if (blCarry == WITH_CARRY) {
	/*@ executing SBC, check borrow */
	if ((ulFlag & C_FLAG) != 0) {
		ulBorrow = 1;	/* borrow */
	}
	else {
		ulBorrow = 0;	/* no borrow */
	}
	}
	else {
	/*@ not executing SBC, assuming no borrow */
	ulBorrow = 0;
	}

	/*@ check borrow when calculate */
	/* bug fix 2005/03/14 T.Tazaki >>> */
	if ( ulDataSrc == 0xffffffff && ulBorrow == 1 )
	{
		ulFlag |= C_FLAG;	/* borrow */
	}
	else
	{
	/* bug fix 2005/03/14 T.Tazaki <<< */
		if (ulDataDest < (ulDataSrc + ulBorrow)) {
//		if (ulDataDest < ulDataSrc + ulBorrow) {		/* change T.Tazaki 2005/03/14 */
		ulFlag |= C_FLAG;	/* borrow */
		}
		else {
		ulFlag &= ~C_FLAG;	/* no borrow */
		}
	}
	/*@ check plus - minus */
	if ((ulDataDest <= LONG_MAX) && (ulDataSrc > LONG_MAX)) {
	/*@ plus - minus, check underflow */
	if (ulDataDest - ulDataSrc - ulBorrow > LONG_MAX) {
		ulFlag |= V_FLAG;	/* underflow */
	}
	else {
		ulFlag &= ~V_FLAG;	/* no underflow */
	}
	}
	/*@ check minus - plus */
	else if ((ulDataDest > LONG_MAX) && (ulDataSrc <= LONG_MAX)) {
	/*@ minus - plus, check underflow */
	if (ulDataDest - ulDataSrc - ulBorrow <= LONG_MAX) {
		ulFlag |= V_FLAG;	/* underflow */
	}
	else {
		ulFlag &= ~V_FLAG;	/* no underflow */
	}
	}
	else {
	/*@ when plus - plus, minus - minus, no underflow */

	ulFlag &= ~V_FLAG;	/* no underflow */
	}

	/*@ return PSR */
	return (ulFlag);
}

/****************************************
 *	fnChangeNZ : check NZ flag	*
 ****************************************/

#ifndef SPARC

unsigned long 
fnChangeNZ (unsigned long ulData, unsigned long ulFlag)
#else /* SPARC */

unsigned long
fnChangeNZ (ulData, ulFlag)
	 unsigned long	 ulData;
	 unsigned long	 ulFlag;	/* PSR */

#endif /* SPARC */

{				/* return PSR */

	/*@ check negative */
	if ((ulData & MSB_BIT) != 0) {
	ulFlag |= N_FLAG;	/* negative */
	}
	else {
	ulFlag &= ~N_FLAG;	/* not negative */
	}

	/* check zero */
	if (ulData == 0) {
	ulFlag |= Z_FLAG;	/* zero */
	}
	else {
	ulFlag &= ~Z_FLAG;	/* not zero */
	}

	/*@ return PSR */
	return (ulFlag);
}

/********************************************************
 *	fnChangeMO : check MO flags for mac 	*
 ********************************************************/

#ifndef SPARC

unsigned long 
fnChangeMO (struct DoubleWord stDest, struct DoubleWord stSrc, unsigned long ulFlag)
#else /* SPARC */

unsigned long
fnChangeMO (stDest, stSrc, ulFlag)
	 struct DoubleWord stDest;
	 struct DoubleWord stSrc;
	 unsigned long	 ulFlag;	/* PSR */

#endif /* SPARC */

{				/* return PSR */
	unsigned long	ulCarry;	/* carry from lower words */
	unsigned long	ulDataDest; /* higher word */
	unsigned long	ulDataSrc;	/* higher word */

	/*@ check carry */
	if ((ULONG_MAX - stSrc.ulLow) < stDest.ulLow) {
		ulCarry = 1;		/* carry */
	}
	else {
		ulCarry = 0;		/* no carry */
	}

	ulDataDest = stDest.ulHigh;
	ulDataSrc = stSrc.ulHigh;

	/*@ check plus + plus */
	if ((ulDataSrc <= LONG_MAX) && (ulDataDest <= LONG_MAX)) {
	/*@ plus + plus, check overflow */
		if (ulDataSrc + ulDataDest + ulCarry > LONG_MAX) {
			ulFlag |= MO_FLAG;	/* mac overflow */
		}
		else {
			;
		}
	}
	/*@ check minus + minus */
	else if ((ulDataSrc > LONG_MAX) && (ulDataDest > LONG_MAX)) {
		/*@ minus + minus, check overflow */
		if (ulDataSrc + ulDataDest + ulCarry <= LONG_MAX) {
			ulFlag |= MO_FLAG;	/* mac overflow */
		}
		else {
			;
		}
	}
	else {
	/*@ when plus + minus, no overflow */
	}

	/*@ return PSR */
	return (ulFlag);
}

/********************************************************
 *	fnExeSpecial : execute nop/slp/halt/int 	*
 ********************************************************/

#ifndef SPARC

int 
fnExeSpecial (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwImm2)
#else /* SPARC */

int
fnExeSpecial (uwOp1, uwOp2, uwImm2)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwImm2;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulAddr;
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iLoop, iRepeat, iExt, iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> add iruma m.takeishi '04.03.26 */
	unsigned long  ulPSR_orig;
	/* <<<<< add iruma m.takeishi '04.03.26 */


	iStatus = 0;		/* initialize */

	/* >>>>> add iruma m.takeishi '03.09.01 */
	switch (uwOp1 << 2 | uwOp2) {
	case 10:
		/* int imm2 */
		iLoop= CMB_LOOP_OK_A1;
		iExt = CMB_EXT_NG;
		break;
	
	/* >>>>> add iruma m.takeishi '04.03.17 */
	case 1: 		/* 0000:01 slp */
	case 2: 		/* 0000:10 halt */
		iLoop= CMB_LOOP_OK;
		iExt = CMB_EXT_NG;
		break;
	/* <<<<< add iruma m.takeishi '04.03.17 */

	default:
		iLoop= CMB_LOOP_OK;
		iExt = CMB_EXT_COND_OK;
		break;
	}
	iRet = fnCheckInstCombi( iLoop, CMB_REPEAT_NG, iExt );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* >>>>> add iruma m.takeishi '03.09.01 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	switch (uwOp1 << 2 | uwOp2) {	/* instruction */

	case 0: 		/* 0000:00 nop */

	vfnIncPC (CODE);	/* PC += CODE */
	gdb_AddCycle(1);
	break;

	case 1: 		/* 0000:01 slp */
	case 2: 		/* 0000:10 halt */

	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		gdb_AddCycle(5);
	}else{
		gdb_AddCycle(1);
	}
	/* >>>>> add iruma m.takeishi '04.03.29 */
	ulPSR = gdbsim_fetch_register( PSR );
	if ( (ulPSR & HE_FLAG) || !(ulPSR & SV_FLAG) ) {
		/* HE=1, or SV=0 */
	/* <<<<< add iruma m.takeishi '04.03.29 */
		iStatus |= SLP_HALT_BREAK;	/* set slp/halt break */
	
		fnDoSlpHalt (SET);	/* enter slp/halt mode */
	/* >>>>> add iruma m.takeishi '04.03.29 */
	}
	/* <<<<< add iruma m.takeishi '04.03.29 */

	vfnIncPC (CODE);	/* PC += CODE */	/* add T.Tazaki 2003/08/08 */

	break;

	case 10:			/* 0010:10 int imm2 */

	if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
	{
		gdb_AddCycle(10);
	}
	else
	{
		gdb_AddCycle(7);
	}

	switch (uwImm2) {	/* software exception number */

	case 0: 	/* int 0x0 */
	case 1: 	/* int 0x1 */
	case 2: 	/* int 0x2 */
	case 3: 	/* int 0x3 */

		/*@ set software exception number 0 */

		vfnIncPC (CODE);	/* PC += CODE */


		/* software exception vector address -> PC */
		/*@ push PC to stack */

		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			ulPSR = gdbsim_fetch_register (PSR);
			/* >>>>> add iruma m.takeishi '04.03.26 */
			ulPSR_orig = ulPSR;
			/* <<<<< add iruma m.takeishi '04.03.26 */
			ulPSR &= ~IE_FLAG;		/* interrupt enable : reset */
			ulPSR &= ~SV_FLAG;		/* supervisor mode */
			gdbsim_store_register ( PSR, ulPSR );
		}
		
		iStatus |= fnPush (PC); 	/* use %ssp */
		if( iStatus == 0 )
		{
			/*@ push PSR to stack */
			iStatus |= fnPush (PSR);
			if( iStatus == 0 )
			{
			
				g_iExtEachInst = FALSE; /* for Not Trace Memory Read : temporary */
				

				/* software exception vector address -> PC */
				if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
				{
					/* >>>>> add iruma m.takeishi '04.03.26 */
					if ( ulPSR_orig & SV_FLAG ) {
						/* changes SV=1 on stack's PSR */
						g_iExtEachInst = FALSE;

						gdbsim_xfer_inferior_memory( gdbsim_fetch_register( SSP ),
							(char *)&ulPSR_orig, 4, 1, NULL, NULL);
					}
					/* <<<<< add iruma m.takeishi '04.03.26 */
					
					ulAddr = gdbsim_fetch_register (TTBR);
					/* >>>>> add iruma m.takeishi '04.03.16 */
					ulPSR = gdbsim_fetch_register (PSR);
					if ( ulPSR & LM_FLAG ) {
						iIsLoop_int = 1;
					}
					/* <<<<< add iruma m.takeishi '04.03.16 */
					/* >>>>> add iruma m.takeishi '04.03.26 */
					ulPSR &= ~RM_FLAG;
					ulPSR &= ~LM_FLAG;
					ulPSR &= ~PM_FLAG;
					gdbsim_store_register( PSR, ulPSR );
					/* <<<<< add iruma m.takeishi '04.03.26 */
					
				}
				else
				{
					/* IE = 0 : add T.Tazaki 2004/02/25 >>> */
					ulPSR = gdbsim_fetch_register (PSR);
					ulPSR &= ~IE_FLAG;		/* interrupt enable : reset */
					gdbsim_store_register ( PSR, ulPSR );
					/* IE = 0 : add T.Tazaki 2004/02/25 <<< */

/* bug fix : T.Tazaki 2005/01/24 >>> */
					if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
					{
						ulAddr = gdbsim_fetch_register (TTBR);
					}
					else
					{
						gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulAddr, 4, 0, NULL, NULL);
					}
/* bug fix : T.Tazaki 2005/01/24 <<< */

				}
				ulAddr += ( 0x30 + uwImm2 * 4 );

				/* >>>>> add iruma m.takeishi '04.03.26 */
				g_iExtEachInst = TRUE;	// for Not TraceMemory Read //
				/* <<<<< add iruma m.takeishi '04.03.26 */

				gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulData, 4, 0, NULL, NULL);
				
				/* >>>>> delete iruma m.takeishi '04.03.26 */
				/*
				g_iExtEachInst = TRUE;	// for Not TraceMemory Read //
				*/
				/* <<<<< delete iruma m.takeishi '04.03.26 */

				gdbsim_store_register( PC, ulData );
				
			}
		}
		break;

	default:		/* error */

		/* error */
		iStatus |= ILL_INST_BREAK;	/* set error break */
	}

	break;
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/******************************************************************************
 *	fnExePushnPopn : execute push/pop, pushn/popn, pushs/pops instruction	  *
 ******************************************************************************/

#ifndef SPARC

int 
fnExePushnPopn (unsigned short uwOp2, unsigned short uwBit4_5, unsigned short uwRegNum)
#else /* SPARC */

int
fnExePushnPopn (uwOp2, uwBit4_5, uwRegNum)
	 unsigned short  uwOp2;
	 unsigned short  uwBit4_5;
	 unsigned short  uwRegNum;	/* rs/rd */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned short	uwTmpReg;
	unsigned long	ulFlag;

	unsigned short uwInstRegNum, uwSimSPReg = 0;
	unsigned short uwStartInstRegNum;	/* add T.Tazaki 2004/02/03 */
	int 			iLoop, iRet;
	/* >>>>> add iruma m.takeishi '04.03.29 */
	int 			iRepeat;
	/* <<<<< add iruma m.takeishi '04.03.29 */


	iStatus = 0;		/* initialize */

	/* >>>>> add iruma m.takeishi '03.09.01 */
	if ( uwOp2 == 3 ) {
		/* pops %ss */
		iLoop = CMB_LOOP_OK_A3;
	} else {
		iLoop = CMB_LOOP_OK;
	}
	/* >>>>> change iruma m.takeishi '04.03.29 */
	/*
	iRet = fnCheckInstCombi( iLoop, CMB_REPEAT_NG, CMB_EXT_COND_OK );
	*/
	iRepeat = CMB_REPEAT_NG;
	switch (uwOp2) {	
	case 0:
	case 1:
		if (uwBit4_5 == 1) {
			/* push %rs */
			/* pop	%rs */
			iRepeat = CMB_REPEAT_OK;
		}
		break;
	}
	iRet = fnCheckInstCombi( iLoop, iRepeat, CMB_EXT_COND_OK );
	/* <<<<< change iruma m.takeishi '04.03.29 */
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.01 */


/* 12/27/2001 N.Komatsu  read PSR for set PM flag */
	ulFlag = gdbsim_fetch_register (PSR);

	/*@ Is instruction pushn or push or pushs or popn or pop or pops? */
	switch (uwOp2) {

	case 0:

	/*@ 00 pushn %rs or push %rs */

		fnInterlock( C33_PUSHN, uwRegNum, 0, 0, 0 );

		if (uwBit4_5 == 1) {

			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
			/* add T.Tazaki 2004/08/25 <<< */
				gdb_AddCycle(1);
			}

			/*@ 01 push %rs */
			/*@ push single register to stack */
			iStatus |= fnPush (uwRegNum);

		}
		else {
			/*@ 00 pushn %rs */

			/*	check if PM flag = 1 */
			if( ulFlag & PM_FLAG )
			{
				uwRegNum = ( ulFlag & RC_BITS ) >> 24;	/* PSR:RC --> Start Register */
				
				if ( uwRegNum == 0 ) {
					iStatus |= PUSHPOP_ILLEGAL;
					break;
				}
				uwRegNum--;
			}
			else
			{
				ulFlag |= PM_FLAG;
				ulFlag &= ~RC_BITS;
				ulFlag |= ( ( uwRegNum << 24 ) & 0x0f000000 );
				gdbsim_store_register( PSR, ulFlag );
			}

			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			/*@ from %r0 to %rs */
			for (uwTmpReg = uwRegNum; (short) uwTmpReg >= R0; uwTmpReg--) {

//				gdb_AddCycle(1);

				/*@ push register to stack */
				iStatus |= fnPush (uwTmpReg);

				/*@ check nomap_mem_break or rom_data_break */
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) != 0) {
					break;
				}
				else {
					ulFlag &= ~RC_BITS;
					ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
					gdbsim_store_register( PSR, ulFlag );
				}
			}
			
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) == 0) 	/* No error ? */
			{
				 ulFlag &= ~PM_FLAG;
				 ulFlag &= ~RC_BITS;
				 gdbsim_store_register( PSR, ulFlag );
			}
		}
		break;

	case 1:
	/*@ 01 popn %rd or pop %rd*/

		if (uwBit4_5 == 1) {
			/*@ 01 pop %rd */
			
			/*@ pop single register from stack */
			fnInterlock( C33_POP, uwRegNum, 0, 1, 2 );
			
			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			iStatus |= fnPop (uwRegNum);
		}
		else {
			/*@ 00 popn %rd */

			fnInterlock( C33_POPN, uwRegNum, 0, 1, 2 );

			/*	check if PM flag = 1 */
			uwTmpReg = R0;
			if( ulFlag & PM_FLAG )
			{
				uwTmpReg = ( ulFlag & RC_BITS ) >> 24;	/* PSR:RC --> Start Register */
				uwTmpReg++;
				if ( uwTmpReg > uwRegNum ) {
					iStatus |= PUSHPOP_ILLEGAL;
					break;
				}
			}
			else
			{
				ulFlag |= PM_FLAG;
				ulFlag &= ~RC_BITS;
				ulFlag |= ( ( uwRegNum << 24 ) & 0x0f000000 );
				gdbsim_store_register( PSR, ulFlag );
			}
			
			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			/*@ from %rd to %r0 */
			for ( ; uwTmpReg <= uwRegNum; uwTmpReg++) {
				/*@ pop register from stack */
//				  gdb_AddCycle(1);
				iStatus |= fnPop (uwTmpReg);	

				/*@ check nomap_mem_break or rom_data_break */
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK)) != 0) {
					break;
				}
				else {
					ulFlag &= ~RC_BITS;
					ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
					gdbsim_store_register( PSR, ulFlag );
				}
			}
			
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) == 0) 	/* No error ? */
			{
				 ulFlag &= ~PM_FLAG;
				 ulFlag &= ~RC_BITS;
				 gdbsim_store_register( PSR, ulFlag );
			}
		}
		break;

	case 2:
	/*@ 10 pushs %ss */

		uwInstRegNum = uwRegNum;

		uwRegNum = SPECIAL_REG(uwRegNum);
		if( uwRegNum >= 29 ){	/* %usp, %ssp, %pc ? */
			uwRegNum-=1;		/* change T.Tazaki 2003/12/03 */
		}

		if (( uwRegNum == (unsigned short) PSR) || ( uwRegNum == (unsigned short) SP))
		{
			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			iStatus |= fnPush (uwRegNum);

			/*@ check nomap_mem_break or rom_data_break */
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) != 0) {
				break;
			}
		}
		else if (( uwRegNum >= (unsigned short) ALR) && ( uwRegNum <= (unsigned short) PC))
		{

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/* >>>>> change T.Tazaki 2004/01/30 */
			if( ulFlag & PM_FLAG )
			{
				uwInstRegNum = ( ulFlag & RC_BITS ) >> 24;	// PSR:RC --> Start Register //

				/* if RC is 0, push was done. */
				if ( uwInstRegNum <= 2 ) {
					iStatus |= PUSHPOP_ILLEGAL;
					break;
				}
			} else {
				ulFlag |= PM_FLAG;
				ulFlag &= ~RC_BITS;
				ulFlag |= ( ( uwInstRegNum << 24 ) & 0x0f000000 );
				gdbsim_store_register( PSR, ulFlag );
			}

			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			for( uwTmpReg = uwInstRegNum; uwTmpReg >= 2; --uwTmpReg )
			{
//				  gdb_AddCycle(1);

				if( uwTmpReg >= 13 ) 
				{
					/* #13=%usp,#14=%ssp,#15=%pc */
					iStatus |= fnPush (uwTmpReg + PSR - 1);		/* push #28,#29,#30 */
				}
				else if( uwTmpReg == 12 || uwTmpReg == 11 || uwTmpReg == 10 ) 
				{
					/* #12=none,#11=%dbbr,#10=%idir */
					iStatus |= fnPush (999);	/* 999 is push data 0x00000000 */
				}
				else
				{	/* #2:%alr --------- #9=%dp */
					iStatus |= fnPush (uwTmpReg + PSR);			/* push #18 --- #25 */
				}
				
				
				/*@ check nomap_mem_break or rom_data_break */
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) != 0) {
					--uwTmpReg;
					ulFlag &= ~RC_BITS;
					ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
					gdbsim_store_register( PSR, ulFlag );
					break;
				}
				else {
					ulFlag &= ~RC_BITS;
					ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
					gdbsim_store_register( PSR, ulFlag );
				}
			}

			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) == 0) 	/* No error ? */
			{
				ulFlag &= ~PM_FLAG;
				ulFlag &= ~RC_BITS;
				gdbsim_store_register( PSR, ulFlag );
			}
			/* <<<<< change T.Tazaki 2004/01/30 */

		}
		else
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		}
		break;

	case 3:
	/*@ 11 pops %sd */

		uwInstRegNum = uwRegNum;

		uwRegNum = SPECIAL_REG(uwRegNum);

		if( uwRegNum >= 29 ){	/* %usp, %ssp, %pc ? */
			uwRegNum-=1;		/* change T.Tazaki 2003/12/03 */
		}

		if (( uwRegNum == (unsigned short) PSR) || ( uwRegNum == (unsigned short) SP))
		{
			/*@ pop special register from stack */
			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */
			iStatus |= fnPop (uwRegNum);

			/*@ check nomap_mem_break or rom_data_break */
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK)) != 0) {
				break;
			}
		}
		else if (( uwRegNum >= (unsigned short) ALR) && ( uwRegNum <= (unsigned short) PC))
		{

			fnInterlock( C33_POPS, uwRegNum, 0, 1, 2 );
			
			/* >>>>> change T.Tazaki 2004/01/28 */
			
			if( ulFlag & PM_FLAG )
			{
				uwStartInstRegNum = ( ulFlag & RC_BITS ) >> 24;  // PSR:RC --> Start Register //
				
				if ( uwStartInstRegNum <= 1 ) { 			// PSR, SP
					iStatus |= PUSHPOP_ILLEGAL;
					break;
				}
			}
			else
			{
				ulFlag |= PM_FLAG;
				ulFlag &= ~RC_BITS;
				ulFlag |= ( ( uwInstRegNum << 24 ) & 0x0f000000 );
				gdbsim_store_register( PSR, ulFlag );
				
				uwStartInstRegNum = 2;
			}

			/* add T.Tazaki 2004/08/25 >>> */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				gdb_AddCycle(3);
			}
			else
			{
				gdb_AddCycle(1);
			}
			/* add T.Tazaki 2004/08/25 <<< */

			for ( uwTmpReg = uwStartInstRegNum; uwTmpReg <= uwInstRegNum; uwTmpReg++) {
				
//				  gdb_AddCycle(1);

				if( uwTmpReg >= 13 ) 
				{
					/* #13=%usp,#14=%ssp,#15=%pc */
					iStatus |= fnPop (uwTmpReg + PSR - 1);		/* pop #28,#29,#30 */
				}
				else if( uwTmpReg == 12 || uwTmpReg == 11 || uwTmpReg == 10 ) 
				{
					/* #12=none,#11=%dbbr,#10=%idir */
					iStatus |= fnPop (999);	/* 999 is no pop( do not update register ) */
				}
				else
				{	/* #2:%alr --------- #9=%dp */

					/* >>>>> add iruma m.takeishi '04.03.16 */
					if ( (uwTmpReg + PSR) ==  LCO ||
						 (uwTmpReg + PSR) ==  LEA ||
						 (uwTmpReg + PSR) ==  LSA ) {
						if ( ulFlag & LM_FLAG ) {
							/* if loop mode, can't pop LCO, LEA, LSA. */
							iStatus |= ILL_INST_BREAK;
							ulFlag &= ~RC_BITS;
							ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
							gdbsim_store_register( PSR, ulFlag );
							
							/* unfrun.c:2225, 
								if (stop_signal == TARGET_SIGNAL_TRAP && stop_after_trap)
								{
								  stop_print_frame = 0;
								  stop_stepping (ecs);
								  return;
								}
							 */
							{ extern int stop_after_trap;
							  stop_after_trap = 1;
							}
							break;
						}
					}
					/* <<<<< add iruma m.takeishi '04.03.16 */

					iStatus |= fnPop (uwTmpReg + PSR);			/* pop #18 --- #25 */
				}

				ulFlag &= ~RC_BITS;
				ulFlag |= ( ( uwTmpReg << 24 ) & 0x0f000000 );
				gdbsim_store_register( PSR, ulFlag );

				/*@ check nomap_mem_break or rom_data_break */
				/* >>>>> change iruma m.takeishi '04.03.16 */
				/*
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK)) != 0)	// error ? //
				*/
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK
								|| ILL_INST_BREAK )) != 0)	/* error ? */
				/* <<<<< change iruma m.takeishi '04.03.16 */
				{
					break;
				}
			}

			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW | DATA_BREAK )) == 0 ) 	/* No error ? */
			{
				ulFlag &= ~PM_FLAG;
				ulFlag &= ~RC_BITS;
				gdbsim_store_register( PSR, ulFlag );
			}
			/* <<<<< change T.Tazaki 2004/01/28 */
		}
		else
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		}
		break;
	}

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (PUSHPOP_ILLEGAL | NOMAP_MEM_BREAK | ROM_DATA_BREAK | ILL_INST_BREAK | STACK_OVERFLOW | DATA_BREAK )) != 0) {
	;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnExeControl : execute call %rb / jp %rb  / jpr %rb *
 ********************************************************/

#ifndef SPARC

int 
fnExeControl (unsigned short uwOp2, unsigned short uwRegNum, int blDelayed)
#else /* SPARC */

int
fnExeControl (uwOp2, uwRegNum, blDelayed)
	 unsigned short  uwOp2;
	 unsigned short  uwRegNum;	/* rb/imm2 */
	 int			 blDelayed; /* if "d" bit set */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulAddr, ulPC;

	iStatus = 0;		/* initialize */


	switch (uwOp2) {		/* instruction */

	case 0: 		/* 00 call %rb */

	fnInterlock( C33_NORMAL, uwRegNum , 0, 0, 0 );

	/*@ check if next inst. is call.d */
	if (blDelayed == YES) {
		/*@ skip delayed inst. */
		vfnIncPC (2 * CODE);	/* PC += CODE + CODE */

		iStatus |= DELAY_FLAG;	/* set delay_flag */

		gdb_AddCycle( 2 );
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */

		gdb_AddCycle( 3 );
	}

	/*@ push PC to stack */
	iStatus |= fnPush (PC);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/* check if next inst. is call.d */
		if (blDelayed == YES) {
			/*@ undo skip */
			vfnIncPC (-2 * CODE);	/* PC -= CODE + CODE */
		}
		else {
			/*@ undo skip */
			vfnIncPC (-1 * CODE);	/* PC -= CODE */
		}
	}
	else {
		/*@ read address to call */
		ulAddr = gdbsim_fetch_register (uwRegNum);
		ulAddr &= (unsigned long)0xfffffffe;		/* LSB = 0 set */

		/*@ set address to PC */
		(void) gdbsim_store_register (PC, ulAddr);
	}

	break;

	case 2: 		/* 10 jp %rb */

	fnInterlock( C33_NORMAL, uwRegNum , 0, 0, 0 );

	/* correct fetch of no use,accumlate before PC changed */
	if (blDelayed == YES) {
		iStatus |= DELAY_FLAG;	/* set delay_flag */
		gdb_AddCycle( 2 );
	}
	else {
		gdb_AddCycle( 1 );
	}

	/*@ read address to jump */
	ulAddr = gdbsim_fetch_register (uwRegNum);
	ulAddr &= (unsigned long)0xfffffffe;		/* LSB = 0 set */

	/*@ set address to PC */
	(void) gdbsim_store_register (PC, ulAddr);

	break;

	case 3: 		/* 11 jpr %rb */

	fnInterlock( C33_NORMAL, uwRegNum , 0, 0, 0 );

	/*@ check if next inst. is jpr.d */
	if (blDelayed == YES) {
		iStatus |= DELAY_FLAG;	/* set delay_flag */
		gdb_AddCycle( 3 );
	}
	else {
		gdb_AddCycle( 4 );
	}

	/*@ read address */
	ulAddr = gdbsim_fetch_register (uwRegNum);
	ulPC = gdbsim_fetch_register (PC);

	/*@ calculate jump address : %pc <- %pc + %rb*/
	ulAddr = ulPC + (ulAddr & (unsigned long)0xfffffffe );
	ulAddr &= (unsigned long)0xfffffffe;		/* LSB = 0 set */

	/*@ set address to PC */
	(void) gdbsim_store_register (PC, ulAddr);

	break;

	}

	vfnInitExtension ();	/* initialize extension */

	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	return (iStatus);		/* return status */
}

/************************************************
 *	fnExeReturn : execute reti/ret		*
 ************************************************/

#ifndef SPARC

int 
fnExeReturn (unsigned short uwOp2, int blDelayed)
#else /* SPARC */

int
fnExeReturn (uwOp2, blDelayed)
	 unsigned short  uwOp2;
	 int			 blDelayed; /* if "d" bit set */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData;

	extern int		nTrapAfterNMI;

	/* >>>>> add iruma m.takeishi '04.03.26 */
	int 			iBuf;
	unsigned long	ulPsr;
	unsigned long	ulData2;
	/* <<<<< add iruma m.takeishi '04.03.26 */

	iStatus = 0;		/* initialize */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	switch (uwOp2) {		/* instruction */

	case 1: 		/* 01 ret */

	/*@ check if next isnt. is ret.d */
	if (blDelayed == YES) {
		iStatus |= DELAY_FLAG;	/* set delay_flag */

		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		{
			gdb_AddCycle(3);
		}
		else
		{
			gdb_AddCycle(4);
		}
	}
	else {
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		{
			gdb_AddCycle(4);
		}
		else
		{
			gdb_AddCycle(5);
		}
	}

	/* pop PC from stack */
	iStatus |= fnPop (PC);

	break;

	case 3: 		/* 11 reti */

	/*@ check if "d" bit is on */
	if (blDelayed == YES) {
		iStatus |= ILL_INST_BREAK;	/* set illegal_inst_break */
	}
	else {
		/*@ check if NMI happened */
		if (nTrapAfterNMI > 0) {
			/*@ count reti execution */
			nTrapAfterNMI--;
		}
		else {
		   ;
		}
		
//		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD || fnCheckAdvancedMode() == C33_PRODUCT_PE ) /* add PE check T.Tazaki 2004/08/05 */
		{
			gdb_AddCycle(5);
			/*@ pop PSR from stack */
			iStatus = fnPop (PSR);

			/*@ check nomap_mem_break */
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
			;
			}
			else {
				/*@ pop PC from stack */
				iStatus |= fnPop (PC);
			}
		}
		else
		{
		
			/* T.Tazaki 2003/08/04 >>> */
			gdb_AddCycle(6);
			/* >>>>> change iruma m.takeishi '04.03.26 */
			/*
			//@ pop PSR from stack //
			iStatus = fnPop (PSR);
			*/
			/* read PSR at first */
			gdbsim_store_register( SP, gdbsim_fetch_register(SP)+4 );
			iStatus = fnPop (PC);
			gdbsim_store_register( SP, gdbsim_fetch_register(SP)-8 );
			/* <<<<< change iruma m.takeishi '04.03.26 */

			/*@ check nomap_mem_break */
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
				;
			}
			else {
				/* >>>>> change iruma m.takeishi '04.03.28 */
				/*
				//@ pop PC from stack //
				iStatus |= fnPop (PC);
				*/
				/*@ pop PSR secondly */
				ulData2 = gdbsim_fetch_register( PSR );
				iStatus |= fnPop (PSR);
				if ( ulData2 & SV_FLAG ) {
					iBuf = USP;
				} else {
					iBuf = SSP;
				}
				gdbsim_store_register( iBuf, gdbsim_fetch_register( iBuf ) - 4 );
				/* <<<<< change iruma m.takeishi '04.03.29 */
				
				/* >>>>> add iruma m.takeishi '04.03.29 */
				if ( iIsLoop_int == 1 ) {
					ulData = gdbsim_fetch_register( PSR );
					ulData |= LM_FLAG;
					gdbsim_store_register( PSR, ulData );
					iIsLoop_int = 0;
				}
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
				
				} else {
					
					if ( ulData2 & SV_FLAG ) {
						gdbsim_store_register( USP,  gdbsim_fetch_register( USP ) + 8 );
					} else {
						/* change SV flag tempolaly */
						ulData2 = gdbsim_fetch_register( PSR );
						if ((ulData2 & SV_FLAG)) {
							/* SUPER ==> user */
							gdbsim_store_register( PSR,  gdbsim_fetch_register( PSR ) & ~SV_FLAG );
							ulData2 = 1;
						} else {
							ulData2 = 0;
						}
						gdbsim_store_register( SSP,  gdbsim_fetch_register( SSP ) + 8 );
						if ( ulData2 == 1 ) {
							/* SUPER ==> user */
							gdbsim_store_register( PSR,  gdbsim_fetch_register( PSR ) | SV_FLAG );
						}
					}
					
				}
				/* <<<<< add iruma m.takeishi '04.03.29 */

			}

#if 0
			/* PC <- Read W[SP+4] */
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register(SP) + 4, (char *)&ulData, 4, 0, NULL, NULL);
			
			/*@ check not exception, not error break */
			if (iStatus == 0) {
				
				(void) gdbsim_store_register (PC, ulData);

				/* PSR <- Read W[SP] */
				iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register(SP), (char *)&ulData, 4, 0, NULL, NULL);
				
				/*@ check not exception, not error break */
				if (iStatus == 0) {
					
					(void) gdbsim_store_register (PSR, ulData);

					/* SP = SP + 8 */
					(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) + 8));
				}
			}
#endif
			/* T.Tazaki 2003/08/04 <<< */
		}
	}

	break;
	}


	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/************************************************
 *	fnExeReturnMMU : execute retm	   *
 ************************************************/

#ifndef SPARC

int 
fnExeReturnMMU (int blDelayed)
#else /* SPARC */

int
fnExeReturnMMU (blDelayed)
	 int			 blDelayed; /* if "d" bit set */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '03.08.19 */
	unsigned long	ulData;
	/* <<<<< add iruma m.takeishi '03.08.19 */

	extern int		nTrapAfterNMI;

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	iStatus = 0;		/* initialize */

	/*@ check if "d" bit is on */
	if (blDelayed == YES) {
		iStatus |= ILL_INST_BREAK;	/* set illegal_inst_break */
	}
	else {

		gdb_AddCycle(6);

		/* >>>>> change iruma m.takeishi '03.08.19 */
		/*@ pop R0 from stack */
		/*
		iStatus = fnPop (R0);
		*/
		iStatus = gdbsim_xfer_inferior_memory ( 0x1c, (char *)&ulData, 4, 0, NULL, NULL);
		/* <<<<< change iruma m.takeishi '03.08.19 */
		

		/*@ check nomap_mem_break */
		if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		;
		}
		else {
			
			
			/* >>>>> change iruma m.takeishi '03.08.19 */
			/*@ pop PSR from stack */
			/*
			iStatus = fnPop (PSR);			// add 2002.06.10 T.Tazaki C33 //
			*/
			gdbsim_store_register ( R0, ulData );
			iStatus = gdbsim_xfer_inferior_memory ( 0x18, (char *)&ulData, 4, 0, NULL, NULL);
			/* <<<<< change iruma m.takeishi '03.08.19 */
			
			
			/*@ check nomap_mem_break */
			if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
			;
			}
			else{
				
				/* >>>>> change iruma m.takeishi '03.08.19 */
				/*@ pop PC from stack */
				/* ME flag clear */
				/*
				ulPSR = ulReg[PSR];
				ulPSR &= ~ME_FLAG;
				(void) gdbsim_store_register (PSR, ulPSR);
				*/

				ulReg[ PC ] = ulData;
				/* <<<<< change iruma m.takeishi '03.08.19 */

			}
		}
	}


	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnExeBranch : execute branch instruction	*
 ********************************************************/

#define Z(ulFlag)	(ulFlag & Z_FLAG)
#define C(ulFlag)	(ulFlag & C_FLAG)
#define NV(ulFlag)	((ulFlag & N_FLAG) ^ ((ulFlag & V_FLAG) >> 2))

#ifndef SPARC

int 
fnExeBranch (unsigned short uwOp1, unsigned short uwImm, int blDelayed)
#else /* SPARC */

int
fnExeBranch (uwOp1, uwImm, blDelayed)
	 unsigned short  uwOp1;
	 unsigned short  uwImm;
	 int			 blDelayed; /* if "d" bit set */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulFlag;
	long			lOffset = 0;
	int 			iCodeSize;	/* skip code size for "call" */
	int 			blBranch;	/* if branch ? */
	unsigned long	ulNowPc;	/* now pc */ /* 04/25/2000 C.Nakayama		 Modify API spec and clock function*/
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	
	/* >>>>> add iruma m.takeishi '04.03.25 */
	unsigned long	 ulBuf;
	/* <<<<< add iruma m.takeishi '04.03.25 */

	/* >>>> add iruma m.takeishi '03.09.01 */
	/* >>>>> change iruma m.takeishi '04.03.29 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_IMM_OK );
	*/
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_NG, CMB_EXT_IMM_OK );
	/* <<<<< change iruma m.takeishi '04.03.29 */
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.01 */


	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	iStatus = 0;		/* initialize */
	blBranch = NO;		/* not branch */


	ulNowPc = gdbsim_fetch_register(PC);	/* save pc */

	/*@ read ulFlag from PSR */
	ulFlag = gdbsim_fetch_register (PSR);

	/*@ calculate offset by extension type */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

	/*@ extend immediate value */
	lOffset = (stExtension.ulData << 6) & 0xffc00000;	/* ext	 -> [31..22] */
	lOffset |= (stExtension.ulData << 9) & 0x003ffe00;	/* ext	 -> [21..9] */
	lOffset |= uwImm << 1;	/* uwImm -> [8..0] */
	break;

	case EXT_L: 	/* ext:target */

	/*@ extend immediate value */
	lOffset = (stExtension.ulData << 9) | (uwImm << 1);

	/*@ check if offset is negative */
	if ((lOffset & 0x200000) != 0) {
		lOffset |= ~0x3fffff;	/* negative */
	}
	else {
		lOffset &= 0x3fffff;	/* positive */
	}
	break;

	case EXT_NONE:		/* no ext */

	/*@ calculate offset by 1bit shift of immediate value */
	lOffset = (unsigned long) uwImm << 1;

	/*@ check if offset is negative */
	if ((lOffset & 0x100) != 0) {
		lOffset |= ~0x1ff;	/* negative */
	}
	else {
		lOffset &= 0x1ff;	/* positive */
	}
	break;
	}

	switch (uwOp1) {		/* instruction */

	case 4: 		/* 0100 jrgt sign9 */

	/*@ check if condition is true */
	if (!(Z (ulFlag)) && !(NV (ulFlag))) {	/* !Z && !(N^V) */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 5: 		/* 0101 jrge sign9 */

	/*@ check if condition is true */
	if (!(NV (ulFlag))) {	/* !(N^V) */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 6: 		/* 0110 jrlt sign9 */

	/*@ check if condition is true */
	if (NV (ulFlag)) {	/* N^V */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 7: 		/* 0111 jrle sign9 */

	/*@ check if condition is true */
	if (Z (ulFlag) || NV (ulFlag)) {	/* Z | (N^V) */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 8: 		/* 1000 jrugt sign9 */

	/*@ check if condition is true */
	if (!Z (ulFlag) && !C (ulFlag)) {	/* !Z && !C */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 9: 		/* 1001 jruge sign9 */

	/*@ check if condition is true */
	if (!C (ulFlag)) {	/* !C */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 10:			/* 1010 jrult sign9 */

	/*@ check if condition is true */
	if (C (ulFlag)) {	/* C */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 11:			/* 1011 jrule sign9 */

	/*@ check if condition is true */
	if (Z (ulFlag) || C (ulFlag)) { 	/* Z | C */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 12:			/* 1100 jreq sign9 */

	/*@ check if condition is true */
	if (Z (ulFlag)) {	/* Z */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 13:			/* 1101 jrne sign9 */

	/*@ check if condition is true */
	if (!Z (ulFlag)) {	/* !Z */

		blBranch = YES; /* branch */
		vfnIncPC (lOffset); /* PC += offset */
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE, no branch */
	}

	break;

	case 14:			/* 1110 call sign9 */

	/*@ check if next inst. is delayed */
	if (blDelayed == YES) {
		iCodeSize = CODE + CODE;	/* skip delayed inst. */
		/* >>>>> change iruma m.takeishi '03.08.20 */
		/*
		gdb_AddCycle( 2 );	// call.d //
		*/
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD ) {
			gdb_AddCycle( 2 );	/* call.d */
		} else {
			gdb_AddCycle( 1 );	/* call.d */
		}
		/* <<<<< change iruma m.takeishi '03.08.20 */

	}
	else {
		iCodeSize = CODE;	/* no delayed inst. */
		/* >>>>> change iruma m.takeishi '03.08.20 */
		/*
		gdb_AddCycle( 3 );	// call //
		*/
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD ) {
			gdb_AddCycle( 3 );	/* call */
		} else {
			gdb_AddCycle( 2 );	/* call */
		}
		/* <<<<< change iruma m.takeishi '03.08.20 */

	}

	vfnIncPC (iCodeSize);	/* increase PC */

	/*@ push PC to stack */
	iStatus |= fnPush (PC);

	/*@ check error break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/*@ error break, restore PC */
		vfnIncPC (-1 * iCodeSize);
	}
	else {
		/*@ branch */
		blBranch = YES;
		vfnIncPC (lOffset - iCodeSize);
	}

	break;

	case 15:			/* 1111 jp sign9 */

	/*@ branch */
	vfnIncPC (lOffset);
	blBranch = YES;

	break;
	}

	/*@ check if next inst. is delayed */
	if (blDelayed == YES) {
		/*@ check if branch */
		if (blBranch == YES) {
			iStatus |= DELAY_FLAG;	/* set delay_flag */
		}
		else {
			;
		}
		
		/* >>>>> change iruma m.takeishi '04.03.25 */
		/*
		if( uwOp1 != 14 ){
			gdb_AddCycle( 1 );	// jp.d, jrxx.d //
		}
		*/
		if( uwOp1 != 14 ){
			if ( uwOp1 != 15 ){  /* jp.d */
				ulBuf = 1;
			} else {
				/* jrxxx.d */
				ulBuf = 1;
				if (blBranch != YES) {
					ulBuf++;
				}
			}
			gdb_AddCycle( ulBuf );
		}
		/* <<<<< change iruma m.takeishi '04.03.25 */
	}
	else
	{
		if (blBranch == YES && uwOp1 != 14) { /* 14 = call, call.d */
			gdb_AddCycle( 2 );
		}
		else
		{
			/* >>>>> change iruma m.takeishi '04.03.25 */
			/*
			if( uwOp1 != 14 ){
				gdb_AddCycle( 1 );	// jrxx //
			}
			*/
			switch ( uwOp1 ) {
			case 14:	   /* call sign */
				break;
			case 15:	   /* jp   sign */
				gdb_AddCycle( 2 );
				break;
			default:	   /* jrxx sign */
				gdb_AddCycle( 1 );
				break;
			}
			/* <<<<< change iruma m.takeishi '04.03.25 */
		}
	}

	/* >>>>> add iruma m.takeishi '04.03.17 */
	if ( (blBranch == YES && uwOp1 != 14 ) ||	 /* jrxxx */
		 (					 uwOp1 == 14 ) ||	 /* call  */	
		 (					 uwOp1 == 15 )		 /* jp	  */	
	   ) {
		iRsvLoop_instCnt = 0;
	}
	/* <<<<< add iruma m.takeishi '04.03.17 */

	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		gdb_AddCycle(1);
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnExeTrans : execute translation instruction	*
 ********************************************************/

#ifndef SPARC

int 
fnExeTrans (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegBase, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeTrans (uwOp1, uwOp2, uwRegBase, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwRegBase;
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData; /* word data to translate */
	unsigned short	uwData; /* half word data to translate */
	short			wData;	/* half word data to translate */
	unsigned char	bData;	/* byte data to translate */
	char			cData;	/* byte data to translate */
	unsigned long	ulAddr;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	iStatus = 0;		/* initialize */

	/* >>>> add iruma m.takeishi '03.09.01 */
	/* >>>>> change iruma m.takeishi '04.03.22 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_IMM_OK );
	*/
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_RS_OP_IMM_OK |	
						CMB_EXT_IMM_OK );
	/* <<<<< change iruma m.takeishi '04.03.22 */
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.01 */

	/* >>>>> add iruma m.takeishi '04.03.22 */
	if ( ( stExtension.iType == EXT_HL || stExtension.iType == EXT_L ) &&	
		   stExtension.iTypeExt == EXT_RS_OP_IMM ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '04.03.22 */

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 ld.b  %rd,[%rb] */

	fnInterlock( C33_INDIRECT, uwRegNum, uwRegBase, 1, 2 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}
	

	/*@ read base address */
	ulAddr = gdbsim_fetch_register (uwRegBase);

	/*@ read data from memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
						 (char *)&ulData, 1, 0, NULL, NULL);
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
						 (char *)&ulData, 1, 0, NULL, NULL);
		}
	}
	else{
		/* post increment */
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 1, 0, NULL, NULL);
	}
	
	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		cData = (char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) cData);

		/*@ check post increment */
		if (uwOp2 == 1) {
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += BYTE_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += BYTE_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 1: 		/* 001 ld.ub %rd,[%rb] */

	fnInterlock( C33_INDIRECT, uwRegNum, uwRegBase, 1, 2 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}

	/*@ read base address */
	ulAddr = gdbsim_fetch_register (uwRegBase);

	/*@ read data from memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}


	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		bData = (unsigned char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) bData);

		/*@ check post increment */
		if (uwOp2 == 1) {
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += BYTE_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += BYTE_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 2: 		/* 010 ld.h  %rd,[%rb] */

	fnInterlock( C33_INDIRECT, uwRegNum, uwRegBase, 1, 2 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}

	/*@ read base address */
	ulAddr = gdbsim_fetch_register (uwRegBase);

	/*@ read data from memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}


	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		wData = (short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) wData);

		/*@ check post increment */
		if (uwOp2 == 1) {
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += HALF_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += HALF_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 3: 		/* 011 ld.uh %rd,[%rb] */

	fnInterlock( C33_INDIRECT, uwRegNum, uwRegBase, 1, 2 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}

	/*@ read base address */
	ulAddr = gdbsim_fetch_register (uwRegBase);

	/*@ read data from memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		uwData = (unsigned short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) uwData);

		/*@ check post increment */
		if (uwOp2 == 1) {
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += HALF_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += HALF_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 4: 		/* 100 ld.w  %rd,[%rb] */

	fnInterlock( C33_INDIRECT, uwRegNum, uwRegBase, 1, 2 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}

	/*@ read base address */
	ulAddr = gdbsim_fetch_register (uwRegBase);

	/*@ read data from memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		gdbsim_store_register (uwRegNum, ulData);

		/*@ check post increment */
		if (uwOp2 == 1) {
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += WORD_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += WORD_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 5: 		/* 101 ld.b [%rb],%rd */

	fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xff;

	/*@ write data to memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else
	{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ check post increment */
		if (uwOp2 == 1) {
			/*@ increase base register */
			ulAddr = gdbsim_fetch_register (uwRegBase);
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += BYTE_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += BYTE_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		   ;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 6: 		/* 110 ld.h [%rb],%rd */

	fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xffff;

	/*@ write data to memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 2, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 2, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else
	{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 2, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}
	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ check post increment */
		if (uwOp2 == 1) {
			/*@ increase base register */
			ulAddr = gdbsim_fetch_register (uwRegBase);
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += HALF_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += HALF_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
			;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 7: 		/* 111 ld.w [%rb],%rd */

	fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum);

	/*@ write data to memory */
	if (uwOp2 == 0) {
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulDataExtRs, 
					  (char *)&ulData, 4, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
		else
		{
			iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase) + stExtension.ulData,	
					  (char *)&ulData, 4, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		}
	}
	else
	{
		iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (uwRegBase), (char *)&ulData, 4, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
	}

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ check post increment */
		if (uwOp2 == 1) {
			/*@ increase base register */
			ulAddr = gdbsim_fetch_register (uwRegBase);
			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
			{
				/* ext imm13, ext %rs */
				if( stExtension.iType == EXT_L || stExtension.iType == EXT_HL || stExtension.iType == EXT_RS )
				{
					ulAddr += stExtension.ulData;	/* %rb1 + %rb2 */
				}
				else if ( stExtension.iTypeExt == EXT_RS_OP_IMM)
				{
					ulAddr += stExtension.ulDataExtRs;	 /* %rb1 + %rb2 */
				}
				else
				{
					/*@ increase base register */
					ulAddr += WORD_DATA;
				}
			}
			else
			{
				/*@ increase base register */
				ulAddr += WORD_DATA;
			}
			gdbsim_store_register (uwRegBase, ulAddr);
		}
		else {
		   ;
		}

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************************************
 *	fnExeTransSpecial : execute special reg. translation instruction	*
 ********************************************************************************/

#ifndef SPARC

int 
fnExeTransSpecial (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeTransSpecial (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;
	 unsigned short  uwRegDest;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData; /* word data to translate */
	unsigned long	ulTemp;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	gdb_AddCycle(1);

	/*@ check if op1 is 0 */
	if (uwOp1 == 0) {	/* ld.w %sd,%rs */

		/* add T.Tazaki 2004/08/25 >>> */
		if ( fnCheckAdvancedMode() == C33_PRODUCT_PE && SPECIAL_REG(uwRegDest) == (unsigned short) PSR )
		{
			gdb_AddCycle(2);		/* ld.w %psr,%rs : 3 cycle */
		}
		/* add T.Tazaki 2004/08/25 <<< */

		if (((SPECIAL_REG(uwRegDest) >= (unsigned short) PSR) &&
			(SPECIAL_REG(uwRegDest) <= (unsigned short) DP )) || (uwRegDest == 13) || (uwRegDest == 14)) {

//			if( ( fnCheckAdvancedMode() != C33_PRODUCT_ADV ) && ( SPECIAL_REG(uwRegDest) >= (unsigned short) LCO))	/* Advanced Mode ? */
			if( ( fnCheckAdvancedMode() == C33_PRODUCT_STD ) && ( SPECIAL_REG(uwRegDest) >= (unsigned short) LCO))	/* Standard Mode ? */
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			}
			else
			{
				if( ( fnCheckAdvancedMode() == C33_PRODUCT_PE ) &&					/* add T.Tazaki 2004/08/05 */
			 			  SPECIAL_REG(uwRegDest) >= (unsigned short) LCO &&
			 			  SPECIAL_REG(uwRegDest) != (unsigned short) TTBR  )
		   		{
					vfnIncPC (CODE);	/* PC += CODE */ /* not error, nop */
				}
				else
				{
					fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

					/* read data from register */
					ulData = gdbsim_fetch_register (uwRegSrc);

					if(uwRegDest == 13 || uwRegDest == 14)
					{
//						uwRegDest -= 3;
						uwRegDest -= 1;	/* change T.Tazaki 2003/12/03 */
					}
					
					if( SPECIAL_REG (uwRegDest) == PSR )
					{
						/* DE(bit14),ME(bit13) flag : Not Change */
						ulTemp = gdbsim_fetch_register( PSR );
						ulTemp &= ( DE_FLAG | ME_FLAG );
						ulData &= ~DE_FLAG;
						ulData &= ~ME_FLAG;
						ulData |= ulTemp;

						/* User mode : HE/SV/IL/IE = Read ONLY Bit */
						ulTemp = gdbsim_fetch_register( PSR );
						if( ulTemp & SV_FLAG )	/* user mode ? */
						{
							ulTemp &= ( HE_FLAG | SV_FLAG | IL_BITS | IE_FLAG );
							ulData &= ~HE_FLAG;
							ulData &= ~SV_FLAG;
							ulData &= ~IL_BITS;
							ulData &= ~IE_FLAG;
							ulData |= ulTemp;
						}
						
						/* >>>>> add iruma m.takeishi '04.03.15 */
						if ( (ulTemp & LM_FLAG) == 0 && (ulData & LM_FLAG) != 0 ) {
							iRsvLoop_instCnt = RESERVELOOP_INSTCNT;
						}
						/* <<<<< add iruma m.takeishi '04.03.15 */

					}

					/* write data to special register */
					gdbsim_store_register (SPECIAL_REG (uwRegDest), ulData);

					vfnIncPC (CODE);	/* PC += CODE */
				}
			}
		}
		else {
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		}
	}
	else {
		/* ld.w %rd,%ss */

		if (((SPECIAL_REG(uwRegSrc) >= (unsigned short) PSR) &&
//			((SPECIAL_REG(uwRegSrc)) <= (unsigned short) DP)) || (uwRegSrc == 13) || (uwRegSrc == 14) || (uwRegSrc == 15)) {
			((SPECIAL_REG(uwRegSrc)) <= (unsigned short) DBBR)) || (uwRegSrc == 13) || (uwRegSrc == 14) || (uwRegSrc == 15)) {/* change T.Tazaki 2003/12/03 */

//			if( ( fnCheckAdvancedMode() != C33_PRODUCT_ADV ) && ( SPECIAL_REG(uwRegSrc) >= (unsigned short) LCO))  /* Not Advanced Mode ? */
			if( ( fnCheckAdvancedMode() == C33_PRODUCT_STD ) && ( SPECIAL_REG(uwRegSrc) >= (unsigned short) LCO))  /* Standard Mode ? */
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			}
			else
			{
				if( ( fnCheckAdvancedMode() == C33_PRODUCT_PE ) &&					/* add T.Tazaki 2004/08/05 */
			 			( SPECIAL_REG(uwRegSrc) >= (unsigned short) LCO ) &&
						( SPECIAL_REG(uwRegSrc) != (unsigned short) TTBR  &&
						  SPECIAL_REG(uwRegSrc) != (unsigned short) IDIR  &&
						  SPECIAL_REG(uwRegSrc) != (unsigned short) DBBR  &&
						  uwRegSrc != 15 )											/* add T.Tazaki 2005/02/28 */
						)
		   		{
					vfnIncPC (CODE);	/* PC += CODE */ /* no error , nop */
				}
				else
				{
					/*@ read data from special register */
					if(uwRegSrc == 13 || uwRegSrc == 14 || uwRegSrc == 15 )
					{
//						uwRegDest -= 3;
//						uwRegDest -= 1;	/* change T.Tazaki 2003/12/03 */ /* del bug fix 2003/12/19 T.Tazaki */
						uwRegSrc  -= 1;	/* bug fix T.Tazaki 2003/12/19 */
					}

					fnInterlock( C33_NORMAL, SPECIAL_REG(uwRegSrc), 0, 0, 0 );

					ulData = gdbsim_fetch_register (SPECIAL_REG (uwRegSrc));

					if( SPECIAL_REG (uwRegSrc) == PC )	/* add T.Tazaki 2003/03/03 */
					{
						if (stDelaySlot.blUsed == YES) {
							ulData = stDelaySlot.ulAddr;
						}
						else
						{
							ulData+=2;
						}
					}
					/*@ write data to register */
					gdbsim_store_register (uwRegDest, ulData);

					vfnIncPC (CODE);	/* PC += CODE */
				}
			}
		}
		else {
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		}
	}


	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/************************************************************************
 *	fnExeTransReg : execute register translation instruction	*
 ************************************************************************/

#ifndef SPARC

int 
fnExeTransReg (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeTransReg (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;	/* rs */
	 unsigned short  uwRegDest; /* rd */

#endif /* SPARC */

{				/* return iStatus( = 0) */
	short			wData;	/* half word data to translate */
	unsigned short	uwData; /* half word data to translate */
	char			cData;	/* byte data to translate */
	unsigned char	bData;	/* byte data to translate */
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );
	gdb_AddCycle(1);

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 ld.b %rd,%rs */

	/*@ read data from register */
	cData = (char) gdbsim_fetch_register (uwRegSrc);

	/*@ write data to register */
	(void) gdbsim_store_register (uwRegDest, (unsigned long) cData);

	break;

	case 1: 		/* 001 ld.ub %rd,%rs */

	/*@ read data from register */
	bData = (unsigned char) gdbsim_fetch_register (uwRegSrc);

	/*@ write data to register */
	(void) gdbsim_store_register (uwRegDest, (unsigned long) bData);

	break;

	case 2: 		/* 010 ld.h %rd,%rs */

	/*@ read data from register */
	wData = (short) gdbsim_fetch_register (uwRegSrc);

	/*@ write data to register */
	(void) gdbsim_store_register (uwRegDest, (unsigned long) wData);

	break;

	case 3: 		/* 011 ld.uh %rd,%rs */

	/*@ read data from register */
	uwData = (unsigned short) gdbsim_fetch_register (uwRegSrc);

	/*@ write data to register */
	(void) gdbsim_store_register (uwRegDest, (unsigned long) uwData);

	break;
	}


	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/********************************************************
 *	fnExeStack : execute stack instruction		*
 ********************************************************/

#ifndef SPARC

int 
fnExeStack (unsigned short uwOp1, unsigned short uwImm, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeStack (uwOp1, uwImm, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwImm;
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus = 0;
	unsigned long	ulData; /* word data to translate */
	unsigned short	uwData; /* half word data to translate */
	short			wData;	/* half word data to translate */
	unsigned char	bData;	/* byte data to translate */
	char			cData;	/* byte data to translate */
	unsigned long	ulAddr;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */


	/* >>>> add iruma m.takeishi '03.09.01 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK | CMB_EXT_IMM_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.01 */


	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	/* Advanced Mode ? */
	{
		gdb_AddCycle(1);
	}else{
		gdb_AddCycle(2);
	}
	
	/*@ read SP */
	ulAddr = gdbsim_fetch_register (SP);

	/*@ check if ext instruction exist */
	if (stExtension.iType != EXT_NONE) {
	/*@ extend immedaite value */

	switch (uwOp1) {	/* data size */

		/*@ byte data */
	case 0: 	/* 000 ld.b  %rd,[%sp+imm6] */
	case 1: 	/* 001 ld.ub %rd,[%sp+imm6] */

		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 ); /* %sp = 1 */

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | uwImm;
		break;

	case 5: 	/* 101 ld.b [%sp+imm6],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | uwImm;
		break;

		/*@ half data */
	case 2: 	/* 010 ld.h  %rd,[%sp+imm7] */
	case 3: 	/* 011 ld.uh %rd,[%sp+imm7] */
		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x1);
		break;

	case 6: 	/* 110 ld.h [%sp+imm7],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x1);
		break;

		/*@ word data */
	case 4: 	/* 100 ld.w  %rd,[%sp+imm8] */

		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x3);
		break;

	case 7: 	/* 111 ld.w [%sp+imm8],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ extend immediate value */
		ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x3);
		break;
	}
	}
	else {
	/*@ use only immediate value */

	switch (uwOp1) {	/* data size */

		/*@ byte data */
	case 0: 	/* 000 ld.b  %rd,[%sp+imm6] */
	case 1: 	/* 001 ld.ub %rd,[%sp+imm6] */
		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

		/*@ use immediate value */
		ulAddr += uwImm;
		break;

	case 5: 	/* 101 ld.b [%sp+imm6],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ use immediate value */
		ulAddr += uwImm;
		break;

		/*@ half data */
	case 2: 	/* 010 ld.h  %rd,[%sp+imm7] */
	case 3: 	/* 011 ld.uh %rd,[%sp+imm7] */
		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

		/*@ shift immediate value by 1 bit */
		ulAddr += uwImm << 1;
		break;

	case 6: 	/* 110 ld.h [%sp+imm7],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ shift immediate value by 1 bit */
		ulAddr += uwImm << 1;
		break;

		/*@ word data */
	case 4: 	/* 100 ld.w  %rd,[%sp+imm8] */
		fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

		/*@ shift immediate value by 2 bits */
		ulAddr += uwImm << 2;
		break;

	case 7: 	/* 111 ld.w [%sp+imm8],%rd */

		fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

		/*@ shift immediate value by 2 bits */
		ulAddr += uwImm << 2;
		break;
	}
	}

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 ld.b  %rd,[%sp+imm6] */

	/*@ read data from stack */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		cData = (char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) cData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 1: 		/* 001 ld.ub %rd,[%sp+imm6] */

	/*@ read data from stack */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		bData = (unsigned char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) bData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 2: 		/* 010 ld.h  %rd,[%sp+imm7] */

	/*@ read data from stack */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		wData = (short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) wData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 3: 		/* 011 ld.uh %rd,[%sp+imm7] */

	/*@ read data from stack */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		uwData = (unsigned short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) uwData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 4: 		/* 100 ld.w  %rd,[%sp+imm8] */

	/*@ read data from stack */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		gdbsim_store_register (uwRegNum, ulData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 5: 		/* 101 ld.b [%sp+imm6],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xff;

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 6: 		/* 110 ld.h [%sp+imm7],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xffff;

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 7: 		/* 111 ld.w [%sp+imm8],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum);

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 4, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnExeDataPoint : execute data pointer instruction	   *
 ********************************************************/

#ifndef SPARC

int 
fnExeDataPoint (unsigned short uwOp1, unsigned short uwImm, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeDataPoint (uwOp1, uwImm, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwImm;
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus = 0;
	unsigned long	ulData; /* word data to translate */
	unsigned short	uwData; /* half word data to translate */
	short			wData;	/* half word data to translate */
	unsigned char	bData;	/* byte data to translate */
	char			cData;	/* byte data to translate */
	unsigned long	ulAddr;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK | CMB_EXT_IMM_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	gdb_AddCycle(1);

	/*@ read SP */
	ulAddr = gdbsim_fetch_register (DP);

	/*@ check if ext instruction exist */
	if (stExtension.iType != EXT_NONE) {
		/*@ extend immedaite value */

		switch (uwOp1) {	/* data size */

			/*@ byte data */
		case 0: 	/* 000 ld.b  %rd,[%dp+imm6] */
		case 1: 	/* 001 ld.ub %rd,[%dp+imm6] */
			fnInterlock( C33_INDIRECT, uwRegNum, SP, 1, 2 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | uwImm;
			break;

		case 5: 	/* 101 ld.b [%dp+imm6],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | uwImm;
			break;

			/*@ half data */
		case 2: 	/* 010 ld.h  %rd,[%dp+imm7] */
		case 3: 	/* 011 ld.uh %rd,[%dp+imm7] */
			fnInterlock( C33_INDIRECT, uwRegNum, DP, 1, 2 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x1);
			break;

		case 6: 	/* 110 ld.h [%dp+imm7],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x1);
			break;

			/*@ word data */
		case 4: 	/* 100 ld.w  %rd,[%dp+imm8] */
			fnInterlock( C33_INDIRECT, uwRegNum, DP, 1, 2 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x3);
			break;

		case 7: 	/* 111 ld.w [%dp+imm8],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ extend immediate value */
			ulAddr += (stExtension.ulData << 6) | (uwImm & ~0x3);
			break;
		}
	}
	else {
		/*@ use only immediate value */

		switch (uwOp1) {	/* data size */

			/*@ byte data */
		case 0: 	/* 000 ld.b  %rd,[%dp+imm6] */
		case 1: 	/* 001 ld.ub %rd,[%dp+imm6] */
			fnInterlock( C33_INDIRECT, uwRegNum, DP, 1, 2 );

			/*@ use immediate value */
			ulAddr += uwImm;
			break;

		case 5: 	/* 101 ld.b [%dp+imm6],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ use immediate value */
			ulAddr += uwImm;
			break;

			/*@ half data */
		case 2: 	/* 010 ld.h  %rd,[%dp+imm7] */
		case 3: 	/* 011 ld.uh %rd,[%dp+imm7] */
			fnInterlock( C33_INDIRECT, uwRegNum, DP, 1, 2 );

			/*@ shift immediate value by 1 bit */
			ulAddr += uwImm << 1;
			break;

		case 6: 	/* 110 ld.h [%dp+imm7],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ shift immediate value by 1 bit */
			ulAddr += uwImm << 1;
			break;

			/*@ word data */
		case 4: 	/* 100 ld.w  %rd,[%dp+imm8] */
			fnInterlock( C33_INDIRECT, uwRegNum, DP, 1, 2 );

			/*@ shift immediate value by 2 bits */
			ulAddr += uwImm << 2;
			break;

		case 7: 	/* 111 ld.w [%dp+imm8],%rd */

			fnInterlock( C33_NORMAL, uwRegNum, 0, 0, 0 );

			/*@ shift immediate value by 2 bits */
			ulAddr += uwImm << 2;
			break;
		}
	}

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 ld.b  %rd,[%dp+imm6] */

	/*@ read data from data pointer */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		cData = (char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) cData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 1: 		/* 001 ld.ub %rd,[%dp+imm6] */

	/*@ read data from data pointer */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		bData = (unsigned char) (ulData & 0xff);
		gdbsim_store_register (uwRegNum, (unsigned long) bData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 2: 		/* 010 ld.h  %rd,[%dp+imm7] */

	/*@ read data from data pointer */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		wData = (short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) wData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 3: 		/* 011 ld.uh %rd,[%dp+imm7] */

	/*@ read data from data pointer */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		uwData = (unsigned short) (ulData & 0xffff);
		gdbsim_store_register (uwRegNum, (unsigned long) uwData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 4: 		/* 100 ld.w  %rd,[%dp+imm8] */

	/*@ read data from pointer */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 4, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		/*@ write data to register */
		gdbsim_store_register (uwRegNum, ulData);

		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 5: 		/* 101 ld.b [%dp+imm6],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xff;

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 6: 		/* 110 ld.h [%dp+imm7],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum) & 0xffff;

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 2, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;

	case 7: 		/* 111 ld.w [%dp+imm8],%rd */

	/*@ read data from register */
	ulData = gdbsim_fetch_register (uwRegNum);

	/*@ write data to memory */
	iStatus = gdbsim_xfer_inferior_memory (ulAddr, (char *)&ulData, 4, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	/*@ check error break & exception */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | S_ADDR_EX)) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}

	break;
	}

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnExeArith : execute arithmetic instruction *
 ********************************************************/




#ifndef SPARC

int 
fnExeArith (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeArith (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;
	 unsigned short  uwRegDest;

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataSrc, ulDataSrc2, ulDataDest = 0;	/* word data */
	unsigned long	ulFlag;
	unsigned long	ulTmpFlag;

	/* >>>>> add iruma m.takeishi '03.9.01 */
	int 	iExt = CMB_EXT_NG, iRet;
	/* <<<<< add iruma m.takeishi '03.9.01 */
	

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	/* >>>>> add iruma m.takeishi '04.03.18 */
	switch( uwOp1 ) {
	case 4: 		/* 100 and	%rd,%rs */
	case 5: 		/* 101 or	%rd,%rs */
	case 6: 		/* 110 xor	%rd,%rs */
	case 7: 		/* 111 not	%rd,%rs */
		iRet = 0;
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
			if ( stExtension.iType == EXT_HL || stExtension.iType == EXT_L ) {
				/*	
				 *	ext %rs,op,imm2 ( ==> ext %rs)
				 *	ext imm13
				 * (ext imm13)
				 *	and/or/xor/not	%rd, %rs
				 */
				return ILL_INST_BREAK;
			}
			/* treat ext %rs,op,imm2 as ext %rs */
			stExtension.iType  = EXT_RS;
			stExtension.ulData = stExtension.ulDataExtRs;
			stExtension.uwDataExtOp = 0;
		}
		break;
	}
	/* <<<<< add iruma m.takeishi '04.03.18 */

	/* >>>> add iruma m.takeishi '03.09.01 */
	switch ( uwOp1 ) {
	case 0: 		/* 000 add	%rd,%rs */
	case 1: 		/* 001 sub	%rd,%rs */
		iExt = CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_RS_OP_IMM_OK
			   | CMB_EXT_OP_IMM_OK | CMB_EXT_IMM_OK;
		break;
	case 2: 		/* 010 cmp	%rd,%rs */
		iExt = CMB_EXT_COND_OK | CMB_EXT_IMM_OK;
		break;
	case 3: 		/* 011 ld.w %rd,%rs */
		iExt = CMB_EXT_COND_OK;
		break;
	case 4: 		/* 100 and	%rd,%rs */
	case 5: 		/* 101 or	%rd,%rs */
	case 6: 		/* 110 xor	%rd,%rs */
		/* >>>>> change iruma m.takeishi '04.03.30 */
		/*
		iExt = CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_IMM_OK;
		*/
		iExt = CMB_EXT_COND_OK | CMB_EXT_RS_OK |	
			   CMB_EXT_RS_OP_IMM_OK |	
				CMB_EXT_IMM_OK ;
		
		/* <<<<< change iruma m.takeishi '04.03.30 */
		break;

	case 7: 		/* 111 not	%rd,%rs */
		/* >>>>> change iruma m.takeishi '04.03.30 */
		/*
		iExt = CMB_EXT_COND_OK;
		*/
		iExt = CMB_EXT_COND_OK | CMB_EXT_RS_OP_IMM_OK;
		/* <<<<< change iruma m.takeishi '04.03.30 */
		break;
	}
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, iExt );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	
	/* <<<<< add iruma m.takeishi '03.09.01 */




	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

	gdb_AddCycle( 1 );

	/*@ read PSR/src */
	ulFlag = gdbsim_fetch_register (PSR);
	ulDataSrc = gdbsim_fetch_register (uwRegSrc);

	/*@ check if ext inst exist */
	if (stExtension.iType != EXT_NONE ) {
		/*@ use data of ext as src2 */
		ulDataSrc2 = stExtension.ulData;
	}
	else {
		/*@ use data of register as src2 */
		ulDataSrc2 = gdbsim_fetch_register (uwRegDest); /* EXT_NONE */
	}

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 add	%rd,%rs */

	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
		ulDataSrc2 = stExtension.ulDataExtRs;
	}

	/*@ calculate CV flag */
	ulTmpFlag = fnChangeCV_ADD (WITHOUT_CARRY, ulDataSrc, ulDataSrc2, ulFlag);

	/*@ calculate */
	ulDataDest = ulDataSrc + ulDataSrc2;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);		/* add T.Tazaki 2002/10/03 */

	/* >>>>> delete iruma m.takeishi '04.03.17 */
	/*
	if(stExtension.iTypeExt == EXT_OP_IMM || stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
	  switch(stExtension.uwDataExtOp & 0xc)
	  {
		case 0x4:
		  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0x8:
		  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0xc:
		  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
		  break;
	  }
	}
	*/
	/* <<<<< delete iruma m.takeishi '04.03.17 */

	break;

	case 1: 		/* 001 sub	%rd,%rs */


	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
		ulDataSrc2 = stExtension.ulDataExtRs;

		/*@ calculate CV flag */
		ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataSrc2, ulDataSrc, ulFlag);

		/*@ calculate */
		ulDataDest = ulDataSrc - ulDataSrc2;

	}
	else if (stExtension.iType != EXT_NONE) {
		/*@ check if ext inst exist */

		/*@ calculate CV flag */
		ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataSrc2, ulDataSrc, ulFlag);

		/*@ calculate */
		ulDataDest = ulDataSrc - ulDataSrc2;
	}
	else {
		/*@ calculate CV flag */
		ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataSrc, ulDataSrc2, ulFlag);

		/*@ calculate */
		ulDataDest = ulDataSrc2 - ulDataSrc;
	}

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);		/* add T.Tazaki 2002/10/03 */

	/* >>>>> delete iruma m.takeishi '04.03.17 */
	/*
	if(stExtension.iTypeExt == EXT_OP_IMM || stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
	  switch(stExtension.uwDataExtOp & 0xc)
	  {
		case 0x4:
		  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0x8:
		  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0xc:
		  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
		  break;
	  }
	}
	*/
	/* <<<<< delete iruma m.takeishi '04.03.17 */

	break;

	case 2: 		/* 010 cmp	%rd,%rs */

	/*@ read dest from register */
	ulDataDest = gdbsim_fetch_register (uwRegDest);

	/*@ check if ext inst exist */
	if (stExtension.iType != EXT_NONE) {
		/*@ calculate CV flag */
		ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataSrc2, ulDataSrc, ulFlag);

		/*@ calculate NZ flag */
		ulFlag = fnChangeNZ ((ulDataSrc - ulDataSrc2), ulTmpFlag);
	}
	else {
		/*@ calculate CV flag */
		ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataSrc, ulDataSrc2, ulFlag);

		/*@ calculate NZ flag */
		ulFlag = fnChangeNZ ((ulDataSrc2 - ulDataSrc), ulTmpFlag);
	}

	break;

	case 3: 		/* 011 ld.w %rd,%rs */

	/*@ transfer data */
	ulDataDest = ulDataSrc;

	break;

	case 4: 		/* 100 and	%rd,%rs */

	/* >>>>> change iruma m.takeishi '04.03.17 */
	/*
	if( stExtension.iTypeExt == EXT_RS_OP_IMM)
	*/
	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	/* <<<<< change iruma m.takeishi '04.03.17 */
	{
		ulDataSrc2 = stExtension.ulDataExtRs;
	}

	/*@ calculate */
	ulDataDest = ulDataSrc & ulDataSrc2;


	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}

	break;

	case 5: 		/* 101 or	%rd,%rs */

	/* >>>>> change iruma m.takeishi '04.03.17 */
	/*
	if( stExtension.iTypeExt == EXT_RS_OP_IMM)
	*/
	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	/* <<<<< change iruma m.takeishi '04.03.17 */
	{
		ulDataSrc2 = stExtension.ulDataExtRs;
	}

	/*@ calculate */
	ulDataDest = ulDataSrc | ulDataSrc2;

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 6: 		/* 110 xor	%rd,%rs */

	/* >>>>> change iruma m.takeishi '04.03.17 */
	/*
	if( stExtension.iTypeExt == EXT_RS_OP_IMM)
	*/
	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	/* <<<<< change iruma m.takeishi '04.03.17 */
	{
		ulDataSrc2 = stExtension.ulDataExtRs;
	}

	/*@ calculate */
	ulDataDest = ulDataSrc ^ ulDataSrc2;

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 7: 		/* 111 not	%rd,%rs */

	/*@ calculate */
	ulDataDest = ~ulDataSrc;

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;
	}


	/* >>>>> add iruma m.takeishi '04.03.17 */
	if( stExtension.iTypeExt == EXT_OP_IMM ||	
		stExtension.iTypeExt == EXT_RS_OP_IMM) {
		switch( uwOp1 ) {
		case 0: 		/* 000 add	%rd,%rs */
		case 1: 		/* 001 sub	%rd,%rs */
		case 4: 		/* 100 and	%rd,%rs */
		case 5: 		/* 101 or	%rd,%rs */
		case 6: 		/* 110 xor	%rd,%rs */
		case 7: 		/* 111 not	%rd,%rs */
			switch(stExtension.uwDataExtOp & 0xc) {
			case 0x4:
			  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
			  break;
			case 0x8:
			  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
			  break;
			case 0xc:
			  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
			  break;
			}
			break;
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.17 */

	/*@ write dest/PSR */
	gdbsim_store_register (uwRegDest, ulDataDest);
	gdbsim_store_register (PSR, ulFlag);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}



/********************************************************
 *	fnExeArithDP : execute arithmetic dp instruction *
 ********************************************************/

#ifndef SPARC

int 
fnExeArithDP (unsigned short uwRegDest)
#else /* SPARC */

int
fnExeArithDP (uwRegDest)
	 unsigned short  uwRegDest;

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataSrc, ulDPData, ulDataDest;  /* word data */
	unsigned long	ulFlag;
	unsigned long	ulTmpFlag;

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	fnInterlock( C33_NORMAL, DP, 0, 0, 0 ); 

	/* DO NOT Change C,V,Z,N flag */

	/*@ read PSR/src */
	ulFlag = gdbsim_fetch_register (PSR);
	
	ulDPData = gdbsim_fetch_register (DP);

	/*@ check if ext inst exist */
	if (stExtension.iType != EXT_NONE) {
		/*@ use data of ext as src2 */
		ulDataSrc = stExtension.ulData;
	}
	else
	{
		/*@ use data of register as src2 */
		ulDataSrc = gdbsim_fetch_register (uwRegDest);
	}

	if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
		ulDataSrc = stExtension.ulDataExtRs;
	}

	/*@ calculate CV flag */
	ulTmpFlag = fnChangeCV_ADD (WITHOUT_CARRY, ulDPData, ulDataSrc, ulFlag);

	/*@ calculate */
	ulDataDest = ulDPData + ulDataSrc;

	/* >>>>> change iruma m.takeishi '03.09.01 */
	/*
	if(stExtension.iTypeExt == EXT_RS_OP_IMM)
	*/
	if(stExtension.iTypeExt == EXT_OP_IMM || stExtension.iTypeExt == EXT_RS_OP_IMM)
	/* <<<<< change iruma m.takeishi '03.09.01 */
	{
	  switch(stExtension.uwDataExtOp & 0xc)
	  {
		case 0x4:
		  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0x8:
		  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0xc:
		  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
		  break;
	  }
	}

	/*@ write dest */
	gdbsim_store_register (uwRegDest, ulDataDest);
	
	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/************************************************************************
 *	fnExeArithImm : execute Arithmetic immediate instruction	*
 ************************************************************************/

#ifndef SPARC

int 
fnExeArithImm (unsigned short uwOp1, unsigned short uwImm6, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeArithImm (uwOp1, uwImm6, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwImm6;
	 unsigned short  uwRegNum;

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataDest; /* word data */
	unsigned long	ulDataImm = 0;	/* word data */
	unsigned long	ulFlag;
	unsigned long	ulTmpFlag;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	/* >>>>> add iruma m.takeishi '04.03.17 */
	int 		   iExtBuf;
	/* <<<<< add iruma m.takeishi '04.03.17 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	/* >>>>> change iruma m.takeishi '04.03.18 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK | CMB_EXT_IMM_OK );
	*/
	iExtBuf = CMB_EXT_COND_OK | CMB_EXT_IMM_OK;
	switch( uwOp1 ) {
	case 0: 		/* 000 add	%rd,imm6 */
	case 1: 		/* 001 sub	%rd,imm6 */
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
			stExtension.iTypeExt = EXT_OP_IMM;
			stExtension.ulDataExtRs = 0;
		}
		iExtBuf |= CMB_EXT_RS_OP_IMM_OK | CMB_EXT_OP_IMM_OK;
		break;
	
	case 4: 		/* 100 and	%rd,sign6 */
	case 5: 		/* 101 or	%rd,sign6 */
	case 6: 		/* 110 xor	%rd,sign6 */
	case 7: 		/* 111 not	%rd,sign6 */
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
			stExtension.iTypeExt = EXT_NONE;
			stExtension.ulDataExtRs = 0;
			stExtension.uwDataExtOp = 0;
		}
		/* >>>>> delete iruma m.takeishi '04.03.30 */
		/*
		iExtBuf |= CMB_EXT_OP_IMM_OK;
		*/
		/* <<<<< delete iruma m.takeishi '04.03.30 */
		break;
	}
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, iExtBuf );
	/* <<<<< change iruma m.takeishi '04.03.18 */

	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );
	gdb_AddCycle(1);

	/*@ read PSR/dest */
	ulFlag = gdbsim_fetch_register (PSR);
	ulDataDest = gdbsim_fetch_register (uwRegNum);

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 add	%rd,imm6 */

	/*@ check if ext inst exist */
	/* >>>>> add iruma m.takeishi '04.03.17 */
	/*
	if (stExtension.iType != EXT_NONE) {
	*/
	if (stExtension.iType != EXT_NONE && stExtension.iTypeExt != EXT_RS ) {
	/* <<<<< change iruma m.takeishi '04.03.17 */
		/*@ expand immediate value with zero */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
	}
	else {
		/*@ use immediate value as is */
		ulDataImm = uwImm6;
	}

	/*@ calculate CV flag */
	ulTmpFlag = fnChangeCV_ADD (WITHOUT_CARRY, ulDataImm, ulDataDest, ulFlag);

	/*@ calculate */
	ulDataDest += ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);

	break;

	case 1: 		/* 001 sub	%rd,imm6 */

	/*@ check if ext inst exist */
	if (stExtension.iType != EXT_NONE) {
		/*@ expand immediate value with zero */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
	}
	else {
		/*@ use immediate value as is */
		ulDataImm = uwImm6;
	}

	/*@ calculate CV flag */
	ulTmpFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataImm, ulDataDest, ulFlag);

	/*@ calculate */
	ulDataDest -= ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);

	break;

	case 2: 		/* 010 cmp	%rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ calculate CV flag */
	ulFlag = fnChangeCV_SUB (WITHOUT_CARRY, ulDataImm, ulDataDest, ulFlag);

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ ((ulDataDest - ulDataImm), ulFlag);

	break;

	case 3: 		/* 011 ld.w %rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ transfer data */
	ulDataDest = ulDataImm;

	break;

	case 4: 		/* 100 and	%rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ calculate */
	ulDataDest &= ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	/* add T.Tazaki 2005/03/14 bug fix >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}
	/* add T.Tazaki 2005/03/14 bug fix <<< */

#if 0	/* del T.Tazaki 2005/03/14 bug fix */
/* 12/19/2001 N.Komatsu  if OC flag is 1, clear V flag */
	if (ulFlag & OC_FLAG) {
		ulFlag &= ~V_FLAG;
	}
#endif

	break;

	case 5: 		/* 101 or	%rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ calculate */
	ulDataDest |= ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	/* add T.Tazaki 2005/03/14 bug fix >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}
	/* add T.Tazaki 2005/03/14 bug fix <<< */

#if 0	/* del T.Tazaki 2005/03/14 bug fix */
/* 12/19/2001 N.Komatsu  if OC flag is 1, clear V flag */
	if (ulFlag & OC_FLAG) {
		ulFlag &= ~V_FLAG;
	}
#endif

	break;

	case 6: 		/* 110 xor	%rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ calculate */
	ulDataDest ^= ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	/* add T.Tazaki 2005/03/14 bug fix >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}
	/* add T.Tazaki 2005/03/14 bug fix <<< */

#if 0	/* del T.Tazaki 2005/03/14 bug fix */
/* 12/19/2001 N.Komatsu  if OC flag is 1, clear V flag */
	if (ulFlag & OC_FLAG) {
		ulFlag &= ~V_FLAG;
	}
#endif

	break;

	case 7: 		/* 111 not	%rd,sign6 */

	/*@ check if ext inst is exist */
	switch (stExtension.iType) {
	case EXT_HL:		/* ext:ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;
		break;

	case EXT_L: 	/* ext:target */

		/*@ expand immediate value */
		ulDataImm = (stExtension.ulData << 6) | uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x40000) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x7ffff;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x7ffff;
		}
		break;

	case EXT_NONE:		/* no ext */

		/*@ use immediate value as is */
		ulDataImm = uwImm6;

		/*@ check negative */
		if ((ulDataImm & 0x20) != 0) {
		/*@ expand with sign */
		ulDataImm |= ~0x3f;
		}
		else {
		/*@ expand with zero */
		ulDataImm &= 0x3f;
		}
		break;
	}

	/*@ calculate */
	ulDataDest = ~ulDataImm;

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	/* add T.Tazaki 2005/03/14 bug fix >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {			// add T.Tazaki 2005/02/01 bug fix
		/*	if OC flag is 1, clear V flag */
		if (ulFlag & OC_FLAG) {
			ulFlag &= ~V_FLAG;
		}
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{		// add T.Tazaki 2005/02/01 bug fix
		ulFlag &= ~V_FLAG;
	}
	/* add T.Tazaki 2005/03/14 bug fix <<< */

#if 0	/* del T.Tazaki 2005/03/14 bug fix */
/* 12/19/2001 N.Komatsu  if OC flag is 1, clear V flag */
	if (ulFlag & OC_FLAG) {
		ulFlag &= ~V_FLAG;
	}
#endif
	break;
	}






	/* >>>>> add iruma m.takeishi '04.03.18 */
	if ( stExtension.iTypeExt == EXT_OP_IMM ) {
		if ( uwOp1 == 0 /* 000 add	%rd,imm6 */ ||	
			 uwOp1 == 1 /* 000 sub	%rd,imm6 */ ) {
			switch(stExtension.uwDataExtOp & 0xc) {
			case 0x4:
			  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
			  break;
			case 0x8:
			  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
			  break;
			case 0xc:
			  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
			  break;
			}
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.18 */





	/*@ write dest/PSR */
	gdbsim_store_register (uwRegNum, ulDataDest);
	gdbsim_store_register (PSR, ulFlag);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/********************************************************
 *	fnExeArithSP : execute add/sub %sp,imm12	*
 ********************************************************/

#ifndef SPARC

int 
fnExeArithSP (unsigned short uwOp1, unsigned short uwImm10)
#else /* SPARC */

int
fnExeArithSP (uwOp1, uwImm10)
	 unsigned short  uwOp1;
	 unsigned short  uwImm10;

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataDest; /* word data */
	unsigned long	ulDataImm;	/* word data */
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	/* >>>>> change iruma m.takeishi '04.03.17 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	*/
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
				  CMB_EXT_RS_OP_IMM_OK | CMB_EXT_OP_IMM_OK | CMB_EXT_COND_OK );
	/* <<<<< change iruma m.takeishi '04.03.17 */
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read SP */
	ulDataDest = gdbsim_fetch_register (SP);

	/*@ calculate immediate value by 2 bit shift */
	ulDataImm = uwImm10 << 2;

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 add %sp,imm12 */

	/*@ calculate */
	ulDataDest += ulDataImm;
	break;

	case 1: 		/* 001 sub %sp,imm12 */

	/*@ calculate */
	ulDataDest -= ulDataImm;
	break;
	}

	/* >>>>> add iruma m.takeishi '04.03.17 */
	if(stExtension.iTypeExt == EXT_OP_IMM || stExtension.iTypeExt == EXT_RS_OP_IMM)
	{
	  switch(stExtension.uwDataExtOp & 0xc)
	  {
		case 0x4:
		  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0x8:
		  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
		  break;
		case 0xc:
		  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
		  break;
	  }
	}
	/* <<<<< add iruma m.takeishi '04.03.17 */


	/* write SP */
	gdbsim_store_register (SP, ulDataDest);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/********************************************************
 *	fnExeArithC : execute adc/sbc instruction	*
 ********************************************************/
#define CARRY(ulFlag)	((ulFlag & C_FLAG) >> 3)

#ifndef SPARC

int 
fnExeArithC (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeArithC (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;	/* rs */
	 unsigned short  uwRegDest; /* rd */

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataSrc;	/* word data */
	unsigned long	ulDataDest; /* word data */
	unsigned long	ulFlag;
	unsigned long	ulTmpFlag;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> add iruma m.takeishi '04.03.10 */
	unsigned long	ulDataDestTmp;
	/* <<<<< add iruma m.takeishi '04.03.10 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_RS_OP_IMM_OK |	
						CMB_EXT_OP_IMM_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read PSR/src/dest */
	ulFlag = gdbsim_fetch_register (PSR);
	ulDataSrc = gdbsim_fetch_register (uwRegSrc);
	ulDataDest = gdbsim_fetch_register (uwRegDest);


	if (uwOp1 == 6) {		/* instruction */

		/*@ Yes uwOp1 == 110 adc */

		/* >>>>> change iruma m.takeishi '04.03.10 */
		//@ calculate CV flag //
		/*
		ulTmpFlag = fnChangeCV_ADD (WITH_CARRY, ulDataSrc, ulDataDest, ulFlag);

		if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			  //@ calculate //
			  ulDataDest = ulDataSrc + stExtension.ulDataExtRs + CARRY (ulFlag);
		*/
		
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM || stExtension.iTypeExt == EXT_OP_IMM )
		{
			  /*@ calculate */
			  if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
				  ulDataDestTmp = stExtension.ulDataExtRs;
			  } else {
				  /* ext   op,imm
				   * adc   %rd,%rs */
				  ulDataDestTmp = ulDataDest;
			  }
			  ulTmpFlag = fnChangeCV_ADD (WITH_CARRY, ulDataSrc, ulDataDestTmp, ulFlag);

			  ulDataDest = ulDataDestTmp + ulDataSrc + CARRY (ulFlag);
				
			  ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);
		/* <<<<< change iruma m.takeishi '04.03.10 */

			  switch(stExtension.uwDataExtOp & 0xc)
			  {
				case 0x4:
				  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
				  break;
				case 0x8:
				  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
				  break;
				case 0xc:
				  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
				  break;
			  }
		}
		else if (stExtension.iType == EXT_RS )
		{
			/*@ check if ext %rs inst exist */
			/*@ calculate */
			/* >>>>> add iruma m.takeishi '04.03.10 */
			ulTmpFlag = fnChangeCV_ADD (WITH_CARRY, ulDataSrc, stExtension.ulData, ulFlag);
			/* <<<<< add iruma m.takeishi '04.03.10 */
			ulDataDest = ulDataSrc + stExtension.ulData + CARRY (ulFlag);
		}
		else
		{
			/* >>>>> add iruma m.takeishi '04.03.10 */
			ulTmpFlag = fnChangeCV_ADD (WITH_CARRY, ulDataSrc, ulDataDest, ulFlag);
			/* <<<<< add iruma m.takeishi '04.03.10 */
			/*@ calculate */
			ulDataDest = ulDataDest + ulDataSrc + CARRY (ulFlag);
		}
		
		/*@ calculate NZ flag */
		/* >>>>> delete iruma m.takeishi '04.03.10 */
		/*
		ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);
		*/
		/* <<<<< delete iruma m.takeishi '04.03.10 */
	}
	else {
		/*@ No	uwOp1 == 111 sbc */

		/* >>>>> change iruma m.takeishi '04.03.10 */
		/*
		if (stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_RS_OP_IMM)
		{
			  //@ calculate CV flag //
			  ulTmpFlag = fnChangeCV_SUB (WITH_CARRY, stExtension.ulDataExtRs, ulDataSrc, ulFlag);
		*/
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM || stExtension.iTypeExt == EXT_OP_IMM )
		{
		/* <<<<< change iruma m.takeishi '04.03.10 */

			  /*@ calculate */
			  /* >>>>> change iruma m.takeishi '04.03.10 */
			  /*
			  ulDataDest = ulDataSrc - stExtension.ulDataExtRs - CARRY (ulFlag);
			  */
			  if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
				  ulTmpFlag = fnChangeCV_SUB (WITH_CARRY, stExtension.ulDataExtRs, ulDataSrc, ulFlag);
				  ulDataDest = ulDataSrc - stExtension.ulDataExtRs - CARRY (ulFlag);
			  } else {
				  ulTmpFlag = fnChangeCV_SUB (WITH_CARRY, ulDataSrc, ulDataDest, ulFlag);
				  ulDataDest = ulDataDest - ulDataSrc - CARRY (ulFlag);
			  }
				
			  ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);
			  /* <<<<< change iruma m.takeishi '04.03.10 */
				
			  switch(stExtension.uwDataExtOp & 0xc)
			  {
				case 0x4:
				  (long)ulDataDest = (long)ulDataDest >> (stExtension.uwDataExtOp & 0x3);
				  break;
				case 0x8:
				  ulDataDest = ulDataDest >> (stExtension.uwDataExtOp & 0x3);
				  break;
				case 0xc:
				  ulDataDest = ulDataDest << (stExtension.uwDataExtOp & 0x3);
				  break;
			  }
		}
		else if (stExtension.iType == EXT_RS )
		{
			/*@ calculate CV flag */
			ulTmpFlag = fnChangeCV_SUB (WITH_CARRY, stExtension.ulData, ulDataSrc, ulFlag);

			/*@ check if ext %rs inst exist */
			/*@ calculate */
			ulDataDest = ulDataSrc - stExtension.ulData - CARRY (ulFlag);
		}
		else
		{
			/*@ calculate CV flag */
			ulTmpFlag = fnChangeCV_SUB (WITH_CARRY, ulDataSrc, ulDataDest, ulFlag);
			
			/*@ calculate */
			ulDataDest = ulDataDest - ulDataSrc - CARRY (ulFlag);
		}
		
		/* >>>>> delete iruma m.takeishi '04.03.10 */
		/*
		//@ calculate NZ flag //
		ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);
		*/
		/* >>>>> delete iruma m.takeishi '04.03.10 */
	}

	/* >>>>> add iruma m.takeishi '04.03.10 */
	/*@ calculate NZ flag */
	if ( stExtension.iTypeExt != EXT_RS_OP_IMM && stExtension.iTypeExt != EXT_OP_IMM )
	{
		ulFlag = fnChangeNZ (ulDataDest, ulTmpFlag);
	}
	/* <<<<< add iruma m.takeishi '04.03.10 */


	/*@ write dest/PSR */
	gdbsim_store_register (uwRegDest, ulDataDest);
	gdbsim_store_register (PSR, ulFlag);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/****************************************************************
 *	fnExeShiftRot : execute Shift/Rotate instruction	*
 ****************************************************************/

#ifndef SPARC

int 
fnExeShiftRot (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwShift, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeShiftRot (uwOp1, uwOp2, uwShift, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwShift;	/* imm4/imm5/rs */
	 unsigned short  uwRegNum;	/* rd */

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataDest; /* word data to shift/rotate */
	unsigned long	ulShift = 0; /* shift/rotate bit number */
	unsigned long	ulFlag, ulCNFlag, ulSOR;
	unsigned long	ulLastBit;	/* shift out at last */
	int 			ndxShift;
	int 			iSignBit;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.01 */
	/* >>>>> change iruma m.takeishi '04.03.22 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK );
	*/
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK |
						CMB_EXT_RS_OP_IMM_OK );
	/* <<<<< change iruma m.takeishi '04.03.22 */
	if ( iRet != 0 ) {
		return ( ILL_INST_BREAK);
	}
	/* <<<<< add iruma m.takeishi '03.09.01 */

	/* >>>>> add iruma m.takeishi '04.03.22 */
	if (stExtension.iTypeExt == EXT_RS_OP_IMM ) {
		if ( stExtension.iType == EXT_HL || 
			 stExtension.iType == EXT_L ) {
			 /* don't permit below inst 
			  * ext   %rs, op, imm2
			  * ext   imm13
			  * (shift inst)
			  */
			return ( ILL_INST_BREAK);
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.22 */

	gdb_AddCycle(1);

	/*	ulSOR initialize */
	ulSOR = 0;	

	/*@ read PSR/dest */
	ulFlag = gdbsim_fetch_register (PSR);
	ulDataDest = gdbsim_fetch_register (uwRegNum);

	/*@ check if size of shift is given by immediate value */
	switch (uwOp2) {  /* op2 */

	case 0:    /* 00 imm4 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	/*@ size of shift is given by register */
	ulShift = (unsigned long) (uwShift & (unsigned short) 0xf);
	break;

	/* change size of shift from 0-8 to 0-31 */
	case 1:    /* 01 %rs */

	fnInterlock( C33_NORMAL, uwShift, 0, 0, 0 );

	/*@ size of shift is given by register */
	ulShift = gdbsim_fetch_register (uwShift) & (unsigned long) 0x1f;
	break;

	/* change size of shift from 0-8 to 0-31 */
	case 3:    /* 11 imm5 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	/*@ size of shift is given by immediate value */
	ulShift = (unsigned long) (uwShift & (unsigned short) 0x1f);
	break;

	}

	/* limit size of shift from 8 to 31 */
//	  if (ulShift > 7) {
//	  ulShift = 8;		  /* limit size */
//	  }
	if (ulShift >= 31){
		ulShift = 31;		/* limit size */
	}
	else {
	   ;
	}

	/* add bug fix T.Tazaki 2007/08/02 >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
	{
		ulShift &= 0x0f;
		if( ulShift >= 8 )
		{
			ulShift = 8;
		}
	}
	/* add bug fix T.Tazaki 2007/08/02 <<< */

	if (stExtension.iType == EXT_RS )				/* ext %rs ? */
	{
		ulDataDest = stExtension.ulData;
	}
	else if (stExtension.iTypeExt == EXT_RS_OP_IMM) /* ext %rs,op,imm2 ? */
	{
		ulDataDest = stExtension.ulDataExtRs;
	}

	/*@ calculate size of shift */
	switch (uwOp1 << 2 | uwOp2) {

	case 3: 		/* 000:11 srl %rd,imm5 */
	case 8: 		/* 010:00 srl %rd,imm4 */
	case 9: 		/* 010:01 srl %rd,%rs */


	/* store the bit which shift out to SOR register */
	/*@ for size of shift */
	for (ndxShift = 0; ndxShift < (int) ulShift; ndxShift++) {
		/*@ lower bit check */
		if ((ulDataDest & LSB_BIT) == 0) {	  /* lower bit is high */
			/*@ make SOR register data */
			ulSOR >>= 1;
		}
		else {
			ulSOR = (ulSOR >> 1) | MSB_BIT;
		}
		/*@ shift */
		ulDataDest >>= 1;
	}

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	the process when it is advanced mode */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */
			/*@ get the status of shift out at last */
	//		  ulLastBit = (ulSOR >> (ulShift - 1)) & LSB_BIT;	/* bug fix T.Tazaki 2003/08/04 */
			ulLastBit = ulSOR & MSB_BIT;						/* bug fix T.Tazaki 2003/08/04 */
			
			if (ulLastBit == 0) {
				/*@ if low, set C_FLAG is 0 */
				ulFlag &= ~C_FLAG;
			}
			else {
				/*@ if high, set C_FLAG is 1 */
				ulFlag |= C_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 7: 		/* 001:11 sll %rd,imm5 */
	case 12:		/* 011:00 sll %rd,imm4 */
	case 13:		/* 011:01 sll %rd,%rs */
	case 15:		/* 011:11 sla %rd,imm5 */
	case 20:		/* 101:00 sla %rd,imm4 */
	case 21:		/* 101:01 sla %rd,%rs */

	/*@ for size of shift */
	for (ndxShift = 0; ndxShift < (int) ulShift; ndxShift++) {

		/*	store the bit which shift out to SOR register */
		if ((ulDataDest & MSB_BIT) == 0) {
			ulSOR <<= 1;
		}
		else {
			ulSOR = (ulSOR << 1) | 1;
		}

		/*@ shift */
		ulDataDest <<= 1;
	}

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	the process when it is advanced mode */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */
			/*@ get the status of shift out at last */
			ulLastBit = ulSOR & LSB_BIT;
			
			if (ulLastBit == 0) {
				/*@ if low, set C_FLAG is 0 */
				ulFlag &= ~C_FLAG;
			}
			else {
				/*@ if high, set C_FLAG is 1 */
				ulFlag |= C_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 11:		 /* 010:11 sra %rd,imm5 */
	case 16:		 /* 100:00 sra %rd,imm4 */
	case 17:		 /* 100:01 sra %rd,%rs */

	/*@ execute sign shift */
	if ((ulDataDest & MSB_BIT) == 0) {
		iSignBit = 0;
	}else{
		iSignBit = 1;
	}
	/*@ for size of shift */
	for (ndxShift = 0; ndxShift < (int) ulShift; ndxShift++) {

		/*@ lower bit check */
		if ((ulDataDest & LSB_BIT) == 0) {	  /* lower bit is high */
			/*@ make SOR register data */
			ulSOR >>= 1;
		}
		else {
			ulSOR = (ulSOR >> 1) | MSB_BIT;
		}
		/*@ shift */
		ulDataDest >>= 1;

		/*@ execute sign shift */
		if (iSignBit == 0) {
			ulDataDest &= ~MSB_BIT; /* positive */
		}
		else {
			ulDataDest |= MSB_BIT;	/* negative */
		}

	}

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	the process when it is advanced mode */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */
			/*@ get the status of shift out at last */
	//		  ulLastBit = (ulSOR >> (ulShift - 1)) & LSB_BIT;	/* bug fix T.Tazaki 2003/08/04 */
			ulLastBit = ulSOR & MSB_BIT;						/* bug fix T.Tazaki 2003/08/04 */
			
			if (ulLastBit == 0) {
				/*@ if low, set C_FLAG is 0 */
				ulFlag &= ~C_FLAG;
			}
			else {
				/*@ if high, set C_FLAG is 1 */
				ulFlag |= C_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 19:		 /* 100:11 rr %rd,imm5 */
	case 24:		 /* 110:00 rr %rd,imm4 */
	case 25:		 /* 110:01 rr %rd,%rs */

	/*@ for size of rotate */
	for (ndxShift = 0; ndxShift < (int) ulShift; ndxShift++) {

		/* Modify T.Tazaki 2003/07/28 >>> */
		/*	store the bit which shift out to SOR register */
/*		  if ((ulDataDest & LSB_BIT) != 0) {
			ulSOR |= (1 << ndxShift);
		}
 */
		if ((ulDataDest & LSB_BIT) == 0) {	  /* lower bit is high */
			/*@ make SOR register data */
			ulSOR >>= 1;
		}
		else {
			ulSOR = (ulSOR >> 1) | MSB_BIT;
		}
		/* Modify T.Tazaki 2003/07/28 <<< */

		/*@ execute sign rotate */
		if ((ulDataDest & LSB_BIT) == 0) {
		ulDataDest >>= 1;

		ulDataDest &= ~MSB_BIT; 	/* positive */
		}
		else {
		ulDataDest >>= 1;

		ulDataDest |= MSB_BIT;	/* negative */
		}
	}

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	the process when it is advanced mode */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */
			/*@ get the status of shift out at last */
	//		  ulLastBit = (ulSOR >> (ulShift - 1)) & LSB_BIT;	/* bug fix T.Tazaki 2003/08/04 */
			ulLastBit = ulSOR & MSB_BIT;						/* bug fix T.Tazaki 2003/08/04 */
			
			if (ulLastBit == 0) {
				/*@ if low, set C_FLAG is 0 */
				ulFlag &= ~C_FLAG;
			}
			else {
				/*@ if high, set C_FLAG is 1 */
				ulFlag |= C_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;

	case 23:		 /* 101:11 rl %rd,imm5 */
	case 28:		 /* 111:00 rl %rd,imm4 */
	case 29:		 /* 111:01 rl %rd,%rs */

	/*@ for size of rotate */
	for (ndxShift = 0; ndxShift < (int) ulShift; ndxShift++) {

		/*@ execute sign rotate */
		if ((ulDataDest & MSB_BIT) == 0) {
			ulDataDest <<= 1;

			ulDataDest &= ~LSB_BIT; 	/* positive */

			/*	 store the bit which shift out to SOR register */
			ulSOR <<= 1;
		}
		else {
			ulDataDest <<= 1;

			ulDataDest |= LSB_BIT;	/* negative */

			/*	store the bit which shift out to SOR register */
			ulSOR = (ulSOR << 1) | 1;
		}
	}

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	the process when it is advanced mode */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */
			/*@ get the status of shift out at last */
	//		  ulLastBit = (ulSOR >> (ulShift - 1)) & LSB_BIT;	/* bug fix T.Tazaki 2003/08/04 */
			ulLastBit = ulSOR & LSB_BIT;						/* bug fix T.Tazaki 2003/08/04 */
			
			if (ulLastBit == 0) {
				/*@ if low, set C_FLAG is 0 */
				ulFlag &= ~C_FLAG;
			}
			else {
				/*@ if high, set C_FLAG is 1 */
				ulFlag |= C_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */

	/*@ calculate NZ flag */
	ulFlag = fnChangeNZ (ulDataDest, ulFlag);

	break;
	}

	/*@ execute shift/rotate */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		/*	set V flag */
		/*@ SE_FLAG check */
		if ((ulFlag & SE_FLAG) != 0) {	  /* SE_FLAG is high */

			/*@ check C, N flag */
			ulCNFlag = ulFlag & (N_FLAG | C_FLAG);

			/*@ set or clear of V flag */
			if (ulCNFlag == 0x9 || ulCNFlag == 0) {  /* (!C & !N) | (C & N) */
				ulFlag &= ~V_FLAG;
			}
			else {									/* (C ^ N) */
				ulFlag |= V_FLAG;
			}
		}
	}	/* add bug fix T.Tazaki 2005/02/01 */


	/* write dest/PSR/SOR */
	gdbsim_store_register (uwRegNum, ulDataDest);
	gdbsim_store_register (PSR, ulFlag);
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )			/* add bug fix T.Tazaki 2005/02/01 */
	{
		gdbsim_store_register (SOR, ulSOR);
	}														/* add bug fix T.Tazaki 2005/02/01 */

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/********************************************************
 *	fnDiv0s : Sign_Div initialize AHR/PSR		*
 ********************************************************/

#ifndef SPARC

int 
fnDiv0s (unsigned short uwRegNum)
#else /* SPARC */

int
fnDiv0s (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR, ulSrc;
	struct DoubleWord stDest;

	iStatus = 0;		/* initialize */

	/*@ read divisor/dividend/PSR */
	ulSrc = gdbsim_fetch_register (uwRegNum);
	stDest.ulLow = gdbsim_fetch_register (ALR);
	ulPSR = gdbsim_fetch_register (PSR);

	/*@ check if dividend is positive */
	if ((stDest.ulLow & MSB_BIT) == 0) {
		ulPSR &= ~DS_FLAG;	/* dividend is positive */
	}
	else {
		ulPSR |= DS_FLAG;	/* dividend is negative */
	}

	/*@ set flag of dividend to DS flag of PSR */

	/*@ check if divisor is positive */
	if ((ulSrc & MSB_BIT) == 0) {
		ulPSR &= ~N_FLAG;	/* divisor is positive */
	}
	else {
		ulPSR |= N_FLAG;	/* divisor is negative */
	}

	/*@ set flag of divisor to N flag of PSR */

	/*@ check if dividend is positive */
	if ((stDest.ulLow & MSB_BIT) == 0) {
		stDest.ulHigh = 0;	/* dividend is positive */
	}
	else {
		stDest.ulHigh = ULONG_MAX;	/* dividend is negative */
	}

	/*@ expand dividend to 64 bits with sign */

	/*@ check if divisor is zero */
	if (ulSrc == 0) {
		iStatus = S_ZERO_EX;	/* set zero division exception */
	}
	else
	{

		/*@ check zero division exception */

		/*@ write PSR/dividend */
		gdbsim_store_register (PSR, ulPSR);
		gdbsim_store_register (AHR, stDest.ulHigh);

		if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
		{
			/* result --> %r5 */
			if( ( ulPSR & HC_FLAG ) != 0 ){
				gdbsim_store_register (R5, stDest.ulHigh);
			}
		}

	}

	return (iStatus);		/* return status */
}

/********************************************************
 *	fnDiv0u : Unsign_Div initialize AHR/PSR 	*
 ********************************************************/

#ifndef SPARC

int 
fnDiv0u (unsigned short uwRegNum)
#else /* SPARC */

int
fnDiv0u (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;

	iStatus = 0;		/* initialize */

	/*@ read PSR */
	ulPSR = gdbsim_fetch_register (PSR);

	/*@ set DS flag & N flag of PSR zero */
	ulPSR &= ~DS_FLAG;
	ulPSR &= ~N_FLAG;

	/*@ write PSR/dividend */
	gdbsim_store_register (PSR, ulPSR);
	gdbsim_store_register (AHR, 0);
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, 0);
		}
	}

	/*@ check if divisor is zero */
	if (gdbsim_fetch_register (uwRegNum) == 0)
	{
		iStatus = S_ZERO_EX;	/* set zero division exception */
	}

	return (iStatus);		/* return status */
}

/****************************************************************
 *	vfnDiv1 : Sign/Unsign Div instruction (step 1)		*
 ****************************************************************/

#ifndef SPARC

void 
vfnDiv1 (unsigned short uwRegNum)
#else /* SPARC */

void
vfnDiv1 (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */

{
	struct DoubleWord stSrc, stTmp, stAHR;
	unsigned long	ulPSR, ulALR;

	/*@ read ALR/PSR */
	ulALR = gdbsim_fetch_register (ALR);
	ulPSR = gdbsim_fetch_register (PSR);

	/*@ read divisor */
	stSrc.ulLow = gdbsim_fetch_register (uwRegNum);

	/*@ expand divisor to 33 bits with sign */
	/*@ check if divisor is positive */
	if ((ulPSR & N_FLAG) == 0) {
	stSrc.ulHigh = 0;	/* divisor is positive */
	}
	else {
	stSrc.ulHigh = 1;	/* divisor is negative */
	}

	/*@ expand higher part of dividend to 33 bits with sign */
	stAHR.ulLow = gdbsim_fetch_register (AHR);
	stAHR.ulHigh = 0;

	/*@ execute a part of division */

	/*@ shift dividend by 1 bit */

	/* stAHR.ulHigh = stAHR.ulLow >> 31;  until ver1.4 */
	/* use DS bit in PSR for ver1.5 */
	if ((ulPSR & DS_FLAG) == 0) {
	stAHR.ulHigh = 0;
	}
	else {
	stAHR.ulHigh = 1;
	}

	stAHR.ulLow = (stAHR.ulLow << 1) + (ulALR >> 31);
	ulALR = ulALR << 1;

	/*@ switch (DS flag , N flag) */
	if ((ulPSR & N_FLAG) == 0) {
	if ((ulPSR & DS_FLAG) == 0) {
		/*@ case: DS=0, N=0 */

		/*@ AHR -= divisor */
		stTmp = fnSubDoubleWord (stAHR, stSrc);

		/*@ check if borrow doesn't exist */
		if ((stTmp.ulHigh & 1) == 0) {
		/*@ renew dividend */
		stAHR.ulLow = stTmp.ulLow;
		/*@ %ALR[0]=1 */
		ulALR |= 1;
		}
		else {
		;
		}
	}
	else {
		/*@ case: DS=1, N=0 */

		/*@ AHR += divisor */
		stTmp = fnAddDoubleWord (stAHR, stSrc);

		/*@ check if carry doesn't exist */
		if ((stTmp.ulHigh & 1) == 1) {
		/*@ renew dividend */
		stAHR.ulLow = stTmp.ulLow;
		/*@ %ALR[0]=1 */
		ulALR |= 1;
		}
		else {
		;
		}
	}
	}
	else {
	if ((ulPSR & DS_FLAG) == 0) {
		/*@ case: DS=0, N=1 */

		/*@ AHR += divisor */
		stTmp = fnAddDoubleWord (stAHR, stSrc);

		/*@ check if carry exist */
		if ((stTmp.ulHigh & 1) == 0) {
		/*@ renew dividend */
		stAHR.ulLow = stTmp.ulLow;
		/*@ %ALR[0]=1 */
		ulALR |= 1;
		}
		else {
		;
		}
	}
	else {
		/*@ case: DS=1, N=1 */

		/*@ AHR -= divisor */
		stTmp = fnSubDoubleWord (stAHR, stSrc);

		/*@ check if borrow exist */
		if ((stTmp.ulHigh & 1) == 1) {
		/*@ renew dividend */
		stAHR.ulLow = stTmp.ulLow;
		/*@ %ALR[0]=1 */
		ulALR |= 1;
		}
		else {
		;
		}
	}
	}

	/*@ write dividend */
	gdbsim_store_register (AHR, stAHR.ulLow);
	gdbsim_store_register (ALR, ulALR);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stAHR.ulLow);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, ulALR);
		}
	}
}

/********************************************************
 *	vfnDiv2s : Sign_Div instruction (step 2)	*
 ********************************************************/

#ifndef SPARC

void 
vfnDiv2s (unsigned short uwRegNum)
#else /* SPARC */

void
vfnDiv2s (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */

{
	struct DoubleWord stSrc, stTmp, stAHR;
	unsigned long	ulPSR, ulALR;

	/*@ read PSR/remainder/quotient/divisor */
	ulPSR = gdbsim_fetch_register (PSR);	/* PSR */
	stAHR.ulLow = gdbsim_fetch_register (AHR);	/* remainder */
	ulALR = gdbsim_fetch_register (ALR);	/* quotient */
	stSrc.ulLow = gdbsim_fetch_register (uwRegNum); 	/* divisor */

	/*@ check if DS flag of PSR is 1 */
	if ((ulPSR & DS_FLAG) != 0) {
	/*@ correct quotient & remainder */

/* Don't need upper 32bit in ver1.5r2 */
/* So, just set 0 to upper 32bit	97/7/30 */

	/*@ expand sivisor to 33 bits with sign */
	/* stSrc.ulHigh = stSrc.ulLow >> 31; until ver1.4 */
	/* use N flag in PSR in ver1.5 */
	/* if ((ulPSR & N_FLAG) == 0) { */
	stSrc.ulHigh = 0;	/* divisor is positive */
	/* } */
	/* else { */
	/* stSrc.ulHigh = 1;*/
	/* } */

	/*@ expand remainder to 33 bits with sign */
	/* stAHR.ulHigh = stAHR.ulLow >> 31; until ver1.4 */
	/* use DS flag in PSR in ver1.5 */
	/* if ((ulPSR & DS_FLAG) == 0) { */
	stAHR.ulHigh = 0;
	/* } */
	/* else { */
	/* stAHR.ulHigh = 1; */
	/* } */


	/*@ check if divisor is positive */
	if ((ulPSR & N_FLAG) == 0) {
		/*@ AHR += divisor */
		stTmp = fnAddDoubleWord (stAHR, stSrc);
	}
	else {
		/*@ AHR -= divisor */
		stTmp = fnSubDoubleWord (stAHR, stSrc);
	}

	/*@ check if carry exist or if result is zero */
	/* if ((stTmp.ulHigh & 1) == 1 || (stTmp.ulLow == 0)) { */
/* In ver1.5r2, just check ulLow == 0	97/7/30 */
	if (stTmp.ulLow == 0) {
		/*@ correct quotient & remainder */
		stAHR.ulLow = stTmp.ulLow;
		ulALR += 1;
	}
	else {
		;
	}
	}
	else {
	;
	}

	/*@ write remainder/quotient */
	gdbsim_store_register (AHR, stAHR.ulLow);
	gdbsim_store_register (ALR, ulALR);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stAHR.ulLow);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, ulALR);
		}
	}
}

/********************************************************
 *	vfnDiv3s : Sign_Div instruction (step 3)	*
 ********************************************************/

void 
vfnDiv3s ()
{
	unsigned long	ulPSR, ulALR;

	/*@ read PSR/quotient */
	ulPSR = gdbsim_fetch_register (PSR);
	ulALR = gdbsim_fetch_register (ALR);

	/*@ check if both divident and divisor have same sign */
	if (((ulPSR & DS_FLAG) == 0 && (ulPSR & N_FLAG) == 0) ||
	((ulPSR & DS_FLAG) != 0 && (ulPSR & N_FLAG) != 0)) {
	;
	}
	else {
	/*@ reverse quotient(plus <-> minus) */
	ulALR = ~ulALR + 1;
	}

	/*@ write quotient */
	gdbsim_store_register (ALR, ulALR);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4 */
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, ulALR);
		}
	}
}

/* 12/13/2001 add function for divu.w instruction */
/********************************************************
 *	vfnDivu : Unsign_Word_Div instruction	  *
 ********************************************************/

#ifndef SPARC

void 
vfnDivu (unsigned short uwRegNum)
#else /* SPARC */

void
vfnDivu (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */
{
	unsigned long  ulALR, ulAHR, ulDiv, ulData, ulPSR;
	
	gdb_AddCycle(35);

	ulPSR = gdbsim_fetch_register (PSR);

	/*@ read ALR */
	ulALR = gdbsim_fetch_register (ALR);

	/*@ read rs register */
	ulDiv = gdbsim_fetch_register (uwRegNum);

	ulData = ulALR;

	/*@ calculate div */
	ulALR = ulData / ulDiv;
	ulAHR = ulData % ulDiv;

	/*@ store */
	(void) gdbsim_store_register (AHR, ulAHR);
	(void) gdbsim_store_register (ALR, ulALR);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, ulAHR);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, ulALR);
		}
	}

}

/* 12/13/2001 add function for divs.w instruction */
/********************************************************
 *	vfnDivs : Sign_Word_Div instruction 	*
 ********************************************************/

#ifndef SPARC

void 
vfnDivs (unsigned short uwRegNum)
#else /* SPARC */

void
vfnDivs (uwRegNum)
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */
{
	long  lALR, lAHR, lDiv, lData, ulPSR;
	

	gdb_AddCycle(35);

	ulPSR = gdbsim_fetch_register (PSR);

	/*@ read ALR */
	lALR = (long) gdbsim_fetch_register (ALR);

	/*@ read rs register */
	lDiv = (long) gdbsim_fetch_register (uwRegNum);

	lData = lALR;

	/*@ calculate div */
	lALR = lData / lDiv;
	lAHR = lData % lDiv;

	/*@ store */
	(void) gdbsim_store_register (ALR, lALR);
	(void) gdbsim_store_register (AHR, lAHR);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, lAHR);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, lALR);
		}

		/* >>>>> add iruma m.takeishi '03.08.29 */
		if ( lALR < 0 )
			ulPSR |= N_FLAG;
		else
			ulPSR &= ~N_FLAG;
		gdbsim_store_register (PSR, ulPSR );
		/* <<<<< add iruma m.takeishi '03.08.29 */
	}

}

/************************************************
 *	fnExeDiv : execute div instruction	*
 ************************************************/

#ifndef SPARC

int 
fnExeDiv (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegNum)
#else /* SPARC */

int
fnExeDiv (uwOp1, uwOp2, uwRegNum)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwRegNum;	/* rs */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */

	/* interlock cycyle count */
	fnInterlock( C33_DIV_MAC, uwRegNum, 0, 2, 0 );
	
	gdb_AddCycle(1);

	switch (uwOp1) {		/* instruction */

	/* >>>>> change irumam.takeishi '03.08.29 */
	/*
	//	divu.w and divs.w instruction //
	case 1: 		// 0001 //
	*/
	/*	divu.w and div.w instruction */
	case 1: 		/* 0001 */
	/* <<<<< change irumam.takeishi '03.08.29 */

	if (gdbsim_fetch_register (uwRegNum) == 0)
	{
		iStatus = S_ZERO_EX;	/* set zero division exception */
	}
	else
	{
		if (uwOp2 == 0) {	 /* 00 divu.w %rs */
			/*@ execute divu.w */
			vfnDivu (uwRegNum);
		}
		/* >>>>> change irumam.takeishi '03.08.29 */
		/*
		else {				 // 01 divs.w %rs //
		*/
		else {				 /* 01 div.w %rs */
		/* <<<<< change irumam.takeishi '03.08.29 */
			/*@ execute divs.w */
			vfnDivs (uwRegNum);
		}
	}
	break;

	case 2: 		/* 010 div0s %rs */

	/*@ execute div0s */
	iStatus = fnDiv0s (uwRegNum);
	break;

	case 3: 		/* 011 div0u %rs */

	/*@ execute div0u */
	iStatus = fnDiv0u (uwRegNum);
	break;

	case 4: 		/* 100 div1 %rs */

	/*@ execute div1 */
	vfnDiv1 (uwRegNum);
	break;

	case 5: 		/* 101 div2s %rs */

	/*@ execute div2s */
	vfnDiv2s (uwRegNum);
	break;

	case 6: 		/* 110 div3s  */

	/*@ execute div3s */
	vfnDiv3s ();
	break;
	}

	/* add T.Tazaki 2005/04/15 >>> */
	if ((iStatus & ( S_ZERO_EX )) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);	/* PC += CODE */
	}
	/* add T.Tazaki 2005/04/15 <<< */
//	  vfnIncPC (CODE);		  // PC += CODE //del T.Tazaki 2005/04/15

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);		/* return status */
}

/************************************************
 *	fnExeScan : execute scan instruction	*
 ************************************************/

#ifndef SPARC

int 
fnExeScan (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeScan (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;	/* rs */
	 unsigned short  uwRegDest; /* rd */

#endif /* SPARC */

{				/* return iStatus ( = 0) */
	unsigned long	ulDataSrc;	/* word data */
	unsigned long	ulBitOffset;
	unsigned long	ulScanMode;
	unsigned long	ulFlag;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read PSR/rs */
	ulFlag = gdbsim_fetch_register (PSR);
	ulDataSrc = gdbsim_fetch_register (uwRegSrc);

	/*@ set Z/N/C/V flag of PSR to zero */
	ulFlag &= ~(Z_FLAG | N_FLAG | C_FLAG | V_FLAG);

/* 12/19/2001 N.Komatsu  for 8bits or 32bits correspondence */
	/*@ get scan mode 8bits or 32bits */
	if ((ulFlag & SW_FLAG) == 0) {
		ulScanMode = 8;
	}
	else {
		ulScanMode = 32;
	}

	/*@ initialize bit offset */
	ulBitOffset = 0;

	/*@ check scan0 */
	if (uwOp1 == 2) {
	/*@ Yes uwOp1 == 010 scan0 */

	/*@ while you don't find 0 and you don't check 8 bits or 32bits */
	while (((ulDataSrc & MSB_BIT) != 0) && (ulBitOffset < ulScanMode)) {
		/*@ shift by 1 bit */
		ulDataSrc <<= 1;

		/*@ count bit offset */
		ulBitOffset++;
	}
	}
	else {
	/*@ No	uwOp1 == 011 scan1 */

	/*@ while you don't find 1 and you don't check 8 bits or 32bits */
	while (((ulDataSrc & MSB_BIT) == 0) && (ulBitOffset < ulScanMode)) {
		/*@ shift by 1 bit */
		ulDataSrc <<= 1;

		/*@ count bit offset */
		ulBitOffset++;
	}
	}

	/*@ check if you don't find */
/* 12/19/2001 N.Komatsu  change for 8bits and 32bits correspondence */
	if (ulBitOffset == ulScanMode) {
	ulFlag |= C_FLAG;	/* not found */
	}
	/*@ check if you find first */
	else if (ulBitOffset == 0) {
	ulFlag |= Z_FLAG;	/* found first */
	}
	else {
	;
	}

	/*@ write rd/PSR */
	gdbsim_store_register (uwRegDest, ulBitOffset);
	gdbsim_store_register (PSR, ulFlag);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0);
}

/****************************************************************
 *	fnExeSwapMirror : execute Swap/Mirror instruction	*
 ****************************************************************/

#ifndef SPARC

int 
fnExeSwapMirror (unsigned short uwOp1, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeSwapMirror (uwOp1, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwRegSrc;	/* rs */
	 unsigned short  uwRegDest; /* rd */

#endif /* SPARC */

{				/* return iStatus (= 0) */
	unsigned long	ulDataSrc;	/* word data */
	unsigned long	ulDataDest; /* word data */
/* 12/11/2001 N.Komatsu  add ulDataDest2 for execute swaph instruction */
	unsigned long	ulDataDest2; /* word data */
	int 			ndxByte, ndxBit;
	unsigned char	bTmpSrc;	/* work area for mirror */
	unsigned long	ulTmpDest;	/* work area for mirror */
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read rs */
	ulDataSrc = gdbsim_fetch_register (uwRegSrc);

	ulDataDest = 0; 	/* initialize rd */
	ulDataDest2 = 0;

	/*@ check instruction */
	if (uwOp1 == 4) {
	/*@ Yes uwOp1 == 100 swap */

	/*@ for 4 bytes */
	for (ndxByte = 0; ndxByte < 4; ndxByte++) {
		/*@ shift rd by 1 byte */
		ulDataDest <<= 8;

		/*@ copy lowest byte of rs to rd */
		ulDataDest |= ulDataSrc & 0xff;

		/*@ shift rs by 1 byte */
		ulDataSrc >>= 8;
	}
	}

/* 12/11/2001 N.Komatsu  execute swaph instruction */
	else if (uwOp1 == 6) {
	/*@ Yes uwOp1 == 110 swaph */

	for (ndxByte = 0; ndxByte < 4; ndxByte++) {
	  if (ndxByte < 2){  /* make lower 2 byte data */
		/*@ shift rd by 1 byte */
		ulDataDest <<= 8;

		/*@ copy lowest byte of rs to rd */
		ulDataDest |= ulDataSrc & 0xff;
	  }
	  else {  /* make higher 2 byte data */
		/*@ shift rd by 1 byte */
		ulDataDest2 <<= 8;

		/*@ copy lowest byte of rs to rd */
		ulDataDest2 |= ulDataSrc & 0xff;
	  }
		/*@ shift rs by 1 byte */
		ulDataSrc >>= 8;
	}
	ulDataDest2 <<= 16;
	ulDataDest |= ulDataDest2;

	}
	else {
	/*@ No	uwOp1 == 101 mirror */

	/*@ for 4 bytes */
	for (ndxByte = 0; ndxByte < 4; ndxByte++) {
		/*@ copy lowest byte of rs to temporary src */
		bTmpSrc = (unsigned char) (ulDataSrc & 0xff);

		/*@ initialize temporary dest */
		ulTmpDest = 0;

		/*@ for 8 bits */
		for (ndxBit = 0; ndxBit < 8; ndxBit++) {
		/*@ shift temporary dest by 1 bit */
		ulTmpDest <<= 1;

		/*@ copy lowest bit of temporary src to temporary dest */
		ulTmpDest |= bTmpSrc & 1;

		/*@ shift temporary src by 1 bit */
		bTmpSrc >>= 1;
		}

		/*@ shift temporary dest by n byte */
		ulTmpDest <<= 8 * ndxByte;

		/*@ copy temporary dest to rd */
		ulDataDest |= ulTmpDest;

		/*@ shift rs by 1 byte */
		ulDataSrc >>= 8;
	}
	}

	/*@ write rd */
	gdbsim_store_register (uwRegDest, ulDataDest);

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return result */
}

/************************************************
 *	fnExeMac1 : execute mac1 instruction  *
 ************************************************/

#ifndef SPARC

int 
fnExeMac1 (unsigned short uwOp1, unsigned short uwRegSrc1, unsigned short uwRegSrc2)
#else /* SPARC */

int
fnExeMac1 (uwOp1, uwRegSrc1, uwRegSrc2)
	 unsigned short  uwOp1; 	 /* op1 */
	 unsigned short  uwRegSrc1;  /* rs1 */
	 unsigned short  uwRegSrc2;  /* rs2 */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;
	unsigned long	ulSrcData1, ulSrcData2; /* %rs, %rd */
	short			wSrcData1 = 0, wSrcData2 = 0;	/* %rs, %rd for calculation */
	long			lSrcData1 = 0, lSrcData2 = 0;	/* %rs, %rd for calculation */
	struct DoubleWord stDest;	/* {AHR, ALR} */
	struct DoubleWord stTmp;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* interlock cycyle count */
	fnInterlock( C33_MLT_MAC1, uwRegSrc1, uwRegSrc2, 2, 0 );

	
	/*@ read addresses, total, and PSR */
	ulSrcData1 = gdbsim_fetch_register (uwRegSrc1); /* data1 */
	ulSrcData2 = gdbsim_fetch_register (uwRegSrc2); /* data2 */
	stDest.ulHigh = gdbsim_fetch_register (AHR);	/* AHR : total(higher part) */
	stDest.ulLow = gdbsim_fetch_register (ALR); /* ALR : total(lower part) */
	ulPSR = gdbsim_fetch_register (PSR);	/* PSR */
	

	switch (uwOp1) {	/* op1 */

	case 1:    /* mac1.h %rd,%rs */

		gdb_AddCycle(1);
		wSrcData1 = (short) (ulSrcData1 & 0xffff);
		wSrcData2 = (short) (ulSrcData2 & 0xffff);
		break;

	case 2:    /* mac1.hw %rd,%rs */
		gdb_AddCycle(2);
		ulSrcData1 &= 0xffff;
		if( ulSrcData1 & 0x8000 ){
			ulSrcData1 |= 0xffff0000;	 /* sign extend */
		}
		lSrcData1 = (long) ulSrcData1;
		lSrcData2 = (long) ulSrcData2;
		break;
	
	case 4:    /* mac1.w %rd,%rs */

		gdb_AddCycle(2);
		lSrcData1 = (long) ulSrcData1;
		lSrcData2 = (long) ulSrcData2;
		break;
	}
	/*@ execute multiplication & addition */

	switch (uwOp1) {	/* op1 */

	case 1:    /* mac1.h %rd,%rs */

		/*@ execute multiplication */
		stTmp.ulLow = (long) wSrcData1 * wSrcData2;
		/*@ check if product is positive */
		if ((stTmp.ulLow & MSB_BIT) == 0) {
			/*@ expand product to 64 bits with sign */
			stTmp.ulHigh = 0;	/* positive */
		}
		else {
			/*@ expand product to 64 bits with sign */
			stTmp.ulHigh = ULONG_MAX;	/* negative */
		}
		break;

	case 2:    /* mac1.hw %rd,%rs */

		/*@ execute multiplication */
		stTmp = fnMltDoubleWord ( lSrcData1, lSrcData2);
		break;

	case 4:    /* mac1.w %rd,%rs */

		/*@ execute multiplication */
		stTmp = fnMltDoubleWord ( lSrcData1, lSrcData2);
		break;
	}


	ulPSR |= fnChangeMO (stDest, stTmp, ulPSR);

	/*@ add product to total */
	stDest = fnAddDoubleWord (stDest, stTmp);

	/*@ write PSR, and total */
	gdbsim_store_register (PSR, ulPSR);
	gdbsim_store_register (AHR, stDest.ulHigh);
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stDest.ulHigh);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	
	/*@ check if calculation is OK */
	if (iStatus == 0) {
		vfnIncPC (CODE);	/* PC += CODE */
	}
	else {
		;
	}
	
	vfnInitExtension ();	/* initialize extension */

   return (iStatus);	   /* return status */
}

/************************************************
 *	fnExeMacW : execute mac instruction  64bit = 64bit + 32bit x 32bit *
 ************************************************/
#ifndef SPARC
int 
fnExeMacW (unsigned short uwOp2,unsigned short uwRegSrc)
#else /* SPARC */

int
fnExeMacW (uwOp2,uwRegSrc)
	 unsigned short  uwOp2; 	/* op2 */
	 unsigned short  uwRegSrc;	/* rs */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;
	unsigned long	ulLoop;
	unsigned long	ulLoopMax;	/* %rs & MAC_COUNT */
	unsigned long	ulRS1, ulRS2;
	unsigned long	ulSrcAddr1, ulSrcAddr2; /* %rs+1, %rs+2 */
	unsigned long	ulSrcData1, ulSrcData2; /* {%rs+1}, {%rs+2} */
	long			lSrcData1, lSrcData2;	/* {%rs+1}, {%rs+2} for calculation */
	short			sSrcData2;	 /* {%rs+1}, {%rs+2} for calculation */
	struct DoubleWord stDest;	/* {AHR, ALR} */
	struct DoubleWord stTmp;

	iStatus = 0;		/* initialize */

	/* interlock cycyle count */
	fnInterlock( C33_DIV_MAC, uwRegSrc, 0, 2, 0 );


	/*@set used register */
	switch(uwRegSrc) {
		default:
			ulRS1 = uwRegSrc+1;
			ulRS2 = ulRS1+1;
			break;
		case 14:
			ulRS1 = 15; 	/* %r15 */
			ulRS2 = 0;		/* %r0 */
			break;
		case 15:
			ulRS1 = 0;		/* %r0 */
			ulRS2 = 1;		/* %r1 */
			break;
	}

	/*@ read loop count, addresses, total, and PSR */
	ulLoopMax = gdbsim_fetch_register (uwRegSrc);	/* loop count */
	ulSrcAddr1 = gdbsim_fetch_register ((unsigned short)ulRS1); /* address1 */
	ulSrcAddr2 = gdbsim_fetch_register ((unsigned short)ulRS2); /* address2 */
	stDest.ulHigh = gdbsim_fetch_register (AHR);	/* AHR : total(higher part) */
	stDest.ulLow = gdbsim_fetch_register (ALR); /* ALR : total(lower part) */
	ulPSR = gdbsim_fetch_register (PSR);	/* PSR */
	
	/*@ check if counter is zero */
	if (ulLoopMax == 0) {
	}
	/*@ check if illegal address exception occured */
	else if ( (((ulSrcAddr1 & 3) != 0) || ((ulSrcAddr2 & 3) != 0)) && uwOp2 == 0 ) {
	iStatus |= S_ADDR_EX;	/* set illegal address exception */
	}
	else if ( (((ulSrcAddr1 & 3) != 0) || ((ulSrcAddr2 & 1) != 0)) && uwOp2 != 0 ) {
	iStatus |= S_ADDR_EX;	/* set illegal address exception */

	}
	else {
	/*@ repeat calculation */

	/*@ for loop count */
	for (ulLoop = 0; ulLoop < ulLoopMax; ulLoop++) {

		gdb_AddCycle(2);

		/*@ read multiplier from memory */
		iStatus |= gdbsim_xfer_inferior_memory (ulSrcAddr1, (char *)&ulSrcData1, WORD_DATA, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		lSrcData1 = ulSrcData1;

		/*@ check nomap_mem_break */
		if ((iStatus & NOMAP_MEM_BREAK) != 0) {
			break;
		}
		else {
			/*@ read multiplicant from memory */
			if(uwOp2 == 0)
			{
			  /* mac.w */
			  iStatus |= gdbsim_xfer_inferior_memory (ulSrcAddr2, (char *)&ulSrcData2, WORD_DATA, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
			  lSrcData2 = ulSrcData2;
			}
			else
			{
			  /* mac.hw */
			  iStatus |= gdbsim_xfer_inferior_memory (ulSrcAddr2, (char *)&sSrcData2, HALF_DATA, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
			  lSrcData2 = sSrcData2;
			}

			/*@ check nomap_mem_break */
			if ((iStatus & NOMAP_MEM_BREAK) != 0) {
				break;
			}
			else {
				/*@ execute multiplication & addition */

				/*@ execute multiplication */
				fnMlt32_32 (&stTmp, lSrcData1, lSrcData2);

				ulPSR |= fnChangeMO (stDest, stTmp, ulPSR);

				/*@ add product to total */
				stDest = fnAddDoubleWord (stDest, stTmp);

				/*@ increment addresses */
				ulSrcAddr1 += WORD_DATA;
				if(uwOp2 == 0)
				{
				  /* mac.w */
				  ulSrcAddr2 += WORD_DATA;
				}
				else
				{
				  /* mac.hw */
				  ulSrcAddr2 += HALF_DATA;
				}
			}
		}
	}

	/*@ write PSR, loop count, addresses, and total */
	gdbsim_store_register (PSR, ulPSR);
	gdbsim_store_register (uwRegSrc, ulLoopMax - ulLoop);
	gdbsim_store_register ((unsigned short)ulRS1, ulSrcAddr1);
	gdbsim_store_register ((unsigned short)ulRS2, ulSrcAddr2);
	gdbsim_store_register (AHR, stDest.ulHigh);
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stDest.ulHigh);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	
	}

	/*@ check if calculation is OK */
	if (iStatus == 0) {
		vfnIncPC (CODE);	/* PC += CODE */
	}
	else {
		;
	}
	vfnInitExtension ();	/* initialize extension */


   return (iStatus);	   /* return status */
}

/************************************************
 *	fnMlt32_32 : Multiplication  32bit x 32bit *
 ************************************************/
#ifndef SPARC
unsigned long 
fnMlt32_32 (struct DoubleWord *stAns, long lSrc1, long lSrc2)
#else /* SPARC */

unsigned long
fnMlt32_32 (stAns, lSrc1, lSrc2)
	 struct DoubleWord *stAns;
	 long Src1;
	 long Src2;

#endif /* SPARC */

{				/* return iStatus */
	int iSign = 1;
	unsigned long ulSrc1L,ulSrc2L,ulSrc1H,ulSrc2H;
  struct DoubleWord stTmp;
  unsigned long ulSrc1,ulSrc2;

	stAns->ulLow = 0;
	stAns->ulHigh = 0;

	ulSrc1 = lSrc1;
	ulSrc2 = lSrc2;

	if(ulSrc1 & MSB_BIT)
	{
		ulSrc1 = lSrc1 - 1;
		ulSrc1 = ~ulSrc1;
		iSign *= -1;
	}
	if(ulSrc2 & MSB_BIT)
	{
		ulSrc2 = lSrc2 - 1;
		ulSrc2 = ~ulSrc2;
		iSign *= -1;
	}

	ulSrc1L = ((unsigned long)ulSrc1) & 0xffff;
	ulSrc2L = ((unsigned long)ulSrc2) & 0xffff;
	ulSrc1H = (((unsigned long)ulSrc1) & 0xffff0000) >> 16;
	ulSrc2H = (((unsigned long)ulSrc2) & 0xffff0000) >> 16;

	stTmp.ulLow  = (ulSrc1L * ulSrc2L);
	stTmp.ulHigh = 0;
	*stAns = fnAddDoubleWord (*stAns, stTmp);

	stTmp.ulLow  = (ulSrc1H * ulSrc2L) << 16;
	stTmp.ulHigh = (ulSrc1H * ulSrc2L) >> 16;
	*stAns = fnAddDoubleWord (*stAns, stTmp);

	stTmp.ulLow  = (ulSrc1L * ulSrc2H) << 16;
	stTmp.ulHigh = (ulSrc1L * ulSrc2H) >> 16;
	*stAns = fnAddDoubleWord (*stAns, stTmp);

	stTmp.ulLow  = 0;
	stTmp.ulHigh = (ulSrc1H * ulSrc2H);
	*stAns = fnAddDoubleWord (*stAns, stTmp);

	if(iSign == -1){
		stAns->ulHigh = ~stAns->ulHigh;
		stAns->ulLow = ~stAns->ulLow + 1;
		if(stAns->ulLow == 0)
		{
			stAns->ulHigh += 1;
		}
	}
	

	return(0);
}

/************************************************
 *	fnExeMac : execute mac instruction	*
 ************************************************/

#ifndef SPARC

int 
fnExeMac (unsigned short uwRegSrc)
#else /* SPARC */

int
fnExeMac (uwRegSrc)
	 unsigned short  uwRegSrc;	/* rs */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;
	unsigned long	ulLoop;
	unsigned long	ulLoopMax;	/* %rs & MAC_COUNT */
	unsigned long	ulRS1, ulRS2;
	unsigned long	ulSrcAddr1, ulSrcAddr2; /* %rs+1, %rs+2 */
	unsigned long	ulSrcData1, ulSrcData2; /* {%rs+1}, {%rs+2} */
	long			lSrcData1;				/* {%rs+1} for calculation */
	short			wSrcData1, wSrcData2;	/* {%rs+1}, {%rs+2} for calculation */
	struct DoubleWord stDest;	/* {AHR, ALR} */
	struct DoubleWord stTmp;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* interlock cycyle count */
	fnInterlock( C33_DIV_MAC, uwRegSrc, 0, 2, 0 );
	gdb_AddCycle(4);

	/*@set used register */
	switch(uwRegSrc) {
		default:
			ulRS1 = uwRegSrc+1;
			ulRS2 = ulRS1+1;
			break;
		case 14:
			ulRS1 = 15; 	/* %r15 */
			ulRS2 = 0;		/* %r0 */
			break;
		case 15:
			ulRS1 = 0;		/* %r0 */
			ulRS2 = 1;		/* %r1 */
			break;
	}

	/*@ read loop count, addresses, total, and PSR */
	ulLoopMax = gdbsim_fetch_register (uwRegSrc);	/* loop count */
	ulSrcAddr1 = gdbsim_fetch_register ((unsigned short)ulRS1); /* address1 */
	ulSrcAddr2 = gdbsim_fetch_register ((unsigned short)ulRS2); /* address2 */
	stDest.ulHigh = gdbsim_fetch_register (AHR);	/* AHR : total(higher part) */
	stDest.ulLow = gdbsim_fetch_register (ALR); /* ALR : total(lower part) */
	ulPSR = gdbsim_fetch_register (PSR);	/* PSR */


	
	/*@ check if counter is zero */
	if (ulLoopMax == 0) {
	vfnIncPC (CODE);	/* PC += CODE */
	}
	/*@ check if illegal address exception occured */
	else if (((ulSrcAddr1 & 1) != 0) || ((ulSrcAddr2 & 1) != 0)) {
	iStatus |= S_ADDR_EX;	/* set illegal address exception */
	}
	else {
	/*@ repeat calculation */

	/*@ for loop count */
	for (ulLoop = 0; ulLoop < ulLoopMax; ulLoop++) {
		/*@ read multiplier from memory */

		gdb_AddCycle(1);

		iStatus |= gdbsim_xfer_inferior_memory (ulSrcAddr1, (char *)&ulSrcData1, HALF_DATA, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		wSrcData1 = (short) (ulSrcData1 & 0xffff);


		/*@ check nomap_mem_break */
		if ((iStatus & NOMAP_MEM_BREAK) != 0) {
		break;
		}
		else {

		gdb_AddCycle(1);

		/*@ read multiplicant from memory */
		iStatus |= gdbsim_xfer_inferior_memory (ulSrcAddr2, (char *)&ulSrcData2, 2, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */;
		wSrcData2 = (short) (ulSrcData2 & 0xffff);

		/*@ check nomap_mem_break */
		if ((iStatus & NOMAP_MEM_BREAK) != 0) {
			break;
		}
		else {
			/*@ execute multiplication & addition */
			stTmp.ulLow = (long) wSrcData1 *wSrcData2;

			/*@ check if product is positive */
			if ((stTmp.ulLow & MSB_BIT) == 0) {
			/*@ expand product to 64 bits with sign */
			stTmp.ulHigh = 0;	/* positive */
			}
			else {
			/*@ expand product to 64 bits with sign */
			stTmp.ulHigh = ULONG_MAX;	/* negative */
			}

			ulPSR |= fnChangeMO (stDest, stTmp, ulPSR);

			/*@ add product to total */
			stDest = fnAddDoubleWord (stDest, stTmp);

			/*@ increment addresses */
			ulSrcAddr1 += HALF_DATA;
			ulSrcAddr2 += HALF_DATA;
		}
		}
	}

	/*@ write PSR, loop count, addresses, and total */
	gdbsim_store_register (PSR, ulPSR);
	gdbsim_store_register (uwRegSrc, ulLoopMax - ulLoop);
	gdbsim_store_register ((unsigned short)ulRS1, ulSrcAddr1);
	gdbsim_store_register ((unsigned short)ulRS2, ulSrcAddr2);
	gdbsim_store_register (AHR, stDest.ulHigh);
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stDest.ulHigh);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	
	/*@ check if calculation is OK */
	if ((iStatus & NOMAP_MEM_BREAK) == 0) {
		vfnIncPC (CODE);	/* PC += CODE */
	}
	else {
		;
	}
	}

	vfnInitExtension ();	/* initialize extension */

   return (iStatus);	   /* return status */
}

/********************************************************
 *	fnExeMacClr : execute AHR, ALR clear instruction	*
 ********************************************************/

#ifndef SPARC

int 
fnExeMacClr (void)
#else /* SPARC */

int
fnExeMacClr (void)

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulPSR;

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(1);


#if 0  /* del T.Tazaki 2003/12/02 >>> */
	/* macclr mode ON */
	iMacclrFlag = 1;
	/* inst count initialize */
	ulMacclrCurrentInst = 0;
#endif

	/*@ clear AHR, ALR */
/* delete 2002/10/09 T.Tazaki >>>> */
/* add 2003/12/02 T.Tazaki >>> */

	(void) gdbsim_store_register (AHR, 0);
	(void) gdbsim_store_register (ALR, 0);

	vfnIncPC (CODE);

	ulPSR = gdbsim_fetch_register(PSR);
	ulPSR &= ~MO_FLAG;
	gdbsim_store_register ( PSR, ulPSR );
	
/* add 2003/12/02 T.Tazaki <<< */
/* delete 2002/10/09 T.Tazaki >>>> */

//	  vfnIncPC (CODE);		/* del T.Tazaki 2003/12/02 */

	return (0);
}

/********************************************************
 *	fnExeSat : execute saturation instruction	 *
 ********************************************************/

#ifndef SPARC

int 
fnExeSat (unsigned short uwClass, unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeSat (uwClass, uwOp1, uwOp2, uwRegSrc, uwRegDest)
	 unsigned short  uwClass;
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwRegSrc;
	 unsigned short  uwRegDest;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData; /* word data to saturate */
	long			lData;	/* word data to saturate */
	unsigned long	ulFlag;
	unsigned long	ulChkFlag;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegSrc, 0, 0, 0 );

	gdb_AddCycle(1);

	/*@ read PSR */
	ulFlag = gdbsim_fetch_register (PSR);

	switch (uwClass) {

	case 4:    /* class 4 */

		switch (uwOp2) {

		case 2:    /* 111:10 sat.b %rd,%rs */

			/*@ get word data from rs register */
			lData = (long) gdbsim_fetch_register (uwRegSrc);
			if (lData < -128) {
				lData = 0xffffff80;
				/*@ set S flag */
				ulFlag |= S_FLAG;
			}
			else if (lData > 127) {
				lData = 0x0000007f;
				/*@ set S flag */
				ulFlag |= S_FLAG;
			}
			/*@ store flag */
			(void) gdbsim_store_register (PSR, ulFlag);

			/*@ store saturated data to rd register */
			(void) gdbsim_store_register (uwRegDest, (unsigned long) lData);

			break;

		case 3:    /* 111:11 sat.ub %rd,%rs */

			/*@ get unsigned word data from rs register */
			ulData = gdbsim_fetch_register (uwRegSrc);
			if (ulData > 255) {
				ulData = 0x000000ff;
				/*@ set S flag */
				ulFlag |= S_FLAG;
			}
			/*@ store flag */
			(void) gdbsim_store_register (PSR, ulFlag);

			/*@ store saturated data to rd register */
			(void) gdbsim_store_register (uwRegDest, ulData);

			break;
		}
		break;

	case 5:    /* class 5 */

		switch (uwOp1) {

		case 5:    /* sat.h or sat.uh */

			switch (uwOp2) {

			case 2:    /* 101:10 sat.h %rd,%rs */

				/*@ get word data from rs register */
				lData = (long) gdbsim_fetch_register (uwRegSrc);
				if (lData < -32768) {
					lData = 0xffff8000;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				else if (lData > 32767) {
					lData = 0x00007fff;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				/*@ store flag */
				(void) gdbsim_store_register (PSR, ulFlag);

				/*@ store saturated data to rd register */
				(void) gdbsim_store_register (uwRegDest, (unsigned long) lData);

				break;

			case 3:    /* 101:11 sat.uh %rd,%rs */

				/*@ get unsigned word data from rs register */
				ulData = gdbsim_fetch_register (uwRegSrc);
				if (ulData > 65535) {
					ulData = 0x0000ffff;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				/*@ store flag */
				(void) gdbsim_store_register (PSR, ulFlag);

				/*@ store saturated data to rd register */
				(void) gdbsim_store_register (uwRegDest, ulData);

				break;
			}
			break;

		case 7:    /* sat.w or sat.uw */

			switch (uwOp2) {

			case 1:    /* 111:01 sat.w %rd,%rs */

				/*@ get word data from rs register */
				lData = (long) gdbsim_fetch_register (uwRegSrc);

				/*@ check N,V flag */
				ulChkFlag = ulFlag & (N_FLAG | V_FLAG);

				if (ulChkFlag == V_FLAG) {	/* N==0 && V==1 */
					lData = 0x80000000;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				else if (ulChkFlag == (N_FLAG | V_FLAG)) {	/* N==1 && V==1 */
					lData = 0x7fffffff;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				/*@ store flag */
				(void) gdbsim_store_register (PSR, ulFlag);

				/*@ store saturated data to rd register */
				(void) gdbsim_store_register (uwRegDest, (unsigned long) lData);

				break;

			case 2:    /* 111:10 sat.uw %rd,%rs */

				/*@ get unsigned word data from rs register */
				ulData = gdbsim_fetch_register (uwRegSrc);

				/*@ check C flag */
				ulChkFlag = ulFlag & C_FLAG;

				if (ulChkFlag != 0) {	 /* C==1 */
					ulData = 0xffffffff;
					/*@ set S flag */
					ulFlag |= S_FLAG;
				}
				/*@ store flag */
				(void) gdbsim_store_register (PSR, ulFlag);

				/*@ store saturated data to rd register */
				(void) gdbsim_store_register (uwRegDest, ulData);

				break;
			}
			break;
		}
		break;
	}
	vfnIncPC (CODE);

	return (iStatus);
}


/************************************************************************
 *	fnMltUnsignedDoubleWord : execute unsigned 32bit * 32bit	*
 ************************************************************************/

#ifndef SPARC

struct DoubleWord 
fnMltUnsignedDoubleWord (unsigned long ulDataDest, unsigned long ulDataSrc)
#else /* SPARC */

struct DoubleWord
fnMltUnsignedDoubleWord (ulDataDest, ulDataSrc)
	 unsigned long	 ulDataDest;
	 unsigned long	 ulDataSrc;

#endif /* SPARC */

{				/* return product */
	struct DoubleWord stDest;	/* product */
	struct DoubleWord stShift;	/* for work */
	unsigned long	ulShift;	/* for work */

	/*@ initialize product */
	stDest.ulHigh = 0;
	stDest.ulLow = 0;

	/*@ for each bit of multiplier */
	for (ulShift = 0; ulShift < 32; ++ulShift) {
	/*@ check if bit of multiplier is 1 */
#ifdef SPARC
	if (ulDataDest & (1 << ulShift)) {
#else /* SPARC */
	if (ulDataDest & (1UL << ulShift)) {
#endif /* SPARC */

		/*@ check if bit0 of multiplier is 1 */
		if (ulShift == 0) {
		/*@ shift 0 bit */
		stShift.ulHigh = 0;
		stShift.ulLow = ulDataSrc;
		}
		else {
		/*@ shift */
		stShift.ulHigh = ulDataSrc >> (32 - ulShift);
		stShift.ulLow = ulDataSrc << ulShift;
		}
	}
	else {
		/*@ shifted value is 0 */
		stShift.ulHigh = 0;
		stShift.ulLow = 0;
	}

	/*@ add shifted value */
	stDest = fnAddDoubleWord (stDest, stShift);
	}

	return (stDest);		/* return product */
}

/********************************************************
 *	fnMltDoubleWord : execute signed 32bit*32bit	*
 ********************************************************/

#ifndef SPARC

struct DoubleWord 
fnMltDoubleWord (long lDataDest, long lDataSrc)
#else /* SPARC */

struct DoubleWord
fnMltDoubleWord (lDataDest, lDataSrc)
	 long			 lDataDest;
	 long			 lDataSrc;

#endif /* SPARC */

{				/* return product */
	unsigned long	ulDataDest; /* for absolute value */
	unsigned long	ulDataSrc;	/* for absolute value */
	int 			iSignFlag;
	struct DoubleWord stDest;	/* product */
	struct DoubleWord stSrc;	/* for calculate 2's complement */

	/*@ get absolute value & sign */

	/*@ check if multiplier is negative */
	if ((lDataSrc & MSB_BIT) != 0) {
	/*@ get absolute value */
	ulDataSrc = ~lDataSrc + 1;

	iSignFlag = -1; 	/* set sign flag negative */
	}
	else {
	iSignFlag = 1;		/* set sign flag positive */
	/*@ get absolute value */
	ulDataSrc = lDataSrc;
	}

	/*@ check if multiplicand is negative */
	if ((lDataDest & MSB_BIT) != 0) {
	/*@ get absolute value */
	ulDataDest = ~lDataDest + 1;

	/*@ calculate sign of product */
	iSignFlag *= -1;
	}
	else {
	/*@ get absolute value */
	ulDataDest = lDataDest;
	}

	/*@ multiply absolute values */
	stDest = fnMltUnsignedDoubleWord (ulDataDest, ulDataSrc);

	/*@ check if sign of product is negative */
	if (iSignFlag != 1) {
	/*@ calculate 2's complement */

	stDest.ulHigh = ~stDest.ulHigh;
	stDest.ulLow = ~stDest.ulLow;

	stSrc.ulHigh = 0;
	stSrc.ulLow = 1;
	stDest = fnAddDoubleWord (stDest, stSrc);
	}
	else {
	;
	}

	return (stDest);		/* return product */
}

/************************************************
 *	fnExeMlt : execute mlt instruction	*
 ************************************************/

#ifndef SPARC

int 
fnExeMlt (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwRegSrc, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeMlt (uwOp1, uwOp2, uwRegSrc, uwRegDest)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwRegSrc;	/* rs */
	 unsigned short  uwRegDest; /* rd */

#endif /* SPARC */

{				/* return iStatus (= 0) */
	unsigned long	ulDataSrc, ulDataDest;	/* word data */
	short			wDataSrc, wDataDest;	/* half data for mlt.h */
	unsigned short	uwDataSrc, uwDataDest;	/* unsigned half data for mltu.h */
	struct DoubleWord stDest;	/* double word destination */
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* interlock cycyle count */
	fnInterlock( C33_MLT_MAC1, uwRegSrc, uwRegDest, 2, 0 );

	/*@ read rs/rd */
	ulDataSrc = gdbsim_fetch_register (uwRegSrc);
	ulDataDest = gdbsim_fetch_register (uwRegDest);
	ulPSR = gdbsim_fetch_register (PSR);	/* PSR */

	switch (uwOp1) {		/* instruction */

	case 0: 		/* 000 mlt.h %rd,%rs , mlt.hw %rd,%rs */

		switch (uwOp2) {	/* instruction */

		case 2: 	/* 10 mlt.h %rd,%rs */

			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {
				gdb_AddCycle(2);
			}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE )	{
				gdb_AddCycle(5);
			}else{
				gdb_AddCycle(1);
			}

			/*@ calculate */
			wDataDest = (short) ulDataDest;
			wDataSrc = (short) ulDataSrc;
			stDest.ulLow = (unsigned long) ((long) wDataDest * (long) wDataSrc);

			/*@ write product */
			gdbsim_store_register (ALR, stDest.ulLow);

			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
			{
				if( ( ulPSR & LC_FLAG ) != 0 ){
					gdbsim_store_register (R4, stDest.ulLow);
				}
			}
			break;

		case 3: 	/* 11 mlt.hw %rd,%rs */

			if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
			{
				return ILL_INST_BREAK;
			}
			
			/*@ calculate */
			ulDataSrc &= 0xffff;
			if( ulDataSrc & 0x8000 ){
				ulDataSrc |= 0xffff0000;	/* sign extend */
			}

			/*@ calculate */
			stDest = fnMltDoubleWord ((long) ulDataDest, (long) ulDataSrc);

			/*@ write product */
			gdbsim_store_register (AHR, stDest.ulHigh);
			gdbsim_store_register (ALR, stDest.ulLow);

			if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
			{
				/* result --> %r4, %r5 */
				if( ( ulPSR & HC_FLAG ) != 0 ){
					gdbsim_store_register (R5, stDest.ulHigh);
				}
				if( ( ulPSR & LC_FLAG ) != 0 ){
					gdbsim_store_register (R4, stDest.ulLow);
				}
			}
			break;
		}
	break;

	case 1: 		/* 001 mltu.h %rd,%rs */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {
		gdb_AddCycle(2);
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE ) {
		gdb_AddCycle(5);
	}else{
		gdb_AddCycle(1);
	}


	/*@ calculate */
	uwDataDest = (unsigned short) ulDataDest;
	uwDataSrc = (unsigned short) ulDataSrc;
	stDest.ulLow = (unsigned long) uwDataDest *(unsigned long) uwDataSrc;

	/*@ write product */
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	break;

	case 2: 		/* 010 mlt.w %rd,%rs */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {
		gdb_AddCycle(5);
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE ) {
		gdb_AddCycle(7);
	}else{
		gdb_AddCycle(1);
	}


	/*@ calculate */
	stDest = fnMltDoubleWord ((long) ulDataDest, (long) ulDataSrc);

	/*@ write product */
	gdbsim_store_register (AHR, stDest.ulHigh);
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stDest.ulHigh);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	break;

	case 3: 		/* 011 mltu.w %rd,%rs */

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV ) {
		gdb_AddCycle(5);
	}else if( fnCheckAdvancedMode() == C33_PRODUCT_PE ) {
		gdb_AddCycle(7);
	}else{
		gdb_AddCycle(1);
	}


	/*@ calculate */
	stDest = fnMltUnsignedDoubleWord (ulDataDest, ulDataSrc);

	/*@ write product */
	gdbsim_store_register (AHR, stDest.ulHigh);
	gdbsim_store_register (ALR, stDest.ulLow);

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		/* result --> %r4, %r5 */
		if( ( ulPSR & HC_FLAG ) != 0 ){
			gdbsim_store_register (R5, stDest.ulHigh);
		}
		if( ( ulPSR & LC_FLAG ) != 0 ){
			gdbsim_store_register (R4, stDest.ulLow);
		}
	}
	break;
	}

	vfnIncPC (CODE);		/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (0); 		/* return OK */
}

/************************************************
 *	fnExeExt : execute ext instruction	*
 ************************************************/

#ifndef SPARC

int 
fnExeExt (unsigned short uwImm13)
#else /* SPARC */

int
fnExeExt (uwImm13)
	 unsigned short  uwImm13;

#endif /* SPARC */

{				/* return iStatus (= 0) */
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
				 CMB_EXT_COND_OK | CMB_EXT_RS_OP_IMM_OK |  CMB_EXT_OP_IMM_OK | CMB_EXT_IMM_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );
	if( fnCheckAdvancedMode() == C33_PRODUCT_STD )	/* ADV => 0 cycle : add T.Tazaki 2003/08/04 */
	{
		gdb_AddCycle(1);
	}
	switch (stExtension.iType) {	/* previous ext inst */

	case EXT_HL:		/* ext:ext */

	/*@ lower 13 bits of extension is renewal */
	stExtension.ulData &= ~0x1fff;
	stExtension.ulData |= uwImm13;

	break;

	case EXT_L: 	/* ext */
	/*@ change extension type */
	stExtension.iType = EXT_HL;

	/*@ save extension */
	stExtension.ulData <<= 13;
	stExtension.ulData |= uwImm13;

	break;

	case EXT_NONE:		/* no ext */

	/*@ change extension type */
	stExtension.iType = EXT_L;

	/*@ save extension */
	stExtension.ulData = uwImm13;

	break;
	}

	
	vfnIncPC (CODE);		/* PC += CODE */

	return (0); 		/* return OK */
}

/***********************************************************
 *	fnExeExtReg : execute ext instruction by the register  *
 ***********************************************************/

#ifndef SPARC

int 
fnExeExtReg (unsigned short uwOp1, unsigned short uwOp2, unsigned short uwOp3)
#else /* SPARC */

int
fnExeExtReg (uwRegSrc)
	 unsigned short  uwOp1;
	 unsigned short  uwOp2;
	 unsigned short  uwOp3;

#endif /* SPARC */

{				/* return iStatus (= 0) */
	
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	
	unsigned long ulPC;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int iFlag, iExt = CMB_EXT_NG, iRet;
	unsigned long ulPSR;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>>> add iruma m.takeishi '03.09.02 */
	if(uwOp1 == 7 && uwOp3 == 0)
	{
		/* ext %rs */
		iExt = CMB_EXT_COND_OK;
	} else if (uwOp1 == 7 && uwOp3 != 0)
	{
		/* ext %rs,op,imm2 */
		iExt = CMB_EXT_COND_OK;
	} else if (uwOp1 == 6 && uwOp3 == 0)
	{
		/* ext cond */
		iExt = CMB_EXT_NG;
	} else if (uwOp1 == 6 && uwOp3 != 0)
	{
		/* ext op,imm2 */
		iExt = CMB_EXT_COND_OK;
	}
	iRet = fnCheckInstCombi( CMB_LOOP_OK_A1, CMB_REPEAT_NG, iExt );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwOp2, 0, 0, 0 );


//	  gdb_AddCycle(1);	/* del 2003/08/04 T.Tazaki */
	
	if(uwOp1 == 7 && uwOp3 == 0)
	{
		/* ext %rs */
		/*@ read rs register */
		/*@ mask lower 26bits data because 26 bits or more data is not ignored */
		stExtension.ulData = gdbsim_fetch_register (uwOp2);
		/*@ change extension type */
		stExtension.iType = EXT_RS;
	}
	else if (uwOp1 == 7 && uwOp3 != 0)
	{
		/* ext %rs,op,imm2 */
		/*@ read rs register */
		stExtension.ulDataExtRs = gdbsim_fetch_register (uwOp2);
		/*@ change operand */
		stExtension.uwDataExtOp = uwOp3;
		/*@ change extension type */
		stExtension.iTypeExt = EXT_RS_OP_IMM;
	}
	else if (uwOp1 == 6 && uwOp3 == 0)
	{
		/* ext cond */
		
		/* >>>>> change iruma m.takeishi '03.09.01 */
		/*
		//@ change operand //
		stExtension.ulDataExtRs = uwOp2;
		//@ change extension type //
		stExtension.iTypeExt = EXT_COND;
		*/
		
		/* set ext cond mode */
		iFlag = 1;
		ulPSR = gdbsim_fetch_register (PSR);
		switch(uwOp2)
		{
			case 4: /* gt */
				if (!(Z (ulPSR)) && !(NV (ulPSR)))
				{
					/* !Z && !(N^V) */
					iFlag = 0;
				}
				break;
			case 5: /* ge */
				if (!(NV (ulPSR)))
				{
					/* !(N^V) */
					iFlag = 0;
				}
				break;
			case 6: /* lt */
				if (NV (ulPSR))
				{
					/* N^V */
					iFlag = 0;
				}
				break;
			case 7: /* le */
				if (Z (ulPSR) || NV (ulPSR))
				{
					/* Z | (N^V) */
					iFlag = 0;
				}
				break;
			case 8: /* ugt */
				if (!Z (ulPSR) && !C (ulPSR))
				{
					/* !Z && !C */
					iFlag = 0;
				}
				break;
			case 9: /* uge */
				if (!C (ulPSR))
				{
					/* !C */
					iFlag = 0;
				}
				break;
			case 10:	/* ult */
				if (C (ulPSR))
				{
					/* C */
					iFlag = 0;
				}
				break;
			case 11:	/* ule */
				if (Z (ulPSR) || C (ulPSR))
				{
					/* Z | C */
					iFlag = 0;
				}
				break;
			case 12:	/* eq */
				if (Z (ulPSR))
				{
					/* Z */
					iFlag = 0;
				}
				break;
			case 13:	/* ne */
				if (!Z (ulPSR))
				{
					/* !Z */
					iFlag = 0;
				}
				break;
		}

		if ( iFlag == 0 ) {
			/* jump : save cycle */
			stExtension.iTypeExtCond = EXTCOND_GO;
			stExtension.ulPushCycle  = gulCycle;
			
		} else {
			stExtension.iTypeExtCond = EXTCOND_EXEC;
			stExtension.ulPushCycle = 0;
		}
		/* <<<<< add iruma m.takeishi '03.09.01 */
	}
	else if (uwOp1 == 6 && uwOp3 != 0)
	{
		/* ext op,imm2 */
		/*@ change operand */
		stExtension.uwDataExtOp = uwOp3;
		/*@ change extension type */
		stExtension.iTypeExt = EXT_OP_IMM;
	}


	vfnIncPC (CODE);		/* PC += CODE */

	return (0); 		/* return OK */
}
/***********************************************************
 *	fnExeCond : PSR condition check *
 ***********************************************************/

#ifndef SPARC

int 
fnExeCond (unsigned short uwCode)
#else /* SPARC */

int
fnExeCond (uwCode )
	unsigned short uwCode;
	
#endif /* SPARC */

{
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	int iFlag = 1;
	unsigned long ulPSR;
	
	if(stExtension.iTypeExt == EXT_COND)
	{
		/* Instruction Check */
		
		/* jrxx, jrxx.d,call,call.d,jp,jp.d ? */
		if( (( uwCode & 0xff00 ) >= 0x0800) && (( uwCode & 0xff00 ) <= 0x1f00 ) )
		{
			iFlag = 0;
		}
		/* jpr %rb ? */
		else if( ( uwCode & 0xfff0 ) == 0x02c0 )
		{
			iFlag = 0;
		}
		/* jpr.d %rb ? */
		else if( ( uwCode & 0xfff0 ) == 0x03c0 )
		{
			iFlag = 0;
		}
		/* brk	? */
		else if( uwCode == 0x0400 )
		{
			iFlag = 0;
		}
		/* retd ? */
		else if( uwCode == 0x0440 )
		{
			iFlag = 0;
		}
		/* int imm2  ? */
		else if( ( uwCode & 0xfffc ) == 0x0480 )
		{
			iFlag = 0;
		}
		/* reti ? */
		else if( uwCode == 0x04c0 )
		{
			iFlag = 0;
		}
		/* call %rb ? */
		else if( ( uwCode & 0xfff0 ) == 0x0600 )
		{
			iFlag = 0;
		}
		/* ret ? */
		else if( uwCode == 0x0640 )
		{
			iFlag = 0;
		}
		/* jp %rd ? */
		else if( ( uwCode & 0xfff0 ) == 0x0680 )
		{
			iFlag = 0;
		}
		/* call.d %rb */
		else if( ( uwCode & 0xfff0 ) == 0x0700 )
		{
			iFlag = 0;
		}
		/* ret.d ? */
		else if( uwCode == 0x0740 )
		{
			iFlag = 0;
		}
		/* jp.d %rb ? */
		else if( ( uwCode & 0xfff0 ) == 0x0780 )
		{
			iFlag = 0;
		}
		else
		{
			/* %psr Condition Check */
			/* >>>>> change iruma m.takeishi '03.09.01 */
			ulPSR = gdbsim_fetch_register (PSR);
			switch(stExtension.ulDataExtRs)
			{
				case 4: /* gt */
					if (!(Z (ulPSR)) && !(NV (ulPSR)))
					{
						/* !Z && !(N^V) */
						iFlag = 0;
					}
					break;
				case 5: /* ge */
					if (!(NV (ulPSR)))
					{
						/* !(N^V) */
						iFlag = 0;
					}
					break;
				case 6: /* lt */
					if (NV (ulPSR))
					{
						/* N^V */
						iFlag = 0;
					}
					break;
				case 7: /* le */
					if (Z (ulPSR) || NV (ulPSR))
					{
						/* Z | (N^V) */
						iFlag = 0;
					}
					break;
				case 8: /* ugt */
					if (!Z (ulPSR) && !C (ulPSR))
					{
						/* !Z && !C */
						iFlag = 0;
					}
					break;
				case 9: /* uge */
					if (!C (ulPSR))
					{
						/* !C */
						iFlag = 0;
					}
					break;
				case 10:	/* ult */
					if (C (ulPSR))
					{
						/* C */
						iFlag = 0;
					}
					break;
				case 11:	/* ule */
					if (Z (ulPSR) || C (ulPSR))
					{
						/* Z | C */
						iFlag = 0;
					}
					break;
				case 12:	/* eq */
					if (Z (ulPSR))
					{
						/* Z */
						iFlag = 0;
					}
					break;
				case 13:	/* ne */
					if (!Z (ulPSR))
					{
						/* !Z */
						iFlag = 0;
					}
					break;
			}
			/* <<<<< change iruma m.takeishi '03.09.01 */
		}
		stExtension.iTypeExt = EXT_NONE;
	}
	return (iFlag);
}

/***************************************************************************
 *	fnExeBit : execute bit operation instruction
 ***************************************************************************/
#ifndef SPARC
int 
fnExeBit (unsigned short uwOp1, unsigned short uwRegBase, unsigned short uwImm)
#else /* SPARC */
int
fnExeBit (uwOp1, uwRegBase, uwImm)
	 unsigned short  uwOp1;
	 unsigned short  uwRegBase;
	 unsigned short  uwImm;

#endif /* SPARC */
{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulData;
	unsigned long	ulImm_Bit;
	unsigned long	ulAddr;
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK | CMB_EXT_IMM_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */

	fnInterlock( C33_BIT, 0, uwRegBase, 0, 0 );

	ulAddr = gdbsim_fetch_register (uwRegBase);

	iStatus = gdbsim_xfer_inferior_memory (ulAddr + stExtension.ulData, (char *)&ulData, 1, 0, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */

	if ((iStatus & NOMAP_MEM_BREAK) != 0) {
	;
	}
	else {
	ulImm_Bit = 1 << uwImm;

	switch (uwOp1) {	/* instruction */

	case 2: 	/* 010 btst  [%rb],imm3 */

		if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
		{
			gdb_AddCycle(2);
		}else{
			gdb_AddCycle(3);
		}
		
		ulPSR = gdbsim_fetch_register (PSR);

		if ((ulData & ulImm_Bit) == 0) {
		ulPSR |= Z_FLAG;
		}
		else {
		ulPSR &= ~Z_FLAG;
		}

		gdbsim_store_register (PSR, ulPSR);

		break;

	case 3: 	/* 011 bclr  [%rb],imm3 */

		gdb_AddCycle(3);

		ulData &= ~ulImm_Bit;
		iStatus |= gdbsim_xfer_inferior_memory (ulAddr + stExtension.ulData, (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		break;

	case 4: 	/* 100 bset  [%rb],imm3 */

		gdb_AddCycle(3);

		ulData |= ulImm_Bit;
		iStatus |= gdbsim_xfer_inferior_memory (ulAddr + stExtension.ulData, (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		break;

	case 5: 	/* 101 bnot  [%rb],imm3 */

		gdb_AddCycle(3);

		ulData ^= ulImm_Bit;
		iStatus |= gdbsim_xfer_inferior_memory (ulAddr + stExtension.ulData, (char *)&ulData, 1, 1, NULL, NULL);/* T.Tazaki 2002.06.05 C33 */
		break;
	}

	if ((iStatus & ROM_DATA_BREAK) != 0) {
		;
	}
	else {
		vfnIncPC (CODE);
	}
	}

	
	vfnInitExtension ();

	return (iStatus);		/* return status */
}

/**************************************************************
 *	fnExePsrSetClr : execute psr set and clear instruction	  *
 **************************************************************/

#ifndef SPARC

int 
fnExePsrSetClr (unsigned short uwOp3, unsigned short uwImm5)
#else /* SPARC */

int
fnExePsrSetClr (uwOp3, uwImm5)
	 unsigned short  uwOp3;
	 unsigned short  uwImm5;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulFlag;
	unsigned long	ulImm_Bit;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */


	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		gdb_AddCycle(3);
	}else{
		gdb_AddCycle(4);
	}


	/* bug fix T.Tazaki 2005/01/11 >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		if( uwImm5 > 4 )
		{
			vfnIncPC (CODE);		/* nop */
			return (iStatus);
		}
	}
	/* bug fix T.Tazaki 2005/01/11 <<< */
	
	/*@ read PSR */
	ulFlag = gdbsim_fetch_register (PSR);

	ulImm_Bit = 1 << (uwImm5 & 0x1f);

	/* >>>>> add iruma m.takeishi '04.03.15 */
	if ( ulImm_Bit == LM_FLAG && (ulFlag & LM_FLAG) == 0) {
		iRsvLoop_instCnt = RESERVELOOP_INSTCNT;
	}
	/* <<<<< add iruma m.takeishi '04.03.15 */

	/* User mode : HE/SV/IL/IE = Read ONLY Bit */
	if( ulFlag & SV_FLAG )	/* user mode ? */
	{
		/* User mode */

		if( ulImm_Bit != HE_FLAG   && ulImm_Bit != DE_FLAG	 && ulImm_Bit != ME_FLAG  && ulImm_Bit != SV_FLAG &&
			ulImm_Bit != C33_BIT11 && ulImm_Bit != C33_BIT10 && ulImm_Bit != C33_BIT9 && ulImm_Bit != C33_BIT8 &&	
			ulImm_Bit != IE_FLAG )
		{
			if (uwOp3 == 1) {	 /* 01 psrset imm5 */
				ulFlag |= ulImm_Bit;
			}
			else {				 /* 10 psrclr imm5 */
				ulFlag &= ~ulImm_Bit;
			}
		}
	}
	else
	{
		/* Supervisor mode */
		if( ulImm_Bit != DE_FLAG && ulImm_Bit != ME_FLAG )
		{
			if (uwOp3 == 1) {	 /* 01 psrset imm5 */
				ulFlag |= ulImm_Bit;
			}
			else {				 /* 10 psrclr imm5 */
				ulFlag &= ~ulImm_Bit;
			}
		}
	}

	(void) gdbsim_store_register (PSR, ulFlag);

	vfnIncPC (CODE);

	
	return (iStatus);
}

/**************************************************************
 *	fnExeLoop : execute loop instruction (loop %rc,%ra)   *
 **************************************************************/

#ifndef SPARC

int 
fnExeLoop (unsigned short uwRegAddr, unsigned short uwRegCount)
#else /* SPARC */

int
fnExeLoop (uwRegAddr, uwRegCount)
	 unsigned short  uwRegAddr;
	 unsigned short  uwRegCount;

#endif /* SPARC */

{				/* return iStatus */
	unsigned long  ulFlag;
	unsigned long  ulLSA, ulLEA, ulLCO;
	unsigned short uwImmAddr;

	/* >>>>> add iruma m.takeishi '04.03.29 */
	int 		   iRet;
	/* <<<<< add iruma m.takeishi '04.03.29 */

	/* >>>>> add iruma m.takeishi '04.03.29 */
	iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '04.03.29 */

	fnInterlock( C33_MLT_MAC1, uwRegAddr, uwRegCount, 0, 0 );

	gdb_AddCycle(5);

	/*@ read loop start address, loop end address, loop count and PSR */
	ulLSA = gdbsim_fetch_register (PC) + (unsigned long) CODE;
	ulLEA = gdbsim_fetch_register (uwRegAddr);
	ulLCO = gdbsim_fetch_register (uwRegCount);
	ulFlag = gdbsim_fetch_register (PSR);

	/*@ set LM flag */
	ulFlag |= LM_FLAG;
	(void) gdbsim_store_register (PSR, ulFlag);

	/*@ write loop start address, loop end address and loop count */
	(void) gdbsim_store_register (LSA, ulLSA);
	(void) gdbsim_store_register (LEA, ulLEA);
	(void) gdbsim_store_register (LCO, ulLCO);

	vfnIncPC (CODE);

	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	return (0);
}

/**************************************************************
 *	fnExeLoopRegImm : execute loop instruction (loop %rc,imm4)*
 **************************************************************/

#ifndef SPARC

int 
fnExeLoopRegImm (unsigned short uwImmAddr, unsigned short uwRegCount)
#else /* SPARC */

int
fnExeLoopRegImm (uwImmAddr, uwRegCount)
	 unsigned short  uwImmAddr;
	 unsigned short  uwRegCount;

#endif /* SPARC */

{				/* return iStatus */
	unsigned long  ulFlag;
	unsigned long  ulLSA, ulLEA, ulLCO;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	/* >>>>> change iruma m.takeishi '04.03.29 */
	/*
	iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	*/
	iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_COND_OK );
	/* <<<<< change iruma m.takeishi '04.03.29 */
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NORMAL, uwRegCount, 0, 0, 0 );

	gdb_AddCycle(5);

	/*@ read loop start address, loop end address, loop count and PSR */
	ulLSA = gdbsim_fetch_register (PC) + (unsigned long) CODE;
	ulLEA = ulLSA + ((unsigned long) (uwImmAddr & 0xf) << 1);
	ulLCO = gdbsim_fetch_register (uwRegCount);
	ulFlag = gdbsim_fetch_register (PSR);

	/*@ set LM flag */
	ulFlag |= LM_FLAG;
	(void) gdbsim_store_register (PSR, ulFlag);

	/*@ write loop start address, loop end address and loop count */
	(void) gdbsim_store_register (LSA, ulLSA);
	(void) gdbsim_store_register (LEA, ulLEA);
	(void) gdbsim_store_register (LCO, ulLCO);

	vfnIncPC (CODE);

	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	return (0);
}

/***************************************************************
 *	fnExeLoopImmImm : execute loop instruction (loop imm4,imm4)*
 ***************************************************************/

#ifndef SPARC

int 
fnExeLoopImmImm (unsigned short uwImmAddr, unsigned short uwImmCount)
#else /* SPARC */

int
fnExeLoopImmImm (uwImmAddr, uwImmCount)
	 unsigned short  uwImmAddr;
	 unsigned short  uwImmCount;

#endif /* SPARC */

{				/* return iStatus */
	unsigned long  ulFlag;
	unsigned long  ulLSA, ulLEA, ulLCO;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		return ILL_INST_BREAK;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(5);

	/*@ read loop start address, loop end address, loop count and PSR */
	ulLSA = gdbsim_fetch_register (PC) + (unsigned long) CODE;
	ulLEA = ulLSA + ((unsigned long) (uwImmAddr & 0xf) << 1);
	ulLCO = (unsigned long) (uwImmCount & 0xf);
	ulFlag = gdbsim_fetch_register (PSR);

	/*@ set LM flag */
	ulFlag |= LM_FLAG;
	(void) gdbsim_store_register (PSR, ulFlag);

	/*@ write loop start address, loop end address and loop count */
	(void) gdbsim_store_register (LSA, ulLSA);
	(void) gdbsim_store_register (LEA, ulLEA);
	(void) gdbsim_store_register (LCO, ulLCO);

	vfnIncPC (CODE);

	/* >>>>> add iruma m.takeishi '04.03.15 */
	iRsvLoop_instCnt = 0;
	/* <<<<< add iruma m.takeishi '04.03.15 */

	return (0);
}

/***************************************************************
 *	fnExeRepeat : execute repeat instruction *
 ***************************************************************/

#ifndef SPARC

int 
fnExeRepeat (unsigned short uwOp2, unsigned short uwRepeatCount)
#else /* SPARC */

int
fnExeRepeat (uwOp2, uwRepeatCount)
	 unsigned short  uwOp2;
	 unsigned short  uwRepeatCount;

#endif /* SPARC */

{				/* return iStatus */
	unsigned long  ulFlag;
	unsigned long  ulLSA;
	unsigned long  ulLCO = 0;

	gdb_AddCycle(4);

	/*@ read repeat start address, and PSR */
	ulLSA = gdbsim_fetch_register (PC) + (unsigned long) CODE;
	ulFlag = gdbsim_fetch_register (PSR);

	switch (uwOp2) {

	case 2:  /* 10 repeat %rb */

		fnInterlock( C33_NORMAL, uwRepeatCount, 0, 0, 0 );

		/*@ read repeat number from register */
		ulLCO = gdbsim_fetch_register (uwRepeatCount);
		/* >>>>> delete iruma m.takeishi '03.09.01 */
		/*
		++ulLCO;
		*/
		/* <<<<< delete iruma m.takeishi '03.09.01 */
		break;

	case 3:  /* 11 repeat imm4 */

		fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

		/*@ get repeat number from immediate */
		ulLCO = (unsigned long) uwRepeatCount & 0xf;
		/* >>>>> delete iruma m.takeishi '03.09.01 */
		/*
		++ulLCO;
		*/
		/* <<<<< delete iruma m.takeishi '03.09.01 */
		break;
	}

	/*@ set RM flag */
	ulFlag |= RM_FLAG;
	(void) gdbsim_store_register (PSR, ulFlag);

	/*@ write repeat start address and repeat number */
	(void) gdbsim_store_register (LSA, ulLSA);
	(void) gdbsim_store_register (LCO, ulLCO);

	vfnIncPC (CODE);

	iRepeatFirst = 1;

	return (0);
}

/********************************************************
 *	fnExeCoproFlag : execute ld.cf instruction	  *
 ********************************************************/

#ifndef SPARC

int 
fnExeCoproFlag ()
#else /* SPARC */

int
fnExeCoproFlag ()

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;

	iStatus = 0;		/* initialize */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		gdb_AddCycle(3);
	}else{
		gdb_AddCycle(1);
	}

	vfnIncPC (CODE);	/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);
}

/********************************************************
 *	fnExeCoproTransReg : execute ld.c instruction	 *
 ********************************************************/

#ifndef SPARC

int 
fnExeCoproTransReg (unsigned short uwImm4, unsigned short uwRegDest)
#else /* SPARC */

int
fnExeCoproTransReg (uwImm4, uwRegDest)
	unsigned short uwImm4;
	unsigned short uwRegDest;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(1);

	vfnIncPC (CODE);	/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);
}

/********************************************************
 *	fnExeCoproTransImm : execute ld.c instruction	 *
 ********************************************************/

#ifndef SPARC

int 
fnExeCoproTransImm (unsigned short uwRegSrc, unsigned short uwImm4)
#else /* SPARC */

int
fnExeCoproTransImm (uwRegSrc, uwImm4)
	unsigned short uwRegSrc;
	unsigned short uwImm4;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */

	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(1);

	vfnIncPC (CODE);	/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);
}

/********************************************************
 *	fnExeCoproCom : execute do.c instruction	*
 ********************************************************/

#ifndef SPARC

int 
fnExeCoproCom (unsigned short uwImm6)
#else /* SPARC */

int
fnExeCoproCom (uwImm6)
	unsigned short uwImm6;

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	/* >>>>> add iruma m.takeishi '03.09.02 */
	int 			iRet;
	/* <<<<< add iruma m.takeishi '03.09.02 */

	iStatus = 0;		/* initialize */


	/* >>>> add iruma m.takeishi '03.09.02 */
	iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
	if ( iRet != 0 ) {
		iStatus = ILL_INST_BREAK;
		return iStatus;
	}
	/* <<<<< add iruma m.takeishi '03.09.02 */

	fnInterlock( C33_NOT_INTERLOCK, 0, 0, 0, 0 );

	gdb_AddCycle(1);

	vfnIncPC (CODE);	/* PC += CODE */

	vfnInitExtension ();	/* initialize extension */

	return (iStatus);
}

/********************************************************
 *	fnExeEachInst : execute each instruction	*
 ********************************************************/

#ifndef SPARC

int 
fnExeEachInst (unsigned short uwCode)
#else /* SPARC */

int
fnExeEachInst (uwCode)
	 unsigned short  uwCode;	/* 16bit code */

#endif /* SPARC */

{				/* return iStatus */
	int 			iStatus;
	unsigned short	uwClass, uwOp1, uwDelay, uwOp2, uwOp3;
	unsigned short	uwBit0_3, uwBit0_4, uwBit0_5, uwBit0_7, uwBit0_9, uwBit0_12;
	unsigned short	uwBit4_5, uwBit4_7, uwBit4_8, uwBit4_9;
	/* >>>>> add iruma m.takeishi '03.09.01 */
	int 			iRet, iExt, iExtType;
	int 			iLoop= CMB_LOOP_NG , iRepeat = CMB_REPEAT_NG;
	/* <<<< add iruma m.takeishi '03.09.01 */

	extern struct DelaySlot stDelaySlot;	/* for Delay slot */
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	extern struct Extension stExtension;
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */



	iStatus = 0;	/* 2002.06.20 add T.Tazaki Status Clear */

	/*@ extract class from code */
	uwClass = uwCode & 0xe000;
	uwClass >>= 13;

	/* >>>>> change iruma m.takeishi '03.09.01 */
	/*
	// ext cond check //
	if(fnExeCond(uwCode))
	*/
	
	/*	each instruction check:   */
	/*	 if CPU doesn't execute the inst in a mode(repeat,loop,ext), */
	/*	 The check is done here. */
	
	iRet = 1;  // exec
	/* in ext-cond skip mode ? */
	if ( stExtension.iTypeExtCond == EXTCOND_SKIP ) {
		iRet = 0;
		// ignore inst-fetch cycle */
		gulCycle = stExtension.ulPushCycle;
	}
	if(iRet)
	/* <<<<< change iruma m.takeishi '03.09.01 */
	{

	switch (uwClass) {		/* class */

	case 0: 		/* class 0 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1e00;
	uwOp1 >>= 9;

	/*@ extract d from code */
	uwDelay = uwCode & 0x0100;
	uwDelay >>= 8;

	if (uwOp1 < 4) {
		/*@ Yes op1 : 0000 - 0011 */

		/*@ extract op2 from code */
		uwOp2 = uwCode & 0x00c0;
		uwOp2 >>= 6;
		/*@ extract bit4_5 from code */
		uwBit4_5 = uwCode & 0x0030;
		uwBit4_5 >>= 4;
		/*@ extract bit0_3 from code */
		uwBit0_3 = uwCode & 0x000f;

		/*@ check if code is OK */
		if (uwBit4_5 == 0) {
		switch (uwOp1 << 2 | uwOp2) {	/* instruction */

		case 0: /* 0000:00 nop */
		case 1: /* 0000:01 slp */
		case 2: /* 0000:10 halt */
		case 10:/* 0010:10 int imm2 */

			/*@ check if code is OK */
			if (uwDelay == 0) {
				/*@ check if code is delayed */
				if (stDelaySlot.blUsed == YES) {
					iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
				}
				else {
					/*@ execute */
					iStatus = fnExeSpecial (uwOp1, uwOp2, uwBit0_3);

				}
			}
			else {
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			}

			break;

		case 4: /* 0001:00 pushn %rs */
		case 5: /* 0001:01 popn %rd */

			/*@ check if code is OK */
			if (uwDelay == 0) {
				/*@ check if code is delayed */
				if (stDelaySlot.blUsed == YES) {
					iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
				}
				else {
					/*@ execute */
					iStatus = fnExePushnPopn (uwOp2, uwBit4_5, uwBit0_3);
				}
			}
			else {
			   iStatus = ILL_INST_BREAK;   /* set illegal inst break */
			}

			break;

/* 12/17/2001 N.Komatsu  add jpr instruction */
		case 7: 	/* 0001:11 jpr %rb , jpr.d %rb */
			if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}
		case 12:	/* 0011:00 call %rb */
		case  14:	 /* 0011:10 jp %rb */

			/*@ check if code is delayed */
			if (stDelaySlot.blUsed == YES) {
			iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
			}
			else {
			
			
			
				/* >>>>> add iruma m.takeishi '03.09.01 */
				switch (uwOp1 << 2 | uwOp2) {
				case 7:
					if (uwDelay == 0) {
						/* 0001:11 jpr %rb */
						iLoop = CMB_LOOP_OK;
					} else {
						/* 0001:11 jpr.d %rb */
						iLoop = CMB_LOOP_OK_A1;
					}
					break;
				case 12:	/* 0011:00 call %rb */
				case 14:	/* 0011:10 jp %rb */
					iLoop = CMB_LOOP_OK_A1;
					break;
				}
				iRet = fnCheckInstCombi( iLoop, CMB_REPEAT_NG, CMB_EXT_NG );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	
				/* <<<<< add iruma m.takeishi '03.09.01 */
					/*@ execute */
					iStatus = fnExeControl (uwOp2, uwBit0_3, (int) uwDelay);
			}

			break;

		case 11:	/* 0010:11 reti */
		case 13:	/* 0011:01 ret */

			/*@ check if code is OK */
			if (uwBit0_3 == 0) {
			/*@ check if code is delayed */
			if (stDelaySlot.blUsed == YES) {
				iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
			}
			else {

				/* >>>>> add iruma m.takeishi '03.09.01 */
				iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_NG );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	
				/* <<<<< add iruma m.takeishi '03.09.01 */
					/*@ execute */
					iStatus = fnExeReturn (uwOp2, (int) uwDelay);
			}
			}
			else {
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			}

			break;

		case 15:	/* 0011:11 retm */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				}
				else
				{
					/* >>>>> add iruma m.takeishi '03.09.01 */
					iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_NG );
					if ( iRet != 0 ) {
						iStatus = ILL_INST_BREAK;
					} else	
					/* <<<<< add iruma m.takeishi '03.09.01 */
						/*@ execute */
						iStatus = fnExeReturnMMU ((int) uwDelay);
				}
				break;

		case 3: /* 0000:11 reserved */
		case 6: /* 0001:10 reserved */
		case 8: /* 0010:00 brk */
		case 9: /* 0010:01 retd */
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
			
		}
		}
		else {

		if (uwOp1 < 2) {
		/*@ Yes op1 : 0000 - 0001 */

			/*@ check if code is OK */
			if (uwDelay == 0) {

			/*@ check if code is OK */
			switch (uwOp1 << 2 | uwOp2) {	/* instruction */

			case 0: /* 0000:00 push %rs */
			case 1: /* 0000:01 pop %rd */
			case 2: /* 0000:10 pushs %ss */
			case 3: /* 0000:11 pops %sd */

				if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}
				/*@ check if code is delayed */
				if (stDelaySlot.blUsed == YES) {
					iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
				}
				else {

					switch (uwOp1 << 2 | uwOp2) {
					case 0: /* 0000:00 push %rs */
					case 1: /* 0000:01 pop %rd */
						iRepeat = CMB_REPEAT_OK;
						iLoop = CMB_REPEAT_OK;
						break;
					
					case 2: /* 0000:10 pushs %ss */
						iRepeat = CMB_REPEAT_NG;
						iLoop = CMB_REPEAT_OK;
						break;
					case 3:
						/* 0000:11 pops %sd */
						iRepeat = CMB_REPEAT_NG;
						iLoop = CMB_LOOP_OK_A3;
						break;
					}
					
					iRet = fnCheckInstCombi( iLoop, iRepeat, CMB_EXT_COND_OK );
					if ( iRet != 0 ) {
						iStatus = ILL_INST_BREAK;
					} else 
						/* increase argument fnExePushnPopn function */
						iStatus = fnExePushnPopn (uwOp2, uwBit4_5, uwBit0_3);
				}
				break;

			case 4: /* 0001:00 divu.w %rb */
			case 5: /* 0001:01 div.w %rb */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}
				
				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_NG, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	
					iStatus = fnExeDiv (uwOp1, uwOp2, uwBit0_3);
				
				break;

			case 6: /* 0001:10 repeat %rb */
			case 7: /* 0001:11 repeat imm4 */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}

				iRet = fnCheckInstCombi( CMB_LOOP_NG, CMB_REPEAT_NG, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	
					/*@ execute */
					iStatus = fnExeRepeat (uwOp2, uwBit0_3);
				
				break;
			}
		}
		else {

			/*@ check if code is OK */
			switch (uwOp1 << 2 | uwOp2) {	/* instruction */

			case 0:    /* 0000:00 mac.w %rs */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}
				
				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_NG, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else {
					gdb_AddCycle(3);
					iStatus = fnExeMacW (uwOp2, uwBit0_3);
				}
				
				break;

			case 1:    /* 0000:01 mac.hw %rs */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}

				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_NG, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else {
					gdb_AddCycle(2);
					iStatus = fnExeMacW (uwOp2, uwBit0_3);
				}
				break;

			case 2:    /* 0000:10 macclr */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}
				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	

					iStatus = fnExeMacClr ();

				break;

			case 3:    /* 0000:11 ld.cf */
				if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}
				/* add T.Tazaki 2004/08/25 >>> */
				if ( fnCheckAdvancedMode() == C33_PRODUCT_PE && stDelaySlot.blUsed == YES)
				{
					iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
					break;
				}
				/* add T.Tazaki 2004/08/25 <<< */

				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK, CMB_EXT_COND_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	
					/*@ execute */
					iStatus = fnExeCoproFlag ();

				break;

			case 4:    /* 0001:00 reserved */
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;

			case 5:    /* 0001:01 add %rd,%dp */
				if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
				{
					iStatus = ILL_INST_BREAK;	/* set illegal inst break */
					break;
				}

				iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_OK,	
						CMB_EXT_COND_OK | CMB_EXT_RS_OK | CMB_EXT_RS_OP_IMM_OK |	
						CMB_EXT_OP_IMM_OK | CMB_EXT_IMM_OK );
				if ( iRet != 0 ) {
					iStatus = ILL_INST_BREAK;
				} else	

					/*@ execute */
					iStatus = fnExeArithDP (uwBit0_3);

				break;

			case 6:    /* 0001:10 reserved */
			case 7:    /* 0001:11 reserved */

				iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
				break;
			}
		}
		}
		else {
		/*@ No op1 0010 - 0011 */

		iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */

		}
		}
	}
	else {
		/*@ No	op1 : 0100 - 0111 */

		/*@ extract bit0_7 from code */
		uwBit0_7 = uwCode & 0x00ff;

		/*@ check if code is delayed */
		if (stDelaySlot.blUsed == YES) {
		iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
		}
		else {
		/*@ execute */
		iStatus = fnExeBranch (uwOp1, uwBit0_7, (int) uwDelay);
		}
	}

	break;

	case 1: 		/* class 1 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	/*@ extract op2 from code */
	uwOp2 = uwCode & 0x0300;	/* uwCode[9:8] */
	uwOp2 >>= 8;
	/*@ extract bit4_7 from code */
	uwBit4_7 = uwCode & 0x00f0;
	uwBit4_7 >>= 4;
	/*@ extract bit0_3 from code */
	uwBit0_3 = uwCode & 0x000f;

	switch (uwOp2) {	/* op2 */

	case 0: 	/* 00 */

		/*@ execute */
		iStatus = fnExeTrans (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

	case 1: 	/* 01 */

		/*@ check if code is delayed */
		if (stDelaySlot.blUsed == YES) {
		iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
		}
		else {
		/*@ execute */
		iStatus = fnExeTrans (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		}
		break;

	case 2: 	/* 10 */

		/*@ execute */
		iStatus = fnExeArith (uwOp1, uwBit4_7, uwBit0_3);
		break;

	case 3: 	/* 11 */

//		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )	/* chenge 2004/08/05 T.Tazaki */
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}
		switch(uwOp1)
		{
			case 6:  /* ext op,imm2 | ext cond */
			case 7:  /* ext %rs | ext %rs,op,imm2 */
					 iStatus = fnExeExtReg (uwOp1,uwBit4_7,uwBit0_3);
					 break;
			default:
					 /*  for 16-31 bits of shift and rotate instruction */
					 /*@ extract bit4_8 from code */
					 uwBit4_8 = uwCode & 0x01f0;
					 uwBit4_8 >>= 4;
					 /*@ execute */
					 iStatus = fnExeShiftRot (uwOp1, uwOp2, uwBit4_8, uwBit0_3);
					 break;
		}
		break;
	}

	break;

	case 2: 		/* class 2 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	/*@ extract bit4_9 from code */
	uwBit4_9 = uwCode & 0x03f0;
	uwBit4_9 >>= 4;
	/*@ extract bit0_3 from code */
	uwBit0_3 = uwCode & 0x000f;

	/*@ execute */
	iStatus = fnExeStack (uwOp1, uwBit4_9, uwBit0_3);

	break;

	case 3: 		/* class 3 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	/*@ extract bit4_9 from code */
	uwBit4_9 = uwCode & 0x03f0;
	uwBit4_9 >>= 4;
	/*@ extract bit0_3 from code */
	uwBit0_3 = uwCode & 0x000f;

	/*@ execute */
	iStatus = fnExeArithImm (uwOp1, uwBit4_9, uwBit0_3);

	break;

	case 4: 		/* class 4 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	if (uwOp1 < 2) {
		/*@ Yes op1 : 000 - 001 */

		/*@ extract bit0_9 from code */
		uwBit0_9 = uwCode & 0x03ff;

		/*@ execute */
		iStatus = fnExeArithSP (uwOp1, uwBit0_9);
	}
	else {
		/*@ No	op1 : 010 - 111 */

		/*@ extract op2 from code */
		uwOp2 = uwCode & 0x0300;
		uwOp2 >>= 8;
		/*@ extract bit4_7 from code */
		uwBit4_7 = uwCode & 0x00f0;
		uwBit4_7 >>= 4;
		/*@ extract bit0_3 from code */
		uwBit0_3 = uwCode & 0x000f;

		switch (uwOp2) {	/* op2 */

		case 0: 	/* 00 */
		case 1: 	/* 01 */

		/*@ execute */
		iStatus = fnExeShiftRot (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

		case 2: 	/* 10 */

		switch (uwOp1) {	/* op1 */

		case 2: /* 010 scan0 %rd,%rs */
		case 3: /* 011 scan1 %rd,%rs */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}

			/*@ execute */
			iStatus = fnExeScan (uwOp1, uwBit4_7, uwBit0_3);
			break;

		case 5: /* 101 mirror %rd,%rs */
			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}
		case 4: /* 100 swap %rd,%rs */
			/*@ execute */
			iStatus = fnExeSwapMirror (uwOp1, uwBit4_7, uwBit0_3);
			break;

		case 6: /* swaph %rd,%rs */
			if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}

			/*@ execute */
			iStatus = fnExeSwapMirror (uwOp1, uwBit4_7, uwBit0_3);
			break;

		case 7: /* 111 sat.b %rd,%rs */

			if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}
			/*@ execute */
			iStatus = fnExeSat(uwClass, uwOp1, uwOp2, uwBit4_7, uwBit0_3);
			break;
		}

		break;

		case 3: 	/* 11 */

		switch (uwOp1) {	/* op1 */

		case 2: /* 010 div0s */
		case 3: /* 011 div0u */
		case 4: /* 100 div1 %rs */
		case 5: /* 101 div2s %rs */
		case 6: /* 110 div3s %rs */

			if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}
			/*@ check if code is delayed */
			if (stDelaySlot.blUsed == YES) {
			iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
			}
			else {

			/*	increase the argument */
			/*@ execute */
			iStatus = fnExeDiv (uwOp1, uwOp2, uwBit4_7);
			}
			break;

		case 7: /* 111 sat.ub %rd,%rs */

			if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}
			/*@ execute */
			iStatus = fnExeSat(uwClass, uwOp1, uwOp2, uwBit4_7, uwBit0_3);
			break;
		}

		break;
		}
	}

	break;

	case 5: 		/* class 5 */

	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	/*@ extract op2 from code */
	uwOp2 = uwCode & 0x0300;
	uwOp2 >>= 8;
	/*@ extract bit4_7 from code */
	uwBit4_7 = uwCode & 0x00f0;
	uwBit4_7 >>= 4;
	/*@ extract bit0_3 from code */
	uwBit0_3 = uwCode & 0x000f;

	switch (uwOp2) {	/* op2 */

	case 0: 	/* 00 */

		switch (uwOp1) {	/* op1 */

		case 0: 	/* 000:00 ld.w	%sd,%rs */
		case 1: 	/* 001:00 ld.w	%rd,%ss */

		/*@ execute */
		iStatus = fnExeTransSpecial (uwOp1, uwBit4_7, uwBit0_3);
		break;

		case 2: 	/* 010:00 btst [%rb],imm3 */
		case 3: 	/* 011:00 bclr [%rb],imm3 */
		case 4: 	/* 100:00 bset [%rb],imm3 */
		case 5: 	/* 101:00 bnot [%rb],imm3 */

		iStatus = fnExeBit (uwOp1, uwBit4_7, uwBit0_3);
		break;

		case 6: 	/* 110:00 adc  %rd,%rs */
		case 7: 	/* 111:00 sbc  %rd,%rs */

		/*@ execute */
		iStatus = fnExeArithC (uwOp1, uwBit4_7, uwBit0_3);
		break;
		}

		break;

	case 1: 	/* 01 */

		switch (uwOp1) {	/* op1 */

		case 0: 	/* 000:01 ld.b	%rd,%rs */
		case 1: 	/* 001:01 ld.ub %rd,%rs */
		case 2: 	/* 010:01 ld.h	%rd,%rs */
		case 3: 	/* 011:01 ld.uh %rd,%rs */

		/*@ execute */
		iStatus = fnExeTransReg (uwOp1, uwBit4_7, uwBit0_3);
		break;

		case 4: 	/* 100:01 ld.c %rd,imm4 */
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeCoproTransReg (uwBit4_7, uwBit0_3);
		break;

		case 5: 	/* 101:01 ld.c imm4,%rs */
		if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeCoproTransImm (uwBit4_7, uwBit0_3);
		break;

		case 6: 	/* 110:01 loop %rc,%ra */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeLoop (uwBit4_7, uwBit0_3);
		break;

		case 7: 	/* 111:01 sat.w %rd,%rs */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		/*@ execute */
		iStatus = fnExeSat (uwClass, uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

		}

		break;

	case 2: 	/* 10 */

		switch (uwOp1) {	/* op1 */

		case 0: 	/* 000:10 mlt.h  %rd,%rs */
		case 1: 	/* 001:10 mltu.h %rd,%rs */
		case 2: 	/* 010:10 mlt.w  %rd,%rs */
		case 3: 	/* 011:10 mltu.w %rd,%rs */

		/*@ check if code is delayed */
		if (stDelaySlot.blUsed == YES) {
			// Delayd instruction bug fix	   T.Tazaki 2001.03.15
			unsigned short usCode = uwCode & 0xff00;
			switch( usCode ){
			case 0xa200:		/* mlt.h  %rd,%rs */
			case 0xa600:		/* mltu.h %rd,%rs */
/* 12/17/2001 N.Komatsu  increase the argument */
				iStatus = fnExeMlt (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
				break;
			default:
				iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
				break;
			// Delayd instruction bug fix	   T.Tazaki 2001.03.15
			}
		}
		else {
			/*@ execute */
			iStatus = fnExeMlt (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		}
		break;

		case 4: 	/* 100:10 mac %rs */

		if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}
		/*@ check if code is delayed */
		if (stDelaySlot.blUsed == YES) {
			iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
		}
		else {
			/*@ execute */
			iRet = fnCheckInstCombi( CMB_LOOP_OK, CMB_REPEAT_NG, CMB_EXT_COND_OK );
			if ( iRet != 0 ) {
				iStatus = ILL_INST_BREAK;
				break;
			}
			iStatus = fnExeMac (uwBit4_7);
		}
		break;

		case 5: 	/* 101:10 sat.h %rd,%rs */
		case 7: 	/* 111:10 sat.uw %rd,%rs */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeSat(uwClass, uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

		case 6: 	/* 110:10 loop %rc,imm4 */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		// iStatus = ILL_INST_BREAK;   /* set illegal inst break */
		iStatus = fnExeLoopRegImm (uwBit4_7, uwBit0_3);
		break;
		}

		break;

	case 3: 	/* 11 */

		switch (uwOp1) {	/* op1 */

		case 0:    /* 000 mlt.hw %rd,%rs */
		
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		/*@ execute */
		iStatus = fnExeMlt (uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

		case 1:    /* 001 mac1.h %rs,%rs */
		case 2:    /* 010 mac1.hw %rs,%rs */
		case 4:    /* 100 mac1.w %rs,%rs */

		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}
		/*@ execute */
		iStatus = fnExeMac1 (uwOp1, uwBit4_7, uwBit0_3);
		break;

		case 5:    /* 101 sat.uh %rd,%rs */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeSat (uwClass, uwOp1, uwOp2, uwBit4_7, uwBit0_3);
		break;

		case 6:    /* 110 loop imm4,imm4 */
		if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
		{
			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;
		}

		iStatus = fnExeLoopImmImm (uwBit4_7, uwBit0_3);
		break;

		/*	in the case of uwOp1 is 7 (for do.c , psrset and psrclr instruction) */
		case 7:    /* 111 */

		/*@ extract op3 from code */
		uwOp3 = uwCode & 0xc0;
		uwOp3 >>= 6;

		/*@ extract bit0_5 from code */
		uwBit0_5 = uwCode & 0x3f;

			switch (uwOp3) {	/* op3 */

			case 0:    /* 00 do.c imm6 */
			if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}

			/* execute */
			iStatus = fnExeCoproCom (uwBit0_5);
			break;

			case 1:    /* 01 psrset imm5 */
			case 2:    /* 10 psrclr imm5 */
			if( fnCheckAdvancedMode() == C33_PRODUCT_STD )
			{
				iStatus = ILL_INST_BREAK;	/* set illegal inst break */
				break;
			}

			/* execute */
			iStatus = fnExePsrSetClr(uwOp3, uwBit0_5);
			break;

			case 4:    /* 11 reserved */

			iStatus = ILL_INST_BREAK;	/* set illegal inst break */
			break;

			}

		break;

		case 3:    /* 011 */

		iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		break;

		}
	}

	break;

	case 6: 		/* class 6 */

	/*@ extract bit0_12 from code */
	uwBit0_12 = uwCode & 0x1fff;

	/*@ check if code is delayed */
	if (stDelaySlot.blUsed == YES) {
		iStatus = ILL_DELAY_BREAK;	/* set illegal delay inst break */
	}
	else {
		/*@ execute */
		iStatus = fnExeExt (uwBit0_12);
	}

	break;

	case 7: 		/* class 7 */
	if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
	{
		iStatus = ILL_INST_BREAK;	/* set illegal inst break */
		break;
	}

	/* 12/10/2001 N.Komatsu  add data pointer instruction */
	/*@ extract op1 from code */
	uwOp1 = uwCode & 0x1c00;
	uwOp1 >>= 10;

	/*@ extract bit4_9 from code */
	uwBit4_9 = uwCode & 0x03f0;
	uwBit4_9 >>= 4;
	/*@ extract bit0_3 from code */
	uwBit0_3 = uwCode & 0x000f;

	/*@ execute */
	iStatus = fnExeDataPoint (uwOp1, uwBit4_9, uwBit0_3);

	break;

	}
	}
	else
	{
		/* PC increment only */
		vfnIncPC (CODE);
	}

	return (iStatus);		/* return status */
}


/************************************************
 *	vfnInitExtension : reset Extension	*
 ************************************************/
void
vfnInitExtension ()
{				/* return void */

	/* initialize extension */
	stExtension.iType = EXT_NONE;
	stExtension.ulData = 0;

	stExtension.iTypeExt = EXT_NONE;
	stExtension.ulDataExtRs = 0;
	stExtension.uwDataExtOp = 0;

	stExtension.iTypeExtCond = EXTCOND_NONE;
	stExtension.ulPushCycle = 0;

	return;
}

/****************************************************************************/
/* Format		 : static void fnInterlock(int iInstMode,unsigned short uwRegno, unsigned shoty uwRegBase,int iCycle1, int iCycle2	  */
/* Arguments																*/
/*	int iInstMode  : instruction mode										*/
/*					 C33_NOT_INTERLOCK : do not add interlock cycle 		*/
/*					 C33_NORMAL 	   : Register load						*/
/*					 C33_DIV_MAC	   : mac,div,mlt Source Reg = 1 		*/
/*					 C33_MLT_MAC1	   : mlt,mac1	 Source Reg = 2 		*/
/*					 C33_BIT		   : bit set [%rb]						*/
/*					 C33_INDIRECT	   : Indirect memory access instruntion */
/*	unsigned short uwRegno	  : %rd or %rs									*/
/*	unsigned short uwRegBase  : %rb or %rs									*/
/*	int iCycle1    : cycyle 1												*/
/*	int iCycle2    : cycyle 2												*/
/* Return value  : void 													*/
/* Explanation	 : Interlock cycle counter addition.						*/
/****************************************************************************/
static void
fnInterlock(int iInstMode,unsigned short uwRegno,unsigned short uwRegBase,int iCycle1, int iCycle2 )
{
	unsigned long ulPSR;

	if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )	/* Not Advanced Mode ? */
	{
		return; 	/* Nop */
	}

	switch( iInstMode )
	{
		case C33_NOT_INTERLOCK:
				
				/* clear interlock mode */
				c33InterLockInfo.iInterlockMode = 0;
				c33InterLockInfo.iCycle1   = 0;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = 0;
				c33InterLockInfo.uwRegno   = 0;
				break;

		case C33_NORMAL:
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					ulPSR = gdbsim_fetch_register( PSR );

					/* Before Instruntion is div, mac ,mlt, mac1 ?*/
					if( c33InterLockInfo.iNextInst == C33_MAC_MLT_DIV )
					{
						if( ulPSR & HC_FLAG )
						{
							if( uwRegno == AHR || c33InterLockInfo.uwRegno == R5 )
							{
								gdb_AddCycle( c33InterLockInfo.iCycle1 );
							}
						}
						else if( uwRegno == AHR )
						{
							gdb_AddCycle( c33InterLockInfo.iCycle1 );
						}

						if( ulPSR & LC_FLAG )
						{
							if( uwRegno == ALR || c33InterLockInfo.uwRegno == R4 )
							{
								gdb_AddCycle( c33InterLockInfo.iCycle1 );
							}
						}
						else if( uwRegno == ALR )
						{
							gdb_AddCycle( c33InterLockInfo.iCycle1 );
						}
					}
					else
					{
						/* Normal Source register */
						if( c33InterLockInfo.uwRegno == uwRegno )	/* %rd == %rs ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle1 );
						}
						else
						{
							if( ulPSR & HC_FLAG )
							{
								if( uwRegno == R5 && c33InterLockInfo.uwRegno == AHR )
								{
									gdb_AddCycle( c33InterLockInfo.iCycle1 );
								}
							}
							if( ulPSR & LC_FLAG )
							{
								if( uwRegno == R4 && c33InterLockInfo.uwRegno == ALR )
								{
									gdb_AddCycle( c33InterLockInfo.iCycle1 );
								}
							}
						}
					}
				}
				
				/* clear interlock mode */
				c33InterLockInfo.iInterlockMode = 0;
				c33InterLockInfo.iCycle1   = 0;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = 0;
				c33InterLockInfo.uwRegno   = 0;
				break;

		case C33_DIV_MAC:	/* div, mac %rs */
				/* check interlock mode */
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					if( c33InterLockInfo.iNextInst != C33_MAC_MLT_DIV )
					{
						if( c33InterLockInfo.uwRegno == uwRegno )	/* %rd == %rs ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle1 );
						}
					}
				}
				
				/* set interlock mode */
				c33InterLockInfo.iInterlockMode = 1;
				c33InterLockInfo.iCycle1   = iCycle1;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = C33_MAC_MLT_DIV;
				c33InterLockInfo.uwRegno   = AHR;
				c33InterLockInfo.uwRegno2  = ALR;
				break;
	

		case C33_MLT_MAC1:	/* mlt, mac1 */
				/* check interlock mode */
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					if( c33InterLockInfo.iNextInst != C33_MAC_MLT_DIV )
					{
						/* Regno = Source Reg1 or Source Reg2 ? */
						if( c33InterLockInfo.uwRegno == uwRegno || c33InterLockInfo.uwRegno2 == uwRegno )	/* %rd == %rs ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle1 );
						}
					}
				}
				
				/* set interlock mode */
				c33InterLockInfo.iInterlockMode = 1;
				c33InterLockInfo.iCycle1   = iCycle1;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = C33_MAC_MLT_DIV;
				c33InterLockInfo.uwRegno   = AHR;
				c33InterLockInfo.uwRegno2  = ALR;
				break;
	
		case C33_INDIRECT:
				/* check interlock mode */
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					if( c33InterLockInfo.iNextInst == C33_INDIRECT )
					{
						if( c33InterLockInfo.uwRegno == uwRegBase )   /* %rd == [%rb] ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle2 );
						}
					}
					else if( c33InterLockInfo.iNextInst == C33_POP )
					{
						/* uwRegBase = from 0 to 15 */
						if( c33InterLockInfo.uwRegno == uwRegBase )   /* pop %rd == [%rb] ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle2 );
						}
					}
					else if( c33InterLockInfo.iNextInst == C33_POPN )
					{
						/* uwRegBase = from 0 to 15 */
						if( c33InterLockInfo.uwRegno == uwRegBase )   /* popn %rd == [%rb] ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle2 );
						}
					}
					else if( c33InterLockInfo.iNextInst == C33_POPS )
					{
						if( (uwRegBase == DP) && (c33InterLockInfo.uwRegno == DP))	   /* pops %dp == [%dp] ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle2 );
						}
					}
				}

				/* set interlock mode */
				c33InterLockInfo.iInterlockMode = 1;
				c33InterLockInfo.iCycle1   = iCycle1;
				c33InterLockInfo.iCycle2   = iCycle2;
				c33InterLockInfo.iNextInst = C33_INDIRECT;
				c33InterLockInfo.uwRegno   = uwRegno;	/* %rd */
				break;

		case C33_BIT:
				/* check interlock mode */
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					if( c33InterLockInfo.iNextInst == C33_INDIRECT )
					{
						if( c33InterLockInfo.uwRegno == uwRegBase )   /* %rd == [%rb] ? */
						{
							gdb_AddCycle( c33InterLockInfo.iCycle2 );
						}
					}
				}

				/* clear interlock mode */
				c33InterLockInfo.iInterlockMode = 0;
				c33InterLockInfo.iCycle1   = 0;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = 0;
				c33InterLockInfo.uwRegno   = 0;
				break;

		case C33_PUSHN:
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					/* uwRegno = from 0 to 15 */
					if( c33InterLockInfo.uwRegno <= uwRegno )	/* pushs %r0->%rd == %rd ? */
					{
						gdb_AddCycle( c33InterLockInfo.iCycle1 );
					}
				}
				
				/* clear interlock mode */
				c33InterLockInfo.iInterlockMode = 0;
				c33InterLockInfo.iCycle1   = 0;
				c33InterLockInfo.iCycle2   = 0;
				c33InterLockInfo.iNextInst = 0;
				c33InterLockInfo.uwRegno   = 0;
				break;

		case C33_POP:
		case C33_POPN:
		case C33_POPS:
				if( c33InterLockInfo.iInterlockMode == 1 )
				{
					gdb_AddCycle( c33InterLockInfo.iCycle1 );
				}

				/* set interlock mode */
				c33InterLockInfo.iInterlockMode = 1;
				c33InterLockInfo.iCycle1   = iCycle1;
				c33InterLockInfo.iCycle2   = iCycle2;
				c33InterLockInfo.iNextInst = iInstMode;
				c33InterLockInfo.uwRegno   = uwRegno;		 /* %rs : when C33_POPS : from %alr to %dp */
				break;

	}
}

/***********************************************************
 *	
 *		Count up clock of instruction
 *		RETURN:void
 *	
 ***********************************************************/
unsigned long ulRealClock;

void
vfnCountUpRealClockInstruction(
	unsigned long ulCount)		/* code */
{
	ulRealClock += ulCount;
}

/***********************************************************
 * Check cpu mode standard mode or advanced mode *
 ***********************************************************/
static unsigned char
fnCheckAdvancedMode()
{
	if( c33TargetInfo.pMapInfo != NULL ){
		return c33TargetInfo.ucAdvanced;
	}
	else
	{
		return 0;
	}

}

/***********************************************************
 * Get memory endian from address *
 ***********************************************************/
static int
fnGetEndian( CORE_ADDR ulAddr)
{
	struct c33_map_info *pstMapInfo;
	int iEndian = C33_LITTLE;
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			/* check if address renge */
			if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
			{
				iEndian = pstMapInfo->iEndian;
				break;
			}
			
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
	}
	
	return iEndian;
}

/***********************************************************
 * Check ROM area from address *
 ***********************************************************/
static int
fnCheckRomArea( CORE_ADDR ulAddr)
{
	struct c33_map_info *pstMapInfo;
	int iStatus = 0;
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			/* check if address renge */
			if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
			{
				if( pstMapInfo->iMapAttr == C33_ROM )
				{
					iStatus = ROM_DATA_BREAK;
				}
				break;
			}
			
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
	}
	
	return iStatus;
}

/***********************************************************
 * Check exist from 0x20000000 to 0x2000000 map area *
 ***********************************************************/

static int
fnCheckExistAdvArea()
{
	struct c33_map_info *pstMapInfo;
	int iStatus = 0;
	unsigned long ulAddr;
	long	l_cnt;

	if( c33TargetInfo.pMapInfo != NULL ){

		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			if( pstMapInfo->iMapAttr == C33_ROM || pstMapInfo->iMapAttr == C33_RAM )
			{
				/* check if address range */
				for( ulAddr = pstMapInfo->ulStartAddr; ulAddr <= pstMapInfo->ulEndAddr; ++ulAddr )
				{
					if( ulAddr == 0x20000000UL )
					{
						iStatus = 1;	/* exist */
						break;
					}
				}
			}

			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
	}
	
	return iStatus;

}

/***********************************************************
 * Get memory access cycle from address , mode(read,write) and access type(byte,half,word)
 ***********************************************************/
static unsigned long
gdb_GetMemoryAccessCycle(unsigned long ulAddr, int iwrite, int iSize)
{
	int i;
	unsigned long ulCycle = 0;	/* access cycle */
	unsigned char ucMemType = 0;	/* memory type(byte,half,word) */

	struct c33_map_info *pstMapInfo;
	int iEndian = C33_LITTLE;
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			/* check if address renge */
			if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
			{
				if(iwrite)
				{
					/* write */
//					  ulCycle = pstMapInfo->ulReadCycle;
					ulCycle = pstMapInfo->ulWriteCycle; /* bug fix T.Tazaki 2003/08/04 */
				}
				else
				{
					/* read */
//					  ulCycle = pstMapInfo->ulWriteCycle;
					ulCycle = pstMapInfo->ulReadCycle;		/* bug fix T.Tazaki 2003/08/04 */
				}
				ucMemType = pstMapInfo->ulDeviceSize;
				break;
			}

			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
			
		}
	}
	else
	{
		ulCycle = MEM_ACCESS_DEFAULT;
		ucMemType = C33_HALF_WORD;
	}

	//memory access number of times
	if(iSize == C33_BYTE)
	{
	}
	else if(iSize == C33_HALF_WORD)
	{
		if(ucMemType == C33_BYTE)
		{
			ulCycle *= 2;
		}
	}
	else if(iSize == C33_WORD)
	{
		if(ucMemType == C33_BYTE)
		{
			ulCycle *= 4;
		}
		else if(ucMemType == C33_HALF_WORD)
		{
			ulCycle *= 2;
		}
	}

	return ulCycle;
}

/****************************************************************************/
/* Format		 : static void c33_SimTraceMode(nt iParamNum, char* pMode, int iOutput, char* pFilename) */
/* Arguments																*/
/*	int iParamNum			  : Parameter number							*/
/*	char *pMode 			  : Trace Mode on or off						*/
/*	int iOutput 			  : Trace output format [1-4]					*/
/*	char* pFilename 		  : Filename									*/
/* Return value  : void 													*/
/* Explanation	 : Trace mode execution.									*/
/****************************************************************************/
void
c33_SimTraceMode(int iParamNum, char* pMode, int iOutput, char* pFilename)
{
	int iResult;
	unsigned long ulSize;
	char cParBuf[256];
	char szBuf[300];

	/* Check number of parameter */
	if ( iParamNum < 2 || iParamNum > 4 )
	{
		c33_error (C33_ERR_COMMAND_PARAM);
		return;
	}


	if( strcmp(pMode,"off") == 0 )
	{
		if ( iParamNum != 2 )
		{
			c33_error (C33_ERR_COMMAND_PARAM);
			return;
		}
		g_iTraceDump = TM_OFF;	/* set trace mode off */
		
		if( Trace_fp != NULL )
		{
			/* trace file close */
			fclose( Trace_fp );
			Trace_fp = NULL;
		}
	}
	else
	{
		if( strcmp(pMode,"on") == 0 )
		{
			if ( iParamNum < 3 || iParamNum > 4 )
			{
				c33_error (C33_ERR_COMMAND_PARAM);
				return;
			}
			if( iOutput < 1 || iOutput > 4 )
			{
				c33_error (C33_ERR_COMMAND_TRACE_NUM);
				return;
			}

			g_iTraceDump = TM_ON;		/* set trace mode on */
			g_ulInstNum = 1;			/* trace number initialize */
			g_iTraceFormat = iOutput;	/* set output format */

			gulCycle = 0;		/* Cycle Counter initialize zero */
		
			if( iParamNum == 3 )
			{
				/* check if file is already opening. 		add T.Tazaki 2005/04/15 >>> */
				if( Trace_fp != NULL )
				{
					/* trace file close */
					fclose( Trace_fp );
					Trace_fp = NULL;
				}
				/* add T.Tazaki 2005/04/15 >>> */
				g_iTraceOutput = TRACE_WINDOW;	/* Trace data output to display */
			}
			else
			{
				if( Trace_fp != NULL )
				{
					/* file open error */
					sprintf( szBuf,C33_ERR_FILE_OPEN, pFilename );
					c33_error (szBuf);
					return;
				}
				/* trace file open */
				if ((Trace_fp = fopen(pFilename, "w")) == NULL)
				{
					/* file open error */
					sprintf( szBuf,C33_ERR_FILE_OPEN, pFilename );
					c33_error (szBuf);
					return;
				}
				else
				{
					g_iTraceOutput = TRACE_FILE;	/* Trace data output to file */
				}
			}
		}
		else
		{
			c33_error (C33_ERR_COMMAND_INVALID);
			return;
		}
	}
}

/****************************************************************
 *	vfnAddMemAccessInfo : add status to MemAccessBuf
 *
 *	(output)g_szStatusBuf[] : trace cycle data "address r/w data cycle"
 *			g_iStatusBufCnt += 1;
 ****************************************************************/

static void 
vfnAddMemAccessInfo (unsigned long ulAddr, int iAccessType, int iDataSize, unsigned long ulData, int blNoData)
{
	char			szTmp[3];
	char			szTmp2[100];
	char			szTmp3[100];
	char			szTmp4[256];

	/*@ check if access type is read */
	if (iAccessType == READ_ACCESS) {
		/*@ concatenate "r" to buf */
		(void) strcpy (szTmp, "r");
	}
	else {
		/*@ concatenate "w" to buf */
		(void) strcpy (szTmp, "w");
	}

	switch (iDataSize) {	/* data size */

	case BYTE_DATA:

		/*@ concatenate data size to buf */
		(void) strcat (szTmp, "B");

		/*@ check if data is read */
		if (blNoData == NO) {
			/*@ make status of memory access on buf */
			(void) sprintf (szTmp2, "%08lX %s       %02lX", ulAddr, szTmp, (ulData & 0xff));
		}
		else {
			/*@ make status of memory access on buf ("--") */
			(void) sprintf (szTmp2, "%08lX %s       --", ulAddr, szTmp);
		}
		break;

	case HALF_DATA:

		/*@ concatenate data size to buf */
		(void) strcat (szTmp, "H");

		/*@ check if data is read */
		if (blNoData == NO) {
			/*@ make status of memory access on buf */
			(void) sprintf (szTmp2, "%08lX %s     %04lX", ulAddr, szTmp, (ulData & 0xffff));
		}
		else {
			/*@ make status of memory access on buf ("----") */
			(void) sprintf (szTmp2, "%08lX %s     ----", ulAddr, szTmp);
		}
		break;

	case WORD_DATA:

	   /*@ concatenate data size to buf */
		(void) strcat (szTmp, "W");

		/*@ check if data is read */
		if (blNoData == NO) {
			/*@ make status of memory access on buf */
			(void) sprintf (szTmp2, "%08lX %s %08lX", ulAddr, szTmp, ulData);
		}
		else {
			/*@ make status of memory access on buf ("--------") */
			(void) sprintf (szTmp2, "%08lX %s --------", ulAddr, szTmp);
		}
		break;
	}

	/* add Clock Count */
	(void) sprintf(szTmp3,"%s %08u", szTmp2, (unsigned int)gulCycle );

	if (g_iStatusBufCnt < STATUS_BUFFER_CNT_MAX) {
		
		if (g_iStatusBufCnt > 0) {
			sprintf( szTmp4,"                                                    %s",szTmp3 );
			(void) strcpy (g_szStatusBuf[g_iStatusBufCnt], szTmp4);
		}
		else {
			(void) strcpy (g_szStatusBuf[g_iStatusBufCnt], szTmp3);
		}

		++g_iStatusBufCnt;
	}

	return;
}


/* >>>>> change iruma m.takeishi '03.12.22 */
/****************************************************************************/
/* Format		 : static void c33_IcdTraceGetSorceCode(unsigned long ulAddr, char* pOut) */
/* Arguments																*/
/*	unsigned long ulAddr : Address											*/
/*	char* pOut			 : Buffer of source information 					*/
/*	int iSize			 : pOut's buffer size ( sizeof(pOut) )				*/
/* Return value  : void 													*/
/* Explanation	 : Get source code from address.							*/
/****************************************************************************/
/*
static void
c33_SimTraceGetSorceCode(unsigned long ulAddr, char* pOut)
*/
static void
c33_SimTraceGetSorceCode(unsigned long ulAddr, char* pOut, int iSize )
/* <<<<< change iruma m.takeishi '03.12.22 */

{
	unsigned long ulLineNo;
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];

	pOut[0] = 0;

	/* Get filename and line number */
	if (c33_FindFilename(ulAddr, cTemp, &ulLineNo) == C33_NO_ERR )
	{
		strcat(pOut, "(" );
		strcat(pOut, cTemp );
		if (strlen (pOut) < 13)
		{
			strcat(pOut, ")" );
		}
		else
		{
			pOut[13] = ')';
			pOut[14] = 0;
		}
		/* Position adjustment */
		while(strlen(pOut) < 15){
			strcat(pOut, " " );
		}
		sprintf(cTemp, "%05u ", (unsigned int)ulLineNo);
		strcat(pOut, cTemp);

		/* Get source code */
		c33_print_source_lines (ulAddr, cTemp);
		
		/* >>>>> change iruma m.takeishi '03.12.22 */
		/*
		strcat(pOut, cTemp);
		*/
		if ( iSize -1 <= strlen(pOut) + strlen(cTemp) ) {
			strncat(pOut, cTemp, iSize-1 - strlen(pOut));
		} else {
			strcat(pOut, cTemp);
		}
		/* <<<<< change iruma m.takeishi '03.12.22 */
	}
}

/*****************************************************************************
   vfnAddRegDump : add register dump to buf

 ( Dump format )

 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 AAAAAAAA AAAAAAAA AAAAAAAA 		 IL:F MO:1 DS:1 IE:1 C:1 V:1 Z:1 N:1
 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 HE:1 RM:1 LM:1 PM:1 RC:F SW:1 OC:1 SE:1 LC:1 HC:1 S:1 DE:1 ME:1 SV:1

*****************************************************************************/
static void
vfnAddRegDump ()
{				/* return void */
	unsigned short	uwRegNum;	/* register number */
	char			szTmpBuf[256];	/* tmp buf */

	char	 szStatusBuf[256];


	szStatusBuf[0] = 0;

	if( g_iTraceFormat != TRACE_REGISTER && g_iTraceFormat != TRACE_REGISTER_SOURCE )
	{
		return;
	}

	/*@ output register dump to buf */

	/*@ from R0 to R7 */
	for (uwRegNum = R0; uwRegNum <= R7; uwRegNum++) {
		/*@ output register data to szTmpBuf */
		(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

		/*@ concatenate szTmpBuf to buf */
		(void) strcat (szStatusBuf, szTmpBuf);
	}

	/*@ concatenate "\n" to buf */
	if( Trace_fp != NULL )
	{
		(void) strcat (szStatusBuf, "\n");
	}

	c33_PutsTraceString( szStatusBuf );

	/*@ from R8 to R15 */
	szStatusBuf[0] = 0;
	for (uwRegNum = R8; uwRegNum <= R15; uwRegNum++) {
		/*@ output register data to szTmpBuf */
		(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

		/*@ concatenate szTmpBuf to buf */
		(void) strcat (szStatusBuf, szTmpBuf);
	}

	/*@ concatenate "\n" to buf */
	if( Trace_fp != NULL )
	{
		(void) strcat (szStatusBuf, "\n");
	}
	c33_PutsTraceString( szStatusBuf );

	/*@ for SP, ALR, AHR */
	(void) sprintf (szStatusBuf, " %08lX %08lX %08lX",
										   ulReg[17]		/* SP */
										  ,ulReg[AHR]
										  ,ulReg[ALR] );


	/*@ format PSR & output to szTmpBuf */
	vfnFormatPSR (ulReg[PSR], szTmpBuf);

	/*@ concatenate szTmpBuf to buf */
	(void) strcat (szStatusBuf, szTmpBuf);

	/*@ concatenate "\n" to buf */
	if( Trace_fp != NULL )
	{
		(void) strcat (szStatusBuf, "\n");
	}
	c33_PutsTraceString( szStatusBuf );

	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{

		/*---------------------------------------------*/
		/* Advanced Mode Register					   */
		/*---------------------------------------------*/

		/*@ for Advancde Spesial Registers( LCO,LSA,LEA,SOR,TTBR,DP,USP,SSP ) */
		szStatusBuf[0] = 0;
		for (uwRegNum = LCO; uwRegNum <= SSP; uwRegNum++) {
			if ( uwRegNum != DBBR && uwRegNum != IDIR )	/* add T.Tazaki 2004/03/10 bug fix */
			{
				/*@ output register data to szTmpBuf */
				(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

				/*@ concatenate szTmpBuf to buf */
				(void) strcat (szStatusBuf, szTmpBuf);
			}
		}

		/*@ concatenate "\n" to buf */
		if( Trace_fp != NULL )
		{
			(void) strcat (szStatusBuf, "\n");
		}
		if( g_iTraceDump == TM_ON ){	/* trace output mode ON ? */
			c33_PutsTraceString( szStatusBuf );
		}

		/*@ format PSR & output to szTmpBuf */
		vfnFormatPSR_adv (ulReg[PSR], szTmpBuf);

		/*@ concatenate szTmpBuf to buf */
		(void) strcpy (szStatusBuf, szTmpBuf);

		/*@ concatenate "\n" to buf */
		if( Trace_fp != NULL )
		{
			(void) strcat (szStatusBuf, "\n");
		}
		c33_PutsTraceString( szStatusBuf );
	}
}
/*****************************************************************************
   vfnAddRegDump_Save_ext : add register dump to buf for PE

 ( Dump format )

 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 AAAAAAAA AAAAAAAA AAAAAAAA 		 IL:F MO:1 DS:1 IE:1 C:1 V:1 Z:1 N:1
 AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA
 HE:1 RM:1 LM:1 PM:1 RC:F SW:1 OC:1 SE:1 LC:1 HC:1 S:1 DE:1 ME:1 SV:1

*****************************************************************************/
static void
vfnAddRegDump_Save_ext ( char *szStatusBuf )
{				/* return void */
	unsigned short	uwRegNum;	/* register number */
	char			szTmpBuf[256];	/* tmp buf */


	szStatusBuf[0] = 0;

	if( g_iTraceFormat != TRACE_REGISTER && g_iTraceFormat != TRACE_REGISTER_SOURCE )
	{
		return;
	}

	/*@ output register dump to buf */

	/*@ from R0 to R7 */
	for (uwRegNum = R0; uwRegNum <= R7; uwRegNum++) {
		/*@ output register data to szTmpBuf */
		(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

		/*@ concatenate szTmpBuf to buf */
		(void) strcat (szStatusBuf, szTmpBuf);
	}

	(void) strcat (szStatusBuf, "\n");

	/*@ from R8 to R15 */
	for (uwRegNum = R8; uwRegNum <= R15; uwRegNum++) {
		/*@ output register data to szTmpBuf */
		(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

		/*@ concatenate szTmpBuf to buf */
		(void) strcat (szStatusBuf, szTmpBuf);
	}

	(void) strcat (szStatusBuf, "\n");

	/*@ for SP, ALR, AHR */
	(void) sprintf (szTmpBuf, " %08lX %08lX %08lX",
										   ulReg[17]		/* SP */
										  ,ulReg[AHR]
										  ,ulReg[ALR] );


	(void) strcat (szStatusBuf, szTmpBuf);

	/*@ format PSR & output to szTmpBuf */
	vfnFormatPSR (ulReg[PSR], szTmpBuf);

	/*@ concatenate szTmpBuf to buf */
	(void) strcat (szStatusBuf, szTmpBuf);

	if( Trace_fp != NULL )
	{
		(void) strcat (szStatusBuf, "\n");
	}

	/* add 2005/04/14 >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )	
	{

		if( Trace_fp == NULL )
		{
			(void) strcat (szStatusBuf, "\n");
		}

		/*---------------------------------------------*/
		/* Advanced Mode Register					   */
		/*---------------------------------------------*/

		/*@ for Advancde Spesial Registers( LCO,LSA,LEA,SOR,TTBR,DP,USP,SSP ) */
		for (uwRegNum = LCO; uwRegNum <= SSP; uwRegNum++) {
			if ( uwRegNum != DBBR && uwRegNum != IDIR )	/* add T.Tazaki 2004/03/10 bug fix */
			{
				/*@ output register data to szTmpBuf */
				(void) sprintf (szTmpBuf, " %08lX",  ulReg[uwRegNum]);

				/*@ concatenate szTmpBuf to buf */
				(void) strcat (szStatusBuf, szTmpBuf);
			}
		}

		/*@ concatenate "\n" to buf */
		(void) strcat (szStatusBuf, "\n");

		/*@ format PSR & output to szTmpBuf */
		vfnFormatPSR_adv (ulReg[PSR], szTmpBuf);

		/*@ concatenate szTmpBuf to buf */
		(void) strcat (szStatusBuf, szTmpBuf);

		if( Trace_fp != NULL )
		{
			(void) strcat (szStatusBuf, "\n");
		}

	}
	/* add 2005/04/14 <<< */
}

/************************************************
 *	vfnFormatPSR : format PSR to output *
 ************************************************/

static void 
vfnFormatPSR (unsigned long ulPSR, char *pszTmpBuf)

{				/* return void */
	int 			ndxFlag;	/* index for tmp buf */
	unsigned long	ulTmp[9];	/* tmp buf for flag & Interrupt Level */

	/*@ extract every flag */
	for (ndxFlag = 0; ndxFlag < 8; ndxFlag++) {
		/*@ extract flag */
		ulTmp[ndxFlag] = ulPSR & 1;

		/*@ prepare next flag */
		ulPSR >>= 1;
	}

	/*@ extract interrupt level */
	ulTmp[ndxFlag] = ulPSR & 0xf;	/* ulTmp = 0 - 0xf */

	/*@ add formatted PSR to buf */
	(void) sprintf (pszTmpBuf,
			"          IL:%1lX MO:%1lX DS:%1lX IE:%1lX C:%1lX V:%1lX Z:%1lX N:%1lX",
			ulTmp[8], ulTmp[7], ulTmp[6], ulTmp[4],
			ulTmp[3], ulTmp[2], ulTmp[1], ulTmp[0]);
}


/************************************************
 *	vfnFormatPSR_adv : format PSR to output *
 ************************************************/

static void 
vfnFormatPSR_adv (unsigned long ulPSR, char *pszTmpBuf)

{				/* return void */
	int 			ndxFlag;	/* index for tmp buf */
	unsigned long	ulTmp[32];	 /* tmp buf for flag & Interrupt Level */
	unsigned long	ulRc;
	
	ulRc = (ulPSR & 0x0f000000UL) >> 24;

	/*@ extract every flag */
	ulPSR >>= 12;
	for (ndxFlag = 0; ndxFlag < 20; ndxFlag++) {
		
		/*@ extract flag */
		ulTmp[ndxFlag] = ulPSR & 1;

		/*@ prepare next flag */
		ulPSR >>= 1;
	}

	/*@ add formatted PSR to buf */
	(void) sprintf (pszTmpBuf,
			" HE:%1lX RM:%1lX LM:%1lX PM:%1lX RC:%1lX SW:%1lX OC:%1lX SE:%1lX LC:%1lX HC:%1lX S:%1lX DE:%1lX ME:%1lX SV:%1lX",
			ulTmp[19], ulTmp[18], ulTmp[17], ulTmp[16], ulRc,	  ulTmp[10], ulTmp[9],	ulTmp[8],	
			ulTmp[5],  ulTmp[4],  ulTmp[3],  ulTmp[2], ulTmp[1],  ulTmp[0]);

}

/****************************************************************************/
/* Format		 : void c33_SimCall(unsigned long ulStartAddr, unsigned long ulArg1  */
/*									unsigned long ulArg2, unsigned long ulArg3	 */
/* Arguments																*/
/*	unsigned long ulStartAddr : Function address							*/
/*	unsigned long ulArg1	  : Argument1									*/
/*	unsigned long ulArg2	  : Argument2									*/
/*	unsigned long ulArg3	  : Argument3									*/
/* Return value  : void 													*/
/* Explanation	 : call subroutine and result output.						*/
/****************************************************************************/
void
c33_SimCall(unsigned long ulStartAddr, unsigned long ulArg1,unsigned long ulArg2, unsigned long ulArg3)
{
	int iStatus;
	unsigned long ulSaveReg[50];
	unsigned long ulReturnPacketAddr = 0;
	unsigned long ulAddr;
	unsigned long ulSize = 0;
	unsigned long i;
	unsigned long ul_loop;
	int 	regno;
	unsigned char ucData;
	unsigned long ulPC, ulR12, ulR13, ulR14;
	unsigned short usTmpData;
	int iFind = 0;
	char	szData[0x400];
	unsigned long ulDataTopAddr;
	unsigned char ucChar;		/* add T.Tazaki 2004/09/28 */
	FILE* fpCallfile = NULL;

	/* Search "jp %r15" */

	ulStartAddr &= 0xfffffffeUL;
	
	for( ulAddr = ulStartAddr; ulAddr <= 0xfffffffeUL; ulAddr += 2 )
	{
		gdbsim_xfer_inferior_memory(ulAddr, (char *)&usTmpData, 2, 0, NULL, NULL);
		if( usTmpData == 0x068f )	/* "jp %r15" ? */
		{
			iFind = 1;
			break;
		}
	}
	if( iFind == 1 )
	{
		/* soft break set */
		usTmpData = SOFT_BREAK_INST;
		gdbsim_xfer_inferior_memory(ulAddr, (char *)&usTmpData, 2, 1, NULL, NULL);

		g_iResume = TRUE;

		/* ALL Registers save */
		for (regno = R0; regno <= PC; regno++)
		{
			ulSaveReg[regno] = ulReg[regno];
		}
		
		 /* Set PC, %r12, %r13, %r14 */
		(void) gdbsim_store_register (PC, ulStartAddr);
		(void) gdbsim_store_register (R12, ulArg1);
		(void) gdbsim_store_register (R13, ulArg2);
		(void) gdbsim_store_register (R14, ulArg3);

		/* Program Start */
	/*	  iStatus = fnGo(0);	*/	
		iStatus = fnGo(2);		/* change T.Tazaki 2005/04/15  2== called by "c33 call" command */
		
		/* Get return packet address( %r10 ) */
		ulReturnPacketAddr = gdbsim_fetch_register (R10);


		/* ALL Registers reload */
		for (regno = R0; regno <= PC; regno++)
		{
			 ulReg[regno] = ulSaveReg[regno];
		}
		
		/* "jp %r15" set back */
		usTmpData = 0x068f;
		gdbsim_xfer_inferior_memory(ulAddr, (char *)&usTmpData, 2, 1, NULL, NULL);

		if( iStatus == 0 )	/* No Error ? */
		{
			if( ulReturnPacketAddr != 0xffffffffUL )
			{
				if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
				{
					/* output to file */
					fpCallfile = fopen(c33TargetInfo.cCallFileName, "wb");
					if (fpCallfile == NULL)
					{
						/* file open error */
						c33_error (C33_ERR_FILE_OPEN, c33TargetInfo.cCallFileName);
						return;
					}
				}

				/* Get Packet size */
				gdbsim_xfer_inferior_memory(ulReturnPacketAddr, (char *)&ulSize, 4, 0, NULL, NULL);


				/* Set Data Top address */
				ulDataTopAddr = ulReturnPacketAddr + 4;

				/* Data Output loop */
				while( 1 )
				{
					if( ulSize >= 0x400 ){
						ul_loop = 0x400;
						ulSize -= 0x400;
					}else{
						ul_loop = ulSize;
						ulSize = 0;
					}

					gdbsim_xfer_inferior_memory(ulDataTopAddr, (char *)szData, ul_loop, 0, NULL, NULL);
					ulDataTopAddr += ul_loop;		//update read address
					
					/* Get Packet data */
					for ( i = 0; i < ul_loop; ++i )
					{
					/* No ANK, change to '.'	add T.Tazaki 2004/09/28 >>> */
						ucChar = (unsigned char)szData[i];
						if( (ucChar >= 0x00 && ucChar <= 0x1f) || 
						    (ucChar >= 0x7f && ucChar <= 0xa0) ||
						    (ucChar >= 0xe0 && ucChar <= 0xff) )
						{
							if( ucChar != 0x0a )	/* add T.Tazaki 2005/05/11 : for Multi line data */
							{
								szData[i] = '.';
							}
						}
						/* add T.Tazaki 2004/09/28 <<< */

						/* output return string */
						if( c33TargetInfo.iCallMode == 1 || c33TargetInfo.iCallMode == 3 )
						{
							/* output to window */
							printf_filtered( "%c",szData[i] );
						}
						if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
						{
							/* output to file */
							if (fputc(szData[i] , fpCallfile) == EOF)
							{
								fclose( fpCallfile );
								c33_error ( C33_ERR_FILE_WRITE);
								return;
							}
						}
					}
					if( ulSize == 0 ){
						break;	//exit while
					}
				}

				/* output return string */
				if( c33TargetInfo.iCallMode == 1 || c33TargetInfo.iCallMode == 3 )
				{
					/* output to window */
					puts_filtered( "\n" );
				}
				if( c33TargetInfo.iCallMode == 2 || c33TargetInfo.iCallMode == 3 )
				{
					if (fpCallfile != NULL)
					{
						fclose( fpCallfile );
					}
				}
			}
		}
		
		g_iResume = FALSE;

		/* update registers information  */
		flush_cached_frames ();
		registers_changed ();
		stop_pc = read_pc ();
		set_current_frame (create_new_frame (read_fp (), stop_pc));
		select_frame (get_current_frame (), 0);

		sim_Gdbtk_Update ();

	}
}

/****************************************************************************/
/* Format		 : void c33_SimInt(int iNmi, int iType, int iLevel) 		*/
/* Arguments																*/
/*	int iNmi	 : NMI flag = 1 NMI, = 0 Not NMI							*/
/*	int iType	 : Interrupt type 0 - 0x7fff(32767) 						*/
/*	int iLevel	 : Interrupt level 0 - 15									*/
/* Return value  : void 													*/
/* Explanation	 : NMI and maskable external interrupt simulation.			*/
/****************************************************************************/
void
c33_SimInt(int iNmi, int iType, int iLevel)
{
	unsigned long	ulIL;
	unsigned long	ulTTBR;
	unsigned long	ulAddress;
	unsigned long	ulVectorAddr;
	unsigned long	ulTmpData;
	unsigned long	ulPSR;
	int iStatus = 0;
	
	if( iType > 0x7fff )
	{
		return;
	}
	
#if 0	/* bug fix : del T.Tazaki 2005/02/01 */

	/*@ push PC to stack */
	iStatus |= fnPush (PC);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
			puts_filtered( "Break by accessing no map.\n" );
			return;
		}
		if ((iStatus & ROM_DATA_BREAK) != 0) {
			puts_filtered( "Break by writing ROM area.\n" );
			return;
		}
		if ((iStatus & STACK_OVERFLOW ) != 0) {
			puts_filtered( "Break by stack overflow.\n" );
			return;
		}
	}

	/*@ push PSR to stack */
	iStatus |= fnPush (PSR);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
			puts_filtered( "Break by accessing no map.\n" );
			return;
		}
		if ((iStatus & ROM_DATA_BREAK) != 0) {
			puts_filtered( "Break by writing ROM area.\n" );
			return;
		}
		if ((iStatus & STACK_OVERFLOW ) != 0) {
			puts_filtered( "Break by stack overflow.\n" );
			return;
		}
	}

#endif
	/*===================*/
	/* Interrupt Process */
	/*===================*/

	ulPSR = gdbsim_fetch_register( PSR );

	if( iNmi == 1 )
	{
		/* NMI Interrupt */

		/* bug fix : add T.Tazaki 2006/11/20 >>> */
		/*@ push PC to stack */
		iStatus |= fnPush (PC);

		/*@ check nomap_mem_break or rom_data_break */
		if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
			if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
				puts_filtered( "Break by accessing no map.\n" );
				return;
			}
			if ((iStatus & ROM_DATA_BREAK) != 0) {
				puts_filtered( "Break by writing ROM area.\n" );
				return;
			}
			if ((iStatus & STACK_OVERFLOW ) != 0) {
				puts_filtered( "Break by stack overflow.\n" );
				return;
			}
		}

		/*@ push PSR to stack */
		iStatus |= fnPush (PSR);

		/*@ check nomap_mem_break or rom_data_break */
		if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
			if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
				puts_filtered( "Break by accessing no map.\n" );
				return;
			}
			if ((iStatus & ROM_DATA_BREAK) != 0) {
				puts_filtered( "Break by writing ROM area.\n" );
				return;
			}
			if ((iStatus & STACK_OVERFLOW ) != 0) {
				puts_filtered( "Break by stack overflow.\n" );
				return;
			}
		}
		/* bug fix : add T.Tazaki 2006/11/20 <<< */

		/* offset set */
		ulAddress = 28;  /* NMI vector address */
		
	}
	else
	{
		/* external Interrupt */
		
		/* check PSR:IE  */
		if( ulPSR & IE_FLAG )			
		{

			/* bug fix : add T.Tazaki 2006/11/24 >>> */
			/* compare interrupt level */
			ulIL = ulPSR & IL_BITS;
			ulIL >>= 8;
			if( ulIL < iLevel )
			{
			/* bug fix : add T.Tazaki 2006/11/24 <<< */

				/* bug fix : add T.Tazaki 2005/02/01 >>> */
				/*@ push PC to stack */
				iStatus |= fnPush (PC);

				/*@ check nomap_mem_break or rom_data_break */
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
					if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
						puts_filtered( "Break by accessing no map.\n" );
						return;
					}
					if ((iStatus & ROM_DATA_BREAK) != 0) {
						puts_filtered( "Break by writing ROM area.\n" );
						return;
					}
					if ((iStatus & STACK_OVERFLOW ) != 0) {
						puts_filtered( "Break by stack overflow.\n" );
						return;
					}
				}

				/*@ push PSR to stack */
				iStatus |= fnPush (PSR);

				/*@ check nomap_mem_break or rom_data_break */
				if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
					if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
						puts_filtered( "Break by accessing no map.\n" );
						return;
					}
					if ((iStatus & ROM_DATA_BREAK) != 0) {
						puts_filtered( "Break by writing ROM area.\n" );
						return;
					}
					if ((iStatus & STACK_OVERFLOW ) != 0) {
						puts_filtered( "Break by stack overflow.\n" );
						return;
					}
				}

				/* bug fix : add T.Tazaki 2005/02/01 <<< */

			/* bug fix : del T.Tazaki 2006/11/24 >>> */
//			/* compare interrupt level */
//			ulIL = ulPSR & IL_BITS;
//			ulIL >>= 8;
//			if( ulIL < iLevel )
//			{
			/* bug fix : del T.Tazaki 2006/11/24 <<< */
				/* offset set */
				ulAddress = (0x40 + iType * 4);

				/* PSR:IL update */
				ulPSR &= ~IL_BITS;
				ulIL = iLevel;
				ulIL <<= 8;
				ulPSR |= ulIL;

			}
			else
			{
				return;
			}
		}
		else
		{
			return;
		}
	}
	ulPSR &= ~SV_FLAG;
	ulPSR &= ~IE_FLAG;
	gdbsim_store_register ( PSR, ulPSR );

//	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV || fnCheckAdvancedMode() == C33_PRODUCT_PE )	/* add PE  T.Tazaki 2004/08/05 */
	{
		/* ADVENCED MACRO or PE */
		ulTTBR = gdbsim_fetch_register( TTBR );
		
		/* Vector address = base + 0x40 + type * 4byte */
		ulTTBR += ulAddress;
		(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	else
	{
		/* STANDARD MACRO */
		(void)gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulVectorAddr, 4, 0, NULL, NULL);	
		/* Vector address = base + 0x40 + type * 4byte */
		ulVectorAddr += ulAddress;
		(void)gdbsim_xfer_inferior_memory(ulVectorAddr, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	
	/*@ set PC */
	(void) gdbsim_store_register (PC, ulTmpData);

	sim_Gdbtk_Update ();

}

/****************************************************************************/
/* Format		 : void c33_SimClock(char *szClock) 						*/
/* Arguments																*/
/*	char *szClock			  : cycle string pointer						*/
/* Return value  : void 													*/
/* Explanation	 : Get clock												*/
/****************************************************************************/
void
c33_SimClock(char *szClock)
{

	sprintf( szClock," %10u",(unsigned int)gulCycle );

}
/****************************************************************************/
/* Format		 : int fnGo( int iretd_flag )								*/
/* Arguments	 : int iret_flag =0 normal,=1 ret.d execute ,=2 c33 call	*/
/* Return value 															*/
/*	int iStatus  : Instruction execute return flag. 						*/
/* Explanation	 : output to display or file.								*/
/****************************************************************************/
static int
fnGo( int iretd_flag )
{
	unsigned long  ulAddr;
	unsigned long  ulPSR;
	unsigned short ulCode;
	unsigned long  ulPC;
	unsigned long  ulPC_temp;
	int 		   iStatus = 0;
	int 		   i_sim_Gdbtk_Update_cnt = 0;
	int 		   iLoop;
	char		   szDisasm[256];			  /* for mnemonic	  */
	char		   szStatusBuf[5000];
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	char		   szSourceLine[256];		  // for Line & Source String //
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	char		   szTmp[100];
	int 		   iRet;
	int 		   iMacFlag;
	int 		   iretd_cnt = 0;
	extern struct DelaySlot stDelaySlot;	  /* for Delayed slot */

	/* >>>>> add iruma m.takeishi '04.03.25 */
	int iIsLcoDown_Loop;
	unsigned long ulTmpPsr;
	int iLSA_HitForLoop = 0;
	struct c33_map_info *stpMapInfo = NULL;
	/* <<<<< add iruma m.takeishi '04.03.25 */

  /* >>>>> add iruma m.takeishi '04.03.29 */
  int			 iLSA_Hit = 0;
  int			 iRepeatInst = 0;
  /* <<<<< add iruma m.takeishi '04.03.29 */
  unsigned short usCode;					/* PE code check  add T.Tazaki 2004/08/05 */
  unsigned long ulExtExceptPC = 0;			/* PE except address		  add T.Tazaki 2005/02/21 */
  int			iExtExceptCnt = 0;			/* PE except ext inst counter add T.Tazaki 2005/02/21 */
  int			iExtExceptFlg = 0;			/* PE ext exception flag	  add T.Tazaki 2005/02/22 */
  int  			iStatusForTrace;			/* add T.Tazaki 2005/03/15 */
  unsigned long ulAddrForTrace;				/* add T.Tazaki 2005/03/15 */
	extern Tcl_Interp *gdbtk_interp;		/* bug fix  add T.Tazaki 2008/06/18 [DDTS gnu33v32-00026] */

//	extern unsigned long ulReg[];

	/* bug fix  add T.Tazaki 2008/06/18 [DDTS gnu33v32-00026] >>> */
	if( g_use_windows == 1 )  /* GUI Mode ? */
	{
		Tcl_Eval( gdbtk_interp, "c33_toolbar_disabled" ); /* gdbtcl/srcbar.itcl */
	}
	/* bug fix  add T.Tazaki 2008/06/18 [DDTS gnu33v32-00026] <<< */

	stDelaySlot.blUsed = NO;

	g_iStop_button = 0;			/* add T.Tazaki 2008/05/27 */

	/* add T.Tazaki 2004/11/10 >>> */
	giKeybreak = 0;
	c33TargetInfo.iKeyBreak = 0;
	/* add T.Tazaki 2004/11/10 <<< */

	do
	{

		if(giKeybreak == 1){
			puts_filtered (C33_MSG_KEY_BREAK);	/* add T.Tazaki 2008/05/27 */
			puts_filtered ("\n");				/* add T.Tazaki 2008/05/27 */
			g_iStop_button = 1;					/* add T.Tazaki 2008/05/27 */
			break;
		}

		if( i_sim_Gdbtk_Update_cnt == 0 )
		{
			c33_Gdbtk_Idle ();
			i_sim_Gdbtk_Update_cnt = 2000;
		}
		else
		{
			--i_sim_Gdbtk_Update_cnt;
		}
		
		/*@ check if instruction is delayed */
		if (stDelaySlot.blUsed == YES) {
			/*@ code is in delayed slot */
			ulAddr = stDelaySlot.ulAddr;

			/*@ you execute delayed instruction */
			stDelaySlot.blRunDelayed = YES;
			
		}
		else {
			/*@ code is in memory */
			ulAddr = gdbsim_fetch_register(PC);

			/*@ you don't execute delayed instruction */
			stDelaySlot.blRunDelayed = NO;
		}

		/* check if ulAddr is hard pc break1 */
		if( c33TargetInfo.ulHardPcBreak[0] != C33_BREAK_DISABLE ){
			if( c33TargetInfo.ulHardPcBreak[0] == ulAddr ){
				puts_filtered( "Break by hard pc break1.\n" );
				break;
			}
		}
		/* check if ulAddr is hard pc break2 */
		if( c33TargetInfo.ulHardPcBreak[1] != C33_BREAK_DISABLE ){
			if( c33TargetInfo.ulHardPcBreak[1] == ulAddr ){
				puts_filtered( "Break by hard pc break2.\n" );
				break;
			}
		}

		stDelaySlot.ulAddr = ulAddr + 2;		/* next address save */

		/* >>>>> change iruma m.takeishi '04.03.25 */
		/*
		gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulCode, 2, 0, NULL, NULL);
		*/
		/****	reads two instruction if bus-size is 32bits, ****/
		/****	and word geometry.							 ****/
	
		/* is word geometry? */
		iRet = 0;
		if ( stC33_CachedCode.iCached == 1 ) {
			if ( stC33_CachedCode.ulPc == ulAddr ) {
				ulCode = (unsigned long)(stC33_CachedCode.usCode);
				iRet = -1;
			}
			stC33_CachedCode.iCached = 0;
		}
	
		if ( iRet != -1 ) {
			iRet = 0;
			if ( ulAddr % 4 == 0 ) {
				/* is 32bit bus? */
				stpMapInfo = c33_GetMapInfo( ulAddr );
				if ( stpMapInfo != NULL ) {
					
					if ( stpMapInfo->ulDeviceSize == C33_WORD ) {
						iRet = 1;
					}
				}
			}
			if ( iRet == 0 ) {
				/* normal fetch */
				g_iExtEachInst = TRUE;
				gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulCode, 2, 0, NULL, NULL);
				g_iExtEachInst = FALSE;
			} else {
				
				/* read 2 inst */
				g_iExtEachInst = TRUE;
				gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulData, 4, 0, NULL, NULL);
				g_iExtEachInst = FALSE;
				
				/* change about Endian */
				iRet = 0; /* litle */
				if ( stpMapInfo != NULL ) {
					if ( stpMapInfo->iEndian == C33_BIG ) {
						iRet = 1; /* Big */
					}
				}
				
				if ( iRet == 0 ) {
					stC33_CachedCode.usCode = (unsigned short)(ulData >> 16);
					ulCode = (unsigned short)(0xffff & ulData);
				} else {
					stC33_CachedCode.usCode = (unsigned short)(0xffff & ulData);
					ulCode = (unsigned short)( 0xffff & (ulData >> 16) );
				}
				stC33_CachedCode.ulPc = ulAddr + 2;
				stC33_CachedCode.iCached = 1;
			}
		}
		/* <<<<< change iruma m.takeishi '04.03.25 */

		/* disp exe code */
		
		/*@ disassemble */
		g_iTraceMode = 1;		 /* ld.w %r0,0x2a (xld.w %r0,0xffffffea) --> (ld.w %r0,0x2a):Not extend */
		c33_Disasm ((unsigned short)ulCode, szDisasm);
		g_iTraceMode = 0;
		
		(void) strcat (szDisasm, "                                   ");
		szDisasm[28] = 0;

		(void) sprintf (szStatusBuf, "%08ld %08lX %04X  %s",
				g_ulInstNum , ulAddr, (int) ulCode, szDisasm);

		g_iStatusBufCnt = 0;	/* for r/w trace */
		ulAddrForTrace = ulAddr;	/* for trace  add T.Tazaki 2005/03/15 */


		/* >>>>> add iruma m.takeishi '04.03.24 */
		if ( iRsvLoop_instCnt != 0 ) {
		
			/* checks PC=LEA. If current is ext-inst, ignored */
			/* Below is copy of LM decrement routine. */
			iRet = 1;
			if ( iLSA_HitForLoop == 1 && stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_NONE ) {
				iRet = 0;
			} else {
				/*@ current PC is equal to LEA */
				if (ulAddr == ulReg[LEA]) {
					
					if ( ( stExtension.iType == EXT_NONE ) &&
						 ( stExtension.iTypeExt == EXT_NONE ) ) {
						iRet = 0;
					}
				}
			}
			if ( iRet == 0 ) {
				/* error */
				puts_filtered( "Too near between LM set and LEA.\n" );
				{ extern int stop_after_trap;
				 stop_after_trap = 1;
				}
			  break;
			}
	
			/* if current inst, which is next to the inst setting LM, is ext-inst,	
			   ext inst is not counted. */
			if ( iRsvLoop_instCnt != RESERVELOOP_INSTCNT || 
				 fnGetExtType( (unsigned short)ulCode ) == EXT_NONE ) { 
				 iRsvLoop_instCnt--;
			}
		}
		/* <<<<< add iruma m.takeishi '04.03.24 */

		/*==================================================================*/
		/* PE undefined instruction interrupt	add T.Tazaki 2004/08/05 >>> */
		/*==================================================================*/
		if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
		{
			usCode = ulCode & 0xff00;
			if( usCode == 0x8a00 || usCode == 0x8e00 || usCode == 0x9600 || usCode == 0xb200 ) /* scan0,scan1,mirror,mac ? */
			{
#if 0	/* del T.Tazaki 2005/03/28 */
				/*   add T.Tazaki 2005/03/15 >>> */
				if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */
					(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulAddrForTrace);
					c33_TraceDump( ulAddrForTrace, szStatusBuf );
				}
				/* add T.Tazaki 2005/03/15 <<< */
#endif
				iStatus = fnPush (PC);
				if( iStatus == 0 )
				{
					iStatus = fnPush (PSR);
					if( iStatus == 0 )
					{
						/* %psr IE = 0 */
						ulPSR = gdbsim_fetch_register (PSR);
						ulPSR &= 0xffffffef;
					   	gdbsim_store_register( PSR, ulPSR );
				
						/* %idir udi set */
						ulData = gdbsim_fetch_register (IDIR);
						ulData = (ulData & 0xffff0000) + (ulCode & 0x0000ffff);
						gdbsim_store_register( IDIR, ulData );

						/* Jump to undefined instruction exception interrupt */
						ulAddr = gdbsim_fetch_register (TTBR);
						ulAddr += 12;
						gdbsim_xfer_inferior_memory(ulAddr, (char *)&ulData, 4, 0, NULL, NULL);
						gdbsim_store_register( PC, ulData );

#if 0	/* del T.Tazaki 2005/03/28 */
						/* add T.Tazaki 2005/03/15 >>> */
						g_ulInstNum++;
						if( g_iTraceDump == TM_ON ){
							vfnAddRegDump ();
						}
						/* add T.Tazaki 2005/03/15 <<< */
#endif
						/* add T.Tazaki 2005/03/28 >>> */
						if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */
							(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulAddrForTrace);
							c33_TraceDump( ulAddrForTrace, szStatusBuf );
							vfnAddRegDump ();
						}
						g_ulInstNum++;
						/* add T.Tazaki 2005/03/28 <<< */

						continue;
					}
				}
				if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
					puts_filtered( "Break by accessing no map.\n" );
					break;
				}
				if ((iStatus & ROM_DATA_BREAK) != 0) {
					puts_filtered( "Break by writing ROM area.\n" );
					break;
				}
				if ((iStatus & STACK_OVERFLOW ) != 0) {
					puts_filtered( "Break by stack overflow.\n" );
					break;
				}
			}
			/* add T.Tazaki 2005/02/21 >>> */
			
			/****************************/
			/* ext exception process	*/
			/****************************/
			
			usCode = ulCode & 0xe000;
			if( usCode == 0xc000 )	/* ext ? */
			{
				if( iExtExceptCnt == 2 )
				{
					iExtExceptCnt = 0;
					iExtExceptFlg = 1;
				}
				else
				{
					if( iExtExceptCnt == 0 )
					{
						ulExtExceptPC = gdbsim_fetch_register( PC ); /* update ext start address */
						g_ulExtStartPC = ulExtExceptPC;				 /*  add T.Tazaki 2005/03/29 */
						g_iExtStartPC_Flag = 1;						 /* used in fnZeroDiv(),fnIlleAddr() */
					}
					++iExtExceptCnt;
				}
			}
			else
			{
				iExtExceptCnt = 0;
				iExtExceptFlg = 0;
			
			}
			/* add T.Tazaki 2005/02/21 <<< */
		}
		/* PE undefined instruction interrupt	add T.Tazaki 2004/08/05 <<< */
		/* STD/ADV undefined instruction interrupt	 add T.Tazaki 2005/03/29 >>> */
		else
		{
			/****************************/
			/* ext exception process	*/
			/****************************/
			
			usCode = ulCode & 0xe000;
			if( usCode == 0xc000 )	/* ext ? */
			{
				if( iExtExceptCnt == 2 )
				{
					iExtExceptCnt = 0;
					iExtExceptFlg = 1;
				}
				else
				{
					if( iExtExceptCnt == 0 )
					{
						ulExtExceptPC = gdbsim_fetch_register( PC ); /* update ext start address */
						g_ulExtStartPC = ulExtExceptPC;				 /*  add T.Tazaki 2005/03/29 */
						g_iExtStartPC_Flag = 1;						 /* used in fnZeroDiv(),fnIlleAddr() */
					}
					++iExtExceptCnt;
				}
			}
			else
			{
				iExtExceptCnt = 0;
				iExtExceptFlg = 0;
			}
		}
		/* STD/ADV undefined instruction interrupt	 add T.Tazaki 2005/03/29 <<< */

		/*==========================================================*/
		/* Instruction execute										*/
		/*==========================================================*/
		
		/* check if PE ext exception? 	add T.Tazaki 2005/02/22 */
		if( iExtExceptFlg == 0 )
		{
			/*==========================================================*/
			/* Instruction execute										*/
			/*==========================================================*/
			g_iExtEachInst = TRUE;

			iStatus = fnExeEachInst( (unsigned short)ulCode);

			g_iExtEachInst = FALSE;

		}
		else
		{
			/* PE only */
			iStatus = fnExtException(ulExtExceptPC);  /* jump to ext exception vector address : add T.Tazaki 2005/02/22 */
			iExtExceptFlg = 0;
			iExtExceptCnt = 0;		/* add T.Tazaki 2005/03/16 */
			g_iExtExceptCnt = 0;	/*  add T.Tazaki 2005/03/16 */

			/*  add T.Tazaki 2005/03/15 >>> */
			if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */
				(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulExtExceptPC);
				c33_TraceDump( ulExtExceptPC, szStatusBuf );
			}
			/*@ count up instruction number */
			g_ulInstNum++;
			if( g_iTraceDump == TM_ON ){
				vfnAddRegDump ();
			}
			/* add T.Tazaki 2005/03/15 <<< */


			continue;								/* add T.Tazaki 2005/03/07 */
		}

		iStatusForTrace = iStatus;									/* add T.Tazaki 2005/03/15 */

		/*==========================================================*/
		/* Error status check										*/
		/*==========================================================*/

		/* >>>>> add iruma m.takeishi '04.03.24 */
		iIsLcoDown_Loop = 1;
		/* this routine is copy of LM=1 */
		if ( stExtension.iTypeExtCond != EXTCOND_SKIP ) {
	
			if ( (ulReg[ PSR ] & LM_FLAG) != 0 ) {
				iRet = 1;
				/* was PC hit i past by ext-inst ? */
				if ( iLSA_HitForLoop == 1 && stExtension.iType == EXT_NONE && stExtension.iTypeExt == EXT_NONE ) {
					iRet = 0;
					iLSA_HitForLoop = 0;
				} else {
				/*@ current PC is equal to LEA */
					if (ulAddr == ulReg[LEA]) {
					
						if ( ( stExtension.iType != EXT_NONE ) ||
							 ( stExtension.iTypeExt != EXT_NONE ) ) {
							iLSA_HitForLoop = 1;
						} else {
							iRet = 0;
						}
					}
				}
				iIsLcoDown_Loop = iRet;
				if ( iRet == 0 ) {
					if ( ulReg[LCO] == 0 ) {
						/* down LM_FLAG */
						ulTmpPsr = gdbsim_fetch_register( PSR );
						ulTmpPsr &= ~LM_FLAG;
						gdbsim_store_register( PSR, ulTmpPsr );
					}
				}
			}
		}
		/* <<<<< add iruma m.takeishi '04.03.24 */

		/* Temporary break ? add T.Tazaki 2003/10/14 >>> */
		ulPC = gdbsim_fetch_register( PC );
		if ( g_temp_brk == 1 && g_ulTemp_brk_addr == ulPC )
		{
			if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
			{
			   printf_filtered( "<fnGo> break by temprary. address=[%x]\n",(int)ulPC);
			}
			break;
		}
		/* Temporary break ? add T.Tazaki 2003/10/14 <<< */

		if((iStatus & DATA_BREAK ) != 0 )
		{
			/* Watch point display */ /* add T.Tazaki 2003/08/01 >>> */
			
			if( g_iWatchBreakpointDisableFlag == 1 )	/* SIM watch + step or next mode ? */
			{
				c33_GetBreakPoint2();	 /* get all breakpoint infomation */ /* add 2003/08/06 T.Tazaki */
			}
			else
			{
				c33_GetBreakPoint();	/* get all breakpoint infomation */
			}
			
			if( g_WriteWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware write watchpoint %d: %s\n\n",g_WatchNum,g_What );
			}
			else if( g_ReadWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware read watchpoint %d: %s\n\n",g_WatchNum,g_What );
			}
			else if( g_WriteReadWatchBreakAddrFlag == 1 )
			{
				printf_filtered( "Hardware access (read/write) watchpoint %d: %s\n\n",g_WatchNum,g_What );
			}
			/* add T.Tazaki 2003/08/01 >>> */
			break;
		}
		if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
			puts_filtered( "Break by accessing no map.\n" );
			g_iExtExceptCnt = 0;	/*  add T.Tazaki 2005/03/16 */
			break;
		}
		if ((iStatus & ROM_DATA_BREAK) != 0) {
			puts_filtered( "Break by writing ROM area.\n" );
			g_iExtExceptCnt = 0;	/*  add T.Tazaki 2005/03/16 */
			break;
		}
		if ((iStatus & STACK_OVERFLOW ) != 0) {
			puts_filtered( "Break by stack overflow.\n" );
			g_iExtExceptCnt = 0;	/* add T.Tazaki 2005/03/16 */
			break;
		}
		if ((iStatus & SLP_HALT_BREAK) != 0) {
			puts_filtered( "Break by sleep or halt.\n" );
			g_iExtExceptCnt = 0;	/* add T.Tazaki 2005/03/16 */
			break;
		}
		if((iStatus & S_ADDR_EX ) != 0) 		/* illigal address exception ? */
		{
			/* add T.Tazaki 2005/03/29 >>>  */
			c33_TraceDump_Output_ext();	
			g_iExtExceptCnt = 0;
			/* add T.Tazaki 2005/03/29 <<< */
			iStatus = fnIlleAddr(); /* jump Illegal address exception vector address */ /* add T.Tazaki 2005/03/15 */
		}
		if((iStatus & S_ZERO_EX ) != 0) 		/* Zero division exception ? */
		{
			/* add T.Tazaki 2005/03/29 >>>  */
			c33_TraceDump_Output_ext();	
			g_iExtExceptCnt = 0;
			/* add T.Tazaki 2005/03/29 <<< */
			iStatus = fnZeroDiv();	/* jump zero division exception vector address */ /* add T.Tazaki 2005/03/15 */
		}

		g_iExtStartPC_Flag = 0;		/* B  add T.Tazaki 2005/04/15 */
	

		if((iStatus & ILL_INST_BREAK ) != 0)	/* illegal instruction ? */
		{

			if( ulCode != SOFT_BREAK_INST ){			/* Not Soft Break ? "brk" */
				puts_filtered( "Illegal instruction.\n" );
				g_iExtExceptCnt = 0;			/*  2005/04/15 */
				{ 
				  extern int stop_after_trap;
				  stop_after_trap = 1;
				}
				break;
			}
			else
			{
				/* add T.Tazaki 2005/04/15 check if CALLED BY "c33 call" command >>> */
				if( iretd_flag == 2 )
				{
					iStatus = 0;
					break;
				}
				/* add T.Tazaki 2005/04/15 <<< */
				
				/* add T.Tazaki 2005/03/28 >>> */
				/* simulated i/o process */
				iRet = fnStdio();
				if( iRet == 1 )
				{
					/* add T.Tazaki 2005/06/29 >>> */
					ulPC =  read_pc();
					
				    if( c33TargetInfo.ulSoftPcBreak[C33_STDIN_BREAK_NUM-1] == ulPC )
			        {
				        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDIN_BREAK_NUM-1];
					}
					else
					{
				        usCode = c33TargetInfo.usSoftPcBreakOrgCode[C33_STDOUT_BREAK_NUM-1];
					}
					gdbsim_xfer_inferior_memory(ulPC, (char *)&usCode, 2, 1, NULL, NULL);

					/* STEP */
					do
					{
						iStatus = fnExeEachInst( usCode);
						if ( fnGetExtType( usCode ) == EXT_NONE ) // not all ext
						{
							break;
						}
						ulAddr = gdbsim_fetch_register(PC);
						gdbsim_xfer_inferior_memory(ulAddr, (char *)&usCode, 2, 0, NULL, NULL);
					}while( 1 );

			        usCode = SOFT_BREAK_INST;
					gdbsim_xfer_inferior_memory(ulPC, (char *)&usCode, 2, 1, NULL, NULL);
					continue;
					/* add T.Tazaki 2005/06/29 <<< */
//					vfnIncPC (CODE);		/* Stdio soft break : PC += CODE */ /* del T.Tazaki 2005/06/29 */
				}
				else
				{
					break;
				}
				/* add T.Tazaki 2005/03/28 <<< */
			}
		}

		/*==========================================================*/
		/* Trace													*/
		/*==========================================================*/
		if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */

			/* Zero devide or address exceotion, inst code is "----"		add T.Tazaki 2005/03/15 >>> */
			if( (iStatusForTrace & S_ZERO_EX ) != 0 ||		   /* Zero division exception ? */
				(iStatusForTrace & S_ADDR_EX ) != 0 )		   /* illigal address exception ? */
			{
				(void) sprintf (szStatusBuf, "%08ld %08lX ----                              ", g_ulInstNum , ulAddr);
			}

			/* add T.Tazaki 2005/03/29 >>> */
			if( ( ulCode & 0xe000 ) == 0xc000 )	/* ext imm13 ? */
			{				
				c33_TraceDump_Save_ext( ulAddr, szStatusBuf );			/* T.Tazaki 2005/04 */
				strcpy( g_szStatusBufForExtExcept[g_iExtExceptCnt] , szStatusBuf );
				++g_iExtExceptCnt;
			}
			else
			{
				if ((iStatusForTrace & 0x3ffff) == 0 || (iStatusForTrace & DELAY_FLAG) == DELAY_FLAG)	/* No error break ? or DELAYED */
				{
					c33_TraceDump_Output_ext();	
					g_iExtExceptCnt = 0;
				}

				c33_TraceDump( ulAddr, szStatusBuf );
			}
			/* add T.Tazaki 2005/03/29 <<< */

		}
		
		/*@ count up instruction number */
		g_ulInstNum++;

		/*==========================================================*/
		/* Delayed Instruntion										*/
		/*==========================================================*/
		
		if ((iStatus & DELAY_FLAG) == DELAY_FLAG)
		{
			/*@ record that next code is read from delayed slot */
			stDelaySlot.blUsed = YES;

			/*@ mask delay flag */
			iStatus &= ~DELAY_FLAG;

			/*@ record next address of ".d" instruction */
			stDelaySlot.ulNewPC = gdbsim_fetch_register(PC);
			
		}
		else
		{
			/*@ record that next code is not read from delayed slot */
			stDelaySlot.blUsed = NO;
		}

		/*@ check if this instruction is delayed */
		if (stDelaySlot.blRunDelayed == YES) {
			/*@ restore address that previous branch instruction set */
			(void) gdbsim_store_register (PC, stDelaySlot.ulNewPC);
			
		}

		/*==========================================================*/
		/* PSR : LM Flag check										*/
		/*==========================================================*/

		/* >>>>> change iruma m.takeishi '04.03.24 */
		/*
		if ((ulReg[PSR] & LM_FLAG) != 0) {

			//@ current PC is equal to LEA //
			if (ulAddr == ulReg[LEA]) {
		*/
		if ((ulReg[PSR] & LM_FLAG) != 0 || iIsLcoDown_Loop == 0 ) {
			if ( iIsLcoDown_Loop == 0 ) {
		/* <<<<< change iruma m.takeishi '04.03.24 */

				if (ulReg[LCO] > 0) {
					
					/* >>>>> change iruma m.takeishi '04.03.24 */
					/*
					(void) gdbsim_store_register (PC, ulReg[LSA]);
					//@ if LCO > 0, load LSA to PC //
					*/
					/*@ if LCO > 0, load LSA to PC */
					if ( ((iStatus & S_ZERO_EX		  ) ==0) && 
						 ((iStatus & S_ADDR_EX		  ) ==0) && 
						 ((ulCode  & 0xfffc) != 0x480 ) /* int imm2 */ ) {
						(void) gdbsim_store_register (PC, ulReg[LSA]);
					}
					/* <<<<< change iruma m.takeishi '04.03.24 */

					/*@ decrement of LCO */
					(void) gdbsim_store_register (LCO, --ulReg[LCO]);
				}
				else {
					/*@ LM flag clear */
					ulReg[PSR] &= ~LM_FLAG;
					(void) gdbsim_store_register (PSR, ulReg[PSR]);
					
					/* >>>>> add iruma m.takeishi '04.03.24 */
					iIsLoop_int = 0;
					/* <<<<< add iruma m.takeishi '04.03.24 */
					
				}
			}
		}

		/*==========================================================*/
		/* PSR : RM Flag check										*/
		/*==========================================================*/

		else if ((ulReg[PSR] & RM_FLAG) != 0) {

			ulPC = gdbsim_fetch_register( PC );

			/* check first insn after "repeat" insn ? */
			if( iRepeatFirst == 1 )
			{
				/* >>>>> change iruma m.takeishi '04.03.29 */
				/*
				ulPC_temp = ulPC;
				*/
				// This is not Reserve-repeat mode.
				iRepeatInst = 1;
				iRet = 0;
				/* <<<<< change iruma m.takeishi '04.03.29 */
				iRepeatFirst = 0;
			}
			else
			{
				/* >>>>> change iruma m.takeishi '04.03.29 */
				/* Fixed for reserved repeat oparation */
				/*
				ulPC_temp = ulPC - CODE;
				*/
				iRet = 1;
				if ( iLSA_Hit == 0 ) {
					if ( ulPC - CODE == ulReg[LSA] ) {
					
						if ( iRepeatInst != 1 &&
							( ( stExtension.iType != EXT_NONE ) ||
							 ( stExtension.iTypeExt != EXT_NONE ) ) ) {
							 // if not repeat inst, and 
							 //  now reserve-repeat mode, executing "ext" inst.
							iLSA_Hit = 1;  // LSA hit, ext inst!
							iRet = 0;	   // do next inst
						}
						
					} else {
						// repeat mode is not started.
						iRet = 0;
					}
				} else {
					// Now Reserve-repeat mode
					if ( ( stExtension.iType != EXT_NONE ) ||
						 ( stExtension.iTypeExt != EXT_NONE ) ) {
						iRet = 0;
					} else {
						iLSA_Hit = 0;
					}
				}
				/* <<<<< change iruma m.takeishi '04.03.29 */
			}

			/* >>>>> change iruma m.takeishi '04.03.29 */
			/*
			if( ulPC_temp == ulReg[LSA] )
			*/
			if ( iRet == 1 )
			/* <<<<< change iruma m.takeishi '04.03.29 */
			{
				if (ulReg[LCO] > 0) {
					
					/*@ load LSA to PC for repeat */
					(void) gdbsim_store_register (PC, ulReg[LSA]);

					/*@ decrement of LCO */
					(void) gdbsim_store_register (LCO, --ulReg[LCO]);
				}
				else
				{
					/*@ RM flag clear */
					ulReg[PSR] &= ~RM_FLAG;
					(void) gdbsim_store_register (PSR, ulReg[PSR]);
					/* >>>>> add iruma m.takeishi '04.03.29 */
					iRepeatInst = 0;
					/* <<<<< add iruma m.takeishi '04.03.29 */
				}
			}
		}
		
		/*@ register dump, if necessary */
		if( g_iTraceDump == TM_ON )
		{
			if( ( ulCode & 0xe000 ) == 0xc000 )	/* ext imm13 ? */
			{
				vfnAddRegDump_Save_ext ( szStatusBuf );	/*  2005/04/14 */
				strcpy( g_szStatusBufForExtExcept[g_iExtExceptCnt] , szStatusBuf );
				++g_iExtExceptCnt;
			}
			else
			{
				vfnAddRegDump ();
			}
		}

		/* ret.d execute mode ? */
		if(iretd_flag == 1 )
		{
			++iretd_cnt;
			if( iretd_cnt == 2 )
			{
				break;	/* exit loop */
			}
		}

	}while( 1 );

	/* bug fix  add T.Tazaki 2008/06/18 [DDTS gnu33v32-00026] >>> */
	if( g_use_windows == 1 )  /* GUI Mode ? */
	{
		Tcl_Eval( gdbtk_interp, "c33_toolbar_enabled" ); /* gdbtcl/srcbar.itcl */
	}
	/* bug fix  add T.Tazaki 2008/06/18 [DDTS gnu33v32-00026] <<< */

	return iStatus;
}

/****************************************************************************/
/* Format		 : void c33_PutsTraceString( char *szString )				*/
/* Arguments																*/
/*	char* szString		 : Buffer of trace data 							*/
/* Return value  : void 													*/
/* Explanation	 : output to display or file.								*/
/****************************************************************************/
static
void c33_PutsTraceString( char *szString )
{
	extern Tcl_Interp *gdbtk_interp;

	if( g_iTraceOutput == TRACE_WINDOW )
	{
		if( g_use_windows == 1 )	 /* GUI mode ? */	/* add T.Tazaki 2003/07/08 */
		{
			/* output to trace window */
			Tcl_SetVar( gdbtk_interp,"c33_trace_string",szString,TCL_GLOBAL_ONLY );
			Tcl_Eval( gdbtk_interp, "c33_trace_out" );
		}
		/* >>>>> add iruma m.takeishi '03.09.09 */
		else {
			printf_filtered( "%s\n", szString );
		}
		/* <<<<< add iruma m.takeishi '03.09.09 */
	}
	else
	{
		if( Trace_fp != NULL )
		{
			fprintf( Trace_fp,"%s",szString );
		}
	}
}

/****************************************************************************/
/* Format		 : static int fnZeroDiv(void)								*/
/* Arguments																*/
/*	void																	*/
/* Return value  : status													*/
/* Explanation	 : Jump zero devision vector.								*/
/****************************************************************************/
static
int fnZeroDiv()
{
	unsigned long	ulTTBR;
	unsigned long	ulVectorAddr;
	unsigned long	ulTmpData;
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '04.03.26 */
	unsigned long	ulData;
	/* <<<<< add iruma m.takeishi '04.03.26 */
	int iStatus;
	
	iStatus = 0;

	vfnInitExtension();	/* add T.Tazaki 2005/03/18 */

	/* >>>>> add iruma m.takeishi '04.03.26 */
	ulPSR = gdbsim_fetch_register( PSR );
	if ( ulPSR & SV_FLAG ) {
		/* changes to SV mode */
		ulData = ulPSR & ~SV_FLAG;
		gdbsim_store_register( PSR, ulData );
	}
	/* <<<<< add iruma m.takeishi '04.03.26 */

	/* AhXsO߂extߕtH 			add T.Tazaki 2005/03/29 >>> */
	if( g_iExtStartPC_Flag == 1 )
	{
		iStatus |= fnPush_ExtExceptPC (g_ulExtStartPC);	
		g_iExtStartPC_Flag = 0;	/* flag clear */
	}
	else
	{
		/* add T.Tazaki 2005/03/29 <<< */
		/*@ push PC to stack */
		iStatus |= fnPush (PC);
	}

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/* >>>>> add iruma m.takeishi '04.03.26 */
		gdbsim_store_register( PSR, ulPSR );
		/* <<<<< add iruma m.takeishi '04.03.26 */
		return iStatus;
	}

	/*@ push PSR to stack */
	iStatus |= fnPush (PSR);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/* >>>>> add iruma m.takeishi '04.03.26 */
		gdbsim_store_register( PSR, ulPSR );
		/* <<<<< add iruma m.takeishi '04.03.26 */
		return iStatus;
	}

	/* >>>>> add iruma m.takeishi '04.03.26 */
	if ( ulPSR & SV_FLAG ) {
		/* rewrite new PSR */
	   g_iExtEachInst = FALSE;
	   gdbsim_xfer_inferior_memory( gdbsim_fetch_register( SSP ),
							(char *)&ulPSR, 4, 1, NULL, NULL);
	   g_iExtEachInst = TRUE;
	}
	/* <<<<< add iruma m.takeishi '04.03.26 */


	ulPSR = gdbsim_fetch_register( PSR );
	ulPSR &= ~SV_FLAG;
	ulPSR &= ~IE_FLAG;
	/* >>>>> add iruma m.takeishi '04.03.26 */
	ulPSR &= ~LM_FLAG;
	ulPSR &= ~RM_FLAG;
	ulPSR &= ~PM_FLAG;
	/* <<<<< add iruma m.takeishi '04.03.26 */
	gdbsim_store_register ( PSR, ulPSR );


//	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV || fnCheckAdvancedMode() == C33_PRODUCT_PE )	/* add PE  T.Tazaki 2004/08/05 */
	{
		/* ADVENCED MACRO or PE */
		ulTTBR = gdbsim_fetch_register( TTBR );
		
		ulTTBR += 16;
		(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	else
	{
		/* STANDARD MACRO */
		(void)gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulVectorAddr, 4, 0, NULL, NULL);	
		ulVectorAddr += 16;
		(void)gdbsim_xfer_inferior_memory(ulVectorAddr, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	
	/*@ set PC */
	(void) gdbsim_store_register (PC, ulTmpData);

	return(iStatus);
}

/* add T.Tazaki 2005/02/21 >>> */

/****************************************************************************/
/* Format		 : static int fnExtException(unsigned long ulExeptPC)		*/
/* Arguments																*/
/*	unsigned long ulExeptPC : ext except first ext PC address.				*/
/* Return value  : status													*/
/* Explanation	 : ext exception vector for C33PE.							*/
/****************************************************************************/
static
int fnExtException(unsigned long ulExeptPC)
{
	unsigned long	ulTTBR;
	unsigned long	ulTmpData;
	unsigned long	ulPSR;
	unsigned long	ulData;
	int iStatus;
	
	iStatus = 0;

	vfnInitExtension();	/* add T.Tazaki 2005/03/18 */

	ulPSR = gdbsim_fetch_register( PSR );/* add  T.Tazaki 2007/07/12 */

	/*@ push PC to stack */
	iStatus |= fnPush_ExtExceptPC (ulExeptPC);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		gdbsim_store_register( PSR, ulPSR );
		return iStatus;
	}

	/*@ push PSR to stack */
	iStatus |= fnPush (PSR);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		gdbsim_store_register( PSR, ulPSR );
		return iStatus;
	}

	ulPSR = gdbsim_fetch_register( PSR );
	ulPSR &= ~IE_FLAG;
	gdbsim_store_register ( PSR, ulPSR );

	ulTTBR = gdbsim_fetch_register( TTBR );
		
	ulTTBR += 8;
	(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
	
	/*@ set PC */
	(void) gdbsim_store_register (PC, ulTmpData);

	return(iStatus);
}

/************************************************
 *	fnPush_ExtExceptPC : push PC to stack		*
 ************************************************/
static int 
fnPush_ExtExceptPC (unsigned long ulExtExeptPC)
{				/* return iStatus */
	int 			iStatus;
	unsigned long	ulAddr;
	int 			iRet = 1;	 // default is NG.

	struct c33_map_info *pstMapInfo;
	

	/* Stack overflow check */
	ulAddr = gdbsim_fetch_register (SP) - 4;
	
	if( c33TargetInfo.pMapInfo != NULL ){
		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			if( pstMapInfo->iMapAttr == C33_STACK )
			{
				iRet = 2;
				if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
				{
					iRet = 0;
					break;
				}
			}
			
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
		
		if ( iRet == 2 ) {
			return STACK_OVERFLOW;
		}
	}

	/*@ change stack pointer */
	(void) gdbsim_store_register (SP, (gdbsim_fetch_register (SP) - 4));

	iStatus = gdbsim_xfer_inferior_memory (gdbsim_fetch_register (SP),	
			   (char *)&ulExtExeptPC, 4, 1, NULL, NULL);

	return (iStatus);		/* return status */
}

/* add T.Tazaki 2005/02/21 <<< */

/****************************************************************************/
/* Format		 : static int fnIlleAddr(void)								*/
/* Arguments																*/
/*	void																	*/
/* Return value  : status													*/
/* Explanation	 : Jump illegal address access vector.						*/
/****************************************************************************/
static
int fnIlleAddr()
{
	unsigned long	ulTTBR;
	unsigned long	ulVectorAddr;
	unsigned long	ulTmpData;
	unsigned long	ulPSR;
	/* >>>>> add iruma m.takeishi '04.03.26 */
	unsigned long	ulData;
	/* <<<<< add iruma m.takeishi '04.03.26 */
	int iStatus;
	
	iStatus = 0;

	vfnInitExtension();	/* add T.Tazaki 2005/03/18 */
	
//	  vfnIncPC (CODE);	  /* PC += CODE */	/* del 2003/11/05 T.Tazaki */

	/* >>>>> add iruma m.takeishi '04.03.26 */
	ulPSR = gdbsim_fetch_register( PSR );
	if ( ulPSR & SV_FLAG ) {
		/* changes to SV mode */
		ulData = ulPSR & ~SV_FLAG;
		gdbsim_store_register( PSR, ulData );
	}
	/* <<<<< add iruma m.takeishi '04.03.26 */

	/* address align error inst is ext ?		add T.Tazaki 2005/03/29 >>> */
	if( g_iExtStartPC_Flag == 1 )
	{
		iStatus |= fnPush_ExtExceptPC (g_ulExtStartPC);
		g_iExtStartPC_Flag = 0;	/* flag clear */
	}
	else
	{
		/* add T.Tazaki 2005/03/29 <<< */
		/*@ push PC to stack */
		iStatus |= fnPush (PC);
	}

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/* >>>>> add iruma m.takeishi '04.03.26 */
		gdbsim_store_register( PSR, ulPSR );
		/* <<<<< add iruma m.takeishi '04.03.26 */
		return iStatus;
	}

	/*@ push PSR to stack */
	iStatus |= fnPush (PSR);

	/*@ check nomap_mem_break or rom_data_break */
	if ((iStatus & (NOMAP_MEM_BREAK | ROM_DATA_BREAK | STACK_OVERFLOW)) != 0) {
		/* >>>>> add iruma m.takeishi '04.03.26 */
		gdbsim_store_register( PSR, ulPSR );
		/* <<<<< add iruma m.takeishi '04.03.26 */
		return iStatus;
	}

	/* >>>>> add iruma m.takeishi '04.03.26 */
	if ( ulPSR & SV_FLAG ) {
		/* rewrite new PSR */
	   g_iExtEachInst = FALSE;
		
	   gdbsim_xfer_inferior_memory( gdbsim_fetch_register( SSP ),
							(char *)&ulPSR, 4, 1, NULL, NULL);
		
	   g_iExtEachInst = TRUE;
	}
	/* <<<<< add iruma m.takeishi '04.03.26 */


	ulPSR = gdbsim_fetch_register( PSR );
	ulPSR &= ~SV_FLAG;
	ulPSR &= ~IE_FLAG;
	/* >>>>> add iruma m.takeishi '04.03.26 */
	ulPSR &= ~LM_FLAG;
	ulPSR &= ~RM_FLAG;
	ulPSR &= ~PM_FLAG;
	/* <<<<< add iruma m.takeishi '04.03.26 */
	gdbsim_store_register ( PSR, ulPSR );

//	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	if( fnCheckAdvancedMode() == C33_PRODUCT_ADV || fnCheckAdvancedMode() == C33_PRODUCT_PE )	/* add PE  T.Tazaki 2004/08/05 */
	{
		/* ADVENCED MACRO or PE */
		ulTTBR = gdbsim_fetch_register( TTBR );
		
		ulTTBR += 24;
		(void)gdbsim_xfer_inferior_memory(ulTTBR, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	else
	{
		/* STANDARD MACRO */
		(void)gdbsim_xfer_inferior_memory(C33_ADDR_TTBR, (char *)&ulVectorAddr, 4, 0, NULL, NULL);	
		ulVectorAddr += 24;
		(void)gdbsim_xfer_inferior_memory(ulVectorAddr, (char *)&ulTmpData, 4, 0, NULL, NULL);
	}
	
	/*@ set PC */
	(void) gdbsim_store_register (PC, ulTmpData);

	return(iStatus);
}
/****************************************************************************/
/* Format		 : void c33_SimFinishCommand(unsigned long ulStartPC)       */
/* Arguments	 : ulStartPC												*/
/* Return value  : void 													*/
/* Explanation	 : finish command. called by infcmd.c						*/
/****************************************************************************/
#define INSN_RET	  0x0640
#define INSN_RET_D	  0x0740
#define INSN_RETI	  0x04c0

void
c33_SimFinishCommand(unsigned long ulStartPC)
{
//	unsigned long ulStartPC;	/* del T.Tazaki 2005/05/17 */
	unsigned long ulCurrentPC;
	unsigned long ulCurrentPC2; /* add T.Tazaki 2005/05/17 */
	unsigned long ulPC;
	unsigned short insn_word;
	unsigned short usTmpData;
	unsigned short usInsn[510];
	int iCondition;
	int iResult;
	int iStatus;
	int i;
	int iSoftPcBreakFlag = 0;
	int iHardPcBreakFlag = 0;
	int iID;
	int iType = 0;
	int iAddr = 0;
	extern struct BreakData stBreakData;

	c33_GetBreakPoint();

	/* Soft PC Break Count Check */
	if( g_SoftPcBreakTable_Index > C33_SOFT_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return;
	}
	/* Hard PC Break Count Check */
	if( g_HardPcBreakTable_Index > C33_HARD_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_HARD_MAX);
		return;
	}

//	ulCurrentPC = ulStartPC = gdbsim_fetch_register (PC);	/* del T.Tazaki 2005/05/17 */
	ulCurrentPC = ulStartPC;								/* add T.Tazaki 2005/05/17 */

	/* Get Instruction Code on current PC */
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
	
	if (insn_word != INSN_RET && insn_word != INSN_RET_D && insn_word != INSN_RETI)
	{
		/* Search Return code */
		c33_InitCancel();	/* bug fix : add 2004/06/22 T.Tazaki : Check Escape Key ON	*/

		while(1)
		{
			if( ulCurrentPC >= 0xfffffffe )
			{
				return; /* address over ==> NOP */
			}
			ulCurrentPC += 2;
			insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
			if (insn_word == INSN_RET || insn_word == INSN_RET_D || insn_word == INSN_RETI)
			{
				/* found ! */
				break;
			}
			/* bug fix : add 2004/06/22 T.Tazaki : Check Escape Key ON	>>>*/
			if( c33TargetInfo.iKeyBreak == 1 || c33_CheckCancel( 0x40000 ) == 1 ) 
			{
				printf_filtered( C33_MSG_KEY_BREAK );
				printf_filtered( "\n" );
				c33_EndCancel();   
				return;
			}
			/* bug fix : add 2004/06/22 T.Tazaki : Check Escape Key ON	<<<*/
		}
		
		c33_EndCancel();   /* bug fix : add 2004/06/22 T.Tazaki : Check Escape Key ON  */
		
		/* add T.Tazaki 2005/05/17 >>> */
		ulCurrentPC2 = gdbsim_fetch_register (PC);
		if( ulStartPC != ulCurrentPC2 )
		{
			ulStartPC = ulCurrentPC2;
		}
		/* add T.Tazaki 2005/05/17 <<< */		
		
		/*==========================================*/
		/* All Breakpoint Insert					*/
		/*==========================================*/

		/* Soft PC Break Insert */

		for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
		{
			if( i < C33_SOFT_PC_BREAK_MAX )
			{
				if( ulStartPC != g_SoftPcBreakTable[ i ] )
				{
					if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
					{
						usInsn[ i ] = read_memory_unsigned_integer (g_SoftPcBreakTable[ i ], 2);
						usTmpData = SOFT_BREAK_INST;
						gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usTmpData, 2, 1, NULL, NULL);
					}
					else
					{
						iSoftPcBreakFlag = 1;
					}
				}
			}
		}

		/* Hard PC Break Insert */
		for( i = 0; i < g_HardPcBreakTable_Index; ++i )
		{
			if( ulStartPC != g_HardPcBreakTable[ i ] )
			{
				if( ulCurrentPC != g_HardPcBreakTable[ i ] )
				{
					if ( i == 0 )
					{
						/* Set hard PC break 1 */
						c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
					}
					else
					{
						/* Set hard PC break 2 */
						c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
					}
				}
				else
				{
					iHardPcBreakFlag = 1;
					if ( i == 0 )
					{
						/* Set hard PC break 1 */
						c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
					}
					else
					{
						/* Set hard PC break 2 */
						c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
					}
				}
			}
		}

		/* Write Watch Break Insert */
		if( g_WriteWatchBreakAddrFlag == 1 || g_ReadWatchBreakAddrFlag == 1 || g_WriteReadWatchBreakAddrFlag == 1 )
		{
			if( g_WriteWatchBreakAddrFlag == 1 )
			{
				stBreakData.iType = WRITE_ACCESS;	/* WRITE */
				stBreakData.ulAddr = g_WriteWatchBreakAddr;
			}
			else if( g_ReadWatchBreakAddrFlag == 1 )
			{
				stBreakData.iType = READ_ACCESS;	/* READ */
				stBreakData.ulAddr = g_ReadWatchBreakAddr;
			}
			else if( g_WriteReadWatchBreakAddrFlag == 1 )
			{
				stBreakData.iType = BOTH_ACCESS;  /* READ/WRITE */
				stBreakData.ulAddr = g_WriteReadWatchBreakAddr;
			}
		}
		else
		{
			stBreakData.iType = NO_ACCESS;
		}
		
		
		/* Temporary Break Set */
		usTmpData = SOFT_BREAK_INST;
		gdbsim_xfer_inferior_memory(ulCurrentPC, (char *)&usTmpData, 2, 1, NULL, NULL);

		/*==========================================*/
		/* GO to ret address						*/
		/*==========================================*/
		g_iResume = TRUE;

		/* Program Start */
		fnGo(0);

		g_iResume = FALSE;
		
		/* original data set */
		gdbsim_xfer_inferior_memory(ulCurrentPC, (char *)&insn_word, 2, 1, NULL, NULL);

		/*==========================================*/
		/* All Breakpoint Remove					*/
		/*==========================================*/
		/* Soft PC Break Remove */
		for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
		{
			if( i < C33_SOFT_PC_BREAK_MAX )
			{
				if( ulStartPC != g_SoftPcBreakTable[ i ] )
				{
					if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
					{
						gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usInsn[ i ], 2, 1, NULL, NULL);
					}
				}
			}
		}
		/* Hard PC Break Remove */
		c33TargetInfo.ulHardPcBreak[0] = C33_BREAK_DISABLE;
		c33TargetInfo.ulHardPcBreak[1] = C33_BREAK_DISABLE;

		/* Write Watch Break Remove */
		stBreakData.iType = NO_ACCESS;
		stBreakData.ulAddr = C33_BREAK_DISABLE;
	}

	ulPC = read_pc();
	
	/* delete temporary break infomation */
	c33_delete_temp_breakpoint( ulPC );

	if( g_iStop_button == 0 )	/* add T.Tazaki 2008/05/27 */
	{
		/*======================================*/
		/* Return Execute						*/
		/*======================================*/
		if( ulCurrentPC == ulPC )
		{
			if( iSoftPcBreakFlag == 0 && iHardPcBreakFlag == 0 ) /* Break by Temprary break ? */
			{
				if(giKeybreak == 0) /* No STOP ? */
				{
					/* STEP */
					if (insn_word != INSN_RET_D )
					{
						/* ret or reti */
						iStatus = fnExeEachInst_trace( insn_word );
					}
					else
					{
						/* ret.d */
						/* GO */
						g_iResume = TRUE;

						/* Program Start ret.d mode */
						fnGo(1);

						g_iResume = FALSE;
					}
				}
				else
				{
					giKeybreak = 0;
				}
			}
		}
	}

	/* refresh window */
	giFrameClear = 0;

	/* update registers information  */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

	sim_Gdbtk_Update ();

}

#define INSN_CALL_RB	 0x0600
#define INSN_CALL_RB_D	 0x0700
#define INSN_CALL_IMM	 0x1c00
#define INSN_CALL_IMM_D  0x1d00
#define INSN_JP_RB		 0x0680
#define INSN_JP_RB_D	 0x0780
#define INSN_JP_IMM 	 0x1e00
#define INSN_JP_IMM_D	 0x1f00
#define INSN_EXT		 0xc000
/****************************************************************************/
/* Format		 : int c33_SimCheckManyCall(unsigned long ulTempAddr,unsigned long ulEndAddr)*/ 
/* Arguments	 : void 													*/
/* Return value  : 0 : not many call, 1 : many call				 			*/
/* Explanation	 : search call inst bitween temp address and end address	*/
/*					sub( sub2(i),sub3(k) ); pattern							*/
/****************************************************************************/

int
c33_SimCheckManyCall(unsigned long ulTempAddr,unsigned long ulEndAddr )
{
	unsigned short insn_word;

	/* search call code */
	while( ulTempAddr < ulEndAddr )
	{
		insn_word = read_memory_unsigned_integer (ulTempAddr, 2);

		/* check if call code found */
		if ( (insn_word & 0xfff0) == INSN_CALL_RB  || (insn_word & 0xfff0) == INSN_CALL_RB_D ||
			 (insn_word & 0xff00) == INSN_CALL_IMM || (insn_word & 0xff00) == INSN_CALL_IMM_D )
		{
			return 1;
		}
		
		if( ulTempAddr <= 0xfffffffcUL )
		{
			ulTempAddr += 2;
		}
		else
		{
			return 0;
		}
	}

	return 0;
}
/****************************************************************************/
/* Format		 : int c33_SimCheckCurrentInst(void)						*/
/* Arguments	 : void 													*/
/* Return value  : 0 : not call, 1 : call and ASM, 2 : jp and C 			*/
/* Explanation	 : check if current pc code is call or call.d.				*/
/****************************************************************************/

int
c33_SimCheckCurrentInst()
{
	unsigned long ulCurrentPC;
	unsigned short insn_word;
	int iStepCount = 0;
	unsigned long ulLineNo;
	char cTemp[C33_TRACE_DATA_LINE_MAX + 1];
	int iLen;

	ulCurrentPC = gdbsim_fetch_register (PC);

	/* check if Current PC Address is in C source */

	/* Get filename and line number */
	if (c33_FindFilename(ulCurrentPC, cTemp, &ulLineNo) == C33_NO_ERR )
	{
		iLen = strlen( cTemp );
//		if( cTemp[iLen-2] == '.' && cTemp[iLen-1] == 'c' )	/* change check extend for C++ T.Tazaki 2005/04/27 */
		if( cTemp[iLen-2] == '.' && cTemp[iLen-1] != 's' )	/* change check extend for C++ T.Tazaki 2005/04/27 */
		{
			/* add T.Tazaki 2003/10/23 >>> */
			/* Get Instruction Code on current PC */
			insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);

			/* search jp code */
			do
			{
				/* check if jp code found */
				if ( (insn_word & 0xfff0) == INSN_JP_RB  || (insn_word & 0xfff0) == INSN_JP_RB_D ||
					 (insn_word & 0xff00) == INSN_JP_IMM || (insn_word & 0xff00) == INSN_JP_IMM_D )
				{
					return 2;
				}
				
				/* check if ext code found */
				if ( (insn_word & 0xe000) != INSN_EXT )
				{
					return 0;
				}
				if( ulCurrentPC <= 0xfffffffcUL )
				{
					ulCurrentPC += 2;
					insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
				}
				else
				{
					return 0;
				}
			}while(1);
			/* add T.Tazaki 2003/10/23 <<< */
		}
	}

	/* Get Instruction Code on current PC */
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);

	/* search call code */
	do
	{
		/* check if call code found */
		if ( (insn_word & 0xfff0) == INSN_CALL_RB  || (insn_word & 0xfff0) == INSN_CALL_RB_D ||
			 (insn_word & 0xff00) == INSN_CALL_IMM || (insn_word & 0xff00) == INSN_CALL_IMM_D )
		{
			return 1;
		}
		
		/* check if ext code found */
		if ( (insn_word & 0xe000) != INSN_EXT )
		{
			return 0;
		}
		if( ulCurrentPC <= 0xfffffffcUL )
		{
			ulCurrentPC += 2;
			insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
		}
		else
		{
			return 0;
		}
	}while(1);

}
/****************************************************************************/
/* Format		 : void c33_SimNextCommand(void)							*/
/* Arguments	 : void 													*/
/* Return value  : void 													*/
/* Explanation	 : next or nexti command. called by infcmd.c				*/
/****************************************************************************/

void
c33_SimNextCommand()
{
	unsigned long ulStartPC;
	unsigned long ulCurrentPC;
	unsigned long ulTempBreakPC;
	unsigned long ulPC;
	unsigned short insn_word;
	unsigned short usTmpData;
	unsigned short usSaveCode;
	unsigned short usInsn[510];
	int iCondition;
	int iResult;
	int iStatus;
	int i;
	int iSoftPcBreakFlag = 0;
	int iHardPcBreakFlag = 0;
	int iID;
	int iType = 0;
	int iAddr = 0;
	extern struct BreakData stBreakData;
	extern int g_iWatchBreakpointDisableFlag;

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "START c33_SimNextCommand()\n");
	}

	c33_GetBreakPoint();

	/* Soft PC Break Count Check */
	if( g_SoftPcBreakTable_Index > C33_SOFT_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return;
	}
	/* Hard PC Break Count Check */
	if( g_HardPcBreakTable_Index > C33_HARD_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_HARD_MAX);
		return;
	}

	ulCurrentPC = ulStartPC = gdbsim_fetch_register (PC);

	/* Skip ext code */
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);

	do
	{
		if ( (insn_word & 0xfff0) == INSN_CALL_RB  || (insn_word & 0xfff0) == INSN_CALL_RB_D ||
			 (insn_word & 0xff00) == INSN_CALL_IMM || (insn_word & 0xff00) == INSN_CALL_IMM_D )
		{
			break;
		}
		if( ulCurrentPC <= 0xfffffffcUL )
		{
			ulCurrentPC += 2;
			insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
		}
		else
		{
			return;
		}
	}while(1);

	/*	(ld.w %rs,[%sp+2])				*/
	/*	(ext)							*/
	/*	(ext)							*/
	/*	call xx 						*/
	/*	(xxxxx)    <--- temporary break */

	/*	(ld.w %rs,[%sp+2])				*/
	/*	(ext)							*/
	/*	(ext)							*/
	/*	call.d xx						*/
	/*	(xxxxx) 						*/
	/*	(xxxxx)    <--- temporary break */

	/* Get Temporary break PC			*/
	insn_word = read_memory_unsigned_integer (ulCurrentPC, 2);
	
	ulCurrentPC += 2;

	/* delayed code ? */
	if ( (insn_word & 0xfff0) == INSN_CALL_RB_D || (insn_word & 0xff00) == INSN_CALL_IMM_D )
	{
		ulCurrentPC += 2;
	}

	ulTempBreakPC = ulCurrentPC;
	
	/* Save temprary break code */

	usSaveCode = read_memory_unsigned_integer (ulTempBreakPC, 2);

	/* Temporary Break Set */
	usTmpData = SOFT_BREAK_INST;
	gdbsim_xfer_inferior_memory(ulTempBreakPC, (char *)&usTmpData, 2, 1, NULL, NULL);

	/* add T.Tazaki 2004/08/02 >>> */
	/*==========================================*/
	/* All Breakpoint Insert					*/
	/*==========================================*/

	/* Soft PC Break Insert */

	for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
	{
		if( i < C33_SOFT_PC_BREAK_MAX )
		{
			if( ulStartPC != g_SoftPcBreakTable[ i ] )
			{
				if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
				{
					usInsn[ i ] = read_memory_unsigned_integer (g_SoftPcBreakTable[ i ], 2);
					usTmpData = SOFT_BREAK_INST;
					gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usTmpData, 2, 1, NULL, NULL);
				}
				else
				{
					iSoftPcBreakFlag = 1;
				}
			}
		}
	}

	/* Hard PC Break Insert */
	for( i = 0; i < g_HardPcBreakTable_Index; ++i )
	{
		if( ulStartPC != g_HardPcBreakTable[ i ] )
		{
			if( ulCurrentPC != g_HardPcBreakTable[ i ] )
			{
				if ( i == 0 )
				{
					/* Set hard PC break 1 */
					c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
				}
				else
				{
					/* Set hard PC break 2 */
					c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
				}
			}
			else
			{
				iHardPcBreakFlag = 1;
				if ( i == 0 )
				{
					/* Set hard PC break 1 */
					c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
				}
				else
				{
					/* Set hard PC break 2 */
					c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
				}
			}
		}
	}

	/* Write Watch Break Insert */
	if( g_WriteWatchBreakAddrFlag == 1 || g_ReadWatchBreakAddrFlag == 1 || g_WriteReadWatchBreakAddrFlag == 1 )
	{
		if( g_WriteWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = WRITE_ACCESS;	/* WRITE */
			stBreakData.ulAddr = g_WriteWatchBreakAddr;
		}
		else if( g_ReadWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = READ_ACCESS;	/* READ */
			stBreakData.ulAddr = g_ReadWatchBreakAddr;
		}
		else if( g_WriteReadWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = BOTH_ACCESS;  /* READ/WRITE */
			stBreakData.ulAddr = g_WriteReadWatchBreakAddr;
		}
	}
	else
	{
		stBreakData.iType = NO_ACCESS;
	}
		
	/* add T.Tazaki 2004/08/02 <<< */

	/*==========================================*/
	/* GO to ret address						*/
	/*==========================================*/
	g_iResume = TRUE;

	/* Program Start */
	fnGo(0);

	g_iResume = FALSE;

	/* original data set */
	gdbsim_xfer_inferior_memory(ulTempBreakPC, (char *)&usSaveCode, 2, 1, NULL, NULL);

	/* add T.Tazaki 2004/08/02 >>> */
	/*==========================================*/
	/* All Breakpoint Remove					*/
	/*==========================================*/
	/* Soft PC Break Remove */
	for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
	{
		if( i < C33_SOFT_PC_BREAK_MAX )
		{
			if( ulStartPC != g_SoftPcBreakTable[ i ] )
			{
				if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
				{
					gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usInsn[ i ], 2, 1, NULL, NULL);
				}
			}
		}
	}
	/* Hard PC Break Remove */
	c33TargetInfo.ulHardPcBreak[0] = C33_BREAK_DISABLE;
	c33TargetInfo.ulHardPcBreak[1] = C33_BREAK_DISABLE;

	/* Write Watch Break Remove */
	stBreakData.iType = NO_ACCESS;
	stBreakData.ulAddr = C33_BREAK_DISABLE;
	/* add T.Tazaki 2004/08/02 <<< */
	
	/* refresh window */
	giFrameClear = 0;

	/* update registers information  */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

	sim_Gdbtk_Update ();

}
/****************************************************************************/
/* Format		 : void c33_SimUntilCommand( unsigned long ulTempBreakAddr) */
/* Arguments																*/
/*	unsigned long ulTempBreakAddr : Temprary break point					*/
/* Return value  : void 													*/
/* Explanation	 : until command.  called by breakpoint.c until_break_command  */
/****************************************************************************/

void
c33_SimUntilCommand( unsigned long ulTempBreakAddr)
{
	unsigned long ulStartPC;
	unsigned long ulCurrentPC;
	unsigned long ulPC;
	unsigned short insn_word;
	unsigned short usTmpData;
	unsigned short usInsn[510];
	unsigned short usSaveCode;		/* add T.Tazaki 2005/03/28 */
	int iCondition;
	int iResult;
	int iStatus;
	int i;
	int iSoftPcBreakFlag = 0;
	int iHardPcBreakFlag = 0;
	int iID;
	int iType = 0;
	int iAddr = 0;
	extern struct BreakData stBreakData;

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "START c33_SimUntilCommand(%x)\n",(int)ulTempBreakAddr );
	}

	c33_GetBreakPoint();

	/* Soft PC Break Count Check */
	if( g_SoftPcBreakTable_Index > C33_SOFT_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_SOFT_MAX);
		return;
	}
	/* Hard PC Break Count Check */
	if( g_HardPcBreakTable_Index > C33_HARD_PC_BREAK_MAX )
	{
		c33_error (C33_ERR_MSG_BRK_HARD_MAX);
		return;
	}

	ulCurrentPC = ulStartPC = gdbsim_fetch_register (PC);

	/* add T.Tazaki 2005/03/28 >>> J */
	/* Save temprary break code */

	usSaveCode = read_memory_unsigned_integer (ulTempBreakAddr, 2);

	/* Temporary Break Set */
	usTmpData = SOFT_BREAK_INST;
	gdbsim_xfer_inferior_memory(ulTempBreakAddr, (char *)&usTmpData, 2, 1, NULL, NULL);
	/* add T.Tazaki 2005/03/28 <<< */

	/*==========================================*/
	/* All Breakpoint Insert					*/
	/*==========================================*/

	/* Soft PC Break Insert */

	for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
	{
		if( i < C33_SOFT_PC_BREAK_MAX )
		{
			if( ulStartPC != g_SoftPcBreakTable[ i ] )
			{
				if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
				{
					usInsn[ i ] = read_memory_unsigned_integer (g_SoftPcBreakTable[ i ], 2);
					usTmpData = SOFT_BREAK_INST;
					gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usTmpData, 2, 1, NULL, NULL);
				}
				else
				{
					iSoftPcBreakFlag = 1;
				}
			}
		}
	}

	/* Hard PC Break Insert */
	for( i = 0; i < g_HardPcBreakTable_Index; ++i )
	{
		if( ulStartPC != g_HardPcBreakTable[ i ] )
		{
			if( ulCurrentPC != g_HardPcBreakTable[ i ] )
			{
				if ( i == 0 )
				{
					/* Set hard PC break 1 */
					c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
				}
				else
				{
					/* Set hard PC break 2 */
					c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
				}
			}
			else
			{
				iHardPcBreakFlag = 1;
				if ( i == 0 )
				{
					/* Set hard PC break 1 */
					c33TargetInfo.ulHardPcBreak[0] = g_HardPcBreakTable[ i ];
				}
				else
				{
					/* Set hard PC break 2 */
					c33TargetInfo.ulHardPcBreak[1] = g_HardPcBreakTable[ i ];
				}
			}
		}
	}

	/* Write Watch Break Insert */
	if( g_WriteWatchBreakAddrFlag == 1 || g_ReadWatchBreakAddrFlag == 1 || g_WriteReadWatchBreakAddrFlag == 1 )
	{
		if( g_WriteWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = WRITE_ACCESS;	/* WRITE */
			stBreakData.ulAddr = g_WriteWatchBreakAddr;
		}
		else if( g_ReadWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = READ_ACCESS;	/* READ */
			stBreakData.ulAddr = g_ReadWatchBreakAddr;
		}
		else if( g_WriteReadWatchBreakAddrFlag == 1 )
		{
			stBreakData.iType = BOTH_ACCESS;  /* READ/WRITE */
			stBreakData.ulAddr = g_WriteReadWatchBreakAddr;
		}
	}
	else
	{
		stBreakData.iType = NO_ACCESS;
	}
		
#if 0	/* del 2005/03/28  */
	/* Temporary Break Set */
	g_temp_brk = 1;
	g_ulTemp_brk_addr = ulTempBreakAddr;
#endif

	/*==========================================*/
	/* GO to ret address						*/
	/*==========================================*/
	g_iResume = TRUE;

	/* Program Start */
	fnGo(0);

	g_iResume = FALSE;
		
	/* original data set 		add T.Tazaki 2005/03/28 >>>  */
	gdbsim_xfer_inferior_memory(ulTempBreakAddr, (char *)&usSaveCode, 2, 1, NULL, NULL);
	/* add T.Tazaki 2005/03/28 <<< */

#if 0	/* del 2005/03/28  */
	/* Temporary Break Clear */
	g_temp_brk = 0;
	g_ulTemp_brk_addr = C33_INVALID_ADDR;
#endif
	/*==========================================*/
	/* All Breakpoint Remove					*/
	/*==========================================*/
	/* Soft PC Break Remove */
	for( i = 0; i < g_SoftPcBreakTable_Index; ++i )
	{
		if( i < C33_SOFT_PC_BREAK_MAX )
		{
			if( ulStartPC != g_SoftPcBreakTable[ i ] )
			{
				if( ulCurrentPC != g_SoftPcBreakTable[ i ] )
				{
					gdbsim_xfer_inferior_memory(g_SoftPcBreakTable[ i ], (char *)&usInsn[ i ], 2, 1, NULL, NULL);
				}
			}
		}
	}
	/* Hard PC Break Remove */
	c33TargetInfo.ulHardPcBreak[0] = C33_BREAK_DISABLE;
	c33TargetInfo.ulHardPcBreak[1] = C33_BREAK_DISABLE;

	/* Write Watch Break Remove */
	stBreakData.iType = NO_ACCESS;
	stBreakData.ulAddr = C33_BREAK_DISABLE;

	/* refresh window */
	giFrameClear = 0;

	if( g_iDebugRun == 1 )	/* add T.Tazaki 2005/04/27 */
	{
		printf_filtered( "END c33_SimUntilCommand\n" );
	}

#if 0

	/* update registers information  */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

	sim_Gdbtk_Update ();
#endif

#if 0 /*del T.Tazaki 2005/03/29 */

	/* update registers information  */
	gdbsim_fetch_register(-1);
	stop_pc = read_pc ();
	registers_changed ();

	giFrameClear = 1;
	c33_Gdbtk_Update (C33_UPDATE_REG);
#endif
	
	/* add T.Tazaki 2005/03/29 >>> */
	/* refresh window */
	giFrameClear = 0;

	/* update registers information  */
	flush_cached_frames ();
	registers_changed ();
	stop_pc = read_pc ();
	set_current_frame (create_new_frame (read_fp (), stop_pc));
	select_frame (get_current_frame (), 0);

	sim_Gdbtk_Update ();
	/* add T.Tazaki 2005/03/29 <<< */

}
/****************************************************************************/
/* Format		 : fnExeEachInst_trace( unsigned short insn_word )			*/
/* Arguments																*/
/*	unsigned short insn_word  : cc33 instruction code						*/
/* Return value  : 0 = OK, 1 = Error										*/
/* Explanation	 : one instruction execute. 								*/
/****************************************************************************/
static int
fnExeEachInst_trace( unsigned short insn_word )
{
	unsigned long  ulAddr;
	int 		   iLoop;
	int 		   iStatus;
	char		   szDisasm[256];			  /* for mnemonic	  */
	char		   szStatusBuf[5000];
	/* >>>>> delete iruma m.takeishi '03.12.19 */
	/*
	char		   szSourceLine[256];		  // for Line & Source String //
	*/
	/* <<<<< delete iruma m.takeishi '03.12.19 */
	char		   szTmp[100];

	/*@ code is in memory */
	ulAddr = gdbsim_fetch_register(PC);

	/*@ disassemble */
	g_iTraceMode = 1;		 /* ld.w %r0,0x2a (xld.w %r0,0xffffffea) --> (ld.w %r0,0x2a):Not extend */
	c33_Disasm ((unsigned short)insn_word, szDisasm);
	g_iTraceMode = 0;
	
	(void) strcat (szDisasm, "                                   ");
	szDisasm[28] = 0;

	(void) sprintf (szStatusBuf, "%08ld %08lX %04X  %s",
			g_ulInstNum , ulAddr, (int) insn_word, szDisasm);

	g_iStatusBufCnt = 0;	/* for r/w trace */

	/*==========================================================*/
	/* Instruction execute										*/
	/*==========================================================*/
	
	g_iExtEachInst = TRUE;

	iStatus = fnExeEachInst((unsigned short)insn_word);

	g_iExtEachInst = FALSE;

	/*==========================================================*/
	/* Error status check										*/
	/*==========================================================*/

	if ((iStatus & NOMAP_MEM_BREAK ) != 0) {
		puts_filtered( "Break by accessing no map.\n" );
		return 1;
	}
	if ((iStatus & ROM_DATA_BREAK) != 0) {
		puts_filtered( "Break by writing ROM area.\n" );
		return 1;
	}
	if ((iStatus & STACK_OVERFLOW ) != 0) {
		puts_filtered( "Break by stack overflow.\n" );
		return 1;
	}

	/*==========================================================*/
	/* Trace													*/
	/*==========================================================*/
	if( g_iTraceDump == TM_ON ){	/* trace file output mode ON ? */

		c33_Gdbtk_Idle ();

		/*==========================================================*/
		/* Trace dump string output 								*/
		/*==========================================================*/

		if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
		{
			/* Get source string */
			/* >>>>> change iruma m.takeishi '03.12.22 */
			/*
			c33_SimTraceGetSorceCode( ulAddr, szSourceLine );
			*/
			c33_SimTraceGetSorceCode( ulAddr, szSourceLine, sizeof(szSourceLine) );
			/* >>>>> change iruma m.takeishi '03.12.22 */
		}

		if (g_iStatusBufCnt == 0)
		{
			(void) strcat (szStatusBuf, "-------- -- --------");
			sprintf( szTmp," %08u",(unsigned int)gulCycle );
			(void) strcat (szStatusBuf, szTmp);
			
			if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
			{
				(void) strcat (szStatusBuf, "   " );
				/* >>>>> change iruma m.takeishi '03.12.22 */
				/*
				(void) strcat (szStatusBuf, szSourceLine);
				*/
				if ( sizeof(szStatusBuf)-1 <= strlen(szStatusBuf) + strlen(szSourceLine) ) {
					strncat( szStatusBuf, szSourceLine, sizeof(szStatusBuf)-1 - strlen(szStatusBuf)  );
					szStatusBuf[ sizeof(szStatusBuf)-1 ] = '\0';	
				} else {
					strcat( szStatusBuf, szSourceLine );
				}
				/* <<<<< change iruma m.takeishi '03.12.22 */
			}
			
			/* >>>>> change iruma m.takeishi '03.12.22 */
			/*
			(void) strcat (szStatusBuf, "\n");
			*/
			if ( sizeof( szStatusBuf)-2 >= strlen( szStatusBuf ) ) {
				strcat( szStatusBuf, "\n" );
			} else {
				
				szStatusBuf[ sizeof(szStatusBuf)-2 ] = '\n';
				szStatusBuf[ sizeof(szStatusBuf)-1 ] = '\0';
			}
			/* <<<<< change iruma m.takeishi '03.12.22 */

			/* >>>>> add iruma m.takeishi '03.12.22 */
			c33_PutsTraceString( szStatusBuf );
			/* <<<<< add iruma m.takeishi '03.12.22 */

		}
		else
		{
			for( iLoop = 0; iLoop < g_iStatusBufCnt; ++iLoop )
			{
				/* >>>>> change iruma m.takeishi '03.12.22 */
				/*
				(void) strcat (szStatusBuf, g_szStatusBuf[iLoop]);
				*/
				if ( iLoop == 0 ) {
					(void) strcat (szStatusBuf, g_szStatusBuf[iLoop]);
				} else {
					(void) strcpy (szStatusBuf, g_szStatusBuf[iLoop]);
				}
				/* <<<<< change iruma m.takeishi '03.12.22 */
				
				if( g_iTraceFormat == TRACE_SOURCE || g_iTraceFormat == TRACE_REGISTER_SOURCE )
				{
					if( iLoop == 0 )
					{
						(void) strcat (szStatusBuf, "   " );
						/* >>>>> change iruma m.takeishi '03.12.22 */
						/*
						(void) strcat (szStatusBuf, szSourceLine);
						*/
						if ( sizeof(szStatusBuf)-1 <= strlen(szStatusBuf)+ strlen( szSourceLine )  ) {
							strncat( szStatusBuf, szSourceLine, sizeof(szStatusBuf)-1 - strlen(szStatusBuf) );
							szStatusBuf[ sizeof(szStatusBuf)-1 ] = '\0';
						} else {
							strcat( szStatusBuf, szSourceLine );
						}
						/* <<<<< change iruma m.takeishi '03.12.22 */
						
					}
				}
				
				/* >>>>> change iruma m.takeishi '03.12.22 */
				/*
				(void) strcat (szStatusBuf, "\n");
				*/
				if( Trace_fp != NULL ) {
					
					if ( strlen( szStatusBuf ) >= sizeof( szStatusBuf ) -2 ) {
						szStatusBuf[ sizeof( szStatusBuf ) -2 ] = '\n';
						szStatusBuf[ sizeof( szStatusBuf ) -1 ] = '\0';
					} else {
						strcat( szStatusBuf, "\n" );
					}
				}
				/* <<<<< change iruma m.takeishi '03.12.22 */
				
				/* >>>>> add iruma m.takeishi '03.12.22 */
				c33_PutsTraceString( szStatusBuf );
				/* <<<<< add iruma m.takeishi '03.12.22 */

			}
			g_iStatusBufCnt = 0;		/* clear buffer memory access(r/w) count */
		}

		/* >>>>> delete iruma m.takeishi '03.12.22 */
		/*
		c33_PutsTraceString( szStatusBuf );
		*/
		/* <<<<< delete iruma m.takeishi '03.12.22 */
	}
	
	/*@ count up instruction number */
	g_ulInstNum++;
	
	return 0;
}

/* >>>>> add iruma m.takeishi '03.09.01 */
/****************************************************************************/
/* Format		 : fnGetExtType( unsigned short usCode )					*/
/* Arguments																*/
/*	unsigned short insn_word  : cc33 instruction code						*/
/* Return value  : EXT_NONE, EXT_L, EXT_RS, EXT_COND, EXT_OP_IMM,			*/
/*				   EXT_RS_OP_IMM											*/
/* Explanation	 : returns ext inst-type from c33 instruction code. 		*/
/****************************************************************************/
static int fnGetExtType( unsigned short usCode )
{
	int iRet;

	if( (usCode & 0xe000 ) == 0xc000 ) {
		// ext imm13
		iRet = EXT_L;
	}
	else if( fnCheckAdvancedMode() == C33_PRODUCT_ADV )
	{
		if ( ((usCode & 0xfff0 ) == 0x3b00) &&	
					 ((usCode & 0x000c) >= 0x4 ) &&
					 ((usCode & 0x000c) <= 0xc ) ) {
			// ext op,imm2
			iRet = EXT_OP_IMM;
		} else if ( ( (usCode & 0xff0f ) == 0x3b00 ) &&
					 ((usCode & 0x00f0) >= 0x40) &&
					 ((usCode & 0x00f0) <= 0xd0) ) {
			// ext cond
			iRet = EXT_COND;
		} else if ((usCode & 0xff0f ) == 0x3f00 ) {
			// ext %rs
			iRet = EXT_RS;
		} else if ( ((usCode & 0xff00 ) == 0x3f00 ) &&
					((usCode & 0xc ) >= 0x4 ) &&	
					((usCode & 0xc ) <= 0xc ) ) {
			// ext %rs, op,imm2
			iRet = EXT_RS_OP_IMM;
		} else {
			// not ext inst
			iRet = EXT_NONE;
		}

	} else {
		// not ext inst
		iRet = EXT_NONE;
	}

	return iRet;
}


/****************************************************************************/
/* Format		 : fnCheckInstCombi( int iLoop, int iRepeat,				*/
/*					   int iExt 											*/
/* Arguments																*/
/*	int iLoop	 : if loop inst check is done or not.						*/
/*	int iRepeat  : about repeat inst check. 								*/
/*	int iExt	 : about ext inst check.									*/
/* Return value  : error inst code if 0, no error ocuured.					*/
/* Explanation	 : Check that CPU can execute the current instruction in	*/
/*					 Current Simlator-state.								*/
/****************************************************************************/
static int fnCheckInstCombi( int iLoop, int iRepeat, int iExt )
{
	int iRet;
	int iCheckList;

	/* >>>>> add iruma m.takeishi '04.03.29 */
	unsigned long	   ulPsr;
	/* <<<<< add iruma m.takeishi '04.03.29 */

	iRet = 0;

#if 0	/* del T.Tazaki 2005/03/07 */
	/* bug fix : add T.Tazaki 2005/02/22 >>> */
	if( fnCheckAdvancedMode() != C33_PRODUCT_ADV )
	{
		return iRet;	/* no operation */
	}
	/* bug fix : add T.Tazaki 2005/02/22 <<< */
#endif

	/* Now loop, repeat nst-check isn't done. */

	iCheckList = iExt & CMB_EXT_MASK;
	/* add T.Tazaki 2005/03/09 >>> */
	if( fnCheckAdvancedMode() == C33_PRODUCT_PE )
	{
		 /* ext imm mode now? */
		if ( !(iCheckList & CMB_EXT_IMM_OK) ) {
			if ( stExtension.iType == EXT_HL || 
				 stExtension.iType == EXT_L) {
				vfnInitExtension();
			}
		}

		return iRet;	/* no operation */
	}
	/* add T.Tazaki 2005/03/09 <<< */

	/* ext cond mode now? */
	if ( !(iCheckList & CMB_EXT_COND_OK) ) {
		if ( stExtension.iTypeExtCond == EXTCOND_EXEC ) {
			iRet = 1;
		}
	}
	 /* ext %rs mode now? */
	if ( !(iCheckList & CMB_EXT_RS_OK) ) {
		if ( stExtension.iType == EXT_RS ) {
			iRet = 1;
		}
	}
	  /* ext %rs,op,imm mode now? */
	if ( !(iCheckList & CMB_EXT_RS_OP_IMM_OK) ) {
		/* >>>>> change iruma m.takeishi '04.03.22 */
		/*
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ||	
			 stExtension.iTypeExt == EXT_OP_IMM) {
		*/
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ) {
		/* <<<<< change iruma m.takeishi '04.03.22 */
			iRet = 1;
		}
	}
	 /* ext op,imm mode now? */
	if ( !(iCheckList & CMB_EXT_OP_IMM_OK) ) {
		/* >>>>> change iruma m.takeishi '04.03.22 */
		/*
		if ( stExtension.iTypeExt == EXT_RS_OP_IMM ||	
			 stExtension.iTypeExt == EXT_OP_IMM) {
		*/
		if ( stExtension.iTypeExt == EXT_OP_IMM) {
		/* <<<<< change iruma m.takeishi '04.03.22 */
			iRet = 1;
		}
	}
	 /* ext imm mode now? */
	if ( !(iCheckList & CMB_EXT_IMM_OK) ) {
		if ( stExtension.iType == EXT_HL || 
			 stExtension.iType == EXT_L) {
			iRet = 1;
		}
	}

	/* >>>>> add iruma m.takeishi '04.03.29 */
	ulPsr = gdbsim_fetch_register( PSR );
	/* is Repreat mode now? */
	if ( ulPsr & RM_FLAG) {
		if (( iRepeat & CMB_REPEAT_NG ) != 0 ) {
			iRet = 1;
		}
	}
	/* <<<<< add iruma m.takeishi '04.03.29 */

	/* add bug fix T.Tazaki 2005/03/14 >>> */
	if( iRet == 1 )
	{
		vfnInitExtension();
	}
	/* add bug fix T.Tazaki 2005/03/14 <<< */

	return iRet;
}
/* <<<<< add iruma m.takeishi '03.09.01 */

/* >>>>> add iruma m.takeishi '04.03.25 */
/****************************************************************************/
/* Format		 : static struct c33_map_info								*/
/*					 c33_GetMapInfo( CORE_ADDR ulAddr)						*/
/* Arguments																*/
/*	CORE_ADDR ulAddr : address whose map is you need						*/
/* Return value 															*/
/*	NULL		 : not found												*/
/*	otherelse	 : pointer to map											*/
/* Explanation	 : Get memory map area's pointer							*/
/****************************************************************************/
static struct c33_map_info * c33_GetMapInfo(CORE_ADDR ulAddr)
{
	struct c33_map_info *pstMapInfo;
	struct c33_map_info *pstRet;
	
	pstRet = NULL;
	
	if( c33TargetInfo.pMapInfo != NULL ){

		pstMapInfo = (struct c33_map_info *)c33TargetInfo.pMapInfo;
		
		while( pstMapInfo )
		{
			/* check if address renge */
			if(( ulAddr >= pstMapInfo->ulStartAddr ) && ( ulAddr <= pstMapInfo->ulEndAddr ))
			{
				pstRet = pstMapInfo;
				break;
			}
			pstMapInfo = (struct c33_map_info *)pstMapInfo->pMapNext;
		}
	}
	
	return pstRet;
}
/* <<<<< add iruma m.takeishi '04.03.25 */
